/*
 *  This file contains the RIO interface that is callable from FORTRAN
 *
 *  Author:   Freddie Akeroyd
 */

#include <dce/pthread.h>
#include <string.hxx>
#include "types.hxx"
#include "status.hxx"
#include "vms_status.hxx"
#include "daerpc.h"
#include "riolibrary.hxx"
#include "rio_interface.hxx"

static RioLibrary rioLibrary;

/*
 * these error codes are "VMS style"
 */
#define	RIO_OPERATION_FAILED		0
#define RIO_OPERATION_SUCCESSFUL	1

#define FORTRAN_ERROR(__status) \
    if (__status.ok()) \
    { \
	return RIO_OPERATION_SUCCESSFUL; \
    } \
    else \
    { \
	__status.Report(); \
	return RIO_OPERATION_FAILED; \
    }

/*
 * see header file for explanation of function arguments
 */

fort_i4 RIO(fort_i4& bus_addr, fort_i2& word, fort_i2& device)
{
    Status status = rioLibrary.readIO(bus_addr, &word, 1);
    FORTRAN_ERROR(status);
}

fort_i4 WIO(fort_i4& bus_addr, fort_i2& word, fort_i2& device)
{
    Status status = rioLibrary.writeIO(bus_addr, &word, 1);
    FORTRAN_ERROR(status);
}

fort_i4 RMEM(fort_i4& bus_addr, fort_i2& word, fort_i2& device)
{
    Status status = rioLibrary.readMem(bus_addr, &word, 1);
    FORTRAN_ERROR(status);
}

fort_i4 WMEM(fort_i4& bus_addr, fort_i2& word, fort_i2& device)
{
    Status status = rioLibrary.writeMem(bus_addr, &word, 1);
    FORTRAN_ERROR(status);
}

fort_i4 RIOD(fort_i4& bus_addr, fort_i2* buffer, fort_i4& nwrds, fort_i2& device)
{
    Status status = rioLibrary.readIO(bus_addr, buffer, nwrds);
    FORTRAN_ERROR(status);
}

fort_i4 WIOD(fort_i4& bus_addr, fort_i2* buffer, fort_i4& nwrds, fort_i2& device)
{
    Status status = rioLibrary.writeIO(bus_addr, buffer, nwrds);
    FORTRAN_ERROR(status);
}

fort_i4 RMEMD(fort_i4& bus_addr, fort_i2* buffer, fort_i4& nwrds, fort_i2& device)
{
    Status status = rioLibrary.readMem(bus_addr, buffer, nwrds);
    FORTRAN_ERROR(status);
}

fort_i4 WMEMD(fort_i4& bus_addr, fort_i2* buffer, fort_i4& nwrds, fort_i2& device)
{
    Status status = rioLibrary.writeMem(bus_addr, buffer, nwrds);
    FORTRAN_ERROR(status);
}

fort_i4 open_dae_c(const char* dae_name)
{
    String cds_name("/.:/servers/");
    cds_name = cds_name + String(dae_name).lower() + String("_dae");
    Status status = rioLibrary.openDae(cds_name);
    FORTRAN_ERROR(status);
}    

fort_i4 OPEN_DAE(STRING& dae_name)
{
    String cds_name("/.:/servers/");
    cds_name = cds_name + String(dae_name.value).lower() + String("_dae");
    Status status = rioLibrary.openDae(cds_name);
    FORTRAN_ERROR(status);
}

fort_i4 RESET_DAE(fort_i4& nn)
{
    Status status = rioLibrary.resetDae(nn);
    FORTRAN_ERROR(status);
}

fort_i4 CLOSE_DAE()
{
    Status status = rioLibrary.closeDae();
    FORTRAN_ERROR(status);
}

fort_i4 RMEMVI(fort_i4& bus_addr, fort_i4* buffer, fort_i4& length, fort_i2& device, fort_i4& wrd_len)
{
    Status status = rioLibrary.readMemVI(bus_addr, buffer, length, wrd_len);
    FORTRAN_ERROR(status);
}

// crpt_section=0 -> all if it
// see crptsect.def for codes
fort_i4 READ_CRPT(fort_i4*& start, fort_i4& crpt_section, fort_i4& size_in_words)
{
    Status status = rioLibrary.readCrpt(start, crpt_section, size_in_words);
    FORTRAN_ERROR(status);
}
