#ifndef RIO_LIBRARY_HXX
#define RIO_LIBRARY_HXX

//+++
//
// CLASS: RioLibrary
//
// CLASS DESCRIPTION: 
//
// The RIO interface called form Fortran
// buffer = pointer to area of nwrds 16bit words to store the data
// bus_addr = the multibus address
//
//---
 
#include "daerequestlist.hxx"

class RioLibrary
{
private:
      // DESCRIPTION: Records whether we are currently connected to a DEA or not
      Boolean connected;
      // DESCRIPTION: The current DAE being accessed, as given by the last open() command
      String currentDae;
      // DESCRIPTION: RPC binding handle
      handle_t daeBinding_h;
      dae_context_handle daeContext_h;

      static int MAX_IO_WORDS;
      static int MAX_MEM_WORDS;

      DaeRequestList* requestList;

      global_section_t *global_section;
      // DESCRIPTION: Read data from DAE memory
      //  PRECOND: 
      //  POSTCOND: 
public:
      Status readMem (fort_i4 bus_addr, fort_i2* buffer, fort_i4 nwrds);
      // DESCRIPTION: Read data from DAE IO space
      //  PRECOND: 
      //  POSTCOND: 
      Status readIO (fort_i4 bus_addr, fort_i2 *buffer, fort_i4 nwrds);
      // DESCRIPTION: Write to DAE memory
      //  PRECOND: 
      //  POSTCOND: 
      Status writeMem (fort_i4  bus_addr, fort_i2 *buffer, fort_i4 nwrds);
      // DESCRIPTION: Write to IO space on DAE
      //  PRECOND: 
      //  POSTCOND: 
      Status writeIO (fort_i4 bus_addr, fort_i2 *buffer, fort_i4 nwrds);
      // DESCRIPTION: Open access to a specific DAE
      //  PRECOND: No DAE must be  currently open
      //  POSTCOND: 
      Status openDae (const char *dae_name);
      // DESCRIPTION: terminate a connection with the dae
      //  PRECOND: There must be a DAE currently open
      //  POSTCOND: 
      Status closeDae ();
      // DESCRIPTION: Default constructor
      //  PRECOND: 
      //  POSTCOND: 
      Status resetDae (fort_i4 nn);
       RioLibrary ();
      // DESCRIPTION: Read memory aligned on longwords ... buffer(length)
      //  PRECOND: Only works for 32 bit alignment
      //  POSTCOND: 
      Status readMemVI (fort_i4 bus_addr, fort_i4* buffer, fort_i4 length, fort_i4 wrd_len);
      Status readCrpt(fort_i4*& start, fort_i4 crpt_section, fort_i4& size_in_words);
private:
      Status sendRequests();
      Status clearRequests();
      Status useIO(fort_i4 bus_addr, fort_i2 *buffer, fort_i4 nwrds, TransferMode direction);
      Status useMem(fort_i4 bus_addr, fort_i2 *buffer, fort_i4 nwrds, TransferMode direction);
};

#endif	/* RIO_LIBRARY_HXX */ 
