#ifndef USEFUL_MACROS_HXX
#define USEFUL_MACROS_HXX

/*
 *  when passed an object of class Status, immediately return if
 *  it contains an error message
 */
#define	RETURN_IF_ERROR(__status) \
    if (!__status.ok()) { return __status; }

/*
 *  call this after using "new" and it will return with a Status object
 *  if the new failed. For it to work you must have an OUT_OF_MEMORY
 *  code defined in the edescript header file. Something like:
 *
 *   edescrip(   OUT_OF_MEMORY,          "Cannot allocate space for @ in @",                 ERROR   )
 */
#define NEW_CHECK(__pointer) \
    if (__pointer == 0) { return Status(OUT_OF_MEMORY, FCODE, #__pointer, __FILE__); } 


// #define ERROR_CHECK(__status, __text) \
//    if (__status != error_status_ok) error_exit(__status, __text)

/*
 *  For it to work you must have an RPC_ERROR
 *  code defined in the edescript header file. Something like:
 *
 *   edescrip(   RPC_ERROR,          "Rpc Error: @ - @",                 ERROR   )
 */

#define RETURN_RPC_STATUS(__status, __message) \
    { \
	if (__status != rpc_s_ok) \
	{ \
	    idl_char error_text[dce_c_error_string_len]; \
	    int dummy; \
	    dce_error_inq_text(__status, error_text, &dummy); \
	    /* cerr << __message << ": " << (dummy == 0 ? (char*)error_text : "error in dce_error_inq_text") << endl;*/ \
	    return Status(RPC_ERROR, FCODE, __message, (char*)error_text); \
	} \
    }

#define APPEND_RPC_STATUS(__status_object, __rpc_status, __message) \
    { \
	if (__rpc_status != rpc_s_ok) \
	{ \
	    idl_char error_text[dce_c_error_string_len]; \
	    int dummy; \
	    dce_error_inq_text(__rpc_status, error_text, &dummy); \
	    cerr << __message << ": " << (dummy == 0 ? (char*)error_text : "error in dce_error_inq_text") << endl; \
	    __status_object.Append(RPC_ERROR, FCODE, __message, (char*)error_text); \
	} \
    }
/*
 *  For it to work you must have an PTHREAD_ERROR
 *  code defined in the edescript header file. Something like:
 *
 *   edescrip(   PTHREAD_ERROR,          "Pthread error: @ - @",                 ERROR   )
 */

#define RETURN_PTHREAD_STATUS(__status, __message) \
	if (__status == -1) \
	{ \
	    /* return Status(PTHREAD_ERROR, FCODE, __message, sys_errlist[errno]); */ \
	    perror(__message); \
	    return Status(PTHREAD_ERROR, FCODE, __message, " "); \
	}

// #define ERROR_CHECK(__status, __mess)	if (__status == -1) perror(__mess)

#define	PRINT_RPC_ERROR(__stream, __status, __message) \
    { \
	if (__status != rpc_s_ok) \
	{ \
	    idl_char error_text[dce_c_error_string_len]; \
	    int dummy; \
	    dce_error_inq_text(__status, error_text, &dummy); \
	    __stream << __message << ": " << (dummy == 0 ? (char*)error_text : "(error in dce_error_inq_text)") << "\n"; \
	} \
    }

#endif /* USEFUL_MACROS_HXX */
