c	RCW 12-12-85
c	OpenGenie function MONITOREFF.FOR
c	Corrects incident spectrum for detector/monitor efficiency.
c	Assume detector/monitor is a PLD scintillator.
c
c........Values for the PLD scintillator detector
c........Values for December 1984 monitor
c........ ie. monitor is 1% efficient
c***********************************************************************
c	modified: Toby Perring 3/8/93
c
c	1) attenuation coefficient for scintillators now 8.3 cf 8.3e-03
c	   Now agrees with CRISP monitor correction, also written by RC Ward,
c	  and my own programs. Data checked with Nigel Rhodes June 1993,
c	  who informed me that all bead detectors were made from the same
c	  Li glass block and therefore have the same doping level.
c	2) ratio of scintillator area to total area calculated using
c	   0.25mm beads spaced 7mm apart -> 1.27e-3 cf 1.325e-2
c	   Some uncertainty whether 0.25mm or 0.5mm beads. Use 0.25mm. It
c	   makes little difference as at IRIS energies the beads are close
c	   to black.
c	
*******************************************************************************
c
c	Adapted for OpenGenie by Phil Marshall 7/96
c
*******************************************************************************
c
	SUBROUTINE MONITOREFF(w1_get,w1_put)
c
	external w1_get,w1_put
	integer lpt,lpt_max
        parameter (lpt_max=6010)
        include 'genie_modules.inc'
	real xin(lpt_max),yin(lpt_max),ein(lpt_max),yout(lpt_max),eout(lpt_max)
	real unt,zz
c
c *** check library version
        IF (MODULE_VERSION_OK(GENIE_MAJOR, GENIE_MINOR) .EQ. 0) RETURN
C
	call module_get_int(w1_get,'ntc',lpt)
	call module_get_real_array(w1_get,'x',xin,lpt)
        call module_get_real_array(w1_get,'y',yin,lpt)
	call module_get_real_array(w1_get,'e',ein,lpt)
c
	unt= 1.276e-03		! ratio of scintillator to total area
				! [0.25mm cubes spaced at 7mm]
	zz = 0.025		! thickness(cm) of scintillator
c
	do int1= 1,lpt
	 yout(int1)= yin(int1)/(unt*(1-exp(-8.3*zz*xin(int1))))
	 eout(int1)= ein(int1)/(unt*(1-exp(-8.3*zz*xin(int1))))
	enddo
c
	call module_put_real_array(w1_put,'y',yout,lpt)  
	call module_put_real_array(w1_put,'e',eout,lpt)  	
	call module_information('Monitor efficiency correction done.')
c
	return
	end
