/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef GENIE_FUNCTIONS_H


#define GENIE_FUNCTIONS_H

#include "g_types.h"
#include "motifdriver_functions.h"

// forward declaration
    class String;	
    class GenieWorkspace;	

// ---
// *** initialisation functions
//
    void init_defaults();

// ---
// *** error handling functions
//
    int error_status();
    void clear_errors();
    const char *error_object(int n);				// n = error number (1 to 6)
    const char *error_failure(int n);
    const char *error_solution(int n);
    const char *error_message(int n);
    extern "C" void error_add(const char *obj, const char *fail, const char *soln);

// ---
// *** functions for device windows
//
    long open_device(const char *device_type,			// device type without "/"
		    const char *filename,			// filename for hardcopy ONLY
		    float width,				// display width in inches
		    float height,				// display height in inches
		    float ra,
		    const char *title, 
		    const char *icont,
		    funcPtrI exf,
		    funcPtr redrf,
		    const char *menutitle,
		    int n,
		    const char *menuitems,
		    funcPtrI menuf);

    void close_device();
    void clear_device();
    void select_device(long device_id);
    g_integer* allocate_colourtable(g_real* ra, g_real* ga, g_real* ba, 
		int len_ra, int& ncol_wanted, g_real contrast = 1.0, 
		int nbands = -1, g_real diffuse = 0.0, g_real shine = 1.0,
		g_real polish = 1.0);
    int deallocate_colourtable(g_integer* table, int ncol);
    int define_hls_colour(float hue, float lightness, float saturation);	// return colour index for new HLS value
    int define_rgb_colour(float r, float g, float b);				// return colour index for new RGB value
    int query_rgb_colour(int index, g_real& r, g_real& g, g_real& b);		// 
    int define_named_colour(const String& name);				// return index for named colour. Colour name
										// must exist in X11 "rgb.txt" file

// ---
// *** functions for picture
//
    int create_picture();				// Returns picture number, -1 on error
    void delete_pictures();
    void redrawcurrentpicture();
    int select_picture(int pictnum);
    void redraw_picture(int pictnum,int reset);		// counting backwards from current picture (0)
    void picture_display(float *);			// array[2] of world co-ordinates
    void get_picture_cursor(int view_num,		// window number in creation order (start at 1)
			    float *world,		// world[2] are window world co-ordinates
			    float *device,		// ** device[2] are the device co-ordinates
			    char  *letter);		// ** address of char letter (key typed)
    int getpictnum();					// returns current picture number
    void undraw(int);
    void undraw_all();
    void window_delete(int window);			// window 1 is the first created
    void undo_undraw();					// undo works on the last undraw (nested with undraw,
		    					//	undraw_all & window_delete)

// ---
// *** rounding functions
//
    float round_up(double);
    float round_down(double);
    void wait();
// ---
// *** functions for viewport
// ** Return a window ID
//
    int create_window_unscaled(float xDlower,		// lower X device co-ordinate
			       float yDlower,		// lower Y device co_ordinate
			       float xDupper,		// upper X device co-ordinate
			       float yDupper,		// upper y device co-ordinate
			       int colour);		// colour index for window

    int new_window_unscaled(float xDlower,		// lower X device co-ordinate
			       float yDlower,		// lower Y device co_ordinate
			       float xDupper,		// upper X device co-ordinate
			       float yDupper,		// upper y device co-ordinate
			       int colour,		// colour index for window
			       float& tmp1,
			       float& tmp2,
			       float& tmp3,
			       float& tmp4,
			       g_objectid objid);	// Array in which to place the Object ID

    int create_window_scaled(float xDlower,		// lower X device co-ordinate
			      float yDlower,		// lower Y device co-ordinate
		              float xDupper,		// upper X device co-ordinate
		              float yDupper,		// upper Y device co-ordinate
		              double *xarray,		// array[=>npts] of X values
		              double *yarray,		// array[=>npts] of Y values
		              double *earray,		// array[=>npts] of errors in Y
			      int npts,			// number of points
			      int colour,		// line colour index
			      int scaling_min_mode,	// (ymax)	0 = use absolute values
			      int scaling_max_mode,	// (ymin)	1 = + 10% to absolute values
			      float LXMIN,			//		2 = + 10% and round to 2 sig figs
			      float LXMAX,			//		3 = round to 2 sig figs
			      float LYMIN,			//		4 = force value to zero
			      float LYMAX);

    int new_window_scaled(float xDlower,		// lower X device co-ordinate
			      float yDlower,		// lower Y device co-ordinate
		              float xDupper,		// upper X device co-ordinate
		              float yDupper,		// upper Y device co-ordinate
		              double *xarray,		// array[=>npts] of X values
		              double *yarray,		// array[=>npts] of Y values
		              double *earray,		// array[=>npts] of errors in Y
			      int npts,			// number of points
			      int colour,		// line colour index
			      int scaling_min_mode,	// (ymax)	0 = use absolute values
			      int scaling_max_mode,	// (ymin)	1 = + 10% to absolute values
			      float LXMIN,		//		2 = + 10% and round to 2 sig figs
			      float LXMAX,		//		3 = round to 2 sig figs
			      float LYMIN,		//		4 = force value to zero
			      float LYMAX,
			      g_objectid objid);	// Array in which to place the Object ID

    int create_window_multiplot(float xDlower,		// lower X device co-ordinate
			         float yDlower,		// lower Y device co-ordinate
		                 float xDupper,		// upper X device co-ordinate
		                 float yDupper,		// upper Y device co-ordinate
				 int colour,		// line colour index
			         int scaling_min_mode,	// (ymax)0 = use absolute values
			         int scaling_max_mode);// (ymin)1 = + 10% to absolute values
				 		//2 = + 10% and round to 2 sig figs
				 		// 3 = round to 2 sig figs
				 		// 4 = force value to zero
				 


  int new_window_multiplot (float xDlower,       // lower X device co-ordinate
                          float yDlower,         // lower Y device co-ordinate
                          float xDupper,         // upper X device co-ordinate
                          float yDupper,         // upper Y device co-ordinate
                          int colour,            // line colour index
                          int scaling_min_mode,  // (ymax)0 = use absolute values
                          int scaling_max_mode,  // (ymin)1 = + 10% to aboslute values
                          float& xmin,            //2 = + 10% and round to 2 sig figs
                          float& xmax,            // 3 = round to 2 sig figs
                          float& ymin,            // 4 = force value to zero
			  float& ymax,
			  g_objectid objid);	 // Array in which to place the Object ID

    void get_window_cursor(float *world,	// world[2] window co-ordinates
			    float *device,	// ** device[2] returned device co-ordinates 
			    char *letter); 	// ** address of a char letter (key typed)

    void get_cursor(float *world,
		    float *device,
		    char *letter,
		    g_objectid id);


// ---
// *** functions for all drawable objects
//
    void draw_axes(int colour,			// line colour
		   float line_width,		// line width (should be whole number)
		   float text_height,		// relative to: 1.0 = (1/40 display height)
		   int quality,			// text font (1 to 4)
		   int xlog,			// 0 = linear, 1 = log
		   int ylog,			// 0 = linear, 1 = log
		   int xnum,			// show x numbers (1=yes)?
		   int ynum,			// show y numbers (1=yes)?
		   int yhoriz,			// Draw Y numbers horizontally?
		   int type);

    void new_axes(int colour,                   // line colour
                   float line_width,            // line width (should be whole number)
                   float text_height,           // relative to: 1.0 = (1/40 display height)
                   int quality,                 // text font (1 to 4)
                   int xlog,                    // 0 = linear, 1 = log
                   int ylog,                    // 0 = linear, 1 = log
                   int xnum,                    // show x numbers (1=yes)?
                   int ynum,                    // show y numbers (1=yes)?
                   int yhoriz,                  // Draw Y numbers horizontally?
		   int type,
		   g_objectid objid);		// array in which to place the Object ID

    void draw_text(float xpos,			// world ordinate (may be outside window)
		    float ypos,			// world ordinate
		    const char* words,		// text string
		    float size,			// relative to: 1.0 = (1/40 display height)
		    float angle,		// in degrees anticlockwise from positive X
		    int colour,			// colour index
		    int quality,		// text font (1 to 4)
		    float linewidth);		// width of lines!

      void new_text(float xpos,			// world ordinate (may be outside window)
		    float ypos,			// world ordinate
		    const char* words,		// text string
		    float size,			// relative to: 1.0 = (1/40 display height)
		    float angle,		// in degrees anticlockwise from positive X
		    int colour,			// colour index
		    int quality,		// text font (1 to 4)
		    float linewidth,		// width of lines!
		    float justify,		// 0.0=left, 0.5=centre etc
		    g_objectid objid);		// Array in which to place the Object ID
   

    void draw_histogram(int npts,		// number of points
			double *xarray,		// array[=>npts] of X values
			double *yarray,		// array[=>npts] of Y values
			int colour,		// line colour
			int bin_centre,		// 0 = X value is bin leading edge, 1 = bin centre
			int type,		// linetype (1=cont,2=dash,3=dot-dash,4=dot)
			float linewidth);	// should be whole number

    void new_histogram(int npts,		// number of points
			double *xarray,		// array[=>npts] of X values
			double *yarray,		// array [=>npts]of Y values
			int colour,		// line colour
			int bin_centre,		// 0 = X value is bin leading edge, 1 = bin centre
			int type,		// linetype (1=cont, 2=dash, 3=dot-dash, 4=dot)
			float linewidth,	// should be whole number
			g_objectid objid);	// Array in which to place the Object ID

    void create_multiplot(int nhist,		// number of histograms
			double *xarray,		// array[>=longest histogram] of X values
			int npoints,		// array[nhist] of Y value lengths
			int chmin,		// first channel to plot
			int chmax,		// last channel to plot
			float ygap,		// Y offset between histograms
			int colour,		// colour index
			int bin,		// 0 = X value is bin leading edge, 1 = bin centre
			int type,		// linetype (1=cont,2=dash,3=dot-dash,4=dot)
			float linewidth);    	// should be whole number

    void add_spectra(const double *yarray,int length);	// add spectra to a multiplot
    void add_spectra(const int *yarray,int length);		// add spectra to a multiplot
    
    void make_multiplot(g_objectid objid);

    void draw_multiplot();

    void create_2d_plot(int *Xdims,		// array of X dimensions	
		int *Ydims,			// array of Y dimensions (NOT POINTS)
		int *Zdims,			// array of Z dimensions
		double *Conts,			// array of contour levels
		int Nconts,			// number of contour levels
		int *Shades,			// array of colours
		int type,			// linetype
		float linewidth);		// linewidth

    void add_2d_spectra(double *x,		// X data 
		 double *y,			// Y data
		 double *z,			// Z data
		 int hist);			// number of current histogram
    
    void draw_2d_plot();
    
    void tdfunct(int *, float *, float *, float *);

    void draw_plot(int npts,			// number of points in plot
		    double *xarray,		// array[npts] of X values
		    double *yarray,		// array[npts] of Y values
		    int colour,			// colour index
		    int type,			// linetype (1=cont,2=dash,3=dot-dash,4=dot)
		    float linewidth);		// should be whole number

    void new_plot(int npts,			// number of points in plot
		    double *xarray,		// array[npts] of X values
		    double *yarray,		// array[npts] of Y values
		    int colour,			// colour index
		    int type,			// linetype (1=cont, 2=dash, 3=dot-dash, 4=dot)
		    float linewidth,		// should be whole number
		    g_objectid objid);		// Array in which to place the Object ID


    void draw_markers(int npts,			// number of points in plot
			double *xarray,		// array[npts] of X values
			double *yarray,		// array[npts] of Y values
			int colour,		// colour index
			int symbol,		// (1=.,2=+,3=*,4=o,5=X.....)
			float charsize,		// relative to: 1.0 = (1/40 display height)
			float linewidth);	// should be a whole number

    void new_markers(int npts,                  // number of points in plot
                        double *xarray,         // array[npts] of X values
                        double *yarray,         // array[npts] of Y values
                        int colour,             // colour index
                        int symbol,             // (1=.,2=+,3=*,4=o,5=X.....)
                        float charsize,         // relative to: 1.0 = (1/40 display height)
                        float linewidth,        // should be a whole number
			g_objectid objid);	// array in which to place the Object ID

    void draw_errors(int npts,			// number of points in plot
			double *xarray,		// array[npts] of X values
			double *yarray,		// array[npts] of Y values
			double *earray,		// array[npts] of delta Y
			int colour,		// colour index
			int angle,		// (1=vertical 0=horizontal)
			float linewidth);	// should be a whole number

    void new_errors(int npts,			// number of points in plot
			double *xarray,		// array[npts] of X values
			double *yarray,		// array[npts] of Y values
			double *earray,		// array[npts] of delta Y
			int colour,		// colour index
			int angle,		// (1 = vertical 0 = horizontal)
			float linewidth,	// should be a whole number
			g_objectid objid);	// Array in which to place the Object ID

    void draw_graticule(int colour,		// colour index
		        float line_width,	// should be whole number
		        int xlog,		// 0 = linear, 1 = log
		        int ylog,		// 0 = linear, 1 = log
			float num_size,
			int xnum,
			int ynum,
			int yhoriz,
			int quality,
			int type);

    void new_graticule(int colour,              // colour index
                        float line_width,       // should be whole number
                        int xlog,               // 0 = linear, 1 = log
                        int ylog,
			float num_size,         // 0 = linear, 1 = log
			int xnum,
			int ynum,
			int yhoriz,
			int quality,
			int type,
			g_objectid objid);	// Array in which places the Object ID	 
    
    void draw_labels(const char* xlabel,	// text of X axis label
		    const char* ylabel,		// text of Y axis label
		    int colour,			// colour index
		    int quality,		// text font (1 to 4)
		    float charsize,		// 1.0 = (1/40 display height) * 0.8
		    float linewidth,		// width of lines!
		    int xvert,			// draw x label vertically?
		    int yvert);			// draw y label vertically?

    void new_labels(const char* xlabel,         // text of X axis label
                    const char* ylabel,         // text of Y axis label
                    int colour,                 // colour index
                    int quality,                // text font (1 to 4)
                    float charsize,             // 1.0 = (1/40 display height) * 0.8
                    float linewidth,            // width of lines!
                    int xvert,                  // draw x label vertically?
                    int yvert,                  // draw y label vertically?
		    g_objectid objid);		// Array in which to place the Object ID

    void draw_title(const char* title,		// text for title
		    int colour,			// line colour index
		    int quality,		// text font (1 to 4)
		    float charsize);		// 1.0 = (1/40 display height) * 1.3

    void new_title(const char* title,           // text for title
                    int colour,                 // line colour index
                    int quality,                // text font (1 to 4)
                    float charsize,             // 1.0 = (1/40 display height) * 1.3
    		    g_objectid objid);		// Array in which to place the Object ID   
 
    void draw_box(float xlower,			// X lower world co-ordinate
		    float ylower,		// Y lower world co-ordinate
		    float xupper,		// X upper world co-ordinate
		    float yupper,		// Y upper world co-ordinate
		    int colour,			// line and fill colour index
		    float linewidth,		// should be whole number
		    int solid);			// 0 = hollow, 1 = filled

    void new_box(float xlower,			// X lower world co-ordinate
		    float ylower,		// Y lower world co-ordinate
		    float xupper,		// X upper world co-ordinate
		    float yupper,		// Y upper world co-ordinate
		    int colour,			// line and fill colour index
		    float linewidth,		// should be whole number
		    int solid,			// 0 = hollow, 1 = filled
                    g_objectid objid);

    void draw_polygon(double x[], 
			double y[], 
			int numsides, 
			int colour, 
			float linewidth, 
			int solid);


    void new_polygon(double x[], 
			double y[], 
			int numsides, 
			int colour, 
			float linewidth, 
			int solid, 
			g_objectid objid);

    void draw_line(float xstart,		// world co-ordinate
		    float ystart,		// world co-ordinate
		    float xend,			// world co-ordinate
		    float yend,			// world co-ordinate
		    int colour,			// colour index
		    int linetype,		// linetype (1=cont,2=dash,3=dot-dash,4=dot)
		    float linewidth);		// should be a whole number

    void new_line(float xstart,                 // world co-ordinate
                    float ystart,               // world co-ordinate
                    float xend,                 // world co-ordinate
                    float yend,                 // world co-ordinate
                    int colour,                 // colour index
                    int linetype,               // linetype (1=cont,2=dash,3=dot-dash,4=dot)
                    float linewidth,            // should be a whole number
                    g_objectid objid);		// Array in which to place the Object ID
 

    void draw_curve(float *xfunct,		// address of function evaluating X = f(T)
		    float *yfunct,		// address of function evaluating Y = g(T)
		    int npts,			// number of points to plot
		    float minT,			// min value of T
		    float maxT,			// max value of T
		    int colour,			// colour index
		    int linethickness);		// should be whole number

    void draw_contour(g_real* values, 
		      int nx_values, 
		      int ny_values, 
		      g_real* tr, 
		      g_real* contours, 
		      int ncont,
		      int line_style, 
		      int line_colour, 
		      g_real line_thickness);
    
    int new_contour(g_real* values,
                      int nx_values,
                      int ny_values,
                      g_real* tr,
                      g_real* contours,
                      int ncont,
                      int line_style,
                      int line_colour,
                      g_real line_thickness,
		      g_objectid objid);	// Array in which to place the object ID


    void draw_contour_function(g_real* values, 
				int nx_values, 
				int ny_values, 
				void* func_ptr, 
				g_real* contours, 
				int ncont,
		               	int line_style, 
				int line_colour, 
				g_real line_thickness);

    int new_contour_function(g_real* values,
                                int nx_values,
                                int ny_values,
                                void* func_ptr,
                                g_real* contours,
                                int ncont,
                                int line_style,
                                int line_colour,
                                g_real line_thickness,
				g_objectid objid);


    void draw_contour_array(g_real* values, 
			    int nx_values, 
			    int ny_values, 
			    g_real* wx, 
			    int* wx_dims, 
			    int wx_ndims, 
			    g_real* wy, 
			    int* wy_dims, 
			    int wy_ndims, 
			    g_real* contours, 
			    int ncont, 
			    int line_type, 
			    int line_colour, 
			    g_real line_thickness);

    int new_contour_array(g_real* values,
                            int nx_values,
                            int ny_values,
                            g_real* wx,
                            int* wx_dims,
                            int wx_ndims,
                            g_real* wy,
                            int* wy_dims,
                            int wy_ndims,
                            g_real* contours,
                            int ncont,
                            int line_type,
                            int line_colour,
                            g_real line_thickness,
			    g_objectid objid);

    g_real* get_contour_levels(g_real* values, 	
				int nvalues, 
				int ncont, 
				int itype);

    void new_label_contour(const char* label, 
			   int start, 
			   int end, 
			   int step, 
			   int intval, 
			   int minint, 
			   int colour, 
			   int font, 
			   g_real height, 
			   g_real line_width,
			   g_objectid contourid,
			   g_objectid);

    void draw_label_contour(const char* label,
                           int start,
                           int end,
                           int step,
                           int intval,
                           int minint,
                           int colour,
                           int font,
                           g_real height,
                           g_real line_width,
                           g_objectid contourid);

    g_real* get_contour_tr_matrix(int nx, 
				  int ny);


    int draw_cellarray_array(g_real* values, 
			     int nx_values, 
			     int ny_values, 
			     g_real* wx, 
			     int* wx_dims, 
			     int wx_ndims, 
			     g_real* wy, 
			     int* wy_dims, 
			     int wy_ndims, 
			     g_real valmax, 
			     g_real valmin, 
			     int itype, 
			     int imode, 
			     g_integer* table, 
			     int ntab);

    int new_cellarray(g_real* values, int nx_values, int ny_values,
                           g_real valmax, g_real valmin,
                           g_real* tr_matrix, int itype, int mode,
                           g_integer* table, int ntab, g_objectid objid);

    int new_cellarray_array(g_real* values, 
                             int nx_values, 
                             int ny_values, 
                             g_real* wx, 
                             int* wx_dims, 
                             int wx_ndims,
                             g_real* wy, 
                             int* wy_dims, 
                             int wy_ndims, 
                             g_real valmax, 
                             g_real valmin, 
                             int itype, 
                             int imode,
                             g_integer* table, 
                             int ntab, 
			     g_objectid objid);


    int draw_cellarray_function(g_real* values, 
				int nx_values, 
				int ny_values, 
				g_real valmax, 
				g_real valmin, 
				void* func_ptr, 
				int itype, 
				int imode, 
				g_integer* table, 
				int ntab);

    int new_cellarray_function(g_real* values, 
                                int nx_values, 
                                int ny_values, 
                                g_real valmax, 
                                g_real valmin,
                                void* func_ptr, 
                                int itype, 
                                int imode, 
                                g_integer* table, 
                                int ntab,
				g_objectid objid);

    int create_window_two_d(g_real dev_xmin, 
				g_real dev_xmax, 
				g_real dev_ymin, 
			    g_real dev_ymax, 
				g_real* x, 
				int nx, 
				g_real* y, 
			    int ny, 
				int colour);

    int new_window_two_d(g_real dev_xmin, 
			g_real dev_xmax, 
			g_real dev_ymin, 
                            g_real dev_ymax, 
			g_real* x, 
			int nx, 
			g_real* y, 
                        int ny, 
			int colour,
			float &LXMIN,
			float &LXMAX,
			float &LYMIN,
			float &LYMAX, 
			g_objectid objid);

    int new_window_three_d(g_real dev_xmin, 
			g_real dev_xmax, 
			g_real dev_ymin, 
            g_real dev_ymax, 
			int colour,
			float &LXMIN,
			float &LXMAX,
			float &LYMIN,
			float &LYMAX, 
            const g_real* eye,
            const g_real* light,
			g_objectid objid);

    int draw_cellarray_wedge(int objnum, int mode);
   
 
    void draw_contour_function(g_real* values, 
				int nx_values, 
				int ny_values, 
				void* func_ptr, 
				g_real* contours, 
				int ncont,
		               int line_style, 
				int line_colour, 
				g_real line_thickness);

    int new_cellarray_wedge(g_objectid cellid, 
			    int mode, 
			    g_objectid objid);
    
    void draw_contour_array(g_real* values, 
				int nx_values, 
				int ny_values, 
			    g_real* wx, 
				int* wx_dims, 
				int wx_ndims, 
				g_real* wy, 
				int* wy_dims, 
				int wy_ndims, 
			    g_real* contours, 
				int ncont, 
				int line_type, 
				int line_colour, 
				g_real line_thickness);


//
//    g_real* get_contour_levels(g_real* values, int nvalues, int ncont, int itype);

//

    void label_contour(const char* label, int start, int end, int step, int intval, int minint, 

			int colour, int font, g_real height, g_real line_width);

//

//    g_real* get_contour_tr_matrix(int nx, int ny);

//
// Functions to access objects on the display list

    int query_object(GenieWorkspace& info, g_objectid objid);

    int display_object(g_objectid objid);

    int display_graphical_object(g_objectid objid, void* oop);

    int remove_object(g_objectid objid);

    int set_current_object_tag(const char* tag);

    String current_device_type();

// Return 0 on success, -1 if unknown colourtable name
    int predefined_colourtable(const String& name, GenieWorkspace& wk);

int three_d_window_flush_bitmap(g_objectid objid, void* oop);


#endif /* GENIE_FUNCTIONS_H */
