/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include "g_types.h"
#include "Pginclude.h"
#include "Drawable_objects.h"
#include "ThreeDObject.h"
#include "ThreeDLattice.h"
#include "LatticePlane.h"
#include "sbroutines.h"

LatticePlane::LatticePlane(const g_real* normal, const g_real* apoint, pg_int edge, 
        pg_int transparency, const g_real* lattice, const g_real* density, 
        pg_int n1, pg_int n2, pg_int n3, const g_real* eye, const g_real* light, const
        ThreeDObjectColours& table) : ThreeDLattice(lattice, density, n1, n2, n3, 
        eye, light, table), m_edge(edge), m_transparency(transparency)
{
    int c;
    m_normal = new pg_float[3];
    m_apoint = new pg_float[3];
    for (c=0;c<3;c++)
    {
        m_normal[c]=F(normal[c]);
        m_apoint[c]=F(apoint[c]);
    }
}

LatticePlane::~LatticePlane()
{
    delete []m_normal;
    delete []m_apoint;
}

void LatticePlane::display()
{
    pg_int tran;
    tran = m_transparency / 25;
    if ((tran < 0) || (tran > 3))
        gerr << "IsoSurface: Invalid transparency value " << m_transparency <<
            "%" << endl;
    else
        sbcpln(m_eye, m_lattice, m_table.ci_low(), m_table.ci_high(), m_light, 
            m_normal, m_apoint, m_edge, tran);
    sbccrt(1,m_table.ci_low(),m_table.ci_high(),m_table.table());
 }
