/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include "g_types.h"
#include "Pginclude.h"
#include "Drawable_objects.h"
#include "Plane.h"
#include "sbroutines.h"

Plane::Plane(const g_real* vertices, pg_int nvertices, pg_int transparency, const g_real* 
    eye, const g_real* light, const ThreeDObjectColours& table) : ThreeDObject(eye, 
    light, table), m_nvertices(nvertices), m_transparency(transparency)
{
    int i;
    m_vertices = new pg_float[3 * nvertices];
    for (i=0;i<3*nvertices;i++)
        m_vertices[i]=F(vertices[i]);
}

Plane::~Plane()
{
	delete []m_vertices;
}

void Plane::display()
{
//    pg_float x0, y0, r0;
    pg_int tran;
    tran = m_transparency / 25;
    if (tran == 0)
        sbplan(m_eye,m_nvertices,m_vertices,m_table.ci_low(),
			m_table.ci_high(),m_light);
    else
    {
        if ((tran < 0) || (tran > 3))
            gerr << "PLANE: Invalid transparency value " << m_transparency <<
            "%" << endl;
        else
            sbplnt(m_eye,m_nvertices,m_vertices,m_table.ci_low(),
		m_table.ci_high(),m_light,tran);
    }
    sbccrt(1,m_table.ci_low(),m_table.ci_high(),m_table.table());
}
