/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include "g_types.h"
#include "ThreeDObjectColours.h"

ThreeDObjectColours::ThreeDObjectColours(pg_int ci_low, pg_int ci_high, pg_int
    shine, pg_int ncband, const g_integer* table) : m_ci_low(ci_low),
    m_ci_high(ci_high), m_shine(shine), m_ncband(ncband)
{
    int c;
    int ncol = m_ci_high - m_ci_low + 1;
    m_table = new pg_int[ncol];
    for(c=0;c<ncol;c++)
        m_table[c]=table[c];
}

ThreeDObjectColours::ThreeDObjectColours(const ThreeDObjectColours& c) :
    m_ci_low(c.m_ci_low), m_ci_high(c.m_ci_high), m_shine(c.m_shine),
    m_ncband(c.m_ncband)
{
    int i;
    int ncol = m_ci_high - m_ci_low + 1;
    m_table = new pg_int[ncol];
    for(i=0;i<ncol;i++)
        m_table[i]=c.m_table[i];
}

ThreeDObjectColours::~ThreeDObjectColours()
{
    delete[] m_table;
}

ThreeDObjectColours& ThreeDObjectColours::operator=(const ThreeDObjectColours&
c)
{
    if (this != &c)
    {
        m_ci_low = c.m_ci_low;
        m_ci_high = c.m_ci_high;
        m_shine = c.m_shine;
        int i;
        int ncol = m_ci_high - m_ci_low + 1;
        delete[] m_table;
        m_table = new pg_int[ncol];
        for(i=0;i<ncol;i++)
            m_table[i]=c.m_table[i];
    }
    return *this;
}

ostream& ThreeDObjectColours::print(ostream& os) const
{
    os << "*** ThreeDObjectColours ***" << endl;
    os << "Emulating colours in range " << m_ci_low << " to " << m_ci_high << endl;
    os << "Shine = " << m_shine << ", number of bands = " << m_ncband << endl;
    os << "Colour table to map to:" << endl;
    for(int i=0; i<m_ci_high-m_ci_low+1; i++)
    {
	os << m_ci_low+i << "->" << m_table[i] << (i % 5 ? " " : "\n");
    }
    return os;
}
