/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef THREEDROUTINES_H
#define THREEDROUTINES_H

#include "ThreeDObjectColours.h"

void new_plane(g_real* vertices, pg_int nvertices, pg_int transparency, 
    g_real* eye, g_real* light, const ThreeDObjectColours& table, g_objectid 
    objid);
void new_threeDText(const String& text, pg_int colour, const g_real* pivot, 
    g_real justify, const g_real* orient, g_real size, g_real* eye, g_real* 
    light, const ThreeDObjectColours& table, g_objectid objid);	        
void new_ellipsoid(const g_real* paxes, const g_real* centre, pg_int icline,
    g_real anglin, g_real* eye, g_real* light, const ThreeDObjectColours& table, 
    g_objectid objid);
void new_threeDLine(const g_real* end1, const g_real* end2, pg_int colour,
    pg_int dash, g_real* eye, g_real* light, const ThreeDObjectColours& table, 
    g_objectid objid);
void new_rod(const g_real* end1, const g_real* end2, g_real radius, pg_int nsides, 
     pg_int lend, g_real* eye, g_real* light, const ThreeDObjectColours& table,
      g_objectid objid);
void new_cone(const g_real* base, const g_real* apex, g_real radius, pg_int nsides, 
    g_real* eye, g_real* light, const ThreeDObjectColours& table, g_objectid 
    objid);
void new_ball(const g_real* centre, g_real radius, pg_int transparency, g_real*
    eye, g_real* light, const ThreeDObjectColours& table, g_objectid objid);
void new_isoSurface(g_real dsurf, pg_int transparency, const g_real* lattice, 
    const g_real* density, pg_int n1, pg_int n2, pg_int n3, g_real* eye,
    g_real* light, const ThreeDObjectColours& table, g_objectid objid);
void new_latticePlane(const g_real* normal, const g_real* apoint, pg_int edge, 
    pg_int transparency, const g_real* lattice, const g_real* density, 
    pg_int n1, pg_int n2, pg_int n3, g_real* eye, g_real* light, const 
    ThreeDObjectColours& table, g_objectid objid);
void new_latticeSlice(g_real dlow, g_real dhigh, const g_real* normal, const 
    g_real* apoint, pg_int edge, pg_int transparency, const g_real* lattice, 
    const g_real* density, pg_int n1, pg_int n2, pg_int n3, g_real* eye, 
    g_real* light, const ThreeDObjectColours& table, g_objectid objid);
void new_surface(g_real dlow, g_real dhigh, g_real dvert,
    const g_real* lattice, const g_real* density, pg_int n1, pg_int n2, 
    const g_real* eye, const g_real* light, const ThreeDObjectColours& table, g_objectid 
    objid);

#endif /* THREEDROUTINES_H */
