/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef COLOURMAP_HXX
#define COLOURMAP_HXX

#define GENIE_COLOUR_RANGE	255.0F

class Colour
{
    public:
        int r;		// in range 0 to GENIE_COLOUR_RANGE
        int g;
        int b;
	int refcount;		// number of devices who have allocated us
	int in_table;		// are we in a colour table?

        Colour() : r(0), g(0), b(0), refcount(0), in_table(0) {}
};

class ColourMap
{
    protected:
	pg_int m_ci_low;			// lowest colour index available
	pg_int m_ci_high;			// highest colour index available
	Colour* m_map;				// 

	int allocTable(g_integer* table, int ncol, const g_real* r, 
				const g_real* g, const g_real* b, int len, 
				g_real contrast, g_real diffuse, g_real shine, 
				g_real polish, int ncband);
	int allocColourShades(g_real rl, g_real gl, g_real bl, 
  		g_integer *shades, int nshades, g_real diffuse, g_real shine, 
				g_real polish);
	int findIndex(pg_float r, pg_float g, pg_float b, int in_table);

    public:
	ColourMap();
	~ColourMap();
	int freeColour(int index);
	int newRGBColour(pg_float r, pg_float g, pg_float b, int in_table = 0, int start = -1);
	int newHLSColour(pg_float h, pg_float l, pg_float s, int in_table = 0, int start = -1);
	int newNamedColour(const char* name, int in_table = 0, int start = -1);
	g_integer* newRGBColourTable(g_real* r, g_real* g, g_real* b, int len, 
				int& ncol_wanted, g_real contrast = 1.0,
		 		int ncband = -1, g_real diffuse = 0.0,
				g_real shine = 1.0, g_real polish = 1.0);

	int freeColourTable(g_integer* table, int ncol);
	int nextFreeIndex(pg_int start = -1);
	int queryColour(int index, g_real& r, g_real& g, g_real& b);
	int numFreeColours();
	void operator=(const ColourMap& colour_map);
	ColourMap(const ColourMap& cm);
	ColourMap(const ColourMap* pcm);
	void copyColours(const ColourMap& colour_map);
	void initialize();
	void install();
	ostream& print(ostream& os);
};

#endif /* COLOURMAP_HXX */
