/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include <string.h>
#include <gnustring.h>
#include "Pginclude.h"
#include "g_types.h"
#include "Drawable_objects.h"
#include "contour_plot.h"
#include "contour_labels.h"
#include "pg_coord_trans_func.h"

static const char* rcsid = "@(#)$Id: contour_labels.cxx,v 1.10 1997/04/01 18:44:45 faa Exp $";
USE_RCSID(rcsid);

ContourLabels::ContourLabels(ContourPlot* last_contour_plot, 
			     const char* label,
			     int start, 
			     int end, 
			     int step, 
			     int intval, 
			     int minint, 
			     int colour, 
			     int font, 
			     float height, 
			     float line_width) :
	
Drawable_objects((int)line_width, colour),m_plot(last_contour_plot), m_label(label), m_start(start),
		m_end(end), m_step(step), m_intval(intval), m_minint(minint), m_font(font), m_height(height)
{
    if (m_end == -1)
    {
	m_end = m_plot->m_ncontours;
    }
}

void ContourLabels::display()
{
    STRING label;
    pg_float level;
    pgscf(m_font);
    pgsch(m_height);
    pgsci(line_colour);
    pgslw(line_thickness);
    DrawableValues& dv = m_plot->m_values;
    PGCoordinateTransformFunction* pg_func = (PGCoordinateTransformFunction*)m_plot->function();
    pg_func->setPointers();
    pg_cont_lab_func_t func = pg_func->contourLabelFunction;
    for(int i=m_start; i<=m_end; i += m_step)
    {
	level = m_plot->m_contours[i-1];			// contour level to plot
	sprintf(label.spot, (const char*)m_label, level);	// translate any "%f" in label to contour value
	label.length = strlen(label.spot);
	pgconlx(dv.values(), dv.dims(0), dv.dims(1), 1, dv.dims(0), 1, dv.dims(1), level,
			func, label, m_intval, m_minint);
    }
}
