/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef COORD_TRANS_FUNC_H
#define COORD_TRANS_FUNC_H

typedef int (*coord_tran_func_t)(int* flags, const g_real* val_in, g_real* val_out);

class CoordinateTransformFunction
{
    protected:
	int m_dimension;		// dimensinality of transform function
// (m_matrix != 0) it should point to an
// array of length (m_dimension*(m_dimension+1)) that contains the coefficients for a linear transform
	pg_float* m_forward_matrix;
	pg_float* m_reverse_matrix;
	coord_tran_func_t m_forward_function;
	coord_tran_func_t m_reverse_function;
	void freeUp();
        int matrixFunction(const pg_float* matrix, const g_real* val_in, g_real* val_out);

    public:
	CoordinateTransformFunction() : m_dimension(0), m_forward_matrix(0), m_reverse_matrix(0), 
				m_forward_function(0), m_reverse_function(0) {}
	CoordinateTransformFunction(int dimension, coord_tran_func_t forward_func, coord_tran_func_t reverse_func = 0);
	CoordinateTransformFunction(int dimension, g_real* forward_matrix);
	void setFunction(int dimension, coord_tran_func_t forward_function, coord_tran_func_t reverse_function = 0);
	void setMatrix(int dimension, g_real* forward_matrix);
	const pg_float* forwardMatrix() { return m_forward_matrix; }
	const pg_float* reverseMatrix() { return m_reverse_matrix; }
	int forwardFunction(int* flags, const g_real* val_in, g_real* val_out);
	int reverseFunction(int* flags, const g_real* val_in, g_real* val_out);
	~CoordinateTransformFunction() { freeUp(); }
};

#endif /* COORD_TRANS_FUNC_H */
