/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef DERIVED_UNIT_H
#define DERIVED_UNIT_H

#include <splash.h>
#include "named_unit.h"
#include "si_unit.h"

class GenieWorkspace;

struct SIUnitElement
{
    SIUnit::Code code;
    SIUnit::Prefix prefix;
    double exponent;
};
    
struct DerivedUnitElement
{
    SIUnit unit;
    double exponent;		// "unit**exponent" is contribution

    DerivedUnitElement() : unit(), exponent(0.0) {}
    DerivedUnitElement(SIUnit _unit, double _exponent) : unit(_unit), exponent(_exponent) {}
    DerivedUnitElement(SIUnit::Code code, SIUnit::Prefix prefix, double _exponent) : unit(code, prefix), exponent(_exponent) {}
};

extern int operator<(const DerivedUnitElement& e1, const DerivedUnitElement& e2);

typedef SPList<DerivedUnitElement> DerivedUnitElementList;

class DerivedUnit : public NamedUnit
{
    private:

    protected:
	String m_name;		// Name of the new unit
	String m_label;		// Graphics character (Character to use when labelling axes etc)
	String m_components;
	double m_scale;		// An overall scaling factor
	DerivedUnitElementList m_unit_list;
	const DerivedUnitElement& list(int i) const { return m_unit_list[i]; }
	void setup();
	
    public:
	DerivedUnit() : m_name("<Undefined>"), m_label(""), m_components(""), m_scale(1.0) {}
	DerivedUnit(const char* name, const char* label, const char* components, double scale = 1.0);
	DerivedUnit(const GenieWorkspace& gw);
	DerivedUnit(const char* name, const SIUnitElement* element_list, int nlist, double scale = 1.0);
	SIDimensionalExponents dimensions() const;
	double convertTo(const DerivedUnit& d) const;
	ostream& print(ostream& os) const;
	const String& name() const { return m_name; }
	String encodeUnit() const;
	operator GenieWorkspace() const;
	int count() const { return m_unit_list.count(); }
	friend DerivedUnit operator*(const DerivedUnit& unit1, const DerivedUnit& unit2);
};

DerivedUnit operator*(const DerivedUnit& unit1, const DerivedUnit& unit2);

inline int operator==(const DerivedUnit& e1, const DerivedUnit& e2)
{
    return e1.name() == e2.name();
}

inline int operator<(const DerivedUnit& e1, const DerivedUnit& e2)
{
    return e1.name() < e2.name();
}

inline ostream& operator<<(ostream& os, const DerivedUnit& d)
{
    return d.print(os);
}

#endif /* DERIVED_UNIT_H */

