/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

//
// $Id: dlfcn_win32.cxx,v 1.5 1998/06/15 09:52:50 faa Exp $
//
// Module: 	dlfcn_win32
// Author:	Chris Moreton-Smith, ISIS Facility, Rutherford Appleton Laboratory
// Purpose:	Emulate the dlopen() set of routines on WINNT WIN95
//		using the native LoadLibrary, GetProcAddress etc. based on FAA's
//		dlfcn_vms routines.
//
#include "geniedefs.h"
#include "module.h"
#include "dlfcn_win32.h"

typedef void (CALLBACK* LPFNDLLFUNC)(void);	// prototype for module routine

static String error_message;			// Used to implement dlerror()


// Open a shared library (DLL)
void* dlopen(const char* path, int mode)
{
    LPVOID lpMsgBuf;
    HINSTANCE hDLL;               // Handle to DLL we are about to open

    // Load the dll
    if ( !(hDLL = LoadLibrary(path)) )
    {
	FormatMessage(
	    FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
	    NULL,
	    GetLastError(),
	    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
	    (LPTSTR) &lpMsgBuf,    0,    NULL );
	error_message = (char *)lpMsgBuf;
	LocalFree(lpMsgBuf);
	return 0;
    }

    ginf << "MODULE: loaded library \"" << path << "\"" << endl;
    return (void *)hDLL;
}


// look up a symbol in a loaded dll
void* dlsym(void* handle, const char* name)
{
    LPVOID lpMsgBuf;
    LPFNDLLFUNC lpfnDllFunc;    // Function pointer
    lpfnDllFunc = (LPFNDLLFUNC)GetProcAddress((HMODULE)handle, (LPCSTR)name);
    if (!lpfnDllFunc)
    { 
	FormatMessage(
	    FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
	    NULL,
	    GetLastError(),
	    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
	    (LPTSTR) &lpMsgBuf,    0,    NULL );
	error_message = (char *)lpMsgBuf;
	LocalFree(lpMsgBuf);
	return 0;
    }

    return (void *)lpfnDllFunc;
}


// return 0 on success, 1 on error
int dlclose(void* handle)
{
    return FreeLibrary((HMODULE)handle);	// Removes one reference count to the library
}


// Emulate error reporting routine
const char* dlerror(void)
{
    return (const char*)error_message;
}

#include <imagehlp.h>
// 0 on success, 1 on failure
int GetExportList(const char* image_name, ModuleFunctionList& exports)
{
    LOADED_IMAGE limage;
    if (!MapAndLoad((char*)image_name, "", &limage, TRUE, TRUE))
    {
	printf("Cannot map");
	return 1;	// cannot map image_name
    }
    PIMAGE_EXPORT_DIRECTORY ped;
    ped = (PIMAGE_EXPORT_DIRECTORY)(limage.FileHeader->OptionalHeader.
		DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT].VirtualAddress);
    if (!ped)
    {
        UnMapAndLoad(&limage);
	return 0;	// no exports
    }
    ped = (PIMAGE_EXPORT_DIRECTORY)ImageRvaToVa(limage.FileHeader, limage.MappedAddress, (DWORD)ped, 0);
    PDWORD* pen = ped->AddressOfNames;
    pen = (PDWORD*)ImageRvaToVa(limage.FileHeader, limage.MappedAddress, (DWORD)pen, 0);
    String name, alias;
    ModuleFunctionType type;
// We are only looking at named functions here i.e. functions that
// are exported purely by ordinal are ignored. The total number of
// functions available would be ped->NumberOfFunctions
    for(unsigned i=0; i<ped->NumberOfNames; i++)
    {
	name = (PSTR)ImageRvaToVa(limage.FileHeader, limage.MappedAddress, (DWORD)pen[i], 0);
	if (name.contains("@"))		// __stdcall mangling
	{
	    alias = name.before("@");
	    type = FortranModuleFunction;
	}
	else
	{	
	    alias = name;
	    type = CModuleFunction;
	}
	ginf << "Found export \"" << name << "\" --> \"" << alias << "\"" << endl;
	module_function item(name, alias, type);
	exports.push(item);
    }
    UnMapAndLoad(&limage);
    return 0;
}
