/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include <math.h>
#include "g_types.h"
#include "Pictlist.h"
#include "Pginclude.h"
#include "Genie_functions.h"
#include "cell_array.h"
#include "polygon_array.h"
#include "dpolygon_array.h"
#include "cell_array_wedge.h"
#include "pg_coord_trans_func.h"

static const char* rcsid = "@(#)$Id: draw_cell_array.cxx,v 1.15 1998/06/22 13:41:31 faa Exp $";
USE_RCSID(rcsid);

static CellArray* last_cell_array_plot = 0; 
static int last_cell_array_picture = -1;    

static int finish_drawing(Drawable_objects* PTR, g_objectid objid)
{
    objid[ObjectPosition] = PicturePTR[Current_Picture]->disp_add(PTR); 
    objid[ObjectPicture] = Current_Picture; 
    return objid[ObjectPosition];
}

int new_cellarray(g_real* values, int nx_values, int ny_values,
			   g_real valmax, g_real valmin,
			   g_real* tr_matrix, int itype, int mode, 
			   g_integer* table, int ntab, g_objectid objid)
{
    TwoDimensionalCoordinateTransformFunction* ctf = new PGCoordinateTransformFunction(tr_matrix);
    
    CellArray* PTR = new CellArray(values, nx_values, ny_values, valmax, 
				   valmin, itype, mode, table, ntab, ctf);
    last_cell_array_plot = PTR;
    last_cell_array_picture = Current_Picture;
    return finish_drawing(PTR, objid);
}

int new_cellarray_function(g_real* values, int nx_values, int ny_values,
			   g_real valmax, g_real valmin,
			   void* func_ptr, int itype, int mode, 
			   g_integer* table, int ntab, g_objectid objid)
 
{

    TwoDimensionalCoordinateTransformFunction* ctf =
	new PGCoordinateTransformFunction(0, (coord_tran_func_t)func_ptr);
						// set reverse function
    
    CellArray* PTR = new CellArray(values, nx_values, ny_values, valmax, 
				   valmin, itype, mode, table, ntab, ctf);
    last_cell_array_plot = PTR;
    last_cell_array_picture = Current_Picture;
    return finish_drawing(PTR, objid);
}

int draw_cellarray_function(g_real* values, int nx_values, int ny_values,
			    g_real valmax, g_real valmin, void* func_ptr,
			    int itype, int mode, g_integer* table, int ntab)
{

    g_integer objid[GENIE_OBJECTID_SIZE];
    new_cellarray_function(values, nx_values, ny_values, valmax, valmin,
			   func_ptr, itype, mode, table, ntab, objid);
    display_object(objid);
    return objid[ObjectPosition];
}

// Return object number

int new_cellarray_array(g_real* values, int nx_values, int ny_values, 
			g_real* wx, int* wx_dims, int wx_ndims, g_real* wy,
			int* wy_dims, int wy_ndims, g_real valmax,
			g_real valmin, int itype, int mode, g_integer* table,
			int ntab, g_objectid objid) 
{
#if 1
    CoordinateTransformMode x_mode, y_mode;
    x_mode = (wx_dims[0] > nx_values ? BinMode : PointMode);
    y_mode = (wy_dims[wy_ndims-1] > ny_values ? BinMode : PointMode);
    TwoDimensionalCoordinateTransformFunction* ctf =
	new PGCoordinateTransformFunction(wx, wx_dims, wx_ndims, x_mode, wy, wy_dims, wy_ndims, y_mode);
#else
// This only works for a regular grid and for WX, WY both 1-D arrays
// Transform assumes first array index is "1" i.e. FORTRAN style
// Matrix gives mapping to bin centres
    g_real matrix[6];
    matrix[2] = 0.0;
    matrix[4] = 0.0;
    matrix[1] = (wx[wx_dims[0] - 1] - wx[0]) / (g_real)(wx_dims[0] - 1);
    matrix[5] = (wy[wy_dims[0] - 1] - wy[0]) / (g_real)(wy_dims[0] - 1);
// Point or Bin mode?
    if (wx_dims[0] == nx_values) 	
        matrix[0] = wx[0] - matrix[1];			// "Point" mode
    else
        matrix[0] = wx[0] - 0.5 * matrix[1];		// "Bin" mode

    if (wy_dims[0] == ny_values) 	
        matrix[3] = wy[0] - matrix[5];			// "Point" mode
    else
        matrix[3] = wy[0] - 0.5 * matrix[5];		// "Bin" mode, shift value to bin centre

    TwoDimensionalCoordinateTransformFunction* ctf =
	new PGCoordinateTransformFunction(matrix);
#endif
    CellArray* PTR;
    if (mode == 2)	// Disjoint POLYGONS
    {
    	PTR = new DPolygonArray(values, nx_values*ny_values, wx, wy,
			wx_dims[0], valmax, valmin, itype, table, ntab, ctf);
    }
    else
    {
    	PTR = new PolygonArray(values, nx_values, ny_values, valmax, valmin,
				   itype, mode, table, ntab, ctf);
    }
    last_cell_array_plot = PTR;
    last_cell_array_picture = Current_Picture;
    return finish_drawing(PTR, objid);
}

int draw_cellarray_array(g_real* values, int nx_values, int ny_values,
			 g_real* wx, int* wx_dims, int wx_ndims, g_real* wy,
			 int* wy_dims, int wy_ndims, g_real valmax,
			 g_real valmin, int itype, int mode, g_integer* table,
			 int ntab) 
{

    g_integer objid[GENIE_OBJECTID_SIZE];
    new_cellarray_array(values, nx_values, ny_values, wx, wx_dims, wx_ndims,
			wy, wy_dims, wy_ndims, valmax, valmin, itype, mode,
			table, ntab, objid);
    
    display_object(objid);
    return objid[ObjectPosition];
}


// Draw a colour wedge into the current window
// mode=0 -> draw colour bars vertically, mode=1 -> draw colour bars horizontally
int new_cellarray_wedge(g_objectid cellid, int mode, g_objectid objid) 
{
    CellArray* PTR;
    CellArrayWedge* wedge;
        
    PTR = (CellArray*)PicturePTR[cellid[ObjectPicture]]-> 
					objectPointer(cellid[ObjectPosition]);

    wedge = new CellArrayWedge(PTR, mode);
    finish_drawing(wedge, objid);
    
    return 0;
}

int draw_cellarray_wedge(int objnum, int mode)
{
    g_integer cellid[GENIE_OBJECTID_SIZE];
    g_integer objid[GENIE_OBJECTID_SIZE];
    if (objnum == -1)
    {
	cellid[ObjectPicture] = last_cell_array_picture;
	cellid[ObjectPosition] = PicturePTR[last_cell_array_picture]->
			objectNumber(last_cell_array_plot);
    }
    else
    {
	cellid[ObjectPicture] = Current_Picture;
	cellid[ObjectPosition] = objnum;
    }
    new_cellarray_wedge(cellid, mode, objid);
    display_object(objid);
    return objid[ObjectPosition];
}

// dim is zero based dimension
DrawableValues get_cellarray_slice(int dim, g_real val)
{
     CellArray* PTR = last_cell_array_plot;
     const DrawableValues& dv = PTR->values();                 // Z values
     g_real val_in[3] = { 0.0, 0.0, 0.0 }, val_out[3];
     int flags(0);
     val_in[dim] = val;
     PTR->reverseFunction(&flags, val_in, val_out);
     return dv.slice(dim, val_out[dim]);
}
