/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
// -----------------------------------------------------------------------------
// ERROR_FUNCTIONS.C
//
// Description: C++ code to allow creation and manipulation of the Genie 
//  object-graphics error handling system.
//
//
// *** include string headers
#include <string.h>
#include <gnustring.h>

#ifdef _WIN32
#	include "strcasecmp.h"
#endif
           
// *** local includes
#include "Errors.h"

#include "Genie_functions.h"

static const char* rcsid = "@(#)$Id: error_functions.cxx,v 1.12 1998/05/07 17:03:59 faa Exp $";
USE_RCSID(rcsid);

static Errors syserr;

// ---
// *** check the error status, return 1 if there are errors
//
    int error_status() {
	int i;
	i = syserr.status();
	return i;

    }

// ---
// *** clear all errors
//
    void clear_errors() {
	syserr.clear();
	}

// ---
// *** return object name causing error (n)
//
    const char *error_object(int n) {
	return syserr.object(n);
	}

// ---
// *** return reason for failure in (n)
//
    const char *error_failure(int n) {

	return syserr.failure(n);
	}

// ---
// *** return possible solution for error (n)
//
    const char *error_solution(int n) {
	return syserr.solution(n);
	}

// ---
// *** return object, failure and solution for error (n)
//
    const char *error_message(int n) {
	return syserr.message(n);
	}

// ---
// *** add an error
//
    void error_add(const char *obj, const char *fail, const char *soln) 
{
//	syserr.add_error(obj, fail,soln);
	if (obj == 0 || *obj == '\0')
	    return;

	if (strcasecmp(obj, "INFORMATION") != 0)
	{
	    (*g_err) << "ERROR returned from: " << obj << endl;
	    (*g_err) << fail << endl;
	}
	else
	{
	    ginf << fail << endl;
	}
        if (soln != 0 && *soln != '\0')
	    ginf << "SOLUTION: " << soln << endl;
}
