/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

// fileset.h: interface for the CFileSet class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FILESET_H__1838AAFD_E981_11D1_A6BB_00609709EB73__INCLUDED_)
#define AFX_FILESET_H__1838AAFD_E981_11D1_A6BB_00609709EB73__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

// This class encapsulates all the dealings of Open GENIE with the outside world in
// terms of files, file systems and symbol translations.

// If you have an object of this class, you _should_ know all you want to know about
// where GENIE is getting its input from, which files exactly it is going to attempt
// to load, and whether a symbol is being translated or not to load the file. Needless
// to say, this class appears where this information is needed within Open GENIE.

// Most of the file members are public as the work has been done by the time they are
// stored and they are just complete, system dependent path strings.

// Access to the base file list is protected as they will have a directory string
// appended locally before use.

// Standard symbols or logical names for Open GENIE components.  This list is intended
// to be minimal.
//
//  GENIE_DIR                   - Essential system files (genie can get most from this anyway)
//  GENIE_TCL_DIR		- startup files for TCL interface
//  GENIE_LIBRARY_DIR           - General modules, bits of GCL etc.
//  GENIE_DOC_DIR		- Location of documentation & help files (mainly html)
//  GENIE_INCLUDE_DIR		- where module headers etc are
//  GENIE_USER_LIBRARY_DIR      - Pointer to user written routines for automatic search
//
//  GENIE_EXE                   - Location of startup executable
//  GENIE_SHARED_LIBRARY        - Bulk of GENIE code shared library/DLL
//
//  GENIE_SMALLTALK_IMAGE       - User Language specific (compiled) image
//  GENIE_TCL_IMAGE             - eg ICE compiled TCL
//
//  GENIE_SMALLTALK_INIT        - User Language specific initialization (needing interpretation)
//  GENIE_GCL_INIT              - User GCL file (eg GENIEINIT.GCL)
//  GENIE_TCL_INIT              - User TCL interface initializtion file
//
//  GENIE_DEFAULT               - User set default directory for running genie in

class CFileSet  
{
private:
    static const char* s_basefiles[];

public:
    GENIEIMPORT static CFileSet* s_this;
    bool GetBaseFile(int& index, String& gstr);

    // persisted directory paths
    String m_gstrHelpDir;
    String m_gstrLibraryDir;
    String m_gstrTCLDir;
    String m_gstrGCLDir;

    // persisted file paths
    String m_gstrSmalltalkImage;
    String m_gstrGCLInitialisation;
    String m_gstrTCLInitialisation;

    // Non persisted members
    String m_gstrHistoryFile;
    String m_gstrFileSetError;		// Use to report errors from the constructor
    String m_gstrParserInstrumentation;
    String m_gstrDebuggingOutput;

#ifdef _WIN32

private:
    static LPCTSTR s_szKey;		// Full Key name for stored registry settings.

public:
    // Win32 registry helper functions
    GENIEIMPORT BOOL GetRegistrySettings(HKEY hk);
    GENIEIMPORT BOOL SetRegistrySettings(HKEY hk);
    String GetRegString(HKEY& rhk, LPTSTR pszValueName);
    int SetRegString(HKEY& rhk, LPTSTR pszValueName, LPCTSTR pszValue);
    int SetRegInt(HKEY& rhk, LPTSTR pszValueName, DWORD dwValue);
    int GetRegInt(HKEY& rhk, LPTSTR pszValueName);
    GENIEIMPORT void SetVars();

    // Persist interface preferences
    BOOL m_bDontShow;
    BOOL m_bFileCheck;
    BOOL m_bSaveImage;

#endif

    CFileSet(
        const char* szGCLDir = "",		    // Main build files in GCL
        const char* szImagePath = "",               // Compiled Smalltalk image file
        const char* szHistoryFile = "",             // GNU command history file
        const char* szInstrumentationFile = "",     // Instrumented parser output
        const char* szDebuggingOutput = "",         // Intermediate smalltalk parser output
        const char* szGCLInitialisation = "",	    // User GCL initialisation file
        const char* szTCLInitialisation = "" );     // User TCL initialisation file
    
    virtual ~CFileSet();

};

#endif // !defined(AFX_FILESET_H__1838AAFD_E981_11D1_A6BB_00609709EB73__INCLUDED_)
