/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#define PRN_BUFFER	256		/* buffer used in WIN32 get_printers() */

#define LINE_BUFFER 100     /* buffers used in UNIX get_printers() */
#define FILE_BUFFER 10000 

                                /* VMS get_printers buffers */
#define QUEUE_BUFFER 100        /* allow up to 100 printer queues */
#define STRING_BUFFER 10000     /* maximum string size */

#include "geniedefs.h"
#include <stdio.h>
#include <ctype.h>
#include <iostream.h>
#include "genie_cmodule.h"

#ifdef _WIN32
#include <strstrea.h>
#include <windows.h>
#include <winspool.h>
#else
#include <strstream.h>
#endif

#ifdef __VMS
#include <starlet.h>
#include <lib$routines.h>
#include <ssdef.h>
#include <lnmdef.h>
#include <descrip.h>
#include <quidef.h>
#include <jbcmsgdef.h>
#endif

#include <string.h>
#include <stdlib.h>
#include <setjmp.h>


#include "smalltalk.h"	    // smalltalk
#include "parsedefs.h"	    // parser
#include "parser.h"
#include "g_tktcl.h"	    // Tcl/Tk
#include "ghistory.h"	    // Access to genie objects
#include "fileset.h"
#include "genie.h"

#include "g_extensions.h"
#include "genievars.h"
#include "parse_funcs.h"
#include "smalltalk_object.h"
#include "gsignals.h"

//#include "extra_genie_commands.h"

//genie_command_t* extra_genie_command_lists = 0;

// Return number of white space separated items on a line
static int num_items(const char* s)
{
    int count = 0;
// skip initial space
    while(*s && isspace(*s))
    {
	s++;
    }
    int in_space = 1;
    while(*s)
    {
	if (in_space && !isspace(*s))
	{
	    in_space = 0;
	    count++;
	}
	if (!in_space && isspace(*s))
	{
	    in_space = 1;
	}
	s++;
    } 
    return count;
}

static OOP string_to_gvar(GenieString& numstring)
{
	    int i;
            g_real realvar;
            g_integer intvar;
            OOP result = (OOP)numstring;
            istrstream iss((char*)(const char*)numstring);
	    int count = num_items(numstring);

            if ( numstring.contains('.') || numstring.contains('e') || numstring.contains('E') )
	    {
		if (count <= 1)
		{
    		    iss >> realvar;
	    	    if( !iss.fail() ) result = (OOP)GenieReal(realvar);
		}
		else
		{
		    GenieRealArray gra(count);
		    for(i=0; i<count; i++)
		    {
			iss >> gra[i];
		    }
	    	    if( !iss.fail() ) result = (OOP)gra;
		}
	    }
	    else
	    {
		if (count <= 1)
		{
		        iss >> intvar;
		        if ( !iss.fail() ) result = (OOP)GenieInteger(intvar);
		}
		else
		{
		    GenieIntegerArray gia(count);
		    for(i=0; i<count; i++)
		    {
			iss >> gia[i];
		    }
	    	    if( !iss.fail() ) result = (OOP)gia;
		}
	    }
	    return result;
}

#ifdef __VMS
static char* get_printers()
{
	$DESCRIPTOR(q_name,"*");    /* wildcard search (all) */
	int c,total = 0,length = 1;
	long int func,item,ret;
	unsigned long int flag;
	short int len;
	struct dsc$descriptor_s pname,qdesc,pname2;
	char printers[STRING_BUFFER];
	char names[QUEUE_BUFFER][32];
	printers[0]='\0';
	qdesc.dsc$w_length = 256;
	qdesc.dsc$a_pointer = (char*)malloc(sizeof(char)*256);
	qdesc.dsc$b_class = DSC$K_CLASS_S;
	qdesc.dsc$b_dtype = DSC$K_DTYPE_T;
	pname.dsc$w_length = 32;
	pname.dsc$b_class = DSC$K_CLASS_S;
	pname.dsc$b_dtype = DSC$K_DTYPE_T;
	pname2.dsc$w_length = 32;
	pname2.dsc$a_pointer = (char*)malloc(sizeof(char)*32);
	pname2.dsc$b_class = DSC$K_CLASS_S;
	pname2.dsc$b_dtype = DSC$K_DTYPE_T;
	func = QUI$_DISPLAY_QUEUE;
	item = QUI$_QUEUE_NAME;         /* get queue names */
	flag = QUI$M_SEARCH_PRINTER;    /* but only printer those of printers */
	do {
		pname.dsc$a_pointer = names[total];
		ret = lib$getqui(&func,&item,0,&q_name,&flag,0,&pname,&len);
		if ((strncmp(pname.dsc$a_pointer,"POST",4) == 0) || 
		(strncmp(pname.dsc$a_pointer,"COLOUR",5) == 0)) {
			pname.dsc$a_pointer[len]='\0';
			total++;
		}
		if (total >= QUEUE_BUFFER) {
			printf("ERROR: Printer queue buffer exceeded");
			return NULL;
		}
	} while (ret != JBC$_NOMOREQUE);
	item = QUI$_QUEUE_DESCRIPTION;  /* retrieve queue descriptions */
	for (c=0;c<total;c++) {
		pname.dsc$a_pointer = names[c];
		lib$getqui(&func,&item,0,&pname,0,0,&qdesc,&len);
		if (len == 0) {			
			item = QUI$_GENERIC_TARGET;     /* get description of target */
			lib$getqui(&func,&item,0,&pname,0,0,&pname2,&len);
			item = QUI$_QUEUE_DESCRIPTION;
			lib$getqui(&func,&item,0,&pname2,0,0,&qdesc,&len);
		}
		strcat(printers,pname.dsc$a_pointer);
		length = length + strlen(pname.dsc$a_pointer) + 1;
		printers[length-2] = '\2';   /* add list separator */
		printers[length-1] = '\0';
		if (len == 0) {
			strcat(printers,"no description");
			len = 14;
		} else {
			strncat(printers,qdesc.dsc$a_pointer,len);
		}
		strcat(printers,"\2");		/* add list separator */
		length = length + len + 1;
		if (length > STRING_BUFFER) {
			printf("ERROR: Printer list buffer exceeded");
			return NULL;
		}
	}
	free(qdesc.dsc$a_pointer);
	free(pname2.dsc$a_pointer);
	return strdup(printers);
}
#else
#ifdef WIN32
static char* get_printers()
{
	unsigned int c;
	char printer[PRN_BUFFER];
	char* printers;
	DWORD flags = PRINTER_ENUM_LOCAL | PRINTER_ENUM_CONNECTIONS;	
	LPPRINTER_INFO_2 pinfo;
	DWORD buffer_size;
	DWORD returned;
	EnumPrinters(flags, NULL, 2, NULL, 0, &buffer_size, &returned);
	pinfo = (LPPRINTER_INFO_2) malloc(buffer_size);
	EnumPrinters(flags, NULL, 2, (LPBYTE)pinfo, buffer_size, &buffer_size, &returned);
	printers = (char*) malloc(returned * PRN_BUFFER);
	printers[0] = '\0';
	for (c=0; c < returned; c++) {
		sprintf(printer,"%s\2",pinfo[c].pPrinterName);
		if (strlen(pinfo[c].pComment) > 0) {
			strcat(printer,pinfo[c].pComment);
			if (strlen(pinfo[c].pLocation) > 0) {
				sprintf(printer,"%s (%s)",printer,pinfo[c].pLocation);
			}
		} else {
			strcat(printer,"no description");
		}
		strcat(printer,"\2");	
		strcat(printers,printer);
	}
	free(pinfo);
	return printers;
}
#else
static char* get_printers()
{
    FILE* pt;
    char temp[LINE_BUFFER];
    char printers[FILE_BUFFER];
    int tlen,len = 1;   /* include string terminator length */
    pt = fopen("/etc/printcap","r");
    if (pt == NULL) {
        printf("ERROR: Could not open /etc/printcap");
        return NULL;
    }
    printers[0] = '\0';
    while ((fgets(temp,LINE_BUFFER,pt)) != NULL)
    {
        if ((temp[0] != '#') && (isalnum(temp[0]))) {
            tlen = 0;
            while ((temp[tlen] != ':') && (temp[tlen] != '|'))
            {
                tlen++;
            }
            temp[tlen]= '\2';    /* separate printer name with \2 */            
            tlen++;
            len = len + tlen + 15;
            if (len > FILE_BUFFER) {
                printf("ERROR: Printcap file buffer exceeded");
                return NULL;
            }
            strncat(printers,temp,tlen);
            strcat(printers,"no description\2\0");
        }
    }
    fclose(pt);
    return strdup(printers);
}
#endif
#endif

static char* tk_inquire(const String& label)
{
    char input[60] = "\0";
    void* dialog;
    dialog = cmodule_inquire_begin("Inquire");
    cmodule_inquire_string(dialog, label, input, 60);
    cmodule_inquire_end(dialog);
    return strdup(input);    
}

//********************************************************************************
// C++ extensions hard coded from the smalltalk internals and loaded via
// stextensions.gcl.
//
// In general these are only things which would be tricky to do in smalltalk
// and are needed by GENIE.
//********************************************************************************

static const char* rcsid = "@(#)$Id: g_extensions.cxx,v 1.42 1998/05/27 19:20:56 faa Exp $";
USE_RCSID(rcsid);

// Callin from Smalltalk
// =====================

// Return nilOOP on failure
OOP g_extensions( extension_function_code function, OOP oopParameters )
{

    OOP result(nilOOP);

    BEGINSAVEBLOCK

    switch(function)				    // NOTE: each part of the CASE is responsible for doing a return OOP;
    {
    case Extension_test:
    	gout << "Extensions test called" << endl;
    	break;

// return a list of available printers
    case Get_printers:
    {
        GenieString printers = get_printers();
        result = printers;
    }
    break;

// return user input from a Tk dialog box
    case Tk_inquire:
    {
        GenieString label(oopParameters, 1);
        GenieString input = tk_inquire(label);
        result = input;
    }
    break;

// convert a stringarray into a variable or array
    case Stringarray_to_gvar:
	{
	    GenieStringArray numstringarray(oopParameters, 1);
	    char* bytes = numstringarray.asString(' ');
	    GenieString numstring = bytes;
	    delete[] bytes;
	    result = string_to_gvar(numstring);
	}
	break;

// convert a string into a variable or array
    case String_to_gvar:
	{
	    GenieString numstring(oopParameters, 1);
	    result = string_to_gvar(numstring);
  	}
        break;

    case StreamMode:        // print a GENIE string on the standard output
        {
            GenieString  fd(oopParameters, 1);
            GenieInteger mode(oopParameters, 2);
            if (mode.defined())
            {
                switch (fd[0])
                {
                case 'o': result = (OOP)GenieInteger(gout.mode(mode)); break;
                case 'i': result = (OOP)GenieInteger(ginf.mode(mode)); break;
                case 'e': result = (OOP)GenieInteger(gerr.mode(mode)); break;
                case 'd': result = (OOP)GenieInteger(gdbg.mode(mode)); break;
                }
            }
	    else
	    {
                switch (fd[0])
                {
		case 'o': result = (OOP)GenieInteger(gout.mode()); break;
		case 'i': result = (OOP)GenieInteger(ginf.mode()); break;
		case 'e': result = (OOP)GenieInteger(gerr.mode()); break;
		case 'd': result = (OOP)GenieInteger(gdbg.mode()); break;
		}
	    }
        }
        break;

    case StringPrint:    // print a GENIE string on the standard output
        {
            GenieString gs(oopParameters, 1);
            GenieString fd(oopParameters, 2);
            if (gs.defined())
            {
                switch (fd[0])
                {
                case 'o': gout << gs << flush; break;
                case 'i': ginf << gs << flush; break;
                case 'E': gerr << gs << flush; break;
                case 'd': gdbg << gs << flush; break;
                case 'e': *g_err << gs << flush; break;  // No line numbers & logging
                }
            }
        }
        break;

    case StringRead:     // print a GENIE string on the standard output
        {
            static char buffer[SHORTCHARBUFF+1];
            gin.getline(buffer, SHORTCHARBUFF);
            result = (OOP)GenieString(buffer);
        }
        break;

    case MangleAs:
	    {
            GenieString self(oopParameters, 1);
            GenieString as(oopParameters, 2);
            char buff[SHORTCHARBUFF+3];		// add on some extra for gX and \0 etc.

	    int len = self.length();
	    if (len < SHORTCHARBUFF)
	    {
                strncpy(buff, (const char*)self, len);
		buff[len] = '\0';
                if ( as == "procedure" )
                    MangleProc(buff);
                else if ( as == "variable" )
                    MangleVar(buff);
                else if ( as == "type" )
                    MangleType(buff);
	    }
	    else
	    {
		gerr << "MangeAs: name too long" << endl;
		buff[0] = '\0';
	    }
            self = buff;
            result = (OOP)self;
        }
        break;

    case Demangle:
	    {
            GenieString self(oopParameters, 1);
            char buff[SHORTCHARBUFF+1];		// for '\0'

	    if (self.length() < SHORTCHARBUFF)
	    {
                strcpy(buff, (const char*)self);
                DeMangle(buff);
	    }
	    else
	    {
		gerr << "DeMangle: name too long" << endl;
		buff[0] = '\0';
	    }
            self = buff;
            result = (OOP)self;
  	    }
        break;

    case QuitTheLot:
        exit(1);         // shouldn't exit this way any more!
        break;

    case LoadTheCode:
	    {
            GenieString fileName(oopParameters, 1);
            Unset_signals();		// Only works to 1 level of recursion !!

	    // Access running genie object to do this
	    ((CGenie*)g_pGenie)->LoadFileByDate(fileName, (time_t) 0, true);

	    Set_signals();
  	    }
        break;

#if 0
    case AddCommandToReadline:
    {
	GenieString command(oopParameters, 1), comment(oopParameters, 2);
	
	break;
    }
#endif

    default:
    	PERROR("Undefined extension");
    }

    ENDSAVEBLOCK

    return result;
}
