/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef G_MODULE
#define G_MODULE

// Forward declarations
struct STRING;
class GenieWorkspace;

#include "module.h"

#include "Pginclude.h"		// for STRING
#include "fortran_types.h"

extern Module genie_modules;

typedef enum {
    Module_test = 0,
    Module_load = 1,
    Module_execute = 2,
    Module_list = 3,
    Module_unload = 4
} module_function_code;

OOP g_module(module_function_code function, OOP arguments);

#if defined(HAVE_LIBF2C)	/* f2c mangles with "__" for names already containing an "_", else as per UNIX */
#define g3_module_get_real g3_module_get_real__
#define g3_module_put_real g3_module_put_real__
#define g3_module_get_double g3_module_get_double__
#define g3_module_put_double g3_module_put_double__
#define g3_module_get_int g3_module_get_int__
#define g3_module_put_int g3_module_put_int__
#define g3_module_get_string g3_module_get_string__
#define g3_module_put_string g3_module_put_string__
#define g3_module_string_array_alloc g3_module_string_array_alloc__
#define g3_module_string_array_add g3_module_string_array_add__
#define g3_module_string_array_free g3_module_string_array_free__
#define g3_module_get_real_array g3_module_get_real_array__
#define g3_module_put_real_array g3_module_put_real_array__
#define g3_module_put_nd_real_array g3_module_put_nd_real_array__
#define g3_module_get_double_array g3_module_get_double_array__
#define g3_module_put_double_array g3_module_put_double_array__
#define g3_module_get_int_array g3_module_get_int_array__
#define g3_module_put_int_array g3_module_put_int_array__
#define g3_module_print g3_module_print__
#define g3_module_open_stdio g3_module_open_stdio__
#define g3_module_close_stdio g3_module_close_stdio__
#define g3_module_get_version g3_module_get_version__
#define g3_module_version_ok g3_module_version_ok__
#define g3_module_get_string_array_go g3_module_get_string_array_go__
#define g3_module_get_string_array_i g3_module_get_string_array_i__
#define g3_module_get_string_array_end g3_module_get_string_array_end__
#define g3_module_malloc	g3_module_malloc__
#define g3_module_free		g3_module_free__
#define g3_module_freeall	g3_module_freeall__
#define g3_module_freeallheaps	g3_module_freeallheaps__
#define g3_module_makeheap	g3_module_makeheap__
#define g3_module_map_real_array		g3_module_map_real_array__
#define g3_module_map_double_array		g3_module_map_double_array__
#elif defined(__unix)		/* UNIX mangles all FORTRAN names with one "_" */
#define g3_module_get_real g3_module_get_real_
#define g3_module_put_real g3_module_put_real_
#define g3_module_get_double g3_module_get_double_
#define g3_module_put_double g3_module_put_double_
#define g3_module_get_int g3_module_get_int_
#define g3_module_put_int g3_module_put_int_
#define g3_module_get_string g3_module_get_string_
#define g3_module_put_string g3_module_put_string_
#define g3_module_string_array_alloc g3_module_string_array_alloc_
#define g3_module_string_array_add g3_module_string_array_add_
#define g3_module_string_array_free g3_module_string_array_free_
#define g3_module_get_real_array g3_module_get_real_array_
#define g3_module_get_double_array g3_module_get_double_array_
#define g3_module_put_real_array g3_module_put_real_array_
#define g3_module_put_nd_real_array g3_module_put_nd_real_array_
#define g3_module_put_double_array g3_module_put_double_array_
#define g3_module_get_int_array g3_module_get_int_array_
#define g3_module_put_int_array g3_module_put_int_array_
#define g3_module_print g3_module_print_
#define g3_module_open_stdio g3_module_open_stdio_
#define g3_module_close_stdio g3_module_close_stdio_
#define g3_module_get_version g3_module_get_version_
#define g3_module_version_ok g3_module_version_ok_
#define g3_module_get_string_array_go g3_module_get_string_array_go_
#define g3_module_get_string_array_i g3_module_get_string_array_i_
#define g3_module_get_string_array_end g3_module_get_string_array_end_
#define g3_module_malloc	g3_module_malloc_
#define g3_module_free		g3_module_free_
#define g3_module_freeall	g3_module_freeall_
#define g3_module_freeallheaps	g3_module_freeallheaps_
#define g3_module_makeheap	g3_module_makeheap_
#define g3_module_map_real_array		g3_module_map_real_array_
#define g3_module_map_double_array		g3_module_map_double_array_
#elif defined(_WIN32)
#define g3_module_get_real G3_MODULE_GET_REAL
#define g3_module_put_real G3_MODULE_PUT_REAL
#define g3_module_get_double G3_MODULE_GET_DOUBLE
#define g3_module_put_double G3_MODULE_PUT_DOUBLE
#define g3_module_get_int G3_MODULE_GET_INT
#define g3_module_put_int G3_MODULE_PUT_INT
#define g3_module_get_string G3_MODULE_GET_STRING
#define g3_module_put_string G3_MODULE_PUT_STRING
#define g3_module_string_array_alloc G3_MODULE_STRING_ARRAY_ALLOC
#define g3_module_string_array_add G3_MODULE_STRING_ARRAY_ADD
#define g3_module_string_array_free G3_MODULE_STRING_ARRAY_FREE
#define g3_module_get_real_array G3_MODULE_GET_REAL_ARRAY
#define g3_module_get_double_array G3_MODULE_GET_DOUBLE_ARRAY
#define g3_module_put_real_array G3_MODULE_PUT_REAL_ARRAY
#define g3_module_put_nd_real_array G3_MODULE_PUT_ND_REAL_ARRAY
#define g3_module_put_double_array G3_MODULE_PUT_DOUBLE_ARRAY
#define g3_module_get_int_array G3_MODULE_GET_INT_ARRAY
#define g3_module_put_int_array G3_MODULE_PUT_INT_ARRAY
#define g3_module_print G3_MODULE_PRINT
#define g3_module_open_stdio G3_MODULE_OPEN_STDIO
#define g3_module_close_stdio G3_MODULE_CLOSE_STDIO
#define g3_module_get_version G3_MODULE_GET_VERSION
#define g3_module_version_ok G3_MODULE_VERSION_OK
#define g3_module_get_string_array_go G3_MODULE_GET_STRING_ARRAY_GO
#define g3_module_get_string_array_i G3_MODULE_GET_STRING_ARRAY_I
#define g3_module_get_string_array_end G3_MODULE_GET_STRING_ARRAY_END
#define g3_module_malloc	G3_MODULE_MALLOC
#define g3_module_free		G3_MODULE_FREE
#define g3_module_freeall	G3_MODULE_FREEALL
#define g3_module_freeallheaps	G3_MODULE_FREEALLHEAPS
#define g3_module_makeheap	G3_MODULE_MAKEHEAP
#define g3_module_map_real_array		G3_MODULE_MAP_REAL_ARRAY
#define g3_module_map_double_array		G3_MODULE_MAP_DOUBLE_ARRAY
#endif


extern "C"
{
    void GENIEMECHANISM g3_module_get_real(GenieWorkspace& pars, const STRING& name, fort_real& val);
    void GENIEMECHANISM g3_module_put_real(GenieWorkspace& res, const STRING& name, const fort_real& val);
    void GENIEMECHANISM g3_module_get_double(GenieWorkspace& pars, const STRING& name, fort_double& val);
    void GENIEMECHANISM g3_module_put_double(GenieWorkspace& res, const STRING& name, const fort_double& val);
    void GENIEMECHANISM g3_module_get_string(GenieWorkspace& pars, const STRING& name, STRING& val);
    void GENIEMECHANISM g3_module_put_string(GenieWorkspace& res, const STRING& name, const STRING& val);
    void GENIEMECHANISM g3_module_string_array_alloc(const fort_int& length, fort_int* handle);
    void GENIEMECHANISM g3_module_string_array_add(fort_int* handle, const fort_int& element, const STRING& val);
    void GENIEMECHANISM g3_module_string_array_free(GenieWorkspace& res, const STRING& name, fort_int* handle);
    void GENIEMECHANISM g3_module_get_int(GenieWorkspace& pars, const STRING& name, fort_int& val);
    void GENIEMECHANISM g3_module_put_int(GenieWorkspace& res, const STRING& name, const fort_int& val);
    void GENIEMECHANISM g3_module_get_real_array(GenieWorkspace& pars, const STRING& name, fort_real* val, fort_int& len);
    void GENIEMECHANISM g3_module_get_double_array(GenieWorkspace& pars, const STRING& name, fort_double* val, fort_int& len);
    void GENIEMECHANISM g3_module_get_int_array(GenieWorkspace& pars, const STRING& name, fort_int* val, fort_int& len);
    void GENIEMECHANISM g3_module_put_real_array(GenieWorkspace& pars, const STRING& name, const fort_real* val, const fort_int& len);
    void GENIEMECHANISM g3_module_put_double_array(GenieWorkspace& pars, const STRING& name, const fort_double* val, const fort_int& len);
    void GENIEMECHANISM g3_module_put_int_array(GenieWorkspace& pars, const STRING& name, const fort_int* val, const fort_int& len);
    void GENIEMECHANISM g3_module_print(const STRING& s, const fort_int& option);
    void GENIEMECHANISM g3_module_open_stdio(char* itty, fort_int& tty_len);
    void GENIEMECHANISM g3_module_close_stdio(fort_int& stdio_unit);

    fort_int GENIEMECHANISM g3_module_put_nd_real_array(GenieWorkspace& pars, const STRING& name, const fort_real* val, 
						const fort_int* dims_array, const fort_int& ndims);
    fort_int GENIEMECHANISM g3_module_get_version(fort_int& major, fort_int& minor);
    fort_int GENIEMECHANISM g3_module_version_ok(const fort_int& major, const fort_int& minor);
    void GENIEMECHANISM g3_module_get_string_array_go(GenieWorkspace& pars, const STRING& iname, fort_int& length, fort_int* handle);
    void GENIEMECHANISM g3_module_get_string_array_i(fort_int* handle, fort_int& element, STRING& ivalue);
    void GENIEMECHANISM g3_module_get_string_array_end(fort_int* handle);
    void GENIEMECHANISM g3_module_freeall(const fort_int* heap);
    void GENIEMECHANISM g3_module_freeallheaps();
    void GENIEMECHANISM g3_module_free(fort_int* address, const fort_int* heap);
    void GENIEMECHANISM g3_module_malloc(const int& nelements, const int& element_size, const void* array, int address[], long& offset, const fort_int* heap);
    void GENIEMECHANISM g3_module_makeheap(fort_int* heap);
    void GENIEMECHANISM g3_module_map_real_array(GenieWorkspace& pars, const STRING& name, fort_real* val, long& offset, fort_int& len);
    void GENIEMECHANISM g3_module_map_double_array(GenieWorkspace& pars, const STRING& name, fort_double* val, long& offset, fort_int& len);
}

// These are only called from C++, so need to be extern "C"
// They allocate space for the array with malloc() if (*val == 0)
void g3_module_get_real_array_alloc(GenieWorkspace& pars, const STRING& name, fort_real** val, fort_int& len);
void g3_module_get_double_array_alloc(GenieWorkspace& pars, const STRING& name, fort_double** val, fort_int& len);
void g3_module_get_int_array_alloc(GenieWorkspace& pars, const STRING& name, fort_int** val, fort_int& len);

extern "C"
{
#if HAVE_LIBF2C
#       define f_map_fortran_stdio f_map_fortran_stdio__
#       define f_unmap_fortran_stdio f_unmap_fortran_stdio__
#       define module_test_function module_test_function__
#       define sample_genie_coord_tran_func sample_genie_coord_tran_func__
#elif defined(__unix)
#       define f_map_fortran_stdio f_map_fortran_stdio_
#       define f_unmap_fortran_stdio f_unmap_fortran_stdio_
#       define module_test_function module_test_function_
#       define sample_genie_coord_tran_func sample_genie_coord_tran_func_
#elif defined(_WIN32)
#       define f_map_fortran_stdio F_MAP_FORTRAN_STDIO_
#       define f_unmap_fortran_stdio F_UNMAP_FORTRAN_STDIO_
#       define module_test_function MODULE_TEST_FUNCTION_
#       define sample_genie_coord_tran_func SAMPLE_GENIE_COORD_TRAN_FUNC
#endif


// The following two are in "module_test_function.f"
    void GENIEMECHANISM module_test_function(GenieWorkspace& pars, GenieWorkspace& res);       // "module_func_t"
    int GENIEMECHANISM sample_genie_coord_tran_func(int* flags, const g_real* val_in, g_real* val_out);        // "coord_tran_func_t"

    void GENIEMECHANISM f_map_fortran_stdio(const char*, fort_int&);
    void GENIEMECHANISM f_unmap_fortran_stdio();
}

#endif /* G_MODULE */
