/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
#======================================================================	#
#======================================================================	#
#									#
#				G_NEWOBJECTS.CC				#
#									#
#======================================================================	#
#======================================================================	#
*/	

#include "geniedefs.h"
#include <iostream.h>
#include <math.h>
#include <limits.h>
#include <string.h>
#include <gnustring.h>
#include "genie_types.h"
#include "smalltalk.h"
#include "g_newgraphics.h"
#include "genievars.h"
#include "Genie_functions.h"
#include "fortran_types.h"
#include "g_fortextern.h"
#include "g_module.h"
#include "Pginclude.h"
#include "Pictlist.h"
#include "smalltalk_object.h"
#include "ThreeDRoutines.h"

static const char* rcsid = "@(#)$Id: g_newgraphics.cxx,v 1.39 1997/11/05 08:57:20 cmm Exp $";
USE_RCSID(rcsid);

// Invert a PGPLOT TR matrix
static g_real* invert_two_by_two(g_real* matrix)
{
    g_real new_matrix[6];
    double det = matrix[1] * matrix[5] - matrix[2] * matrix[4];
    new_matrix[0] = (matrix[2] * matrix[3] - matrix[5] * matrix[0]) / det;
    new_matrix[1] = + matrix[5] / det;
    new_matrix[2] = - matrix[2] / det;
    new_matrix[3] = (matrix[4] * matrix[0] - matrix[1] * matrix[3]) / det;
    new_matrix[4] = - matrix[4] / det;
    new_matrix[5] = + matrix[1] / det;
    memcpy(matrix, new_matrix, 6*sizeof(g_real));
    return matrix;
}

// Callin from Smalltalk
// =====================


OOP g_newgraphics( NewGraphicsCode function, OOP arguments )
{
    static int debug_callout = (getenv("GENIE_DEBUG_CALLOUT") != 0 ? 1 : 0);

    OOP result(nilOOP);

    BEGINSAVEBLOCK
 
    if (debug_callout)
	gdbg << "CALLOUT: calling g_newgraphics with function \"" << NewGraphicsNames[function] << "\"\n";

    switch(function)	
    {
        case gxPolygonCreate:
	{
	    GenieRealArray x(arguments,1), y(arguments, 2);
	    GenieInteger colour(arguments, 3), solid(arguments, 4);
	    GenieReal lw(arguments, 5);
	    GenieObjectID  objid(GENIE_OBJECTID_SIZE);

	    if(x.defined() && y.defined())
	    {
		new_polygon(x, y, x.length(), colour, lw, solid, objid);
	    	
		result = (OOP) objid; 
	    }
	    break;
	}

	case gxPlaneCreate:
	{
	    GenieRealArray eye(arguments,1), light(arguments,2);
	    GenieWorkspace table(arguments,3);
	    GenieRealArray vertices(arguments,4);
	    GenieInteger transparency(arguments,5);
        GenieIntegerArray ci = table("CI");
        GenieInteger ci_low = table("ic1");
        GenieInteger ci_high = table("ic2");
        ThreeDObjectColours c(ci_low,ci_high,0,0,ci);
        GenieInteger nvertices = vertices.length() / 3;
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    new_plane(vertices,nvertices,transparency,eye,light,c,objid); 
	   	result = (OOP) objid;
	    break;
    }

    case gxThreeDTextCreate:
    {
	    GenieRealArray eye(arguments,1), light(arguments,2);
	    GenieWorkspace table(arguments,3);
	    GenieString text(arguments,4);
	    GenieInteger colour(arguments,5);
	    GenieRealArray pivot(arguments,6);
	    GenieReal justify(arguments,7);
	    GenieRealArray orient(arguments,8);
	    GenieReal size(arguments,9);        
	    long ci[2]={1,2};
        ThreeDObjectColours c(0,0,0,0,ci);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    new_threeDText(text,colour,pivot,justify,orient,size,eye,light,c,objid); 
	   	result = (OOP) objid;
	    break;
    }

    case gxEllipsoidCreate:
    {
	    GenieRealArray eye(arguments,1), light(arguments,2);
	    GenieWorkspace table(arguments,3);
	    GenieRealArray paxes(arguments,4), centre(arguments,5);
	    GenieInteger icline(arguments,6);
	    GenieReal anglin(arguments,7);  
	    GenieIntegerArray ci = table("CI");
	    GenieReal shin = table("SHINE");
        g_integer shine = 1;
        if ((g_real)shin == 0.0)
            shine=0;
        GenieInteger ci_low = table("ic1");
        GenieInteger ci_high = table("ic2");
        ThreeDObjectColours c(ci_low,ci_high,shine,0,ci);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    new_ellipsoid(paxes,centre,icline,anglin,eye,light,c,objid); 
	   	result = (OOP) objid;
	    break;
    }

    case gxThreeDLineCreate:
    {
	    GenieRealArray eye(arguments,1), light(arguments,2);
	    GenieWorkspace table(arguments,3);
	    GenieRealArray end1(arguments,4), end2(arguments,5);
	    GenieInteger colour(arguments,6), dash(arguments,7);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    long ci[2]={1,2};
        ThreeDObjectColours c(0,0,0,0,ci);
	    new_threeDLine(end1,end2,colour,dash,eye,light,c,objid); 
	   	result = (OOP) objid;
	    break;
    }

	case gxRodCreate:
	{
	    GenieRealArray eye(arguments,1), light(arguments,2);
	    GenieWorkspace table(arguments,3);
        GenieRealArray end1(arguments,4), end2(arguments,5);
	    GenieReal radius(arguments,6);
	    GenieInteger nsides(arguments,7), lend(arguments,8);
	    GenieIntegerArray ci = table("CI");
	    GenieInteger ci_low = table("IC1");
	    GenieInteger ci_high = table("IC2");
        ThreeDObjectColours c(ci_low,ci_high,0,0,ci);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    new_rod(end1,end2,radius,nsides,lend,eye,light,c,objid); 
	   	result = (OOP) objid;
	    break;
    }

	case gxConeCreate:
	{
	    GenieRealArray eye(arguments,1), light(arguments,2);
	    GenieWorkspace table(arguments,3);
        GenieRealArray base(arguments,4), apex(arguments,5);
	    GenieReal radius(arguments,6);
	    GenieInteger nsides(arguments,7);
	    GenieIntegerArray ci = table("CI");
        GenieInteger ci_low = table("ic1");
        GenieInteger ci_high = table("ic2");
        ThreeDObjectColours c(ci_low,ci_high,0,0,ci);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    new_cone(base,apex,radius,nsides,eye,light,c,objid); 
	   	result = (OOP) objid;
	    break;
    }

    case gxBallCreate:
    {
	    GenieRealArray eye(arguments,1), light(arguments,2);
	    GenieWorkspace table(arguments,3);
	    GenieRealArray centre(arguments,4);
	    GenieReal radius(arguments,5);
	    GenieInteger transparency(arguments,6);  
	    GenieIntegerArray ci = table("CI");
	    GenieReal shin = table("SHINE");
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
        g_integer shine = 1;
        if ((g_real)shin == 0.0)
            shine=0;
	    GenieInteger ci_low = table("ic1");
        GenieInteger ci_high = table("ic2");
        ThreeDObjectColours c(ci_low,ci_high,shine,0,ci);
	    new_ball(centre,radius,transparency,eye,light,c,objid); 
	   	result = (OOP) objid;
	    break;
    }

    case gxIsoSurfaceCreate:
    {
	    GenieRealArray eye(arguments,1), light(arguments,2);
	    GenieWorkspace table(arguments,3);
	    GenieRealArray lattice(arguments,4), density(arguments,5);
	    GenieReal dsurf(arguments,6);
	    GenieInteger transparency(arguments,7);  
	    GenieIntegerArray ci = table("CI");
	    GenieReal shin = table("SHINE");
        g_integer shine = 1;
        if ((g_real)shin == 0.0)
            shine=0;
        GenieInteger ci_low = table("ic1");
        GenieInteger ci_high = table("ic2");
        ThreeDObjectColours c(ci_low,ci_high,shine,0,ci);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    new_isoSurface(dsurf, transparency, lattice, density, 
	       density.dims(1)-1, density.dims(2)-1, density.dims(3)-1, eye, 
	    light, c, objid); 
	   	result = (OOP) objid;
	    break;
    }

    case gxLatticePlaneCreate:
    {
	    GenieRealArray eye(arguments,1), light(arguments,2);
	    GenieWorkspace table(arguments,3);
	    GenieRealArray lattice(arguments,4), density(arguments,5);
	    GenieRealArray normal(arguments,6), apoint(arguments,7);
	    GenieInteger edge(arguments,8), transparency(arguments,9);  
	    GenieIntegerArray ci = table("CI");
        GenieInteger ci_low = table("ic1");
        GenieInteger ci_high = table("ic2");
        ThreeDObjectColours c(ci_low,ci_high,0,0,ci);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    new_latticePlane(normal, apoint, edge, transparency, lattice, density,
	        0, 0, 0, eye, light, c, objid); 
	   	result = (OOP) objid;
	    break;
    }

    case gxLatticeSliceCreate:
    {
	    GenieRealArray eye(arguments,1), light(arguments,2);
	    GenieWorkspace table(arguments,3);
	    GenieRealArray lattice(arguments,4), density(arguments,5);
	    GenieRealArray normal(arguments,6), apoint(arguments,7);
	    GenieInteger edge(arguments,8), transparency(arguments,9);  
        GenieReal dlow(arguments,10), dhigh(arguments,11);
	    GenieIntegerArray ci = table("CI");
        GenieInteger ci_low = table("ic1");
        GenieInteger ci_high = table("ic2");
        ThreeDObjectColours c(ci_low,ci_high,0,0,ci);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    new_latticeSlice(dlow, dhigh, normal, apoint, edge, 0, lattice, density, 
	        density.dims(1)-1, density.dims(2)-1, density.dims(3)-1, eye, light, 
	        c, objid); 
	   	result = (OOP) objid;
	    break;
    }

    case gxSurfaceCreate:
    {
	    GenieRealArray eye(arguments,1), light(arguments,2);
	    GenieWorkspace table(arguments,3);
	    GenieRealArray lattice(arguments,4), density(arguments,5);
	    GenieReal dlow(arguments,6), dhigh(arguments,7), dvert(arguments,8); 
	    GenieIntegerArray ci = table("CI");
	    GenieReal shin = table("SHINE");
	    GenieInteger ncband = table("NBANDS");
        g_integer shine = 1;
        if ((g_real)shin == 0.0)
            shine=0;
        GenieInteger ci_low = table("ic1");
        GenieInteger ci_high = table("ic2");
        ThreeDObjectColours c(ci_low,ci_high,shine,ncband,ci);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    new_surface(dlow, dhigh, dvert, lattice, density, 
	        density.dims(1)-1, density.dims(2)-1, eye, light, c, objid); 
	   	result = (OOP) objid;
	    break;
    }

	case gxPolylineCreate:
        {
           GenieReal xstart(arguments,1), ystart(arguments,2);
           GenieReal xend(arguments,3), yend(arguments, 4);
           GenieInteger colour(arguments, 5), linetype(arguments,6);
           GenieReal linewidth(arguments,7);
	   GenieObjectID  objid(GENIE_OBJECTID_SIZE);

           if(xstart.defined() && ystart.defined() && xend.defined() &&
              yend.defined())
           {
                new_line(xstart, ystart, xend, yend, colour, linetype,
                          linewidth, objid);
          	
		result = (OOP) objid; 
	   }
           break;
        }

	case gxTextCreate:
	{
	   GenieReal xpos(arguments, 1), ypos(arguments, 2);
	   GenieString WORDS(arguments, 3);
     	   GenieReal size(arguments, 4);
	   GenieReal angle(arguments, 5);
	   GenieInteger colour(arguments, 6);
	   GenieInteger quality(arguments, 7);
	   GenieReal width(arguments, 8);
	   GenieReal justify(arguments, 9);
	   GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    
           if(xpos.defined() && ypos.defined()) 
	   {

	     new_text(xpos, ypos, WORDS, size, angle, colour, quality, 
		      width, justify, objid);
		
	     result = (OOP) objid;

	   }
	   break;
	}

	case gxAxesCreate:
	{
	   GenieInteger colour(arguments,1);
	   GenieReal line_width(arguments,2), number_size(arguments,3);
	   GenieInteger quality(arguments,4), xlog(arguments,5);
	   GenieInteger ylog(arguments,6), xnum(arguments,7);
	   GenieInteger ynum(arguments,8), yhoriz(arguments,9);
	   GenieInteger type(arguments,10);
	   GenieObjectID objid(GENIE_OBJECTID_SIZE);
	   new_axes( colour, line_width, number_size, quality, xlog, 
		      ylog, xnum,  ynum, yhoriz,type,objid); 
	   
	   result = (OOP) objid;
	   break;
	}

	case gxGraticuleCreate:
        {
           GenieInteger colour(arguments,1);
           GenieReal line_width(arguments,2);
           GenieInteger xlog(arguments,3), ylog(arguments,4);
	   GenieReal num_size(arguments,5);
	   GenieInteger xnum(arguments,6), ynum(arguments,7);
	   GenieInteger yhoriz(arguments,8), quality(arguments,9);
	   GenieInteger type(arguments,10);
	   GenieObjectID objid(GENIE_OBJECTID_SIZE);

           new_graticule(colour, line_width, xlog, ylog, num_size, xnum, ynum,
		         yhoriz, quality, type, objid);

	   result = (OOP) objid;
           break;
        }

	case gxLabelsCreate:
        {
            GenieString xlabel(arguments,1), ylabel(arguments,2);
            GenieInteger colour(arguments,3), quality(arguments, 4);
            GenieReal charsize(arguments,5), linewidth(arguments, 6);
            GenieInteger xvert(arguments, 7), yvert(arguments, 8);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);

            if( xlabel.defined() && ylabel.defined())
            {
                new_labels(xlabel, ylabel, colour, quality, charsize,
                            linewidth, xvert, yvert, objid);
            
	   	result = (OOP) objid; 
	    }
            break;
        }

	case gxTitleCreate:
        {
           GenieString title(arguments,1);
           GenieInteger colour(arguments,2) , quality(arguments,3);
           GenieReal charsize(arguments, 4);
	   GenieObjectID objid(GENIE_OBJECTID_SIZE);

           if( title.defined())
           {
                new_title(title, colour, quality, charsize, objid);
           
		result = (OOP) objid;
	   }
           break;
        }

	case gxPlotCreate:
	{
	   GenieRealArray xarray(arguments,1), yarray(arguments,2);
           GenieInteger colour(arguments,3), type(arguments,4);
	   GenieReal linewidth(arguments, 5);
	   GenieObjectID objid(GENIE_OBJECTID_SIZE);
	   
	   if(xarray.defined() && yarray.defined())
	   { 
	      	   
		new_plot(xarray.length(), xarray, yarray, colour, type, 
			  linewidth, objid);
	  	result = (OOP) objid; 
	   }
	   else
	   {
	      gerr << "xarray or yarray is not defined" << endl;
	   }

	   break;
	}

	case gxHistogramCreate:
	{
	   GenieRealArray xarray(arguments,1), yarray(arguments,2);
	   GenieInteger colour(arguments,3), bin_centre(arguments,4);
	   GenieInteger type(arguments,5);
	   GenieReal linewidth(arguments,6);
	   GenieObjectID objid(GENIE_OBJECTID_SIZE);

	   if(xarray.defined() && yarray.defined())
           {
		new_histogram(yarray.length(), xarray, yarray, colour, 
			       bin_centre, type, linewidth, objid);
           	result = (OOP) objid;
	    }
           else
           {
              gerr << "xarray or yarray is not defined" << endl;
           }

	   break;
	}

	case gxMarkersCreate:
	{
           GenieRealArray xarray(arguments,1), yarray(arguments,2);
           GenieInteger colour(arguments,3), symbol(arguments,4);
	   GenieReal charsize(arguments,5), linewidth(arguments,6);
	   GenieObjectID objid(GENIE_OBJECTID_SIZE);
	   
	   if(xarray.defined() && yarray.defined())
           {
              new_markers(xarray.length(), xarray, yarray, colour, 
			  symbol, charsize, linewidth, objid); 
			 
              result = (OOP) objid; 
	   }
           else
           {
              gerr << "xarray or yarray is not defined" << endl;
	   }

	   break;
	}

	case gxHErrorsCreate:
        {
           GenieRealArray xarray(arguments,1), yarray(arguments,2);
           GenieRealArray earray(arguments,3);
           GenieInteger colour(arguments, 4), horiz(arguments,5);
           GenieReal linewidth(arguments,6);
	   GenieObjectID objid(GENIE_OBJECTID_SIZE);
	   
           if(xarray.defined() && yarray.defined() && earray.defined())
           {
		if(yarray.length() == earray.length())
		{
                    new_errors(yarray.length(), xarray, yarray, earray, 
			   	colour, horiz, linewidth, objid);
		    result = (OOP) objid;
		}
		else
		{
		    gerr << "earray does not have the same number of ";
		    gerr << "elements as x/yarray " << endl;
		}
	   }
           else
           {
                gerr << "xarray , earray or yarray is not ";
		gerr << "defined" << endl;
           }

           break;
        }

	case gxVErrorsCreate:
	{
           GenieRealArray xarray(arguments,1), yarray(arguments,2);
           GenieRealArray earray(arguments,3);
           GenieInteger colour(arguments, 4), vert(arguments,5);
	   GenieReal linewidth(arguments,6);
           GenieObjectID objid(GENIE_OBJECTID_SIZE); 

	   if(xarray.defined() && yarray.defined() && earray.defined())
           {
                new_errors(yarray.length(), xarray, yarray, earray, 
			   colour, vert, linewidth, objid); 
			 
          	result = (OOP) objid; 
	   }
           else
           {
                gerr << "xarray , earray or yarray is not ";
		gerr << "defined" << endl;
           }

           break;
	}

	case gxErrorsCreate:
	{
           GenieRealArray xarray(arguments,1), yarray(arguments,2);
           GenieRealArray earray(arguments,3);
           GenieInteger colour(arguments, 4), vert(arguments,5);
	   GenieReal linewidth(arguments,6);
           GenieObjectID objid(GENIE_OBJECTID_SIZE); 

	   if(xarray.defined() && yarray.defined() && earray.defined())
           {
                new_errors(yarray.length(), xarray, yarray, earray, 
			   colour, vert, linewidth, objid); 
			 
          	result = (OOP) objid; 
	   }
           else
           {
                gerr << "xarray , earray or yarray is not ";
		gerr << "defined" << endl;
           }
 	   break;
	}

	case gxContourCreate:
	{
	    GenieRealArray values(arguments,1);
    	    GenieRealArray tr (arguments,2), contours(arguments,3);
	    GenieInteger line_style(arguments,4); 
	    GenieInteger line_colour(arguments,5);
	    GenieReal line_thickness(arguments,6);
	    GenieInteger itype(arguments,7);
	    GenieInteger ncont(arguments, 8);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);

	    g_real* tr_matrix;
	    g_real* cont;
	    
	    if (values.defined())
	    {
	    	if (tr.defined())
            	{
                    tr_matrix = tr;
                }
                else
            	{
//               ginf << "Tr not defined, using substitute values" << endl;
                    tr_matrix = get_contour_tr_matrix(values.dims(1), 
		  				  values.dims(2));
            	}
	    	if (contours.defined())
            	{
                    cont = contours;
                    ncont = contours.length();
            	}
            	else
            	{
//                    ginf << "Contours not defined, using substitute values" << endl;
                    cont = get_contour_levels(values, values.length(), 
					      ncont, itype);
	    	}
	
	    	    new_contour (values, values.dims(1), values.dims(2), tr_matrix,
                                 cont, ncont, line_style, line_colour,
                                 line_thickness, objid);
            
		    result = (OOP) objid;	
	    	}

	    else
	    {
		gerr << "ERROR, values is not defined" << endl;
	    }

	    break;
	}

	case gxContourCreateArray:
	{
	    GenieRealArray values(arguments,1);
	    GenieRealArray wx(arguments,2), wy(arguments,3);
	    GenieRealArray contours(arguments,4);
	    GenieInteger line_style(arguments,5), line_colour(arguments,6);
	    GenieReal line_thickness(arguments,7);
	    GenieInteger itype(arguments,8), ncont(arguments,9);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);

	    g_real* cont = 0;
	    
	    if (values.defined())
	    {
	    	if (contours.defined())
            	{
                    cont = contours;
                    ncont = contours.length();
            	}
                else
            	{
//                    ginf << "Contours not defined, using substitute ";
//		    ginf << "values" << endl;
                    cont = get_contour_levels(values, values.length(), 
					      ncont, itype);
	    	}
	    }
	    else
	    {
		gerr << "ERROR, values is not defined" << endl;
	    }
	    
	    if(wx.defined() && wy.defined() && values.defined())
	    {
		new_contour_array (values, values.dims(1), values.dims(2),
				    wx, wx.dimsArray(), wx.dimsSize(), wy, 
				    wy.dimsArray(), wy.dimsSize(),cont, ncont, 
				    line_style, line_colour, line_thickness,
				    objid);
	   	
	   	result = (OOP) objid; 
	    }
	    else
	    {
		gerr << "ERROR values, wx or wy is not defined" << endl;
	    }

	    break;
        }

	case gxContourCreateFunction:
	{
	    GenieRealArray values(arguments,1); 
	    GenieString funcPtr(arguments,2);
            GenieRealArray contours(arguments,3);
            GenieInteger line_style(arguments,4), line_colour(arguments,5);
            GenieReal line_thickness(arguments,6);
	    GenieInteger itype(arguments,7), ncont(arguments,8);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);

	    g_real* cont = 0;
	    void* func_ptr;
        
	    if (values.defined())
	    {
	    	if (contours.defined())
            	{
                    cont = contours;
                    ncont = contours.length();
            	}
            	else
            	{
//                    ginf << "Contours not defined, " ; 
//		    ginf << "using substitute values" << endl;
                    cont = get_contour_levels(values, values.length(), 
					      ncont, itype);
            	}
	    }
	    else
	    {
		gerr << "ERROR, values is not defined " << endl;
	    }
 	    
	    func_ptr = genie_modules.getFunctionPointer(funcPtr);

	    if ((func_ptr != 0) && values.defined())
            {
                new_contour_function(values, values.dims(1), values.dims(2), 
				      func_ptr, cont, ncont, line_style, 
				      line_colour, line_thickness, objid);
            
		result = (OOP) objid;	
	    }
            else
            {
                gerr << "ERROR values or funcPtr is not defined" << endl;
            }

            break;
        }

	case gxContourCreateLabel:
	{    
	    GenieString label(arguments,1);
	    GenieInteger start(arguments,2), end(arguments,3);
	    GenieInteger step(arguments,4), intval(arguments,5);
	    GenieInteger minint(arguments, 6), colour(arguments,7);
	    GenieInteger font(arguments,8);
	    GenieReal height(arguments,9), line_width(arguments,10);
	    GenieObjectID contourid(arguments,11);

	    GenieObjectID objid(GENIE_OBJECTID_SIZE);

	    if(contourid.defined())
	    {
	    	new_label_contour(label, start, end, step, intval, minint, 
				  colour, font, height, line_width, contourid,
				  objid);
	   	result = (OOP) objid;	 
	    }
	    else
	    {
		gerr << "No Contour Specified" << endl;
	    }
	   
	    break;
	}

	case gxCellCreate:
	{
	    GenieRealArray values(arguments,1), tr(arguments, 2);
	    GenieReal valmax(arguments,3),valmin(arguments,4);
	    GenieInteger itype(arguments,5), mode(arguments,6);
	    GenieWorkspace colourtable(arguments,7);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
  	    g_real* tr_matrix;

	    if (colourtable.defined() && values.defined())
	    {
	    	if (!valmax.defined())
			valmax = values.array_max(1);
	    	if (!valmin.defined())
			valmin = values.array_min(1);
	    	if (tr.defined())
			tr_matrix = (g_real*)tr;
	    	else
			tr_matrix = get_contour_tr_matrix(values.dims(1), values.dims(2));
	  	GenieIntegerArray table = colourtable("CI");
		new_cellarray(values, values.dims(1), values.dims(2),valmax,
				     valmin, tr_matrix, itype, mode, table,
                                     table.length(), objid);
	    
		result = (OOP) objid;
	    }
	    else
	    {
		gerr << "ERROR: values or colourtable is not defined" << endl;
	    }
	    break;

	}

	case gxCellArrayCreate:
	{
	    GenieRealArray values(arguments,1), wx(arguments,2);
	    GenieRealArray wy(arguments,3);
	    GenieReal valmax(arguments,4),valmin(arguments,5);
	    GenieInteger itype(arguments,6), mode(arguments,7);
	    GenieWorkspace colourtable(arguments,8);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);

	    if(colourtable.defined() && values.defined() && wx.defined()
	       && wy.defined())
	    {
	    	if (!valmax.defined())
			valmax = values.array_max(1);
	    	if (!valmin.defined())
			valmin = values.array_min(1);
	    
	  	GenieIntegerArray table = colourtable("CI");

		new_cellarray_array(values, values.dims(1), values.dims(2),
				     wx, wx.dimsArray(), wx.dimsSize(), wy,
				     wy.dimsArray(), wy.dimsSize(), valmax, 
				     valmin, itype, mode, table,
                                     table.length(), objid);
	    
		result = (OOP) objid;
	    }
	    else
	    {
		gerr << "ERROR, table, values, wx or wy is ";
		gerr << "not defined" << endl;
	    }
	    break;
	}
	case gxCellWedgeCreate:
	{
	    GenieObjectID  cellid(arguments,1);
	    GenieInteger mode(arguments,2), cellPic(arguments,3); 
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    
    	    new_cellarray_wedge(cellid, mode, objid);
	    result = (OOP) objid; 
	    break;
	}
	
	case gxNewUnScaledWindow:
	{
	    GenieReal xDlower(arguments,1), xDupper(arguments,2);
	    GenieReal yDlower(arguments,3), yDupper(arguments,4);
	    GenieInteger colour(arguments,5);
	    GenieReal LXMIN(arguments,6), LXMAX(arguments,7);
            GenieReal LYMIN(arguments,8), LYMAX(arguments,9);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    float tmp1, tmp2, tmp3, tmp4;

            tmp1 = LXMIN;
            tmp2 = LXMAX;
            tmp3 = LYMIN;
            tmp4 = LYMAX;

	    new_window_unscaled(xDlower, yDlower, xDupper, yDupper, colour, 
				tmp1, tmp2, tmp3, tmp4, objid);

	    GenieWorkspace wk;
            wk("lxmin", tmp1);
            wk("lxmax", tmp2);
            wk("lymin", tmp3);
            wk("lymax", tmp4);
            wk("objid", objid);

            result = (OOP)wk;
	    break;
	}
	
	case gxNewScaledWindow:
	{
	    GenieReal xdl(arguments,1), ydl(arguments,2);
            GenieReal xdu(arguments,3), ydu(arguments,4);
	    GenieRealArray xa(arguments,5), ya(arguments,6);
	    GenieRealArray ea (arguments,7);
	    GenieInteger col(arguments,8), mins(arguments,9);
	    GenieInteger maxs(arguments,10);
	    GenieReal LXMIN (arguments,11), LYMIN (arguments,12);
	    GenieReal LXMAX (arguments,13), LYMAX (arguments,14);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE); 
	    int np;

	    np = xa.length();
	    if (ya.length() < np)
	    {
		np = ya.length();
	    }
	    if (ea.defined() && (ea.length() < np)) 
		np = ea.length();
		
	    new_window_scaled(xdl, ydl, xdu, ydu, xa, ya, ea, np, col,
			     mins, maxs, LXMIN, LXMAX, LYMIN, LYMAX,
			     objid);
		
	    result = (OOP) objid;
	    break;
	}
	
	case gxCreateMultiplotWindow:
	{
	    GenieReal xDlower(arguments,1), yDlower(arguments,2);
            GenieReal xDupper(arguments,3), yDupper(arguments,4);
	    GenieInteger colour(arguments,5), scaling_max_mode(arguments,6);
	    GenieInteger scaling_min_mode(arguments,7);
	    GenieReal LXMIN(arguments,8), LXMAX(arguments,9);
            GenieReal LYMIN(arguments,10), LYMAX(arguments,11);
            GenieObjectID objid(GENIE_OBJECTID_SIZE);
            float tmp1, tmp2, tmp3, tmp4;

            tmp1 = LXMIN;
            tmp2 = LXMAX;
            tmp3 = LYMIN;
            tmp4 = LYMAX;
	
	    new_window_multiplot(xDlower, yDlower, xDupper, yDupper, colour,
			         scaling_max_mode, scaling_min_mode, tmp1,
				 tmp2, tmp3, tmp4, objid); 

	    GenieWorkspace wk;
            wk("lxmin", tmp1);
            wk("lxmax", tmp2);
            wk("lymin", tmp3);
            wk("lymax", tmp4);
            wk("objid", objid);

            result = (OOP)wk; 
	    break;

	}
	
	case gxCreateWindowTwoD:
	{
	    GenieReal dev_xmin(arguments,1), dev_xmax(arguments,2);
	    GenieReal dev_ymin(arguments,3), dev_ymax(arguments,4);
	    GenieRealArray x(arguments,5), y(arguments,6);
	    GenieInteger colour(arguments,7);
	    GenieReal LXMIN(arguments,8), LXMAX(arguments,9);
	    GenieReal LYMIN(arguments,10), LYMAX(arguments,11);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    float tmp1, tmp2, tmp3, tmp4;
	
	    tmp1 = LXMIN; 
	    tmp2 = LXMAX; 
	    tmp3 = LYMIN; 
	    tmp4 = LYMAX;
 
	    if(x.defined() && y.defined())	    
	    {
		new_window_two_d (dev_xmin, dev_xmax, dev_ymin, dev_ymax,
			          x, x.length(), y, y.length(), colour, tmp1,
			   	      tmp2, tmp3, tmp4, objid);
	        GenieWorkspace wk;
	        wk("lxmin", tmp1);
	        wk("lxmax", tmp2);
	        wk("lymin", tmp3);
	        wk("lymax", tmp4);
	    	wk("objid", objid);
	
	        result = (OOP)wk;
	    }
	    else
	    {
		gerr << "x or y is not defined" << endl;
	    } 
	    break;
	}

	case gxCreateMultiplot:
	{
	    GenieInteger nhist(arguments,1);
	    GenieRealArray xarray(arguments,2);
	    GenieInteger chmin(arguments,3);
	    GenieInteger chmax(arguments,4);
	    GenieReal ygap(arguments,5);
	    GenieInteger colour(arguments,6), bin(arguments,7);
	    GenieInteger type(arguments,8);
	    GenieReal linewidth(arguments, 9);
	    long int npp;
	    if(xarray.defined())
            {
	    	npp = xarray.length();
	    	if((int)chmin == -1)
	    	{
		   chmin = 0;	
	    	}
	    	if((int)chmax == -1)
	    	{
		   chmax = npp -1;
	    	}

		create_multiplot(nhist, xarray, npp, chmin, chmax, ygap,
			  	 colour, bin, type, linewidth);
	    }
	    else
	    {
		gerr << "XARRAY ERROR, xarray is not defined" << endl;
	    }

	    break;
	}
	
	case gxAddSpectra:
	{
	    GenieRealArray yarray(arguments,1);
	    GenieInteger nhist(arguments,2);
	    const g_real* ydata = yarray.data();
	    int step, ndims = yarray.dimsSize();
	    int num=nhist;
	    int number = yarray.dims(1);

	    if (num > number)
	    {
		gerr << "ERROR, " ;
		ginf << "There are not enough yarray's to plot " << endl;
		ginf << num << " histograms " << endl;
	    }
	    else
	    {
	    	if (ndims == 1)
	    	{
			step = 0;
	    	}
	    	else
	    	{
			step = yarray.dope(1);
	    	}
	    	int YP = yarray.dims(ndims);
	    	for(int i=0; i<num; i++)
	    	{

	    		add_spectra(ydata, YP);
	    		ydata += step;
	    	}
	    }
	    break;
	}

	case gxAddSpectraArray:
	{
	    int np, all_ok(1);
            GenieInteger npoints(arguments, 1), ngroup(arguments, 2);
	    int* ya = new int[(size_t)npoints];
	    GenieString file(arguments, 3);
	    GenieIntegerArray spectra_array(arguments,4);
	    for(int i=0; i<spectra_array.length() && all_ok; i++)
            {
                np = get_spectrum(file, spectra_array[i], ya, npoints, ngroup);
// If get_spectrum() failed, we will already have an error message
		if (np == (g_integer)npoints)
                    add_spectra(ya, np);
		else
		    all_ok = 0;
            }
            delete[] ya;
	    break;
        }

	case gxMultiplotMake:
	{
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    
	    make_multiplot(objid);
	    result = (OOP) objid;
	    break;	

	}
   	
	case gxDeviceOpen:
 	{

	    funcPtrI MEF = 0;
            funcPtrI MHF = 0;
            funcPtr MRF = 0;

	    GenieInteger device_id;
	    GenieString device_type(arguments,1), filename(arguments,2);
	    GenieReal width(arguments,3), height(arguments,4);
	    GenieReal ra(arguments,5);
	
	    // device_id == -1 on error
	    device_id = open_device(device_type, filename, width, height, ra, 
			"Open GENIE","Genie",MEF,MRF,"Hardcopy",0,NULL,MHF);

	    clear_device();
//	    create_picture();
	    result = (OOP)device_id;
	    
	    break;

	}

	case gxDeviceClose:
	{
	    close_device();
	    break;

	}

	case gxDeviceClear:
	{
	    clear_device();
	    break;

	}

	case gxDeviceSelect:
   	{
	    GenieInteger device_id(arguments, 1);
	    select_device(device_id);
	    break;
	}

	case gxPictureSelect:
   	{
	    GenieInteger pictnum(arguments, 1), ret;
	    ret = select_picture(pictnum);
	    result = (OOP)ret;
	    break;
	}

//	case gxPictureCreate:
//	{
//	    create_picture();
//	    break;
//	
//	}

	case gxPictureDelete:
	{
	    delete_pictures();
	    break;

	}

	case gxPictureRedraw:
	{
	    GenieInteger pictnum(arguments,1), reset(arguments,2);
	    
	    clear_device();
	    redraw_picture(pictnum, reset);
	    
	    break;
	
	}

	case gxPictureUndraw:
	{
	    GenieInteger num(arguments,1);

	    undraw(num);
	    clear_device();
	    cout << "Device clearing" << endl;
	    redraw_picture(1,1);
	    break;

	}

	case gxPictureUndrawAll:
	{
	    undraw_all();
	    break;

	}

	case gxPictureWindowDelete:
	{
	    GenieInteger viewnum(arguments,1);
	
	    window_delete(viewnum);
	    break;
	
	}

	case gxPictureUndoUndraw:
	{
	    undo_undraw();
	    break;

	}

	case gxDisplay:
	{
	    GenieObjectID objid(arguments,1);
	    SmalltalkObject graphical_object(arguments,2);
	    display_graphical_object(objid, (OOP)graphical_object);
	    break;
	}
	
	 case gxCellFunctionCreate:
        {
            GenieRealArray values(arguments,1); 
            GenieReal valmax(arguments,2),valmin(arguments,3);
            GenieString func_string(arguments,4); 
	    GenieInteger itype(arguments,5), mode(arguments,6);
            GenieWorkspace colourtable(arguments,7);
            GenieObjectID objid(GENIE_OBJECTID_SIZE);
#if HAVE_DLFCN_H || defined(__VMS) || defined(__hpux)
            void *func_ptr = genie_modules.getFunctionPointer(func_string);  // (genie_cont_func_t)
#else
            void *func_ptr = sample_genie_coord_tran_func;    // in module_test_function.f
	    func_string = "sample_genie_coord_tran_func";
#endif /* HAVE_DLFCN_H */
	    if (func_ptr == 0)
	    {
		gerr << "CELL_FUNCTION: No function \"" << func_string << "\" currently loaded" << endl;
	 	break;
	    }
	    if (!valmax.defined())
		valmax = values.array_max(1);
	    if (!valmin.defined())
		valmin = values.array_min(1);
            if(colourtable.defined() && values.defined()) 
            {
                GenieIntegerArray table = colourtable("CI");
                new_cellarray_function(values, values.dims(1), values.dims(2),
                                       valmax, valmin, func_ptr, itype,
                                       mode, table, table.length(), objid);
                result = (OOP) objid;
            }
            else
            {
                gerr << "CELL_FUNCTION: table or values is not defined" << endl;
            }
            break;
	}
	
	case gxNewPicture:
	{
	    GenieInteger pictnum;
	    pictnum = create_picture();
	    return (OOP)GenieInteger(pictnum);
	    break;
  	
	}
    
	case gxGetCursor:
	{
	    GenieWorkspace wk;
	    GenieRealArray world(arguments,1); 
	    GenieObjectID id(arguments,2);
	    GenieInteger colour(arguments, 3);
	    float d[2] = { 0.0, 0.0 }, *w = world; 
	    char letter[2] = { '\0', '\0' };
	    pg_int old_colour, new_colour;

	    new_colour = colour;
	    pgqci(old_colour);
	    pgsci(new_colour);	// should really pass colour to cursor
	    get_cursor(w, d, letter, id);
	    pgsci(old_colour);

	    wk("w_x", w[0]);
	    wk("w_y", w[1]);
	    wk("d_x", d[0]);
	    wk("d_y", d[1]);
	    wk("char", letter);
	    result = (OOP)wk;
	    
	    break;
	}

	case gxGetObjectFromTag:
	{	
// Tags stored by a sprintf("lu") and are "pg" + pointer address
// However, we get passed just the pointer address by tkdriv.c
	    GenieString tag(arguments, 1);
	    result = (OOP)(unsigned long)atol(tag);
	    break;
	}

	case gxThreeDWindowCreate:
	{
	    GenieReal dev_xmin(arguments,1), dev_xmax(arguments,2);
	    GenieReal dev_ymin(arguments,3), dev_ymax(arguments,4);
	 	GenieInteger colour(arguments,5);
	    GenieReal LXMIN(arguments,6), LXMAX(arguments,7);
	    GenieReal LYMIN(arguments,8), LYMAX(arguments,9);
	    GenieRealArray eye(arguments,10), light(arguments,11);
	    GenieObjectID objid(GENIE_OBJECTID_SIZE);
	    float tmp1, tmp2, tmp3, tmp4;
	
	    tmp1 = LXMIN; 
	    tmp2 = LXMAX; 
	    tmp3 = LYMIN; 
	    tmp4 = LYMAX;
 

		new_window_three_d(dev_xmin, dev_xmax, dev_ymin, dev_ymax,
			          colour, tmp1, tmp2, tmp3, tmp4, eye, light, objid);
	        GenieWorkspace wk;
	        wk("lxmin", tmp1);
	        wk("lxmax", tmp2);
	        wk("lymin", tmp3);
	        wk("lymax", tmp4);
	    	wk("objid", objid);
	
	        result = (OOP)wk;

	    break;
	}

	case gxThreeDWindowFlushBitmap:
	{
	    GenieObjectID objid(arguments,1);
	    SmalltalkObject graphical_object(arguments,2);
	    three_d_window_flush_bitmap(objid, (OOP)graphical_object);
	    break;
	}

	default:
	    gerr << "Invalid graphics callout number: " << function << endl;
	    break;
    }
    ENDSAVEBLOCK
    return result;
}

