/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"

#include <iostream.h>

#include "smalltalk.h"
#include "genievars.h"
#include "g_peak.h"
#include "callout_list.h"

static const char* rcsid = "@(#)$Id: g_peak.cxx,v 1.13 1997/11/12 15:59:22 faa Exp $";
USE_RCSID(rcsid);

OOP g_peak( peak_function_code function, OOP arguments)
{
    GenieCalloutList gcl(arguments);

    BEGINSAVEBLOCK

    // Get data arrays & length
    GenieRealArray x(gcl, "x"), y(gcl, "y"), e(gcl, "e");
    x.setFlag(WriteBackInternalArrays);
    y.setFlag(WriteBackInternalArrays);
    e.setFlag(WriteBackInternalArrays);

    // Get result parameters & initial value options
    GenieRealArray pars(gcl, "pars"), sigpar(gcl, "sigpar");
    GenieIntegerArray ipropt(gcl, "ipropt");
    pars.setFlag(WriteBackInternalArrays);
    sigpar.setFlag(WriteBackInternalArrays);
    ipropt.setFlag(WriteBackInternalArrays);

//    GenieFloat xgrd, konst, x0, xavg, amp, var;

    // status return
    GenieInteger igood(gcl, "igood");
    fort_int igood_tmp;

    // work arrays needed by the routines
    fort_int n = x.length();
    fort_real* xw = new fort_real[n];
    fort_real* yw = new fort_real[n];
    fort_real* ew = new fort_real[n];

    switch(function)
    {
        case Peak_test:
//	    cout << "Peak/function module test called" << endl;
//            x.print << endl << y.print << endl << n.print << endl;
	    break;

        case Pkgaus: pkgaus(x, y, e, n, xw, yw, ew, pars, sigpar, ipropt, igood_tmp); break;
        case Pkgexp: pkgexp(x, y, e, n, xw, yw, ew, pars, sigpar, ipropt, igood_tmp); break;
        case Pklexp: pklexp(x, y, e, n, xw, yw, ew, pars, sigpar, ipropt, igood_tmp); break;
        case Pklorz: pklorz(x, y, e, n, xw, yw, ew, pars, sigpar, ipropt, igood_tmp); break;
        case Pkvogt: pkvogt(x, y, e, n, xw, yw, ew, pars, sigpar, ipropt, igood_tmp); break;
        case Pkvexp: pkvexp(x, y, e, n, xw, yw, ew, pars, sigpar, ipropt, igood_tmp); break;
        case Pkpoly: pkpoly(x, y, e, n, xw, yw, ew, pars, sigpar, ipropt, pars.length(), igood_tmp); break;

        case Fngaus: fngaus(x, y, n, pars); break;
        case Fngexp: fngexp(x, y, n, pars); break;
        case Fnlexp: fnlexp(x, y, n, pars); break;
        case Fnlorz: fnlorz(x, y, n, pars); break;
        case Fnvogt: fnvogt(x, y, n, pars); break;
        case Fnvexp: fnvexp(x, y, n, pars); break;
        case Fnpoly: fnpoly(x, y, n, pars, pars.length()); break;
        
        case Pkmulti: 
	{
	    GenieIntegerArray types(gcl, "TYPES");
	    pkmulti(x, y, e, n, xw, yw, ew, pars, sigpar, ipropt, types, pars.dims(1), pars.dims(2), igood_tmp);
	    break;
	}

        case Fnmulti: 
	{
	    GenieIntegerArray types(gcl, "TYPES");
	    fnmulti(x, y, yw, n, pars, pars.dims(1), pars.dims(2), types);
	    break;
	}

	default:
            PERROR("Peak routines called with invalid function(g_peak)");
            break;
    }
    delete[] xw;
    delete[] yw;
    delete[] ew;
    igood = igood_tmp;

    ENDSAVEBLOCK

    return arguments;
}
