/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef  G_PEAK__
#define G_PEAK__

#include "fortran_types.h"

// Header file for peak fitting module
typedef enum {
    Peak_test = 0,
    Pkinit = 1,
    Pkgaus = 2,
    Pkgexp = 3,
    Pklexp = 4,
    Pklorz = 5,
    Pkvogt = 6,
    Fngaus = 7,
    Fngexp = 8,
    Fnlexp = 9,
    Fnlorz = 10,
    Fnvogt = 11,
    Pkpoly = 12,
    Fnpoly = 13,
    Pkvexp = 14,
    Fnvexp = 15,
    Pkmulti = 16,
    Fnmulti = 17
} peak_function_code;

OOP g_peak(peak_function_code function, OOP arguments);

//
// C Prototypes for all fortran Peak/Function calls
//

#if HAVE_LIBF2C || defined(__unix)
#	define pkgaus pkgaus_
#	define pkgexp pkgexp_
#	define pklexp pklexp_
#	define pklorz pklorz_
#	define pkvogt pkvogt_
#	define pkvexp pkvexp_
#	define pkpoly pkpoly_
#	define pkmulti pkmulti_
#	define fngaus fngaus_
#	define fngexp fngexp_
#	define fnlexp fnlexp_
#	define fnlorz fnlorz_
#	define fnvogt fnvogt_
#	define fnvexp fnvexp_
#	define fnpoly fnpoly_
#	define fnmulti fnmulti_
#endif /* HAVE_LIBF2C || defined(__unix) */

#ifdef _WIN32
#	define pkgaus  PKGAUS
#	define pkgexp  PKGEXP
#	define pklexp  PKLEXP
#	define pklorz  PKLORZ
#	define pkvogt  PKVOGT
#	define pkvexp  PKVEXP
#	define pkpoly  PKPOLY
#	define pkmulti PKMULTI
#	define fngaus  FNGAUS
#	define fngexp  FNGEXP
#	define fnlexp  FNLEXP
#	define fnlorz  FNLORZ
#	define fnvogt  FNVOGT
#	define fnvexp  FNVEXP
#	define fnpoly  FNPOLY
#	define fnmulti FNMULTI
#endif /* _WIN32 */

extern "C"
{
    // peak fitting
    void pkgaus(FRA x, FRA y, FRA e, CFI n, FRA xw, FRA yw, FRA ew, FRA pars, FRA sigpar, FIA ipropt, FI igood);
    void pkgexp(FRA x, FRA y, FRA e, CFI n, FRA xw, FRA yw, FRA ew, FRA pars, FRA sigpar, FIA ipropt, FI igood);
    void pklexp(FRA x, FRA y, FRA e, CFI n, FRA xw, FRA yw, FRA ew, FRA pars, FRA sigpar, FIA ipropt, FI igood);
    void pklorz(FRA x, FRA y, FRA e, CFI n, FRA xw, FRA yw, FRA ew, FRA pars, FRA sigpar, FIA ipropt, FI igood);
    void pkvogt(FRA x, FRA y, FRA e, CFI n, FRA xw, FRA yw, FRA ew, FRA pars, FRA sigpar, FIA ipropt, FI igood);
    void pkvexp(FRA x, FRA y, FRA e, CFI n, FRA xw, FRA yw, FRA ew, FRA pars, FRA sigpar, FIA ipropt, FI igood);
    void pkpoly(FRA x, FRA y, FRA e, CFI n, FRA xw, FRA yw, FRA ew, FRA pars, FRA sigpar, FIA ipropt, CFI npars, FI igood);

    // peak generating
    void fngaus(FRA x, FRA y, CFI n, FRA pars);
    void fngexp(FRA x, FRA y, CFI n, FRA pars);
    void fnlexp(FRA x, FRA y, CFI n, FRA pars);
    void fnlorz(FRA x, FRA y, CFI n, FRA pars);
    void fnvogt(FRA x, FRA y, CFI n, FRA pars);
    void fnvexp(FRA x, FRA y, CFI n, FRA pars);
    void fnpoly(FRA x, FRA y, CFI n, FRA pars, CFI npars);

    void pkmulti(FRA x, FRA y, FRA e, CFI n, FRA xw, FRA yw, FRA ew, FRA pars, FRA sigpar, FIA ipropt, FIA types, CFI npeaks, CFI npkpars, CFI igood);
    void fnmulti(FRA x, FRA y, FRA yw, CFI n, FRA pars, CFI npeaks, CFI npars, FIA types);
}

#endif // G_PEAK__
