C   Open GENIE, a program for neutron scattering data analysis & display
C   ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
C   Copyright (C) 1996,1997
C
C   This program is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
C
C 
C Program:	GAUSS_ELIM
C Author:	Freddie Akeroyd, ISIS
C Description:	Solve Ax=B by Gaussian elimination with partial pivoting 
C               A(N,N) B(N,M) x(N,M) i.e. M solutions of NxN system
C               A overwritten and solutions returned in B
C
      SUBROUTINE GAUSS_ELIM(A,NAMAX,N,B,NBMAX,M)
      IMPLICIT NONE
      INTEGER I,J,N,M,NAMAX,NBMAX,IPIVOT,L
      REAL AMAX,A(NAMAX,N),B(NBMAX,M),TEMP,FRAC
C      CALL PRINT_MATRIX(A,NAMAX,N,N)
      DO J=1,N
C Find pivot element (the largest) and swap rows if necessary
          AMAX=ABS(A(J,J))
          IPIVOT=J
          DO I=J,N
              IF (ABS(A(I,J)) .GT. AMAX) THEN
                  AMAX = ABS(A(I,J))
                  IPIVOT = I
              ENDIF
          ENDDO
          IF (AMAX .LT. 1.E-20) THEN
              WRITE(6,*) 'GAUSS_ELIM: The matrix is singular'
              RETURN
          ENDIF
          CALL SWAP_ROWS(A,NAMAX,N,J,IPIVOT)
          CALL SWAP_ROWS(B,NBMAX,M,J,IPIVOT)
C Now subtract the pivot row J from the others
          DO I=J+1,N
              FRAC = A(I,J) / A(J,J)
              CALL SUBTRACT_ROWS(A, NAMAX, N, FRAC, J, I)
              CALL SUBTRACT_ROWS(B, NBMAX, M, FRAC, J, I)
          ENDDO
C          CALL PRINT_MATRIX(A,NAMAX,N,N)
      ENDDO
C      CALL PRINT_MATRIX(B,NBMAX,N,M)
C A is now upper triangular - solve equations by back substitution
C Overwrite B with the solution
      DO L=1,M
          DO I=N,1,-1
              TEMP = B(I,L)
              DO J=I+1,N
                  TEMP = TEMP - A(I,J) * B(J,L)
              ENDDO
              B(I,L) = TEMP / A(I,I)
          ENDDO
      ENDDO
C      WRITE(6,*) 'Solution'
C      CALL PRINT_MATRIX(B,NBMAX,N,M)
      RETURN
      END
          
C Swap rows I and J of matrix A(N,N)
      SUBROUTINE SWAP_ROWS(A,NMAX,N,I,J)
      IMPLICIT NONE
      INTEGER I,J,K,NMAX,N
      REAL TMP,A(NMAX,N)
      DO K=1,N
          TMP=A(I,K)
          A(I,K)=A(J,K)
          A(J,K)=TMP
      ENDDO
      RETURN
      END

C Subtract FRAC of row I from row J
      SUBROUTINE SUBTRACT_ROWS(A,NMAX,N,FRAC,I,J)
      IMPLICIT NONE
      INTEGER I,J,K,NMAX,N
      REAL A(NMAX,N),FRAC
      DO K=1,N
          A(J,K) = A(J,K) - FRAC * A(I,K)
      ENDDO
      RETURN
      END

C print out a matrix
      SUBROUTINE PRINT_MATRIX(A,NMAX,N,M)
      IMPLICIT NONE
      INTEGER I,J,N,NMAX,M
      REAL A(NMAX,M)
      WRITE(6,*) '-------------'
      DO I=1,N
          WRITE(6,*) (A(I,J),J=1,M)
          WRITE(6,*) ' '
      ENDDO
      WRITE(6,*) '-------------'
      RETURN
      END
