/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *	Module:	 	GenieArray
 *	Author:	 	Freddie Akeroyd, ISIS
 *	Purpose: 	Provide template classes for definition of arrays of genie variables
 */
#include "geniedefs.h"
#include <fstream.h>

#include <string.h>
#include <gnustring.h>
#include <ctype.h>

#include "smalltalk.h"

static const char* rcsid = "@(#)$Id: genie_array.cxx,v 1.25 1997/11/04 17:54:27 cmm Exp $";
USE_RCSID(rcsid);

int associateGenieVariables(OOP oopParameters, ... );

char* Strdup(const char* old_string);
char* Strdup(const char* old_string, int length);
char* Strdup(const unsigned char* old_string);
char* Strdup(const unsigned char* old_string, int length);

class GenieVariable;	// forward declaration --- needed for workspace

#include "genievars.h"

const int INST_VAR_OFFSET = 2;

#define Array GenieArray<GenieType, CDataType, GXType>

template <class GenieType, class CDataType, GenieVariableType GXType>
OOP GenieArray<GenieType, CDataType, GXType>::allocOOP(OOP a_dimsOOP)
{
    static OOP gxnew = registerOOP(internString("new:"));
    static OOP gxarray = registerOOP(typeNameToOOP(GXVariableNames[GXType]));
    return saveOOP(msgSend(gxarray, gxnew, a_dimsOOP, 0)); 
}

template <class GenieType, class CDataType, GenieVariableType GXType>
OOP GenieArray<GenieType, CDataType, GXType>::allocOOP(int dim1, int dim2)
{
    return allocOOP(makeDimsOOP(dim1, dim2));
}

template <class GenieType, class CDataType, GenieVariableType GXType>
OOP GenieArray<GenieType, CDataType, GXType>::become(OOP oop)
{
    GenieArrayBase::become(oop);
    updateDimsAndDope();
    return oop;
}
    

template <class GenieType, class CDataType, GenieVariableType GXType>
GenieArray<GenieType, CDataType, GXType>::GenieArray(int dim1, int dim2) : GenieArrayBase(nilOOP)
{
    alloc(dim1, dim2);
}

#if 0
template <class GenieType, class CDataType, GenieVariableType GXType>
void GenieArray<GenieType, CDataType, GXType>::alloc(int dim1, int dim2)
{
    static OOP gxnew = registerOOP(internString("new:"));
    static OOP gxarray = registerOOP(typeNameToOOP(GXVariableNames[GXType]));
    OOP a_dimsOOP = makeDimsOOP(dim1, dim2);
//    m_value = instantiateOOPWith(findClass(internString(Strdup(GXVariableNames[GXType]))), len);
//    saveOOP(m_value);
    m_value = saveOOP(msgSend(gxarray, gxnew, a_dimsOOP, 0));
//    setDims(dim1, dim2);
    return;
}
#endif

template <class GenieType, class CDataType, GenieVariableType GXType>
OOP GenieArray<GenieType, CDataType, GXType>::shallowCopy(const GenieArray<GenieType, CDataType, GXType>& ga)
{
    return GenieArrayBase::shallowCopy(ga);
}

template <class GenieType, class CDataType, GenieVariableType GXType>
void GenieArray<GenieType, CDataType, GXType>::operator=(const GenieArray<GenieType, CDataType, GXType>& ga)
{
    if (this != &ga)
    {
	deepCopy(ga);
//    }
// Copy elements
//    for(int i=1; i<=m_length; i++)
//    {
//	arrayAtPut(m_value, i+INST_VAR_OFFSET, arrayAt(ga.m_value, i+INST_VAR_OFFSET));
//    }
    }
}

// nil_flag: 0=pass nils, 1=skip nils
template <class GenieType, class CDataType, GenieVariableType GXType>
void GenieArray<GenieType, CDataType, GXType>::withCollect(CDataType (*func)(CDataType, CDataType), Array& ga2, int nil_flag)
{
    Array& ga1 = *this;
    OOP element1, element2;
    for(int i=1; i<=length(); i++)
    {
	element1 = indexOOP(ga1.value(), i);
	element2 = indexOOP(ga2.value(), i);
	if ( (isNil(element1) || isNil(element2)) && nil_flag )
	{
	    indexOOPPut(m_value, i, nilOOP);
	}
	else
	{
	    indexOOPPut(m_value, i, GenieType::toOOP(func(GenieType::fromOOP(element1),GenieType::fromOOP(element2))));
	}
    }
    return;
}

template <class GenieType, class CDataType, GenieVariableType GXType>
void GenieArray<GenieType, CDataType, GXType>::collect(CDataType (*func)(CDataType))
{
    OOP element;
    for(int i=1; i<=length(); i++)
    {
	element = indexOOP(m_value, i);
	if (!isNil(element))
	{
	    indexOOPPut(m_value, i, GenieType::toOOP(func(GenieType::fromOOP(element))));
	}
    }
    return;
}

template <class GenieType, class CDataType, GenieVariableType GXType>
void GenieArray<GenieType, CDataType, GXType>::operator()(const CDataType* ca, int len)
{
  BEGINSAVEBLOCK
    alloc(len);
    saveOOP(m_value);
    for(int i=1; i<=len; i++)
    {
	arrayAtPut(m_value, i+INST_VAR_OFFSET, GenieType::toOOP(ca[i-1]));
    }
  ENDSAVEBLOCK
    return;
}

#if 0
template <class GenieType, class CDataType, GenieVariableType GXType>
void GenieArray<GenieType, CDataType, GXType>::alloc(const GenieArrayBase& ga)
{
    int len = ga.length();
  BEGINSAVEBLOCK
    m_value = instantiateOOPWith(findClass(internString(Strdup(GXVariableNames[GXType]))), len);
    saveOOP(m_value);
    copyDims(ga);
  ENDSAVEBLOCK
    return;
}

template <class GenieType, class CDataType, GenieVariableType GXType>
//int GenieArray<GenieType, CDataType, GXType>::alloc(const GenieArray<GenieType, CDataType, GXType>& ga, CDataType fill)
void GenieArray<GenieType, CDataType, GXType>::alloc(const GenieArrayBase& ga, CDataType fill)
{
    int len = ga.length();
  BEGINSAVEBLOCK
    m_value = instantiateOOPWith(findClass(internString(Strdup(GXVariableNames[GXType]))), len);
    saveOOP(m_value);
    copyDims(ga);
// Now fill the array up with whatever we wished
    for(int i=1; i<=len; i++)
    {
	arrayAtPut(m_value, i+INST_VAR_OFFSET, GenieType::toOOP(fill));
    }
  ENDSAVEBLOCK
    return;
}
#endif


template <class GenieType, class CDataType, GenieVariableType GXType>
OOP GenieArray<GenieType, CDataType, GXType>::deepCopy(const GenieArray<GenieType, CDataType, GXType>& ga)
{
  BEGINSAVEBLOCK
    alloc(ga.length());
    saveOOP(m_value);
// Now fill the array up with whatever we wished
    for(int i=1; i<=length(); i++)
    {
	arrayAtPut(m_value, i+INST_VAR_OFFSET, arrayAt(ga.value(), i+INST_VAR_OFFSET));
    }
  ENDSAVEBLOCK
    return m_value;
}

template <class GenieType, class CDataType, GenieVariableType GXType>
OOP GenieArray<GenieType, CDataType, GXType>::deepCopyOOP(OOP oop)
{
    GenieArray<GenieType, CDataType, GXType> temp(oop);
    return temp.deepCopy();
}
    
template <class GenieType, class CDataType, GenieVariableType GXType>
ostream& GenieArray<GenieType, CDataType, GXType>::print(ostream& os, const char* note) const
{
    GenieArrayBase::print(os, note);
    long old_state = os.setf(ios::showpoint);
    os << GXVariableNames[GXType] << " with " << length() << " elements";
    if (dimsSize() > 1)
	os << " and " << dimsSize() << " dimensions" << endl;
    else
	os << endl;
//    for(int j=1; j<=dimsSize(); j++)
//    {
	os << "\t";
    	for(int i=1; i<=length(); i++)
    	{
	    os << GenieType::fromOOP(indexOOP(m_value, i)) << " ";
	    if (i > 6)
	    	break;
    	}
	os << "... (" << length() << " elements)" << endl;
//    }
    os.setf(old_state);
    return os;
}

ostream& operator<<(ostream& os, const GenieIntegerArray& ga)
{
    return ga.print(os);
}

ostream& operator<<(ostream& os, const GenieRealArray& ga)
{
    return ga.print(os);
}

template <class GenieType, class CDataType, GenieVariableType GXType>
bool GenieArray<GenieType, CDataType, GXType>::operator==(const GenieArray<GenieType, CDataType, GXType>& ga)
{
    if (length() != ga.length())
	return false;
    for(int i=1; i<=length(); i++)
    {
	if ((*this)(i) != ga(i))
	    return false;
    }
    return true;
}

template <class GenieType, class CDataType, GenieVariableType GXType>
GenieArray<GenieType, CDataType, GXType>::GenieArray(const GenieArray<GenieType, CDataType, GXType>& ga) : GenieArrayBase(ga.m_value)
{
}

template <class GenieType, class CDataType, GenieVariableType GXType>
OOP GenieArray<GenieType, CDataType, GXType>::shallowCopyOOP(OOP oop)
{
    return GenieVariableBase::shallowCopyOOP(oop);
}

#if 0
template <class GenieType, class CDataType, GenieVariableType GXType>
void GenieArray<GenieType, CDataType, GXType>::add(int dim1, CDataType val)
{
    (*this)(dim1, (*this)(dim1) + val);
}
#endif

// Instantiate templates - this is compiler specific
#if defined(__GNUG__) || defined(_MSC_VER)

#ifdef _MSC_VER
#pragma warning(disable:4660)
#endif

template class GenieArray<GenieWorkspace, g_work, GenieWorkspaceArrayType>;
template class GenieArray<GenieString, g_string, GenieStringArrayType>;

#ifdef _MSC_VER
#pragma warning(default:4660)
#endif

#endif /* __GNUG__ */

#ifdef __DECCXX
#pragma define_template GenieArray<GenieWorkspace, g_work, GenieWorkspaceArrayType>
#pragma define_template GenieArray<GenieString, g_string, GenieStringArrayType>
#endif /* __DECCXX */
