/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *      Module:         genie_base
 *      Author:         Freddie Akeroyd, ISIS
 *      Purpose:        Definitions common to all genie variables
 *
 */
#include "geniedefs.h"
#include "genievars.h"
#include "callout_list.h"

static const char* rcsid = "@(#)$Id: genie_base.cxx,v 1.26 1997/05/06 09:56:57 cmm Exp $";
USE_RCSID(rcsid);

ostream& GenieVariableBase::print(ostream& os, const char* note) const
{
    if (note != NULL) 
    { 
	os << note << endl; 
        os << "OOP Constructor" << endl;
        if (isOOP(m_value))
        {
	    printOOPConstructor(m_value);
        }
        else
        {
	    os << "Integer = " << toInt(m_value) << endl;
        }
    }
    return os;
}

void GenieVariableBase::init(OOP oop, OOP params, int _location)
{
    m_value = oop;
    m_old_value = oop;
    m_location = _location;
    m_params = params;
    m_old_class = (m_value != nilOOP ? getClass() : undefinedObjectClass);
//    m_old_class = stclass;
    return;
}

GenieVariableBase::~GenieVariableBase()
{
    if ( (m_location != 0) && (m_params != nilOOP) )	// A zero location means no association to a named smalltalk Genie variable
    {
        if ( (m_old_class != getClass()) && (!isNil(m_old_value)) )
        {
	    gerr << "GenieVariables(Illegal class change on associate return at location " << m_location << ")" << endl;
	    this->print(gerr, "The Variable(C++ print)");
	    gerr << "New and old values" << endl;
	    if (isOOP(m_value)) { printOOPConstructor(m_value); }
	 	else { gerr << "Currently an Integer: " << toInt(m_value) << endl; }
	    if (isOOP(m_old_value)) { printOOPConstructor(m_old_value); }
	 	else { gerr << "Was an Integer: " << toInt(m_old_value) << endl; }
	    gerr << "New class: " << getClassString(getClass()) << endl;
	    gerr << "Old class: " << getClassString(m_old_class) << endl;
	    m_value = nilOOP;
    	    throwException(InvalidType);
        }
//	maybeMoveOOP(m_params);
//	if (isOOP(m_old_value) && !isNil(m_old_value) && isOOP(m_value) &&!isNil(m_value))
//  	{
//	    cout << "New putback\n";
//	    swapObjects(m_value, m_old_value);
//	}
//	else
//	{
            arrayAtPut(m_params, m_location, m_value);
//	}
    }
}       

// returns 0 on success, 1 on error
int GenieVariableBase::associate(OOP parameters, const char* name)
{
    return ::associateGenieVariables(parameters, name, this, NULL);
}

// returns 0 on success, 1 on error
int GenieVariableBase::associate(const GenieCalloutList& gcl, const char* name)
{
    return gcl.associate(name, this);
}

GenieVariableBase& GenieVariableBase::operator=(const GenieVariableBase& gvb)
{
    if (this != &gvb)
    {
	shallowCopy(gvb);
    }
    return *this;
}

// These routines are called to write an "Undefined" variable
GenieVariableType GenieVariableBase::skip(IntermediateFile* gif)
{
    return GenieBaseType;
}

GenieVariableType GenieVariableBase::writeAscii(fstream& fs, int indent)
{
    return GenieBaseType;
}

GenieVariableType GenieVariableBase::readAscii(fstream& fs)
{
    setOOP(nilOOP);
    return GenieBaseType;
}

GenieVariableType GenieVariableBase::read(IntermediateFile* gif)
{
    setOOP(nilOOP);
    return GenieBaseType;
}

GenieVariableType GenieVariableBase::write(IntermediateFile* gif)
{
    return GenieBaseType;
}

ostream& operator<<(ostream& os, const GenieVariableBase& gv)
{
    return gv.print(os);
}

void saveGenieVariable(const GenieVariableBase& gvb)
{
    incAddOOP(gvb.value());
}

OOP GenieVariableBase::deepCopyOOP(OOP oop)
{
    static OOP deepcopy = registerOOP(internString("deepCopy"));
    return saveOOP(msgSend(oop, deepcopy, 0));
}

OOP GenieVariableBase::shallowCopyOOP(OOP oop)
{
    static OOP shallowcopy = registerOOP(internString("shallowCopy"));
    return saveOOP(msgSend(oop, shallowcopy, 0));
}

OOP GenieVariableBase::inspectOOP(OOP oop)
{
    static OOP Inspect = registerOOP(internString("inspect"));
    return msgSend(oop, Inspect, 0);
}

OOP GenieVariableBase::inspect()
{
    return inspectOOP(m_value);
}

OOP GenieVariableBase::at(OOP pos) const
{
    static OOP at = registerOOP(internString("at:"));
    return msgSend(m_value, at, pos, 0);
}

// Like at, but return "undefined" if field is not present rather than a smalltalk error message
OOP GenieVariableBase::atOrNil(OOP pos) const
{
//    static OOP atIfAbsent = registerOOP(internString("at:ifAbsent:"));
//    static OOP absent_block = registerOOP(evalExpr("[ nil ]"));
//    return msgSend(m_value, atIfAbsent, pos,  absent_block, 0);
    return at(pos);
}

OOP GenieVariableBase::atPut(OOP pos, OOP val)
{
    static OOP atput = registerOOP(internString("at:put:"));
    return msgSend(m_value, atput, pos, val, 0);
}

OOP saveOOP(OOP oop)
{
    incAddOOP(oop);
    return oop;
}

void GenieVariableBase::deepCopy(const GenieVariableBase& gvb)
{
    m_value = gvb.deepCopy();
}

void GenieVariableBase::shallowCopy(const GenieVariableBase& gvb)
{
    m_value = gvb.shallowCopy();
}

int GenieVariableBase::associate(OOP parameters, int location) 
{
    OOP values = arrayAt(parameters, 2);
    int value_location = location - 1;	// location starts at 1, but Genie callout starts at 2
    init(instVarAt(values, value_location), values, value_location);
    return 0;
}

GenieVariableType GenieVariableBase::type() const
{
    return getGenieVariableType(m_value);
}

int GenieVariableBase::indentAscii(fstream& fs, int indent)
{
    for(int i=0; i<indent; i++)
    {
	fs.put(' ');
    }
    return 0;
}
