/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *      Module:         genie_cmodule
 *      Author:         Freddie Akeroyd, ISIS
 *      Purpose:        C MODULE interface to calling user programs
 *
 */
#include "geniedefs.h"
#include "genie_cmodule.h"
#include "smalltalk.h"
#include "genievars.h"
#include "module.h"
#include "g_module.h"
#include "g_tktcl.h"
#include "module_inquire.h"

static const char* rcsid = "@(#)$Id: genie_cmodule.cxx,v 1.14 1998/07/07 15:41:33 faa Exp $";
USE_RCSID(rcsid);

GENIEEXPORT void cmodule_get_real(GenieWorkspace* pars, const char* name, fort_real* val)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_get_real(*pars, name_s, *val);
}

GENIEEXPORT void cmodule_get_double(GenieWorkspace* pars, const char* name, fort_double* val)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_get_double(*pars, name_s, *val);
}

GENIEEXPORT void cmodule_put_real(GenieWorkspace* res, const char* name, fort_real val)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_put_real(*res, name_s, val);
}

GENIEEXPORT void cmodule_print(const char* s, fort_int option)
{
    STRING name_s; name_s.length = strlen(s); strcpy(name_s.spot, s);
    g3_module_print(name_s, option);
}

GENIEEXPORT fort_int cmodule_get_version(fort_int* major, fort_int* minor)
{
    return g3_module_get_version(*major, *minor);
}

GENIEEXPORT fort_int cmodule_version_ok(fort_int major, fort_int minor)
{
    return g3_module_version_ok(major, minor);
}

GENIEEXPORT void cmodule_put_double(GenieWorkspace* res, const char* name, fort_double val)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_put_double(*res, name_s, val);
}

GENIEEXPORT void cmodule_get_string(GenieWorkspace* pars, const char* name, char** val)
{
    STRING val_s; 
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_get_string(*pars, name_s, val_s);
    *val = (char*)cmodule_malloc(1+val_s.length, NULL);
    strcpy(*val, val_s.spot);
}

GENIEEXPORT void cmodule_put_string(GenieWorkspace* res, const char* name, const char* val)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    STRING val_s; val_s.length = strlen(val); strcpy(val_s.spot, val);
    g3_module_put_string(*res, name_s, val_s);
}

GENIEEXPORT void cmodule_put_string_array(GenieWorkspace* res, const char* name, const char* val[], fort_int len)
{
    STRING val_s;
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    fort_int handle[2];
    g3_module_string_array_alloc(len, handle);
    for(int i=0; (i < len) && (val[i] != 0); i++)
    {
 	val_s.length = strlen(val[i]);
	strcpy(val_s.spot, val[i]);
	g3_module_string_array_add(handle, i+1, val_s);
    }
    g3_module_string_array_free(*res, name_s, handle);
}

GENIEEXPORT void cmodule_get_int(GenieWorkspace* pars, const char* name, fort_int* val)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_get_int(*pars, name_s, *val);
}

GENIEEXPORT void cmodule_put_int(GenieWorkspace* res, const char* name, fort_int val)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_put_int(*res, name_s, val);
}

// Allocates space for returned array with malloc()
GENIEEXPORT void cmodule_get_real_array(GenieWorkspace* pars, const char* name, fort_real** val, fort_int* len)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_get_real_array_alloc(*pars, name_s, val, *len);
}

// Allocates space for returned array with malloc()
GENIEEXPORT void cmodule_get_double_array(GenieWorkspace* pars, const char* name, fort_double** val, fort_int* len)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_get_double_array_alloc(*pars, name_s, val, *len);
}
    
// Allocates space for returned array with malloc()
GENIEEXPORT void cmodule_get_int_array(GenieWorkspace* pars, const char* name, fort_int** val, fort_int* len)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_get_int_array_alloc(*pars, name_s, val, *len);
}

GENIEEXPORT void cmodule_put_real_array(GenieWorkspace* pars, const char* name, const fort_real* val, fort_int len)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_put_real_array(*pars, name_s, val, len);
}

GENIEEXPORT void cmodule_put_double_array(GenieWorkspace* pars, const char* name, const fort_double* val, fort_int len)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_put_double_array(*pars, name_s, val, len);
}

GENIEEXPORT void cmodule_put_int_array(GenieWorkspace* pars, const char* name, const fort_int* val, fort_int len)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_put_int_array(*pars, name_s, val, len);
}

GENIEEXPORT fort_int cmodule_put_nd_real_array(GenieWorkspace* pars, const char* name, const fort_real* val, 
	const fort_int* dims_array, fort_int ndims)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    return g3_module_put_nd_real_array(*pars, name_s, val, dims_array, ndims);
}

GENIEEXPORT void* cmodule_inquire_begin(const char* title)
{
    int* dialog;
    STRING title_s; 
    title_s.length = strlen(title); 
    strcpy(title_s.spot, title);
    dialog = (int*)malloc(sizeof(DialogueBox));  
    g3_module_tk_inquire_begin(title_s, dialog); 
    return (void*)dialog;
}   

GENIEEXPORT void cmodule_inquire_int(void* dialog, const char* label, fort_int* val)
{
    STRING label_s; 
    label_s.length = strlen(label); 
    strcpy(label_s.spot, label);    
    g3_module_tk_inquire_int((int*)dialog, label_s, *val);
    return;
}

GENIEEXPORT void cmodule_inquire_real(void* dialog, const char* label, fort_real* val)
{
    STRING label_s; 
    label_s.length = strlen(label); 
    strcpy(label_s.spot, label);    
    g3_module_tk_inquire_real((int*)dialog, label_s, *val);
    return;
}

GENIEEXPORT void cmodule_inquire_double(void* dialog, const char* label, fort_double* val)
{
    STRING label_s; 
    label_s.length = strlen(label); 
    strcpy(label_s.spot, label);    
    g3_module_tk_inquire_double((int*)dialog, label_s, *val);
    return;
}

GENIEEXPORT void cmodule_inquire_string(void* dialog, const char* label, char* val, fort_int len)
{
    STRING label_s; 
    label_s.length = strlen(label); 
    strcpy(label_s.spot, label);  
    g3_module_tk_inquire_string((int*)dialog, label_s, val, len);
    return;
}

GENIEEXPORT void cmodule_inquire_end(void* dialog)
{
    g3_module_tk_inquire_end((int*)dialog);
    free(dialog);
    return;
}

GENIEEXPORT void* cmodule_malloc(int nbytes, void* heap)
{
    void* address;
    int array[1];
    long offset;
    g3_module_malloc(nbytes, 1, array, (int*)&address, offset, (fort_int*)&heap);   
    return address;
}

GENIEEXPORT void cmodule_free(void* address, void* heap)
{
    g3_module_free((fort_int*)&address, (fort_int*)&heap);
}

GENIEEXPORT void cmodule_freeall(void* heap)
{
    g3_module_freeall((fort_int*)&heap);
}

GENIEEXPORT void cmodule_freeallheaps()
{
    g3_module_freeallheaps();
}

GENIEEXPORT void* cmodule_makeheap()
{
    void* heap;
    g3_module_makeheap((fort_int*)&heap);
    return heap;
}
