/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *      Module:         genie_int
 *      Author:         Freddie Akeroyd, ISIS
 *
 */
#include "geniedefs.h"
#include <stdlib.h>
#include "genievars.h"
#include "intermediate_file.h"

static const char* rcsid = "@(#)$Id: genie_int.cxx,v 1.19 1998/02/01 14:31:12 faa Exp $";
USE_RCSID(rcsid);

ostream& GenieInteger::print(ostream& os, const char* note) const
{
    GenieVariableBase::print(os, note);
    return os << (g_integer)(*this);
}

static const unsigned char g_int_size = (unsigned char)sizeof(g_integer);

GenieVariableType GenieInteger::write(IntermediateFile* gif)
{
    if (gif->writeInteger(*this))
    {
        return GenieIntegerType;
    }
    else
    {
	return GenieTypeError;
    }
}

GenieVariableType GenieInteger::skip(IntermediateFile* gif)
{
    if (gif->skipInteger())
    {
        return GenieIntegerType;
    }
    else
    {
	return GenieTypeError;
    }
}

GenieVariableType GenieInteger::read(IntermediateFile* gif)
{
    g3io_int_t temp;
    if (gif->readInteger(temp))
    {
	*this = temp;
        return GenieIntegerType;
    }
    else
    {
	return GenieTypeError;
    }
}

GenieVariableType GenieInteger::writeAscii(fstream& fs, int indent)
{
    fs << setindent(indent) << (*this) << "\n";
    return GenieIntegerType;
}

GenieVariableType GenieInteger::readAscii(fstream& fs)
{
    g3io_int_t temp;
    fs >> temp;
    *this = temp;
    return GenieIntegerType;
}

g_integer GenieInteger::fromOOP(OOP oop) 
{ 
    char* temp;
    g_integer ret;
    if (oop == nilOOP)
	return UndefinedInteger;
    if (isInt(oop))
	return toInt(oop);
// We keep fonts and colours etc. as symbols, so handle these
    if (isAKindOf(oopClass(oop), symbolClass))
    {
	temp = (char*)toCString(oop);
	ret = atol(temp);
	free(temp);
	return ret;
    }
    gerr << "GenieInteger: being passed invalid type" << endl;
    return UndefinedInteger;
}
