/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef GENIE_INT
#define GENIE_INT

#include "g_types.h"

class GenieInteger;
class IntermediateFile;

class GenieInteger : public GenieVariableBase
{
public:
    g_integer operator=(g_integer i) { return(setOOP(toOOP(i)), i); }
    inline GenieInteger& operator=(const GenieInteger& gi);
    operator g_integer() const { return fromOOP(m_value); }
    friend inline g_integer operator+(const GenieInteger& gi, g_integer i);
    friend inline g_integer operator+(const GenieInteger& gi1, const GenieInteger& gi2);
    GenieInteger(g_integer i) : GenieVariableBase(toOOP(i)) { }
    GenieInteger(const GenieCalloutList& gcl, const char* name) : GenieVariableBase(gcl, name) {}
    GenieInteger(OOP params, int location) : GenieVariableBase(params, location) {}
    GenieInteger(const GenieInteger& gi) : GenieVariableBase(gi.deepCopy()) { }
    GenieInteger(OOP oop = nilOOP) : GenieVariableBase(deepCopyOOP(oop)) {};
    static OOP toOOP(g_integer gi) { return (IS_UNDEFINED_INTEGER(gi) ? nilOOP : fromInt(gi)); }
    static g_integer fromOOP(OOP oop);
    static OOP deepCopyOOP(OOP oop) { return (oop == nilOOP ? nilOOP : toOOP(fromOOP(oop))); }
    OOP deepCopy() const { return deepCopyOOP(m_value); }
    ostream& print(ostream& os, const char* note=NULL) const;
    GenieVariableType read(IntermediateFile* gif);
    GenieVariableType write(IntermediateFile* gif);
    GenieVariableType skip(IntermediateFile* gif);
    GenieVariableType readAscii(fstream& fs);
    GenieVariableType writeAscii(fstream& fs, int indent = 0);
    static bool isDefined(g_integer gi) { return !IS_UNDEFINED_INTEGER(gi); }
    static g_integer undefine(g_integer& gi) { return UNDEFINE_INTEGER(gi); }
};
    
inline g_integer operator+(const GenieInteger&gi, g_integer i)
{
    return GenieInteger::fromOOP(gi.value()) + i;
}

inline g_integer operator+(const GenieInteger& gi1, const GenieInteger& gi2)
{
    return GenieInteger::fromOOP(gi1.value()) + GenieInteger::fromOOP(gi2.value());
}

inline g_integer operator-(const GenieInteger& gi1, const GenieInteger& gi2)
{
    return GenieInteger::fromOOP(gi1.value()) - GenieInteger::fromOOP(gi2.value());
}

inline GenieInteger& GenieInteger::operator=(const GenieInteger& gi)
{
    if (this != &gi)
    {
    	setOOP(gi.deepCopy());
    }
    return *this;
}

#endif	/* GENIE_INT */
