/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *      Module:         genie_real
 *      Author:         Freddie Akeroyd, ISIS
 *
 */
#include "geniedefs.h"
#include "genievars.h"
#include "intermediate_file.h"

static const char* rcsid = "@(#)$Id: genie_real.cxx,v 1.13 1997/05/06 09:57:02 cmm Exp $";
USE_RCSID(rcsid);

ostream& GenieReal::print(ostream& os, const char* note) const
{
    GenieVariableBase::print(os, note);
    long old_state = os.setf(ios::showpoint);
    os << (g_real)(*this);
    os.setf(old_state);
    return os;
}

GenieVariableType GenieReal::write(IntermediateFile* gif)
{
    return gif->writeReal(*this) ? GenieRealType : GenieTypeError;
}

GenieVariableType GenieReal::skip(IntermediateFile* gif)
{
    return gif->skipReal() ? GenieRealType : GenieTypeError;
}

GenieVariableType GenieReal::read(IntermediateFile* gif)
{
    g3io_real_t temp;
    if (gif->readReal(temp))
    {
        *this = temp;
        return GenieRealType;
    }
    else
    {
        return GenieTypeError;
    }
}

GenieVariableType GenieReal::writeAscii(fstream& fs, int indent)
{
    fs << setindent(indent) << (*this) << "\n";
    return GenieRealType;
}

GenieVariableType GenieReal::readAscii(fstream& fs)
{
    g3io_real_t temp;
    fs >> temp;
    *this = temp;
    return GenieRealType;
}

