/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef GENIE_REAL
#define GENIE_REAL

#include "g_types.h"

class GenieReal;

class GenieReal : public GenieVariableBase
{
public:
    inline GenieReal& operator=(const GenieReal& gr);
    g_real operator=(g_real d) { return(setOOP(toOOP(d)), d); }
    operator g_real() const { return fromOOP(m_value); }
    GenieReal(g_real d) : GenieVariableBase(toOOP(d)) {}
    friend inline g_real operator+(const GenieReal& gr, g_real d);
    friend inline g_real operator-(const GenieReal& gr, g_real d);
    friend inline g_real operator+(const GenieReal& gr1, const GenieReal& gr2);
    GenieReal(OOP parameters, int location) : GenieVariableBase(parameters, location) {}
    GenieReal(const GenieCalloutList& gcl, const char* name) : GenieVariableBase(gcl, name) {}
    GenieReal(const GenieReal& gr) : GenieVariableBase(gr.deepCopy()) {}
    GenieReal(OOP oop = nilOOP) : GenieVariableBase(deepCopyOOP(oop)) {};
    operator float() { return m_value == nilOOP ? FLT_MAX : F(fromOOP(m_value)); }
    static g_real fromOOP(OOP oop) { return oop == nilOOP ? UndefinedReal : floatOOPValue(oop); }
    static OOP toOOP(g_real gr) { return IS_UNDEFINED_REAL(gr) ? nilOOP : saveOOP(floatNew(gr)); }
    static OOP deepCopyOOP(OOP oop) { return oop == nilOOP ? nilOOP : toOOP(fromOOP(oop)); }
    OOP deepCopy() const { return deepCopyOOP(m_value); }
    ostream& print(ostream& os, const char* note=NULL) const;
    GenieVariableType read(IntermediateFile* gif);
    GenieVariableType write(IntermediateFile* gif);
    GenieVariableType skip(IntermediateFile* gif);
    GenieVariableType readAscii(fstream& fs);
    GenieVariableType writeAscii(fstream& fs, int indent = 0);
    static bool isDefined(g_real gr) { return !IS_UNDEFINED_REAL(gr); }
    static g_real undefine(g_real& gr) { return UNDEFINE_REAL(gr); }
};

inline g_real operator+(const GenieReal& gr, g_real d)
{
    return GenieReal::fromOOP(gr.value()) + d;
}

inline g_real operator-(const GenieReal& gr, g_real d)
{
    return GenieReal::fromOOP(gr.value()) - d;
}

inline g_real operator+(const GenieReal& gr1, const GenieReal& gr2)
{
    return GenieReal::fromOOP(gr1.value()) + GenieReal::fromOOP(gr2.value());
}

inline GenieReal& GenieReal::operator=(const GenieReal& gr)
{
    if (this != &gr)
    {
	setOOP(gr.deepCopy());
    }
    return *this;
}

#endif	/* GENIE_REAL */
