/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef GENIE_REAL_ARRAY
#define GENIE_REAL_ARRAY

#include "genie_flat_array.h"

class GenieIntegerArray;

// Typedef for doing a collect with a FORTRAN function loaded from a module
typedef g_real (*real_array_collect_ref_t)(g_real&);
typedef g_real (*real_array_collect3_ref_t)(g_real&, int[], int&);

class GenieRealArray : public GenieFlatArray<GenieReal, g_real, GenieRealArrayType>
{
protected:
    float* m_float;
    double* m_double;
    typedef GenieFlatArray<GenieReal, g_real, GenieRealArrayType> BaseClass;

public:
    GenieRealArray(OOP params, int location) : BaseClass(params, location), m_float(0), m_double(0) {}
    GenieRealArray(const float* fp, int dim1, int dim2 = 0);
    GenieRealArray(const float* fp, const int* dims_array, int ndims);
    GenieRealArray(const double* dp, int dim1, int dim2 = 0);
    GenieRealArray(const double* dp, const int* dims_array, int ndims);
    g_real& operator()(int i) { return BaseClass::operator()(i); }
    g_real& operator()(int i, int j) { return BaseClass::operator()(i,j); }
    void operator()(const float* fp, int dim1, int dim2);
    void operator()(const float* fp, const int* dims_array, int ndims);
    void operator()(const float* fp, int dim1) { operator()(fp, dim1, 0); }
    void operator()(const double* dp, int dim1, int dim2);
    void operator()(const double* dp, const int* dims_array, int ndims);
    void operator()(const double* dp, int dim1) { operator()(dp, dim1, 0); }
    bool operator==(const GenieRealArray& gra);
//    g_real operator()(int i) const { return GenieArray<GenieReal, g_real, GenieRealArrayType>::operator()(i); }
//    GenieRealArray(const double* dp, int length);
//    void setup(float* fp, int length);
//    void setup(double* fp, int length);
//    GenieRealArray(const GenieIntegerArray& gia);
    GenieRealArray(OOP oop = nilOOP) : BaseClass(oop), m_float(0), m_double(0) {}
    GenieRealArray(int dim1, int dim2 = 0) : BaseClass(dim1, dim2), m_float(0), m_double(0) {}
    GenieRealArray(const int* dims_array, int ndims) : BaseClass(dims_array, ndims), m_float(0), m_double(0) {}
    GenieRealArray(const GenieCalloutList& gcl, const char* name) : BaseClass(gcl, name), m_float(0), m_double(0) {}
    GenieRealArray(const GenieRealArray& gra) : BaseClass(gra), m_float(0), m_double(0) {}
    GenieRealArray(const GenieRealArray& gra, g_real val) : BaseClass(gra, val), m_float(0), m_double(0) {}
//    ostream& print(ostream& os, const char* note = 0) const { return os; }
    operator float*() { return setupFloat(); }
    operator double*() { return setupDouble(); }
    float* setupFloat();
    double* setupDouble();
//    void add(int dim1, double val) { (*this)(dim1, (*this)(dim1) + val); }
    virtual ~GenieRealArray();
    void removeInternalArrays();
//    GenieRealArray operator+(const GenieRealArray& gra);
//    GenieRealArray operator-(const GenieRealArray& gra);
//    GenieRealArray operator*(const GenieRealArray& gra);
//    GenieRealArray operator/(const GenieRealArray& gra);
//    g_real min();
//    g_real max();
    GenieVariableType readAscii(fstream& fs);
    GenieVariableType writeAscii(fstream& fs, int indent = 0);
    GenieVariableType read(IntermediateFile* gif);
    GenieVariableType write(IntermediateFile* gif);
    GenieVariableType skip(IntermediateFile* gif);
    OOP become(OOP oop);
    OOP become(const BaseClass& ga) { return become(ga.value()); }
    OOP become(const GenieVariable& gv);
    GenieRealArray binsToPoints();
    GenieRealArray pointsToBins(g_real* halfwidth);
    GenieRealArray slice(int dimension, g_real value);
};

#endif /* GENIE_REAL_ARRAY */
