/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *      Module:         genie_string
 *      Author:         Freddie Akeroyd, ISIS
 *
 */
#include "geniedefs.h"
#include <stdlib.h>
#include "genievars.h"
#include "intermediate_file.h"

static const char* rcsid = "@(#)$Id: genie_string.cxx,v 1.28 1998/05/05 10:32:32 faa Exp $";
USE_RCSID(rcsid);

void GenieString::setupString(OOP oop)
{
    setOOP(oop);
    char* tmp_str = fromOOP(oop);
    String::operator=(tmp_str);
    if (tmp_str != 0)
	free(tmp_str);
}

void GenieString::setupString(const char* s)
{
    if (s != NULL)
    {
        String::operator=(s);
        setOOP(toOOP(s));
    }
    else
    {
	String::operator=("");
	setOOP(nilOOP);
    }
}

void GenieString::init(OOP oop, OOP params, int _location)
{
    GenieVariableBase::init(oop, params, _location);
    setupString(oop);
}

GenieString& GenieString::operator=(const String& s)
{
    setupString(s);
    return *this;
}

GenieString& GenieString::operator=(const char* s)
{
    setupString(s);
    return *this;
}

GenieVariableBase& GenieString::operator=(const GenieVariableBase& gvb)
{
    ginf << "Calling dubious code" << endl;
    GenieVariableBase::operator=(gvb);
    String::operator=(fromOOP(m_value));
    return *this;
}

GenieString& GenieString::operator=(const GenieString& gs)
{
    if (this != &gs)
    {
	setupString((const char*)gs);
    }
    return *this;
}

// We copy the string into an OOP
// The GenieVariableBAse destructor is called before
// That of the String class, and this will copy the value
// back to smalltalk
GenieString::~GenieString()
{
    if (rep != 0)
    {
	m_value = toOOP((const char*)(*this));
    }
    else
    {
	m_value = nilOOP;
    }
}

OOP GenieString::setOOP(OOP oop)
{
    return GenieVariableBase::setOOP(oop);
}

ostream& GenieString::print(ostream& os, const char* note) const
{
    GenieVariableBase::print(os, note);
    return os << (const String&)(*this);
}

GenieVariableType GenieString::write(IntermediateFile* gif)
{
    return gif->writeString(*this) ? GenieStringType : GenieTypeError;
}

GenieVariableType GenieString::skip(IntermediateFile* gif)
{
    return gif->skipString() ? GenieStringType : GenieTypeError;
}

GenieVariableType GenieString::read(IntermediateFile* gif)
{
    char* temp;
    if (gif->readString(temp))
    {
	*this = temp;
	gif->freeString(temp);
 	return GenieStringType;
    }
    else
    {
	return GenieTypeError;
    }
}

// Enclose string in " " and map internally ( " ->  \" ) and  (  \ -> \\ )
// Undefined strings (null pointers) are output as ""
GenieVariableType GenieString::writeAscii(fstream& fs, const char* s, int indent)
{
    fs << setindent(indent);
    fs.put('\"');
    if (s != 0)
    {
        while(*s != '\0')
        {
	    if ( (*s == '\"') || (*s == '\\') )
	        fs.put('\\');
	    fs.put(*s);
	    ++s;
        }
    }
    fs.put('\"');
    fs.put('\n');
    return GenieStringType;
}

GenieVariableType GenieString::writeAscii(fstream& fs, int indent)
{
    return writeAscii(fs, *this, indent);
}

GenieVariableType GenieString::readAscii(fstream& fs, String& s)
{
    s = "";
    int i;
    while(fs.get() != '\"')
	;
    while(true)
    {
	i = fs.get();
	if (i == '\"')
	    break;
	if (i == '\\')
	    i = fs.get();
	s += (char)i;
    }
//    cerr << "Read: \"" << s << "\"" << endl;
    return GenieStringType;
}

GenieVariableType GenieString::readAscii(fstream& fs)
{
    GenieVariableType type = readAscii(fs, *this);
    setOOP(toOOP(*this));
    return type;
}

OOP GenieString::toOOP(c_g_string gs, int len)
{
    if (IS_UNDEFINED_STRING(gs))
	return nilOOP;

    if (nilOOP == symbolClass)
    {
	gerr << "Returning symbol" << endl;
	return saveOOP(symbolToOOP((char*)gs));	// what about len?
    }
    else
    {
	if (len == -1)
	{
	    return saveOOP(stringNew((char*)gs));
	}
	else
	{
	    return saveOOP(countedStringNew((char*)gs, len));
	}
//	return saveOOP(stringToOOP(gs));	/* does a registerOOP() */
    }
}

OOP GenieString::deepCopyOOP(OOP oop) 
{ 
    if (oop == nilOOP)
	return nilOOP;
    const char* tmp_str = fromOOP(oop);
    OOP res = toOOP(tmp_str);
    free((void *)tmp_str);
    return res;
}
