/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef GENIE_STRING
#define GENIE_STRING

#define duplicateString(__string) strcpy((char*)malloc((strlen(__string)+1)*sizeof(char)), __string)

#ifndef _WIN32
	extern "C" void* malloc(size_t size);
#endif

class GenieString;

class GenieString : public String, public GenieVariableBase	// We need them this way round
{
public:
    virtual GenieVariableBase& operator=(const GenieVariableBase& gvb);
    GenieString& operator=(const GenieString& gs);
    GenieString& operator=(const String& s);
    GenieString& operator=(const char* gs);
    GenieString(const char* gs) : String(gs), GenieVariableBase(toOOP(gs)) {};
    GenieString(const char* gs, int len) : String(gs, len), 
		GenieVariableBase(toOOP(gs, len)) {};
    GenieString(const String& s) : String(s), GenieVariableBase(toOOP(s)) {};
    GenieString(const GenieCalloutList& gcl, const char* name) : String(), GenieVariableBase() { associate(gcl, name); }
    GenieString(OOP params, int location) : String(), GenieVariableBase(params, location) { init(m_value); }
    GenieString(const GenieString& gs) : String((const char*)gs), GenieVariableBase(gs.deepCopy()) {};
    GenieString(OOP oop) : String(fromOOP(oop)), GenieVariableBase(deepCopyOOP(oop)) {};
    GenieString() : String(), GenieVariableBase(nilOOP) {};
    OOP deepCopy() const { return deepCopyOOP(m_value); }
    static OOP toOOP(c_g_string gs, int len = -1);
    static g_string fromOOP(OOP oop) { return (oop != nilOOP ? OOPToString(oop) : UndefinedString); }	// need to free() string
//    static OOP deepCopyOOP(OOP oop) { c_g_string tmp(fromOOP(oop)); return toOOP(tmp); } // tmp is essential for g++ 2.7.1
    static OOP deepCopyOOP(OOP oop);
    virtual ostream& print(ostream& os, const char* note=NULL) const;
    virtual void init(OOP oop = nilOOP, OOP params = nilOOP, int location = 0);
    virtual OOP setOOP(OOP oop);
    void setupString(OOP oop);
    void setupString(const char* s);
    GenieVariableType read(IntermediateFile* gif);
    GenieVariableType write(IntermediateFile* gif);
    GenieVariableType skip(IntermediateFile* gif);
    static GenieVariableType readAscii(fstream& fs, String& s);
    static GenieVariableType writeAscii(fstream& fs, const char* s, int indent = 0);
    GenieVariableType readAscii(fstream& fs);
    GenieVariableType writeAscii(fstream& fs, int indent = 0);
//    const GenieString* operator&() const { return this; }
    virtual ~GenieString();
};
    
inline String operator+(const GenieString& gs, const char* c)
{
    return c == 0 ? (String)gs : (String)gs + String(c);
}

#endif	/* GENIE_STRING */
