/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *      Module:         genie_string_array
 *      Author:         Freddie Akeroyd, ISIS
 *
 */
#include "geniedefs.h"
#include "genievars.h"
#include "intermediate_file.h"

static const char* rcsid = "@(#)$Id: genie_string_array.cxx,v 1.11 1997/11/12 15:59:34 faa Exp $";
USE_RCSID(rcsid);

GenieStringArray::~GenieStringArray()
{

}

GenieVariableType GenieStringArray::skip(IntermediateFile* gif)
{
    return gif->skipStringArray() != 0 ? GenieStringArrayType : GenieTypeError;
}

GenieVariableType GenieStringArray::write(IntermediateFile* gif)
{
    return gif->writeStringArray(*this) != 0 ? GenieStringArrayType : GenieTypeError;
}

GenieVariableType GenieStringArray::read(IntermediateFile* gif)
{
    return gif->readStringArray(*this) != 0 ? GenieStringArrayType : GenieTypeError;
}

GenieVariableType GenieStringArray::writeAscii(fstream& fs, int indent)
{
    writeAsciiDims(fs, indent);
    for(int i=1; i<=length(); i++)
    {
	GenieString::writeAscii(fs, (*this)[i-1], indent);
    }
    return GenieStringArrayType;
}

GenieVariableType GenieStringArray::readAscii(fstream& fs)
{
    OOP a_dimsOOP = readAsciiDims(fs);
    alloc(a_dimsOOP);
    for(int i=0; i<length(); i++)
    {
        String temp;
	GenieString::readAscii(fs, temp);
        (*this)(i, (const char*)temp);
    }
    return GenieStringArrayType;
}

GenieStringArray::GenieStringArray(const char* sa[], long length) :
    GenieArray<GenieString, g_string, GenieStringArrayType>(length)
{
    for(int i=0; i<length; i++)
    {
	(*this)(i, sa[i]);
    }
}

// Concatenate all array elements together into one long string,
// with each element separated from the next by "delim"
char* GenieStringArray::asString(char delim)
{
    int i, len = 0;
    for(i=0; i<length(); i++)
    {
	len += (strlen((*this)[i]) + 1); // +1 for delim on each record
    }
    len++;	// for final '\0';
    char* bytes = new char[len];
    char* p = bytes;
    for(i=0; i<length(); i++)
    {
	len = strlen((*this)[i]);
        strncpy(p, (*this)[i], len);
	p[len] = delim;
	p[len+1] = '\0';
	p += (len + 1);
    }
    return bytes;
}
