/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *      Module:         genie_work
 *      Author:         Freddie Akeroyd, ISIS
 *
 */
#include "geniedefs.h"
#include <stdlib.h>
#include "parse_funcs.h"
#include "genievars.h"
#include "intermediate_file.h"


static const char* rcsid = "@(#)$Id: genie_work.cxx,v 1.35 1998/10/01 14:05:39 faa Exp $";
USE_RCSID(rcsid);

const int GenieWorkspace::m_default_fields = 4;

enum id_inst_vars  { TALLY=1, VALUES=2 };

extern "C" int identityDictionaryFindKeyOrNil(OOP, OOP);

// return a symbol OOP
OOP GenieWorkspace::getKey(const char* field_name) const
{
    unsigned i;
    char buffer[128];
    for(i=0; i<strlen(field_name); i++)
    {
	buffer[i] = tolower(field_name[i]);
    }
    buffer[i] = '\0';
    Garble(buffer);
    return internString(buffer);
}

OOP GenieWorkspace::getFieldOOP(const char* field_name) const
{
    return atOrNil(getKey(field_name));
}

GenieVariable GenieWorkspace::operator()(const char* field_name) const
{
#if 0
    int index = identityDictionaryFindKeyOrNil(m_value, getKey(field_name));
//    OOP m_field_values = instVarAt(m_value, VALUES);
    return GenieVariable(getOOP(index+1));
#else
    return GenieVariable(getFieldOOP(field_name));
#endif
}    

void GenieWorkspace::alloc(int nfields)
{
    static OOP gxworkspace = registerOOP(typeNameToOOP("GXWorkspace"));
    static OOP gxnew = registerOOP(internString("new:"));
    m_value = saveOOP(msgSend(gxworkspace, gxnew, fromInt(nfields), 0));
//        m_value = instantiateOOPWith(findClass(internString(Strdup("GXWorkspace"))), nfields);
//        instVarAtPut(m_value, TALLY, intToOOP(0));
//        instVarAtPut(m_value, VALUES, arrayNew(nfields));
}

void GenieWorkspace::initWorkspace(OOP oop, OOP subclass_oop, OOP params, int _location)
{
    static OOP gxnew = registerOOP(internString("new"));
    if (subclass_oop == undefinedObjectClass)
    {
	initWorkspace(oop, m_default_fields, params, _location);
    }
    else
    {
        GenieVariableBase::init(oop, params, _location);
	if (oop == nilOOP)
 	{
            m_value = saveOOP(msgSend(subclass_oop, gxnew, 0));
	}
    }
}

void GenieWorkspace::initWorkspace(OOP oop, int nfields, OOP params, int _location)
{
    GenieVariableBase::init(oop, params, _location);
    if (oop == nilOOP)
    {
	alloc(nfields);
    }
}

GenieWorkspace&  GenieWorkspace::operator=(const GenieWorkspace& gw)
{
    if (this != &gw)
    {
	setOOP(gw.shallowCopy());
    }
    return *this;
}

ostream& GenieWorkspace::print(ostream& os, const char* note) const
{

    GenieVariableBase::print(os, note);
    const char* field_name;
    os << "Genie Workspace with length = " << length() << ", tally = " << tally() << endl;
    for(int i=1; i<=length(); i++)
    {
	GenieVariable gv(getOOP(i));
	if (gv.defined())
	{
	    field_name = field(i);
	    if (field_name)
		os << "(" << field_name << ")\t = " << gv << endl;
	}
    }
  return os;
}

int GenieWorkspace::tally() const        // number of used fields
{
    return toInt(instVarAt(m_value, TALLY));
}

#if 0
virtual void read(const char *file, int block)
{
    int ntc, ntcmax, ndet, ndetmax, nuse, err, quick;
    int ispec;
    fort_real *x, user_tables[30];
    ispec = block;
    ntcmax = ntc + 1;
    ndetmax = ndet + 1;
    delt_work = new fort_real[ndetmax];    
    l2_work = new fort_real[ndetmax];    
    tthe_work = new fort_real[ndetmax];    
    spec_work = new fort_int[ndetmax];    
    idat = new fort_int[ntcmax];
    x = new fort_real[ntc+1];
    const char* runid = translatePath(file, 'f');	// make a fortran style name
    OPEN_DATA_FILE_S(runid, ntc, ndet, nuse, err);
    if (err != 0) 
    {
	cerr << "Open error" <, endl;
	return;
    }
    READ_DATA_S(runid, err, ispec, delt_work, spec_work, tthe_work, l2_work,
              ndetmax, x, idat, ntcmax, l1, l2, tt, d,
              tm, un, in, rn, ut, nuse, quick);
    this("x") = x;
    this("xlabel", unit_name(2, 'x'));
    this("ylabel", unit_name(2, 'y'));
    return;
}

#endif

const char* GenieWorkspace::field(int i) const
{
// key is a symbol OOP
    static char buff[256];
    OOP key = indexOOP(m_value, i);
    if (isNil(key))
    {
	return 0;
    }
    int len = numIndexableFields(key);
    strncpy(buff, symbolAsString(key), len);
    buff[len] = '\0';
    UnGarble(buff);
    return buff;
}

#if 0
OOP GenieWorkspace::deepCopyOOP(OOP oop)
{
    OOP key;
    GenieWorkspace old_wk(oop);
    GenieWorkspace new_wk(old_wk.length());
    for(int i=1; i<=old_wk.tally(); i++)
    {
	key = indexOOP(old_wk.value(), i);
 	identityDictionaryAtPut(new_wk.value(), key, GenieVariable::deepCopyOOP(old_wk.getOOP(i)));
    }
    return new_wk.value();
}
#endif

ostream& operator<<(ostream& os, const GenieWorkspace& gw)
{
    return gw.print(os);
}

GenieVariableType GenieWorkspace::write(IntermediateFile* gif)
{
    unsigned res(1);
    GenieVariable gv;
    char* field_name;
    res &= gif->writeInteger(tally());		// number of populated fields
    for(int i=1; i<=length(); i++)
    {
	field_name = OOPToString(indexOOP(m_value, i)); 
	if (field_name != 0)
	{
	    gv.become(getOOP(i));
	    res &= gif->writeString(field_name);
	    res &= (gv.write(gif) != 0 ? 1 : 0);
	    free(field_name);
	}
    }
    return res != 0 ? GenieWorkspaceType : GenieTypeError; 
}

GenieVariableType GenieWorkspace::read(IntermediateFile* gif)
{
    unsigned res(1);
    g3io_int_t len;
    GenieVariable gv;
    char* field_name;
    res &= gif->readInteger(len);
    for(int i=1; i<=len; i++)
    {
	res &= gif->readString(field_name);
	res &= (gv.read(gif) != 0 ? 1 : 0);
	atPut(internString(field_name), gv.value());
	res &= gif->freeString(field_name);
    }
    return res != 0 ? GenieWorkspaceType : GenieTypeError; 
}

GenieVariableType GenieWorkspace::skip(IntermediateFile* gif)
{
    unsigned res(1);
    g3io_int_t len;
    res &= gif->readInteger(len);
    for(int i=1; i<=len; i++)
    {
	res &= gif->skipString();					// Field label
	res &= (GenieVariable::skip(gif) != 0 ? 1 : 0);			// Field Value
    }
    return res != 0 ? GenieWorkspaceType : GenieTypeError; 
}

GenieVariableType GenieWorkspace::writeAscii(fstream& fs, int indent)
{
    GenieVariable gv;
    char* field_name;
    fs << setindent(indent) << tally() << "\n";				// Number of used fields
    for(int i=1; i<=length(); i++)
    {
	field_name = OOPToString(indexOOP(m_value, i)); 
	if (field_name != 0)
	{
	    gv.become(getOOP(i));
	    UnGarble(field_name);
	    GenieString::writeAscii(fs, field_name, indent+2);
	    gv.writeAscii(fs, indent+4);
	    free(field_name);
	}
    }
    return GenieWorkspaceType;
}

GenieVariableType GenieWorkspace::readAscii(fstream& fs)
{
    GenieVariable gv;
    int len;
    String field_name;
    char* buffer;
    fs >> len;			// The workspace tally
    for(int i=1; i<=len; i++)
    {
	GenieString::readAscii(fs, field_name);
	buffer = new char[strlen(field_name) + 3];	// +3 for "gX" (from Garble()) plus the usual '\0'
	strcpy(buffer, field_name);
	Garble(buffer);					// Name mangle field name for Smalltalk
	gv.readAscii(fs);				// Read field value
	atPut(internString(buffer), gv.value());
	delete[] buffer;
    }
    return GenieWorkspaceType;
}

// subclass must start GX and the third letter should be a capital, then the rest lowercase
GenieWorkspace::GenieWorkspace(const char* subclass)
{ 
    char buffer[300];
// We use subclass+2 to skip the GX bit which seems to be added by subclass: for us
    sprintf(buffer, "Smalltalk at: #%s ifAbsent: [ GXStructure subclass: '%s' fields: '' comment: 'Dynamically created'. 'Creating class %s as subclass of Structure' prinl ]!", subclass, subclass+2, subclass+2);
    evalCode(buffer);
    initWorkspace(nilOOP, typeNameToOOP(subclass)); 
}
