/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef GENIE_WORK
#define GENIE_WORK

class GenieWorkspace;

class GenieWorkspace : public GenieVariableBase
{
protected:
    static const int m_default_fields;		// default number of fields to allocate

public:
    enum id_inst_vars  { TALLY=1, VALUES=2 };
    OOP getKey(const char* field_name) const;
    OOP getFieldOOP(const char* field_name) const;
    inline void operator()(const char* field_name, int i);
    inline void operator()(const char* field_name, const GenieVariableBase& gv);
    inline void operator()(const char* field_name, GenieReal& gr);
    inline void operator()(const char* field_name, g_real gr);
    inline void operator()(const char* field_name, GenieInteger& gi);
    inline void operator()(const char* field_name, g_integer gi);
    inline void operator()(const char* field_name, const GenieString& gs);
    inline void operator()(const char* field_name, const String& s);
    inline void operator()(const char* field_name, c_g_string gs);
    GenieVariable operator()(const char* field_name) const;
    GenieWorkspace(const char* subclass);
    GenieWorkspace(int nfields) { initWorkspace(nilOOP, nfields); }
    GenieWorkspace(OOP params, int location) : GenieVariableBase(params, location) { initWorkspace(m_value); }
    GenieWorkspace(const GenieCalloutList& gcl, const char* name) { initWorkspace(nilOOP, 1); associate(gcl, name); }
    void alloc(int nfields = 1);
    GenieWorkspace(OOP value_oop = nilOOP, OOP subclass_oop = undefinedObjectClass) { initWorkspace(value_oop, subclass_oop); };
    GenieWorkspace(const GenieWorkspace& gw) : GenieVariableBase(gw.deepCopy()) {}
    GenieWorkspace& operator=(const GenieWorkspace& gw);
    inline int length() const;
    int tally() const;
    ostream& print(ostream& os, const char* note=NULL) const;
//    void read(const char *file, int block = -1);
//    void read(const char *file, const char* label);
    OOP getOOP(int i) { return arrayAt(instVarAt(m_value, VALUES), i); }
    const OOP getOOP(int i) const { return arrayAt(instVarAt(m_value, VALUES), i); }
    const OOP operator()(int i) const { return getOOP(i); }
    const char* field(int i) const;
    void initWorkspace(OOP value = nilOOP, int nfields = 1, OOP params = nilOOP, int location = 0);
    void initWorkspace(OOP value, OOP subclass_oop, OOP params = nilOOP, int location = 0);
    GenieVariableType read(IntermediateFile* gif);
    GenieVariableType write(IntermediateFile* gif);
    GenieVariableType skip(IntermediateFile* gif);
    GenieVariableType readAscii(fstream& fs);
    GenieVariableType writeAscii(fstream& fs, int indent = 0);
//    void init(OOP value = nilOOP, OOP params = nilOOP, int location = 0) { initWorkspace(value, 1, params, location); }
//    OOP deepCopy() const { return deepCopyOOP(m_value); }
    static OOP toOOP(const g_work& gw) { return gw.value(); }
    static g_work fromOOP(OOP oop) { return g_work(oop); }
//    static OOP deepCopyOOP(OOP oop);
//    static OOP shallowCopyOOP(OOP oop) { return dictionaryCopy(oop); }
    bool operator==(const GenieWorkspace& gw) const { gerr << "== not implemented for workspace" << endl; return 1; }
    bool operator!=(const GenieWorkspace& gw) const { gerr << "!= not implemented for workspace" << endl; return 0; }
};

inline g_work operator+(const GenieWorkspace& gw, const g_work& d)
{
    return g_work(nilOOP);
}

inline void GenieWorkspace::operator()(const char* field_name, const GenieVariableBase& gv)
{
    atPut(getKey(field_name), gv.value());
//    identityDictionaryAtPut(m_value, getKey(field_name), gv.value());
}

inline void GenieWorkspace::operator()(const char* field_name, GenieReal& gr)
{
    atPut(getKey(field_name), gr.shallowCopy());
//    identityDictionaryAtPut(m_value, getKey(field_name), gr.shallowCopy());
}

inline void GenieWorkspace::operator()(const char* field_name, g_real gr)
{
    atPut(getKey(field_name), GenieReal::toOOP(gr));
//    identityDictionaryAtPut(m_value, getKey(field_name), GenieReal::toOOP(gr));
}

inline void GenieWorkspace::operator()(const char* field_name, g_integer gi)
{
    atPut(getKey(field_name), GenieInteger::toOOP(gi));
//    identityDictionaryAtPut(m_value, getKey(field_name), GenieInteger::toOOP(gi));
}

inline void GenieWorkspace::operator()(const char* field_name, int i)
{
    atPut(getKey(field_name), GenieInteger::toOOP(i));
//    identityDictionaryAtPut(m_value, getKey(field_name), intToOOP(i));
}

inline void GenieWorkspace::operator()(const char* field_name, c_g_string gs)
{
    atPut(getKey(field_name), GenieString::toOOP(gs));
//    identityDictionaryAtPut(m_value, getKey(field_name), GenieString::toOOP(gs));
}

inline void GenieWorkspace::operator()(const char* field_name, GenieInteger& gi)
{
    atPut(getKey(field_name), gi.shallowCopy());
//    identityDictionaryAtPut(m_value, getKey(field_name), gi.shallowCopy());
}

inline void GenieWorkspace::operator()(const char* field_name, const String& s)
{
    (*this)(field_name, (c_g_string)s); 
}

inline void GenieWorkspace::operator()(const char* field_name, const GenieString& gs)
{
    atPut(getKey(field_name), gs.deepCopy());
//    identityDictionaryAtPut(m_value, getKey(field_name), gs.deepCopy());
}
    
inline int GenieWorkspace::length() const	// total length (including undefined fields)
{
    return numIndexableFields(m_value);
}

ostream& operator<<(ostream& os, const GenieWorkspace& gw);

#endif /* GENIE_WORK */
