/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *      Module:         genie_workspace_array
 *      Author:         Freddie Akeroyd, ISIS
 *
 */
#include "geniedefs.h"
#include "genievars.h"
#include "intermediate_file.h"

static const char* rcsid = "@(#)$Id: genie_workspace_array.cxx,v 1.9 1997/05/06 09:57:04 cmm Exp $";
USE_RCSID(rcsid);

void GenieWorkspaceArray::alloc(const GenieArrayBase& gab)
{ 
    GenieArray<GenieWorkspace, g_work, GenieWorkspaceArrayType>::alloc(gab);
}

// We write out a GenieVariable rather than a GenieWorkspace so we can preserve
// subclass types ... should really build subclass types into the workspace
GenieVariableType GenieWorkspaceArray::read(IntermediateFile* gif)
{
#if 1
    unsigned res = gif->readArrayDims(*this);
    for(int i=0; i<length(); i++)    
    {
        GenieVariable gv;
	res &= ( gv.read(gif) != 0 ? 1 : 0 );
	(*this)(i, (GenieWorkspace)gv);
    }
    return res != 0 ? GenieWorkspaceArrayType :  GenieTypeError;
#else
    return GenieTypeError;
#endif
}

GenieVariableType GenieWorkspaceArray::skip(IntermediateFile* gif)
{
    GenieWorkspaceArray gwa;
    return gwa.read(gif);
}

GenieVariableType GenieWorkspaceArray::write(IntermediateFile* gif)
{
#if 1
    unsigned res = gif->writeArrayDims(*this);
    GenieVariable gv;
    for(int i=0; i<length(); i++)    
    {
	gv.become((*this)[i]);
	res &= ( gv.write(gif) != 0 ? 1 : 0 );
    }
    return res != 0 ? GenieWorkspaceArrayType :  GenieTypeError;
#else
    return GenieTypeError;
#endif
}

GenieVariableType GenieWorkspaceArray::writeAscii(fstream& fs, int indent)
{
    writeAsciiDims(fs, indent);
    GenieVariable gv;
    for(int i=0; i<length(); i++)    
    {
	gv.become((*this)[i]);
	gv.writeAscii(fs, indent+2);
    }
    return GenieWorkspaceArrayType;
}

GenieVariableType GenieWorkspaceArray::readAscii(fstream& fs)
{
    OOP a_dimsOOP = readAsciiDims(fs);
    alloc(a_dimsOOP);
    for(int i=0; i<length(); i++)
    {
        GenieVariable gv;
	gv.readAscii(fs);
	(*this)(i, (GenieWorkspace)gv);
    }
    return GenieWorkspaceArrayType;
}
