/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *  function to log a message to a server process via UDP
 *  Freddie Akeroyd, 28/6/94
 */
#include "geniedefs.h"
/* Fiddle these to allow non-posix types which are used in socket.h and in.h */
//#undef _POSIX_SOURCE

#if defined(_WIN32)
#	include "vms_password_funcs.h"
#	include "strcasecmp.h"
#endif /* _WIN32 */

static const char* rcsid = "@(#)$Id: genielog.cxx,v 1.33 1998/06/22 08:07:30 cmm Exp $";
USE_RCSID(rcsid);

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_UNISTD_H
#	include <unistd.h>
#else
#	include <io.h>
#endif

#include <sys/types.h>
#ifdef _WIN32
#	include <winsock.h>
#   define INTERNET_SOCKET PF_INET
#else
#	include <sys/socket.h>
#	include <netdb.h>
#	include <netinet/in.h>
#	include <arpa/inet.h>
#	include <pwd.h>
#   define SOCKET int
#   define INVALID_SOCKET -1
#   define INTERNET_SOCKET AF_INET
#   define closesocket close
#endif
#include <string.h>
#ifdef _WIN32
#include <time.h>
#else
#include <sys/time.h>
#endif /* _WIN32 */
#include "genielog.h"

#ifndef SIABUFSIZ
const unsigned long SIABUFSIZ = 2048;		// Room for password structure
#endif
static char buffer1[SIABUFSIZ]; 

// Sort out a mess on Digital UNIX 4.0 
#ifdef HAVE_LIBC_R
#	if _POSIX_C_SOURCE>=199506L	/* P1003.1c */
		extern "C" int getpwnam_r(const char*, struct passwd *, char *, size_t, struct passwd **);
		extern "C" int getpwuid_r(uid_t, struct passwd *, char *, size_t, struct passwd **);
#		define GETPWNAM(name,pwname) getpwnam_r(name, NULL, buffer1, (int)sizeof(buffer1),&pwname)
#		define GETPWUID(uid,pwname) getpwuid_r(getuid(), NULL, buffer1, sizeof(buffer1),&pwname)
#	else
#		define GETPWNAM(name,pwname) getpwnam_r(name, pwname, buffer1, (int)sizeof(buffer1))
#		define GETPWUID(uid,pwname) getpwuid_r(getuid(), pwname, buffer1, sizeof(buffer1))
#	endif
#else
#	define GETPWNAM(name,pwname) pwname = getpwnam(name)
#       define GETPWUID(uid,pwname) pwname = getpwuid(uid)
#endif /* HAVE_LIBC_R */

#if defined(__ultrix) && defined(__DECCXX)
extern "C" u_short htons(u_short s);
#endif


#define	MACHINE		"isolde.nd.rl.ac.uk"	/* where to send message */
#define	MACHINE_IP	"130.246.49.21"		/* where to send message */
/* #define USE_GETHOSTBYNAME 	/* Lookup MACHINE rather than just use MACHINE_IP - can cause a non-networked machine to hang */
#define	PORT	1157			/* udp port on MACHINE */

/* #define	DEBUG				/* if want errors printed to stdout */

// On success, return socket file descriptor and populate sockaddr_in
// On error, Return -1
static SOCKET get_socket_descriptor(struct sockaddr_in* ps_in)
{
    SOCKET s;
    struct hostent* hostp;
#ifdef USE_GETHOSTBYNAME
    if ((hostp = gethostbyname(MACHINE)) == 0)
    {
#    ifdef	DEBUG
	gprintef("Cannot determine address of \"%s\"\n", MACHINE);
	herror("gethostbyname");
#    endif
	return -1;
    }
#else	/* emulate gethostbyname() */
    in_addr_t a = inet_addr(MACHINE_IP);
    if (a == (in_addr_t)-1)
    {
#    ifdef	DEBUG
	gprintef("Cannot convert address \"%s\"\n", MACHINE_IP);
#    endif
	return -1;
    }
    hostp = new hostent;
    hostp->h_addr_list = new char*[1];
    hostp->h_addr_list[0] = (char*)&a;
    hostp->h_length = sizeof(in_addr_t);
#endif /* USE_GETHOSTBYNAME */
    if ((s = socket(INTERNET_SOCKET, SOCK_DGRAM, 0)) == -1)
    {
#ifdef	DEBUG
	PERROR("genielog - socket");
#endif
	return -1;
    }
    memset(ps_in, 0, sizeof(struct sockaddr_in));
    ps_in->sin_family = AF_INET;
    ps_in->sin_port = htons(PORT);
    memcpy(&(ps_in->sin_addr.s_addr), hostp->h_addr_list[0], hostp->h_length);
    return s;
}
/*
 *  attempt to send message to udp port PORT at internet host MACHINE
 *  return 0 on success, -1 on error
 */
int genielog(const char* message)
{
    static int first_call = 1;
    static SOCKET s = INVALID_SOCKET;	// open socket file descriptor
    static struct sockaddr_in s_in;
    static time_t session_id;
    int blen;
    int retval = 0;
    char header[256];
    char *name, *buffer;
#ifdef HAVE_LIBC_R
    struct passwd* pwname = new passwd;
#else
    struct passwd* pwname = 0;
#endif /* HAVE_LIBC_R */

    if (first_call == 1)
    {
	first_call = 0;
#ifdef _WIN32
	WSADATA wsaData;
	WORD wVersionRequested = MAKEWORD( 1, 1 );
	WSAStartup( wVersionRequested, &wsaData );
#endif /* _WIN32 */
	session_id = time(NULL);
	s = get_socket_descriptor(&s_in);
    }
    if (s == INVALID_SOCKET)
        return -1;

	// Do we allow logging?
    name = getenv("GENIE_LOGGING");
    if (name == 0 || strcasecmp(name, "yes") != 0)
	return 0;

    if (message == 0)
	return -1;

#ifndef _WIN32
    name = getlogin();
    if (name != 0)	// interactive
    {
	GETPWNAM(name,pwname);
    }
    else		// non-interactive
    {
	GETPWUID(getuid(),pwname);
	if (pwname != 0)
	{
	    name = pwname->pw_name;
	}
    }
#else
    unsigned long ul = SIABUFSIZ;
    if (GetUserName(buffer1, &ul) != 0)
    {
	name = buffer1;
    }
    else
    {
	name = 0;
    }
#endif

#if defined(__VMS) || defined(_WIN32)
    sprintf(header, "From user %s\nSession-id: %ld\n", (name == 0 ? "unknown" : name), (unsigned long)session_id);
#else
    sprintf(header, "From user %s (%s)\nSession-id: %ld\n", (name == 0 ? "unknown" : name), 
				(pwname == 0 ? "" : pwname->pw_gecos), (unsigned long)session_id);
#endif /* __VMS || _WIN32 */

#ifdef HAVE_LIBC_R
    delete pwname;
    pwname = 0;
#endif /* HAVE_LIBC_R */
    buffer = (char*)malloc(1+strlen(header)+strlen(message));
    if (buffer == 0)
    {
#ifdef	DEBUG
	fprintf(stderr, "genielog: malloc failure\n");
#endif
	return -1;
    }
    buffer[0] = '\0';
    strcat(buffer, header);
    strcat(buffer, message);
    blen = strlen(buffer);

    if (sendto(s, buffer, blen, 0, (struct sockaddr*)&s_in, sizeof(struct sockaddr_in)) != blen)
    {
	retval = -1;
#ifdef	DEBUG
	PERROR("genielog - sendto");
#endif
    }
    free(buffer);
    return retval;
}

// 0 on success, 1 on failure
// On entry, *reply_length is space available in "reply"; on exit it is amount read
#define SELECT_TIMEOUT	5
int tcpip_get(const char* server, short port, const char* message, int mess_length, 
		char* reply, int* reply_length)
{
    int n, nfds, replen, max_replen;
    SOCKET sd;
    struct hostent *hostp;
    struct sockaddr_in address;
    struct timeval call_timeout = { SELECT_TIMEOUT, 0};
#if 0
    int readfds=0, writefds=0, exceptfds=0;
#else
    fd_set readfds, writefds, exceptfds;
    FD_ZERO(&readfds);
    FD_ZERO(&writefds);
    FD_ZERO(&exceptfds);
#endif
    max_replen = *reply_length;
    *reply_length = 0;
    if ((sd = socket(INTERNET_SOCKET, SOCK_STREAM, 0)) == INVALID_SOCKET)
    {
	PERROR("tcpip_get - socket");
	return 1;
    }
    memset(&address, 0, sizeof(struct sockaddr_in));
    address.sin_family = AF_INET;
    if ((hostp = gethostbyname(server)) == NULL)
    {
	gprintef("tcpip_get - gethostbyname failed for \"%s\"\n", server);
	return 1;
    }
    memcpy(&(address.sin_addr.s_addr), hostp->h_addr, hostp->h_length);
    address.sin_port = htons(port);
    if (connect(sd, (struct sockaddr*)&address, sizeof(address)) == -1) 
    {
	PERROR("tcpip_get - connect");
	return 1;
    }
    if (send(sd, message, mess_length, 0) == -1) 
    {
	PERROR("tcpip_get - send");
	return 1;
    }
#if 0
    readfds = (1 << sd);
#else
    FD_SET(sd, &readfds);
#endif
    nfds = sd + 1;
    replen = 0;
// poll for a reply
    while(max_replen > 0 && select(nfds, &readfds, NULL, NULL, &call_timeout) > 0)
    {
	if (FD_ISSET(sd, &readfds))
	{
            n = recv(sd, reply, max_replen, 0);
	    if (n > 0)
	    {
	        reply += n;
	        replen += n;
	        max_replen -= n;
	    }
	    else
	    {
		max_replen = 0;
	    }
	}
        FD_ZERO(&readfds);
        FD_SET(sd, &readfds);
    }
    *reply_length = replen;
    if (shutdown(sd, 2) == -1)
    {
	PERROR("tcpip_get - shutdown");
    }
    if (closesocket(sd) == -1)
    {
	PERROR("tcpip_get - close");
    }
    return 0;
}
