/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *      Module:         genievars
 *      Author:         Freddie Akeroyd, ISIS
 *      Purpose:        General routines for manipulating "genie variables"
 *
 */
#include "geniedefs.h"

#include "genievars.h"

static const char* rcsid = "@(#)$Id: genievars.cxx,v 1.13 1997/04/02 22:24:59 faa Exp $";
USE_RCSID(rcsid);

OOP getClassOOP(OOP __oop)
{
    return isInt(__oop) ? integerClass : oopClass(__oop);
}

char* getClassString(OOP oop)
{
//    static OOP class_msg = registerOOP(internString("classNameString"));
    static OOP class_msg = registerOOP(internString("class"));
    static OOP print_msg = registerOOP(internString("printString"));
    OOP class_oop = msgSend(oop, class_msg, 0);
    return OOPToString(msgSend(class_oop, print_msg, 0));
}

static const int ngxtypes = sizeof(GXVariableNames) / sizeof(const char*);
static OOP gxtypes[ngxtypes];
static bool done_types = false;

void init_types_if_needed()
{
    if (!done_types)
    {
        for(int i=0; i<ngxtypes; i++)
        {
	    if (strncmp(GXVariableNames[i], "GX", 2) == 0)
	    {
	        gxtypes[i] = registerOOP(typeNameToOOP(GXVariableNames[i]));
		if (gxtypes[i] == nilOOP)
		{
		    gerr << "init_types_if_needed(): registering nilOOP!!!" << endl;
		}
	    }
	    else
	    {
		gxtypes[i] = nilOOP;
	    }
        }
	done_types = true;
    }
    return;
}

OOP getGenieVariableOOP(GenieVariableType type)
{
    init_types_if_needed();
    return gxtypes[type];
}

// Get type from a value oop
GenieVariableType getGenieVariableType(OOP oop)
{
    GenieVariableType type = getGenieVariableTypeFromClass(getClassOOP(oop));
    if (type == GenieUndefinedArrayType)
    {
 	int i, length = numIndexableFields(oop);
	for(i=1; i <= length; i++)
	{
	    if (getClassOOP(indexOOP(oop, i)) != undefinedObjectClass)
		break;
 	}    
	if (i > length) 
        { 	
	    return GenieUndefinedArrayType;		// All elements are undefined
	}
	switch(getGenieVariableType(indexOOP(oop, i)))
	{
	    case GenieIntegerType:
		return GenieIntegerArrayType;
		break;

	    case GenieRealType:
		return GenieRealArrayType;
		break;

	    case GenieStringType:
		return GenieStringArrayType;
		break;

	    case GenieWorkspaceType:
		return GenieWorkspaceArrayType;
		break;

	    case GenieUndefinedType:
		return GenieUndefinedArrayType;		// An array with all elements undefined
		break;

	    default:
		gerr << "getGenieVariableType: Undefined array type" << endl;
		return GenieUndefinedArrayType;
		break;
	}
    }    
    return type;
}

// Get type from a class oop
GenieVariableType getGenieVariableTypeFromClass(OOP classy)
{
    init_types_if_needed();
    for(int i=0; i<ngxtypes; i++)
    {
	if (classy == gxtypes[i])
	{
	    return (GenieVariableType)i;
	}
    }
    if (classy == integerClass)
    {
	return GenieIntegerType;
    }
    else if (classy == floatClass)
    {
	return GenieRealType;
    }
    else if (classy == stringClass)
    {
	return GenieStringType;
    }
    else if (isAKindOf(classy, identityDictionaryClass))
    {
	return GenieWorkspaceType;
    }    
    else if (isAKindOf(classy, arrayClass))
    {
	return GenieUndefinedArrayType;   
    }
    else if (classy == undefinedObjectClass)
    {
	return GenieUndefinedType;
    }
    else
    {
	gerr << "getGenieVariableType: undefined type" << endl;
	printOOPConstructor(classy);
	return GenieUndefinedType;
    }
    return GenieUndefinedType;	/* Not reached - just keeps CXX happy */
}


char* Strdup(const char* old_string)
{
    return Strdup(old_string, strlen(old_string));
}

char* Strdup(const unsigned char* old_string)
{
    return Strdup(old_string, strlen((const char*)old_string));
}

char* Strdup(const unsigned char* old_string, int length)
{
    char* new_string = new char[length+1];
    new_string[length] = '\0';
    return strncpy(new_string, (const char*)old_string, length);
}

char* Strdup(const char* old_string, int length)
{
    char* new_string = new char[length+1];
    new_string[length] = '\0';
    return strncpy(new_string, old_string, length);
}

