/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

// ghistory.cpp: implementation of the CHistory class.
//
//////////////////////////////////////////////////////////////////////

#include "geniedefs.h"      // Project header file always include

#include <stdlib.h>
#include "history.h"        // GNU history header
#include "ghistory.h"       // Class header

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

unsigned CHistory::s_dwSessionID = 0;

CHistory::CHistory(const char* filename) :
    m_gstrHistoryFile(filename)

{
    if (s_dwSessionID != 0)
    {
	    gerr << "Not allowed to create more than one history object per process - aborting" << endl;
            exit(1);
    }
    else
    {
	s_dwSessionID++;

        using_history();        // GNU initialization

        // load any history commands or create a new history file
        if (read_history((char*)(const char*)m_gstrHistoryFile) != 0) /* read in previous session commands */
        {
            gprintif("WARNING: unable to read commands from %s\n", (const char*)m_gstrHistoryFile);
            gprintif("--------- creating a new history file\n");
	        add_history("");

            if (write_history((char*)(const char*)m_gstrHistoryFile) != 0)
            {// read in previous session commands
                gprintif("WARNING: cannot create a history file", (const char*)m_gstrHistoryFile);
            }
        }
        using_history();        // Set history pointer to end of read in list
        m_search_pos = history_length - 1;
    }
}


CHistory::~CHistory()
{
    // save commands from this session
/*    if (history_truncate_file((char*)(const char*)m_gstrHistoryFile, 500) != 0)
    {
	gprintif("WARNING: error closing history session log file %s\nOld commands may be lost",
            (const char*)m_gstrHistoryFile);
    }
*/    
    // Do this last in case something bombs and we don't get here
    s_dwSessionID--;
}

void CHistory::AddCommand(const char * szCommand)
{
    add_history((char*)szCommand);
    history_set_pos(history_length);
    m_search_pos = history_length-1;
    append_history(1, (char*)(const char*)m_gstrHistoryFile);
    history_truncate_file((char*)(const char*)m_gstrHistoryFile, 500);
}

const char* CHistory::GetLastCommand()
{
    return current_history() == 0 ? "<none>" : current_history()->line;
}

const char* CHistory::GetPreviousCommand()
{
    HIST_ENTRY* history;
    history = previous_history();
    if (history == NULL) {
	    return NULL;
    } else {
        return history->line;
    }
}

const char* CHistory::GetNextCommand()
{
    HIST_ENTRY* history;
    history = next_history();
    if (history == NULL) {
	    return NULL;
    } else {
	    return history->line;
    }
}

const char* CHistory::Search(const char* text)
{
    int pos;
    if (text == NULL) {
        m_search_pos = history_length - 1;
        return NULL;
    }        
    pos = history_search_pos((char*)text, -1, m_search_pos);
    if (pos == -1) {
	    return NULL;
    } else {
	    m_search_pos = pos - 1;
	    return history_get(pos+history_base)->line;
    }	 
}
