/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include "hdf_utils.h"
#include "hdf_object.h"
#include "hdf_vgroup.h"

int HDFObject::addToFile(HDFFile& hdf)
{
    gerr << "Attempt to write unknown object" << endl;
    return -1;
}

int HDFObject::insertInVgroup(Vgroup& vg)
{
    int status = Vaddtagref(vg.vid(), m_tag, m_ref);
    HDF_ERROR("HDFObject::insertInVgroup", status, -1);
    return 0;
}

int HDFObject::addAnnotatation(const char* annotation, ann_type type)
{
    int32 status, an_id = ANstart(m_file_id);
    HDF_ERROR("HDFObject::addAnnotatation", an_id, -1);
    int32 ann_id = ANcreate(an_id, m_tag, m_ref, type);
    HDF_ERROR("HDFObject::addAnnotatation", ann_id, -1);
    status = ANwriteann(ann_id, annotation, HDstrlen(annotation));
    HDF_ERROR("HDFObject::addAnnotatation", status, -1);
    status = ANendaccess(ann_id);
    HDF_ERROR("HDFObject::addAnnotatation", status, -1);
    status = ANend(an_id);
    HDF_ERROR("HDFObject::addAnnotatation", status, -1);
    return 0;
}

int HDFObject::numberOfAnnotations(ann_type type) const
{
    int32 status, an_id = ANstart(m_file_id);
    HDF_ERROR("HDFObject::numberOfAnnotations", an_id, -1);
    int32 ann_num = ANnumann(an_id, type, m_tag, m_ref);
    status = ANend(an_id);
    HDF_ERROR("HDFObject::numberOfAnnotations", status, -1);
    return ann_num;
}

char* HDFObject::readAnnotation(int index, ann_type type) const
{
    if (index >= numberOfAnnotations(type))
    {
	gerr << "Attempt to read invalid annotation number: " << index << endl;
	return 0;
    }
    int32 status, an_id = ANstart(m_file_id);
    HDF_ERROR("HDFObject::readAnnotation", an_id, 0);
    int32 ann_id = ANselect(an_id, index, type);
    HDF_ERROR("HDFObject::readAnnotation", ann_id, 0);
    int len = ANannlen(ann_id);
    char* buffer = new char[len+2];
    status = ANreadann(ann_id, buffer, len+1);
    HDF_ERROR("HDFObject::readAnnotation", status, 0);
    status = ANendaccess(ann_id);
    HDF_ERROR("HDFObject::readAnnotation", status, 0);
    status = ANend(an_id);
    HDF_ERROR("HDFObject::readAnnotation", status, 0);
    return buffer;
}

ostream& HDFObject::print(ostream& os) const
{
    int i, n;
    os << "HDFObject with tag " << m_tag << " and reference " << m_ref << endl;
    n = numberOfLabels();
    if (n > 0)
    {
        os << "*** Object Labels ***" << endl;
	for(i=0; i<n; i++)
	    os << readLabel(i) << endl;
    }
    n = numberOfDescriptions();
    if (n > 0)
    {
        os << "*** Object Descriptions ***" << endl;
	for(i=0; i<n; i++)
	    os << readDescription(i) << endl;
    }
    return os;
}

