/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"

#ifndef _WIN32
#	include <sys/time.h>
#	include <pwd.h>
#else
#	include <time.h>
#endif

#include "genievars.h"
#include "intermediate_file.h"

#if defined(__sgi) || defined(__osf__)
extern "C" int gethostname(char*, int);
#endif

#if defined(__VMS) && __VMS_VER < 70000000
//#include "vms_password_funcs.h"
#endif

#if HAVE_LIBC_R
#include <sia.h>
#endif

IntermediateFile::~IntermediateFile()
{
}

// Sort out a mess on Digital UNIX 4.0 
#ifdef HAVE_LIBC_R
#  if _POSIX_C_SOURCE>=199506L	/* P1003.1c */
        extern "C" int getpwuid_r(uid_t, struct passwd *, char *, size_t, struct passwd **);
#       define GETPWUID(uid,pwname) getpwuid_r(getuid(), NULL, buffer1, sizeof(buffer1),&pwname)
#  else
#       define GETPWUID(uid,pwname) getpwuid_r(getuid(), pwname, buffer1, sizeof(buffer1))
#  endif
#else
#       define GETPWUID(uid,pwname) pwname = getpwuid(uid)
#endif /* HAVE_LIBC_R */

#ifndef SIABUFSIZ
#define SIABUFSIZ 128
#endif

String IntermediateFile::creator()
{
    char buffer[128], buffer1[SIABUFSIZ];
#ifdef HAVE_LIBC_R
    struct passwd* pwname = new passwd;
#else
    struct passwd* pwname = 0;
#endif

    // Write out the name of the user
    if (gethostname(buffer, sizeof(buffer)) != 0)
    {
        strcpy(buffer, "<unknown>");
    }

#ifndef _WIN32
    GETPWUID(getuid(), pwname);
#endif

    String user_name;
#if defined(__VMS)                    /* we don't have pw_gecos */
    user_name = "unknown";
#elif defined(_WIN32)
    DWORD buflen1 = sizeof(buffer1);
    GetUserName(buffer1, &buflen1);
    user_name = buffer1;
#else
    if (pwname != 0)
        user_name = pwname->pw_gecos;
    else
	user_name = "unknown";
#endif 

    user_name += " <";
#ifdef _WIN32
    user_name += user_name += "@"; user_name += buffer;
#else
    if (pwname != 0)
        user_name += pwname->pw_name; 
    else
	user_name += "unknown";
    user_name += "@"; user_name += buffer;
#endif
    user_name += ">";
                                       
#ifdef HAVE_LIBC_R
    delete pwname;
#endif
    return user_name;
}

String IntermediateFile::dateAndTime()
{
    static char buffer[128];
// Write out the date and time of the update
    time_t clock;
    time(&clock);
    struct tm* tm_struct = localtime(&clock);
    if (strftime(buffer, sizeof(buffer), "%a %d %h at %H:%M:%S %Z", tm_struct) == 0)
    {
        strcpy(buffer, "<unknown>");
    }
    return String(buffer);
}
