/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#define IN_IOFUNCS_CC
#include "geniedefs.h"
#undef IN_IOFUNCS_CC
#include <string.h>
#include <stdio.h>
#include <stdarg.h>


//
// Used to keep all GENIE I/O under some form of control
//

static const char* rcsid = "@(#)$Id: iofuncs.cxx,v 1.17 1997/11/12 15:59:49 faa Exp $";
USE_RCSID(rcsid);


// Define all GENIE I/O stream pointers here
istream              *g_in;       // No special processing needed on input
genie_ostream        *g_out;      // equivalent to cout
genie_ostream        *g_inf;      // filter for informational messages/warnings
genie_ostream        *g_err;      // filter for error messages (to stderror)
genie_ostream        *g_dbg;      // filter for debug output (to stdlog)


FILE *__si = stdin;    /* temps for stdin, stdout & stderr */
FILE *__so = stdout;
FILE *__se = stderr;


// Unlikely to print out more this in one go!
#define PRINT_BUFFER LONGCHARBUFF*2
static char buffer[PRINT_BUFFER+1];

void gwrite(const void *data, size_t size, size_t num_items)
{
    g_out->write((const char*)data, size * num_items);
}

void gwritei(const void *data, size_t size, size_t num_items)
{
    g_inf->write((const char*)data, size * num_items);
}

void g_writee(const void *data, size_t size, size_t num_items)
{
    g_err->write((const char*)data, size * num_items);
}

void gwrited(const void *data, size_t size, size_t num_items)
{
    g_dbg->write((const char*)data, size * num_items);
}

void gprintf(const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    vsprintf(buffer, fmt, args);
    va_end(args);
    if (strlen(buffer) >= PRINT_BUFFER)
        gerr << "Buffer too short - increase PRINT_BUFFER & recompile" << endl;
    *g_out << buffer << flush;
}

void gprintif(const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    vsprintf(buffer, fmt, args);
    va_end(args);
    if (strlen(buffer) >= PRINT_BUFFER)
        gerr << "Buffer too short - increase PRINT_BUFFER & recompile" << endl;
    *g_inf << buffer << flush;
}

void g_printef(const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    vsprintf(buffer, fmt, args);
    va_end(args);
    if (strlen(buffer) >= PRINT_BUFFER)
        gerr << "Buffer too short - increase PRINT_BUFFER & recompile" << endl;
    *g_err << buffer << flush;
//    logit(buffer);      /* Report back problems in the field to HQ */
}

void gprintdf(const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    vsprintf(buffer, fmt, args);
    va_end(args);
    if (strlen(buffer) >= PRINT_BUFFER)
        gerr << "Buffer too short - increase PRINT_BUFFER & recompile" << endl;
    *g_dbg << buffer << flush;
}



// IO manipulators

ostream& g_indent(ostream& os, int n)
{
    for(int i=0; i<n; i++)
    {
        os.put(' ');
    }
    return os;
}



// These templates aren't defined in libg++ for some reason
#if defined(__GNUG__) /* || defined(_MSC_VER) */
template class omanip<int>;
template ostream& operator<<(ostream&, const omanip<int>&);
#endif /* __GNUG__ */
