/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef ISIS_RAW_FILE
#define ISIS_RAW_FILE

const char* translatePath(const char *path, char opt);
void translatePath(String& path, char opt);

class ISISRawFile
{
    private:
	String m_runid;
    	fort_int* m_spec_work;
        fort_int* m_idat;
    	fort_real* m_delt_work;
    	fort_real* m_tthe_work;
    	fort_real* m_l2_work;
    	fort_real* m_tcb;
	fort_real* m_user_tables;
	fort_int m_ntc, m_ndet, m_nuse, m_ndetmax, m_ntcmax;

	g_real* m_xdata;		// TCB
   	int readSpectrum(GenieWorkspace* pgw, int spec_no); 

    public:
 	ISISRawFile();
    	int open(const char* runid);
   	int readSpectrum(GenieWorkspace& gw, int spec_no)
   	    { return readSpectrum(&gw, spec_no); }
	int readSpectra(GenieWorkspace& gw, const GenieIntegerArray& spectrum_numbers, int nspec);
    	~ISISRawFile();
	static int checkAndOpen(String& runid, fort_int& ntc, fort_int& ndet, fort_int& nuse);
	int numberOfTimeChannels() { return m_ntc; }
	const g_real* timeChannels() const { return m_xdata; }
	int readSpectrum(g_real* ydata, g_real* earray, fort_int spec_no);
};

#endif /* ISI_SRAW_FILE */
