%{
#include "geniedefs.h"

static const char* rcsid = "@(#)$Id: lexcommands.l,v 1.25 1998/05/18 14:57:18 cmm Exp $";
USE_RCSID(rcsid);


#if defined (HAVE_UNISTD_H)
#  include <unistd.h>
#endif /* HAVE_UNISTD_H */

#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>

#define INSIDE_PARSER__
#include "parsedefs.h"
#include "yacccommands.h"

#ifndef FALSE
#define FALSE (0)
#endif
#ifndef TRUE
#define TRUE (1)
#endif

#ifdef yywrap				/* if yywrap is defined as a macro eg "#define yywrap() 1" then clear it */
#undef yywrap
#endif

static char buffer[SHORTCHARBUFF];
#ifdef _WIN32
extern int yyback(int *p, int m);
#endif /* _WIN32 */

#ifdef ECHO				/* if ECHO is defined as a macro replace it with a hard error */
#undef ECHO
#endif
#define ECHO { sprintf(buffer,"%c <-- ECHOing without translation",yytext[0]); yyerror("buffer"); }

#ifdef YYLMAX				/* Need to up the size of Token buffer so redefine YYLMAX */
#undef YYLMAX
#endif
#define YYLMAX LEXTOKENBUFFERSIZ


/* Make up our own definitions for input and output to allow full control over the getc */
#undef input
#undef unput
#define input() (((yytchar=yysptr>yysbuf?U(*--yysptr):\
        getc_parser())==10?(yylineno++,yytchar):yytchar)==EOF?0:yytchar)
#define unput(c) {yytchar= (c);if(yytchar=='\n')yylineno--;*yysptr++=yytchar;}


/* Macros for returning apprpriate token values */
/* NOTE: do not change the position at which oldyy0 is set, uplus, uminus & special rely on it ! */

#define RETTOKVAL(x) {\
	oldyy0 = yytext[0];\
	if ( x == ENDOFLINE ) strcpy((char *)yytext,"<eol>");\
	strcpy(yylval.terminal, (char *)yytext);\
	return(last=x); }

#define RETFVAL(x) {\
	oldyy0 = yytext[0];\
	sscanf((char *)yytext, "%lf", &yylval.areal);\
	return(last=x); }

#define RETIVAL(x) {\
	oldyy0 = yytext[0];\
	yylval.aninteger = atol((char *)yytext);\
	return(last=x); }

#define RETSTRVAL yylval.astring

#define RETDEBUGSTRING(x) {\
	oldyy0 = yytext[0];\
	if ( x == ENDOFLINE ) strcpy((char *)yytext,"<eol>");\
	strcpy(yylval.debugging_string, (char *)yytext);\
	return(last=x); }

#ifdef _DEB_
#undef RETTOKVAL
#undef RETFVAL
#undef RETIVAL
#undef RETSTRVAL
#define RETTOKVAL(x) RETDEBUGSTRING(x)
#define RETFVAL(x) RETDEBUGSTRING(x)
#define RETIVAL(x) RETDEBUGSTRING(x)
#define RETSTRVAL yylval.debugging_string
#endif


/* VARIABLES
 * =========
 */


/* ancillary local definitions */

static char *strp;			/* general string pointer */
static int oldyy0;			/* Information kept to allow left context sensitivity on a Unary minus */
static int last;			/* The last token value returned to the calling parser */
static unsigned char *last_lexeme;	/* Keep our own pointer to the last lexeme parsed (ie yytext) */


/* Any functions to be used by yylex */

int yywrap(void){
	strcpy((char *)last_lexeme, "<eof>");	/* not strictly necessary but does help debugging */
	return(1);			/* 1 indicates end of file OK, stop lexing */
}

/* called by yacc or lex on a syntax error */

void ggerror(const char err[])
{
    g_printef("%d: %s\nThe last token read was \"%s\" (%d decimal), Token value = %d\n",
	    lines_parsed+1, err, last_lexeme, (int)(last_lexeme[0]), last);
    errors_found++;
    return;
}

int yylook();

/* First define all the token strings and regular expressions to get them
   note that the "special" definition is weird check out RE syntax very
   carefully for this one !*/

%}

digit		    [0-9]
hexdigit	    [0-9A-Fa-f]
octdigit	    [0-7]
alpha		    [A-Za-z_$]
special		    [\-\]^+*/|<>=%().:@,[&]
white		    [ \t]
eol		        (\r\n|\n)
any		        (.|{eol})
comment		    "#".*
continuation	"&"{white}*({eol}|{comment}{eol})

identifier	    {alpha}({alpha}|{digit})*

uinteger	    {digit}+

hexconst	    [Xx]{hexdigit}+
octconst	    0{octdigit}+
decconst	    [1-9]{digit}*

uplus		    \+/({alpha}|{digit}|{special})
uminus		    -/({alpha}|{digit}|{special})

mantissa	    (\.{uinteger}|{uinteger}\.{uinteger}|{uinteger}\.)
exponent	    [Ee]({uinteger}|[+\-]{uinteger})
real		    ({mantissa}|{mantissa}{exponent}|{uinteger}{exponent})

/*%p 3000		/* Increase DFA move space from default of 2500 */
%start STRN NORM CSTATE


%%

%{
last_lexeme = (unsigned char *) yytext;			/* keep a pointer to yytext for diagnostics (cannot be done earlier) */
continuation_line = FALSE;
%}

<NORM>OR		    RETTOKVAL(OR);			/* Keywords */
<NORM>AND		    RETTOKVAL(AND);
<NORM>NOT		    RETTOKVAL(NOT);
<NORM>FORWARD		RETTOKVAL(FORWARD);
<NORM>PROCEDURE		RETTOKVAL(PROCEDURE);
<NORM>ENDPROCEDURE	RETTOKVAL(ENDPROCEDURE);
<NORM>PARAMETERS	RETTOKVAL(PARAMETERS);
<NORM>QUALIFIERS	RETTOKVAL(QUALIFIERS);
<NORM>RESULT		RETTOKVAL(RESULT);
<NORM>GLOBAL		RETTOKVAL(GLOBAL);
<NORM>LOCAL		    RETTOKVAL(LOCAL);
<NORM>LOOP		    RETTOKVAL(LOOP);
<NORM>FROM		    RETTOKVAL(FROM);
<NORM>TO		    RETTOKVAL(TO);
<NORM>STEP		    RETTOKVAL(STEP);
<NORM>ENDLOOP		RETTOKVAL(ENDLOOP);
<NORM>EXITIF		RETTOKVAL(EXITIF);
<NORM>CASE		    RETTOKVAL(CASE);
<NORM>IS		    RETTOKVAL(IS);
<NORM>ENDCASE		RETTOKVAL(ENDCASE);
<NORM>OTHERWISE		RETTOKVAL(OTHERWISE);
<NORM>IF		    RETTOKVAL(IF);
<NORM>ELSEIF		RETTOKVAL(ELSEIF);
<NORM>ELSE		    RETTOKVAL(ELSE);
<NORM>ENDIF		    RETTOKVAL(ENDIF);
<NORM>RETURN            RETTOKVAL(RETURN);

<NORM>"<~"              RETTOKVAL(ALIAS);		/* Compound operators */
<NORM>"!="              RETTOKVAL(NOTEQUALS);
<NORM>">="              RETTOKVAL(GEQUAL);
<NORM>"<="              RETTOKVAL(LEQUAL);

<NORM>"%("          {
			            BEGIN CSTATE;
			            strp = RETSTRVAL;
			        }

<NORM>{uplus}		{ if ( strchr("(><=!+-*/| \t&,[^", oldyy0) )
				        { RETTOKVAL(UPLUS); } else RETTOKVAL(oldyy0); }		/* NOTE: for uplus, uminus & special,	 */
<NORM>{uminus}		{ if ( strchr("(><=!+-*/| \t&,[^", oldyy0) )
				        { RETTOKVAL(UMINUS); } else RETTOKVAL(oldyy0); }	/* oldyy0 =~= yytext[0] (see RETTOKVAL)  */

<NORM>{real}		RETFVAL(UREAL); 		/* Real numbers */
<NORM>{uinteger}	RETIVAL(UINTEGER);		/* Unsigned integers */
<NORM>{identifier}	RETTOKVAL(IDENTIFIER);		/* identifiers (not a keyword) */

<NORM>\"	        {
        			    BEGIN STRN;			/* Sub-syntax for strings (similar to C strings) */
        			    strp = RETSTRVAL;		/* initialize string pointer to build up string value */
        			}

<NORM>{special}		RETTOKVAL(oldyy0);		/* Return the character as a special */

<NORM>{white}		oldyy0 = yytext[0];		/* Match whitespace */
<NORM>{eol}		    RETTOKVAL(ENDOFLINE);   /* Match an end of line */

<NORM>;			    {				            /* Pseudo end of line */
            			lines_parsed--;			/* fix the line count as the yacc parser will increment it */
            			RETTOKVAL(ENDOFLINE);
        			}

<NORM>{comment}		RETTOKVAL(COMMENT);		/* Match a comment */
<NORM>.			    RETTOKVAL(RUBBISH);		/* Anything else is an illegal character */


<STRN>\\{decconst}	{ *strp++ = (unsigned char)strtol((char *)yytext+1, (char **)0, 10); }			/* Convert the constant */
<STRN>\\{octconst}	{ *strp++ = (unsigned char)strtol((char *)yytext+1, (char **)0, 8); }			/* Convert the constant */
<STRN>\\{hexconst}	{ *strp++ = (unsigned char)strtol((char *)yytext+2, (char **)0, 16); }			/* Convert the constant */
<STRN>\\[rbfntv]	{ *strp++ = *(strchr("\rr\bb\ff\nn\tt\vv", yytext[1]) - 1); }	/* lookup table for substitute values */

<STRN>(\\{eol}|{eol})	{
                			yyerror("Unterminated string");
                			BEGIN NORM;
                			oldyy0 = yytext[0];
                			return(last=RUBBISH);		/* Return an error */
            			}

<STRN>\"		    {
            			*strp = '\0';
            			BEGIN NORM;
            			oldyy0 = yytext[0];
            			return(last=STRING);			/* Return the bits */
        			}

<STRN>\\.		    { *strp++ = yytext[1]; }
<STRN>'			    { *strp++ = yytext[0]; *strp++ = yytext[0]; }		/* change ' to '' for smalltalk */
<STRN>.			    { *strp++ = yytext[0]; }


<CSTATE>\\{decconst}	{ *strp++ = (unsigned char)strtol((char *)yytext+1, (char **)0, 10); }			/* Convert the constant */
<CSTATE>\\{octconst}	{ *strp++ = (unsigned char)strtol((char *)yytext+1, (char **)0, 8); }			/* Convert the constant */
<CSTATE>\\{hexconst}	{ *strp++ = (unsigned char)strtol((char *)yytext+2, (char **)0, 16); }			/* Convert the constant */
<CSTATE>\\[rbfntv]	    { *strp++ = *(strchr("\rr\bb\ff\nn\tt\vv", yytext[1]) - 1); }	/* lookup table for substitute values */

<CSTATE>(\\{eol}|{eol}) {
                			*strp = '\0';
                			oldyy0 = yytext[0];
                			strp = RETSTRVAL;
                			return(last=CODELINE);			/* Return the bits */
            			}

<CSTATE>")%"    {
        			*strp = '\0';
        			BEGIN NORM;
        			oldyy0 = yytext[0];
        			strp = RETSTRVAL;
        			return(last=ENDCODE);			/* Return the bits */
    			}

<CSTATE>\\.		{ *strp++ = yytext[1]; }
<CSTATE>.		{ *strp++ = yytext[0]; }

{continuation}	{ oldyy0 = yytext[0]; continuation_line = TRUE; lines_parsed++; } /* Handle line extension using "&" (all states) */

{any}			{ unput(yytext[0]); BEGIN NORM; }	/* Get into initial state on any character (then relex it)*/
