/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
//+++
//
// CLASS: Marker
//
//
//---
// ----
// define String data type
//
#include <gnustring.h>
#include "Drawable_objects.h"
#include "Plot.h"
#include "Marker.h"
#include "Pginclude.h"
#include "Errors.h"
#include "SinglePrecision.h"
 
static const char* rcsid = "@(#)$Id: marker.cxx,v 1.8 1997/04/18 09:08:41 cmm Exp $";
USE_RCSID(rcsid);

Marker::Marker     (int size,
		   double* xdata,
		   double* ydata,
		   int symb,
		   int colour,
		   float csize,
		   int thickness) {

//	    int c;

	    // *** set visibility
	    invisible = 0;

	    // *** get size (check for zero)
	    if (size <= 0) {
			Errors syserr;
			syserr.add_error ("Marker plot ",
				"Illegal number of data points ",
				"make number_of_points greater than 0");
			return;
			}
	    number_of_points = size;

	    // *** allocate X array
	    X_array = new float[size];
	    if (X_array == 0) {
			Errors syserr;
			syserr.add_error ("Marker plot ",
			    "Insufficient virtual memory for X array ",
			    "call procedure DELETE_PICTURES");
			return;
			}
	    else
		    DoubleArrayToFloatArray(X_array, xdata,size);

	    // *** allocate Y array
	    Y_array = new float[size];
	    if (Y_array == 0) {
			Errors syserr;
			syserr.add_error ("Marker plot ",
			    "Insufficient virtual memory for Y array ",
			    "call procedure DELETE_PICTURES");
			return;
			}
	    else
		    DoubleArrayToFloatArray(Y_array, ydata,size);

	    // *** fill in line colour
	    if (colour == -1)
		    line_colour = LINE_COLOUR;
	    else
		    line_colour = colour;

	    // *** fill in symbol
	    if (symb == 0)
		    symbol = SYMBOL;
	    else
		    symbol = symb;

	    // *** fill in thickness
	    if (thickness == 0)
		    line_thickness = LINE_THICKNESS;
	    else
		    line_thickness = thickness;

	    // *** fill in size
	    if (csize == 0.0)
		    character_size = CHARACTER_SIZE;
	    else
		    character_size = csize;

	   }

Marker::~Marker() {

	    // *** delete the NEWed arrays
	    delete [] X_array;
	    delete [] Y_array;

	    }

void Marker::display() {

	    // *** set the colour
	    pgsci(line_colour);

	    // *** set the line thickness
	    pgslw(line_thickness);

	    // *** character size
	    pgsch(character_size);

	    // *** plot the line
	    pgpoint(number_of_points, X_array, Y_array,symbol);

	    }
