/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef MODULE_H
#define MODULE_H

#ifndef _WIN32
#	include <sys/time.h>
#else
#	include <time.h>
#endif

#include "splash.h"


enum ModuleFunctionType
{
    FortranModuleFunction = 0, CModuleFunction = 1
};

struct module_function
{
    String name;		// name of function
    String alias;		// GENIE name of function
    ModuleFunctionType type;	// type of function
    module_function() : name("<unknown>"), alias("<unknown>"), type(FortranModuleFunction) { }
    module_function(String n, String a, ModuleFunctionType t) :
	name(n), alias(a), type(t) { }
};

typedef SPList<module_function> ModuleFunctionList;

struct module_item
{
    void* handle;		// Pointer to library handle
    String path;		// Name of shared library file
    time_t load_time;		// time() when file was loaded
    String description;		// Description of the module (for the list command)
    ModuleFunctionList funcs;	// known functions
};

typedef SPList<module_item> ModuleList;

class Module
{
    protected:
	ModuleList m_list;

    private:
	int unload(int i);
	const char* getPath(int i);
	int getModulePosition(const char* path);

    public:
	int load(const char* pathname, const char* description);
	int unload(const char* pathname);
	int list(genie_ostream& output_stream);
	void* getFunctionPointer(const char* name, ModuleFunctionType type = FortranModuleFunction);
	Module() {};
	~Module();
};

int operator<(const module_item& a, const module_item& b);
int operator<(const module_function& a, const module_function& b);

#endif /* MODULE_H */
