/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include "g_types.h"
#include "module_inquire.h"

char* DialogueElement::getVarString(int index, Tcl_Interp* interp)
{
    char var[6];
    sprintf(var, "g_me%d", index);
    return Tcl_GetVar(interp, var, TCL_GLOBAL_ONLY);
}

void IntegerDialogueElement::constructElement(int index, Tcl_Interp* interp)
{
    char output[100];
    sprintf(output, "g_inquire_num \"%s\" %d %d", (const char*)m_label, *m_value, index);
    Tcl_Eval(interp, output);    
}

void IntegerDialogueElement::returnValue(int index, Tcl_Interp* interp)
{
    *m_value = atoi(getVarString(index, interp));
}

void RealDialogueElement::constructElement(int index, Tcl_Interp* interp)
{
    char output[100];
    sprintf(output, "g_inquire_num \"%s\" %f %d", (const char*)m_label, *m_value, index);
    Tcl_Eval(interp, output);    
}

void RealDialogueElement::returnValue(int index, Tcl_Interp* interp)
{
    *m_value = F(atof(getVarString(index, interp)));
}

void DoubleDialogueElement::constructElement(int index, Tcl_Interp* interp)
{
    char output[100];
    sprintf(output, "g_inquire_num \"%s\" %f %d", (const char*)m_label, *m_value, index);
    Tcl_Eval(interp, output);    
}

void DoubleDialogueElement::returnValue(int index, Tcl_Interp* interp)
{
    *m_value = atof(getVarString(index, interp));
}

void StringDialogueElement::constructElement(int index, Tcl_Interp* interp)
{
    char output[100];
    sprintf(output, "g_inquire_string \"%s\" \"%s\" %d %d", (const char*)m_label, m_value, m_length, index);
    Tcl_Eval(interp, output);    
}

void StringDialogueElement::returnValue(int index, Tcl_Interp* interp)
{
    char* result = getVarString(index, interp);
    strncpy(m_value, result, m_length);
}

void DialogueBox::display()
{
    char output[100];
    int c;
    sprintf(output, "g_start_inquire \"%s\"", (const char*)m_title);
    Tcl_Eval(m_interp, output);
    for (c=0; c<m_elist.count(); c++)
    {
        m_elist[c]->constructElement(c, m_interp);
    }
    sprintf(output, "g_stop_inquire");
    Tcl_Eval(m_interp, output);        
    for (c=0; c<m_elist.count(); c++)
    {
        m_elist[c]->returnValue(c, m_interp);
        delete m_elist[c];
    }   
}

// Instantiate templates
#if defined(__GNUG__)
template class SPList<DialogueElement*>;
template class SPListBase<DialogueElement*>;
#endif /* __GNUG__ */

#ifdef __DECCXX
#pragma define_template SPList<DialogueElement*>
#pragma define_template SPListBase<DialogueElement*>
#endif /* __DECCXX */
