/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef MODULE_INQUIRE_H
#define MODULE_INQUIRE_H

#include "splash.h"
#include <tk.h>

class DialogueElement
{
    protected:
        String m_label;
        char* getVarString(int index, Tcl_Interp* interp);
    public:
        virtual void constructElement(int index, Tcl_Interp* interp) {}
        virtual void returnValue(int index, Tcl_Interp* interp) {}
        DialogueElement(String label): m_label(label) {}
	virtual ~DialogueElement() {}
};

class IntegerDialogueElement: public DialogueElement
{
    private:
        int* m_value;
    public:
        void constructElement(int index, Tcl_Interp* interp);
        void returnValue(int index, Tcl_Interp* interp);
        IntegerDialogueElement(String label, int* value): DialogueElement(label), m_value(value) {}
};

class RealDialogueElement: public DialogueElement
{
    private:
        float* m_value;
    public:
        void constructElement(int index, Tcl_Interp* interp);
        void returnValue(int index, Tcl_Interp* interp);
        RealDialogueElement(String label, float* value): DialogueElement(label), m_value(value) {}
};

class DoubleDialogueElement: public DialogueElement
{
    private:
        double* m_value;
    public:
        void constructElement(int index, Tcl_Interp* interp);
        void returnValue(int index, Tcl_Interp* interp);
        DoubleDialogueElement(String label, double* value): DialogueElement(label), m_value(value) {}
};

class StringDialogueElement: public DialogueElement
{
    private:
        char* m_value;
        int m_length;
    public:
        void constructElement(int index, Tcl_Interp* interp);
        void returnValue(int index, Tcl_Interp* interp);
        StringDialogueElement(String label, char* value, int length): DialogueElement(label), m_value(value), m_length(length) {}
};

typedef SPList<DialogueElement*> DialogueElementList;

class DialogueBox
{
    private:
        DialogueElementList m_elist; 
        String m_title;
        Tcl_Interp* m_interp;
    public:
        DialogueBox(String title, Tcl_Interp* interp) : m_title(title), m_interp(interp) {}
        void addElement(DialogueElement* e) {m_elist.push(e);}
        void display();        
};

#endif /* MODULE_INQUIRE_H */
