/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"

#if defined (HAVE_UNISTD_H)
#  include <unistd.h>
#endif /* HAVE_UNISTD_H */


static const char* rcsid = "@(#)$Id: motifdriver.c,v 1.12 1997/04/18 09:08:43 cmm Exp $";
USE_RCSID(rcsid);

#if defined(__unix) || defined(__unix__)
#   define newxedriver newxedriver_	    /* UNIX and F2C APPEND AN "_" */
#endif

#ifdef __cplusplus
extern "C"
#endif
void newxedriver(int *intarr, float *rbuf, char *chr);

#ifdef HAVE_LIBXM

#undef HAVE_LIBPTHREADS
/* 
    This is an X windows PGPLOT driver for VAX & AXp VMS, however, it should
    be possible to make it function on any system supporting the intrinsic X
    toolkit, the Motif widget set and Posix threads.

    Both parts of the driver (C and Fortran) MUST be used together, the two
    parts conbined may be substituted directly for the XEDRIV.F driver. The
    device then replaces the XWINDOWS device.

    The driver attempts to solve the existing X driver's problems that have
    been the most inconvenient to us:

	o  The new driver does not allocate reserved colour cells,
	   but uses shared colour planes in the Default colourmap.

	o  It is now possible to resize the window using the mouse
           by dragging and dropping the resize border.

	o  The 'Close' option in the system menu is now inoperative.
	   Instead there is a menu bar at the top, this includes a
	   'Close window' option, it closes the display.

	o Posix Threads have been used to control the asynchronous
	  event handler. All operating system dependant parts have
	  been modularised for porting convenience.

	o For colour terminals, a 'Background Colour' menu has been
	  added so that the user can change it during run-time.

	0 To aid further diagnostics, there is a 'Diagnostics ON'
	  item in the file menu. usefull if there are problems.

    -----------------------------------------------------------------------

    The driver has been compiled on the following operating systems:

	VAX-VMS version 5.5-2
	AXP-VMS version 1.5
	AXP-VMS version 6.1
	DEC-Ultrix version 4.4

	DECwindows Motif version 1.1 and 1.2.

    -----------------------------------------------------------------------

    The driver opens a display on the default display set by the user, if
    the user cannot open the display the the program aborts.

    The orientation is landscape, a call to PGPAPER sets it to the user
    required size.

    The resolution is 75 or 100DPI.

    The cursor is used for input (Button 1 returns A) 
				 (Button 2 returns D) 
				 (Button 3 returns X) 
    During cursor input, the asynchronous event handler is stopped and
    events are handled in the main process. The thread is re-started
    when the cursor routine terminates.

    -----------------------------------------------------------------------

    Author:  S. Johnston
	     ISIS Instrumentation Division,
	     Rutherford Appleton Laboratory.
	     (SDJ@isise.rl.ac.uk)

    Date:    22-July-1994.
    
    Notes to programmers:  If you have problems or feel that this could be 
    improved (or have made your own improvements) please mail to the above
    address so that the world may hear of your great deed!

    There are some utilities available for the X driver:

	1) Set the size and position of the window (before it appears)
	   from a Fortran or C callable function.

	2) Set the window title/icon name before the window appears
	   from C or Fortran.

	3) Change the title/icon name while the window is on screen.

	4) Make the drawing area a child of a Widget within one of
	   your own applications (You supply parent widget and event
	   handler - you could even use the xeventhandler supplied).

	5) Give a user supplied re-draw function, when the window is
	   re-sized using the mouse, PGPLOT calls are used to redraw
	   the picture.

--------------------------------------------------------------------------------
MODIFICATIONS::-

22 August 1994  Version 1.1
		The exception handling for the threads has been improved
		and will be omitted for POSIX compilation.
		The ability to toggle between buffered and non-buffered
		modes has been added.
		The ApplicationContext is not now destroyed after a
		CloseDevice call, instead it is used for any further
		OpenDevice calls, the OS is used to clear up any X
		resources allocated. Tests have shown that the device
		may be OPENED and CLOSED over 890 times from within
		a single image before memory is exhausted, therefore,
		it is not seen as a problem.

		Hooks for the redraw function have been added.

13 Sept 1994	S. Johnston
		Check that COLBUT is a valid Widget before setting
		foreground colour or sensitivity.

14 Nov 1994	S. Johnston
		When the code was optimised for speed, the cursor routine was
		broken. The driver no longer responded to keyboard input.
		The driver has been further enhanced so that the size may
		be changed for each invocation of the driver within the 
		same image execution.
*/

#include <stdio.h>
#include <stdlib.h>
#ifdef __VMS_POSIX			    /* stops interferance in VMS native driver */
#   include <unistd.h>
#endif
#include <setjmp.h>
#include <signal.h>
#include <X11/Intrinsic.h>
#include <X11/cursorfont.h>		    /* HOLD CURSOR DEFINITIONS */
#include <Xm/PushB.h>
#include <Xm/DrawingA.h>
#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/MainW.h>
#include <X11/Xlib.h>
#include <X11/X.h>

#ifndef HAVE_LIBPTHREADS
#   include "gsignals.h"
#endif


static Boolean DEBUG = False;		    /* GIVES THE DEBUG MESSAGES */
static Boolean ASYNC = True;		    /* ALLOWS ASYNC EVENT HANDLER */
static Boolean BUFFERING = True;
static Boolean UPDATEOFTEN = False;
static int BUFFERCOUNT = 100;

/* local stuff */
static Boolean FirstTime = True;	     /* FIRST DEVICE OPENED         */
static Boolean InitTime = True;		    /* FIRST PASS THROUGH FUNCTION */
static Boolean MouseMove = True;	    /* FALSE FOR PGWINDOW CALLS */

/* Allocated colour information */
/* The magic number used occurs
	 only in these lines */
#define MAXCOLOUR 145			    /* MAXIMUM ALLOWED COLOURS */
static int MAXALLOCCOLOUR = 0;		    /* NO COLOURS YET ALLOCATED */
static XColor Colours[MAXCOLOUR];	    /* ARRAY OF COLOURS */
static int AllocColour[MAXCOLOUR];	    /* IS IT ALLOCATED??? */
static unsigned short RGB[MAXCOLOUR][3]={65535, 65535, 65535,  /* WHITE    */
					    0,     0,     0,  /* BLACK    */
					65535,     0,     0,  /* RED      */
					    0, 65535,     0,  /* GREEN    */
					    0,     0, 65535,  /* BLUE     */
					    0, 65535, 65535,  /* CYAN     */
					65535,     0, 65535,  /* MAGENTA  */
					65535, 65535,     0,  /* YELLOW   */
					65535, 32767,     0,  /* ORANGE   */
					32767, 65535,     0,  /* LIGHT GR */
					    0, 65535, 32767,  /* Sea GR   */
					    0, 32767, 65535,  /* LT BLUE  */
					32767,     0, 65535,  /* PURPLE   */
					65535,     0, 32767,  /* MAROON   */
					20000, 20000, 20000,  /* DK GRAY  */
					53690, 53690, 53690};  /* LT GRAY  */

static char labels[16][15] = {"White","Black","Red","Green","Blue","Cyan",
	"Magenta","Yellow","Orange","Light Green","Sea Green","Light Blue",
	"Purple","Maroon","Dark Gray","Light Gray"};

/* widgets */
static Widget DrawingArea;
static Pixmap pixmap;
static GC gca, gcb;
static unsigned long Blackpix, Whitepix;
static Widget MenuBar,pulldown1,pulldown2,pulldown3,pulldown4;
static Widget filem, Userbutton;
static Widget buttonFD, buttonFQ, buttonFM,buttonFB,buttonMU;
static Widget buttonFB0, buttonFB1, buttonFB2;
static Widget COLBUT[16],Usersubmenu[20];
static Widget Colourem;

/* X information */
static Display *display;
static int screen;
static float resolution[2];
static int depth;
static Boolean MONO = True;
static Visual *visual;
static Colormap Colourmap;

/* screen and window sizes */
static int xpix, ypix;
static int xmm, ymm;
static int width, height;
static int PixWidth, PixHeight;
static int xpos, ypos;
static int MAX_X_REDRAW = 0, MAX_Y_REDRAW = 0;		/* USED FOR EXPOSE */
static int MIN_X_REDRAW = 2000, MIN_Y_REDRAW = 2000;    /* EVENTS          */
static int CURSOR_X_HIT;
static int CURSOR_Y_HIT;
static char CURSOR_B_HIT[2];

#include "motifdriver_functions.h"
#include "motifdriver.h"			/* EXTERNALS FOR UTILITIES */

int childwindow;				/* INTERNAL DECLARATIONS */
XtAppContext ApplicationContext;	    
int DPI;
funcPtr savedfunct;
funcPtrI exitfunct;
funcPtrI menufunct;
Widget TopWindow;
Widget ParentWindow;
struct user1 usersize;
struct user2 userpos;    
struct user3 userinfo;
struct user4 usermenu;


/*
 ==============================================================================
 **************************************
 ALLOCATES COLOURS AS REQUIRED TO DO SO 
 **************************************
*/
void AllocateColour(int ColIndex) {
	XColor exact;
	int status;

	if (AllocColour[ColIndex] != 0) return;
	if (MONO) {
		if (DEBUG) printf("Mono terminal\n");

		Colours[0].pixel = WhitePixel(display, screen);
		Colours[1].pixel = BlackPixel(display, screen);
		AllocColour[0] = 1;
		AllocColour[1] = 1;
		Blackpix = Colours[0].pixel;
		Whitepix = Colours[1].pixel;
		return;
	}

	if (DEBUG) printf("Allocating colour index %d",ColIndex);
	if (DEBUG) printf(", RGB values: %d, %d, %d\n",
			RGB[ColIndex][0],RGB[ColIndex][1],RGB[ColIndex][2]);

	/* set the colour index */
	Colours[ColIndex].red   = RGB[ColIndex][0];
	Colours[ColIndex].green = RGB[ColIndex][1];
	Colours[ColIndex].blue  = RGB[ColIndex][2];

	/* allocate it */
	status = (int)XAllocColor(display, Colourmap, &Colours[ColIndex]);
	AllocColour[ColIndex] = status;

	if (DEBUG) printf(" -- Status returned: %d\n",status);

	if (ColIndex ==1)
		Blackpix = Colours[ColIndex].pixel;
	if (ColIndex ==0)
		Whitepix = Colours[ColIndex].pixel;
	
}

/*
 ==============================================================================
 ************************************** 
 REDRAWS THE WINDOW FOR AN EXPOSE EVENT
 ************************************** 
*/
void RedrawWindow(Widget wg, XtPointer p1, XtPointer p2) {
    Window window = XtWindow(wg);
    int x, y, w, h;
    XEvent *event = ((XmDrawingAreaCallbackStruct *)p2)->event;

    /* get position and size of exposed area */
    x = event->xexpose.x;
    y = event->xexpose.y;

    w = event->xexpose.width;
    h = event->xexpose.height;

     if (DEBUG) {
		printf("EXPOSE, re-drawing window - Position:");
		printf("%d, %d - Size: %d, %d\n",x,y,w,h);
     }

    /* do the redraw */
    if (w != 0 && h != 0)
        XCopyArea(display, pixmap, XtWindow(DrawingArea),
			     gca, x,y, w,h, x,y);

}

/*
 ==============================================================================
 ******************************************
 REMAPS THE WINDOW FOR AN UPDATE AREA EVENT
 ******************************************
*/
void updatepicture(Widget wg, XtPointer p1, XtPointer p2) {

    if (DEBUG) printf("Picture update called\n");

		if (MAX_X_REDRAW != 0 && MAX_Y_REDRAW != 0) {

		    /* check limits */
		    if (MIN_X_REDRAW < 0) MIN_X_REDRAW = 0;
		    if (MIN_Y_REDRAW < 0) MIN_Y_REDRAW = 0;
		    if (MAX_X_REDRAW > width) MAX_X_REDRAW = width;
		    if (MAX_Y_REDRAW > height) MAX_Y_REDRAW = height;

		    if (DEBUG) printf("Update area: (%d,%d)%d, %d\n",
			     MIN_X_REDRAW, MIN_Y_REDRAW,
			     MAX_X_REDRAW, MAX_Y_REDRAW);

		    XCopyArea(display, pixmap, XtWindow(DrawingArea),
			     gca, MIN_X_REDRAW, MIN_Y_REDRAW,
			     1 + MAX_X_REDRAW - MIN_X_REDRAW,
			     1 + MAX_Y_REDRAW - MIN_Y_REDRAW,
			     MIN_X_REDRAW, MIN_Y_REDRAW);

		}

		MIN_X_REDRAW = width;
		MAX_X_REDRAW = 0;
		MIN_Y_REDRAW = height;
		MAX_Y_REDRAW = 0;

		XtVaSetValues(buttonMU, XmNsensitive, False, NULL);
}

/*
 ==============================================================================
 *********************************
 MAKES A NEW PIXMAP AFTER A RESIZE
 *********************************
*/
void ResizePixmap(int NewX, int NewY, Boolean forced) {
		Pixmap OldPixmap;
		int x, y;

	/* trap for invalid pixmap */
	if (pixmap == 0) return;

	if (DEBUG && forced) printf("Pixmap resize forced on CLS\n");
	if (width > PixWidth || height > PixHeight || forced) {

		/* copy pixmap */
		OldPixmap = pixmap;

		/* make a new pixmap */
		pixmap = XCreatePixmap(display, XtWindow(DrawingArea),
				    NewX, NewY, depth);
		XFillRectangle(display, pixmap, gcb, 0,0, NewX, NewY);

		/* copy and free */
		x = width;
		y = height;
		if (NewX < x) x = NewX;
		if (NewY < y) y = NewY;

		XCopyArea(display, OldPixmap, pixmap,
					     gca, 0, 0, x,y, 0,0 );
		XFreePixmap(display,OldPixmap);

		/* set the statics */
		PixWidth = NewX;
		PixHeight = NewY;

		if (DEBUG) printf("New pixmap size:%d, %d\n",PixWidth,
						PixHeight);
	}
	
	/* clear the screen */
		if (DEBUG) printf("New pixmap Window:%d, Pixmap:%d\n",
			    DrawingArea,pixmap);
	XClearWindow(display, XtWindow(DrawingArea));
	XCopyArea(display, pixmap, XtWindow(DrawingArea),
				gca, 0, 0, width,height, 0,0 );
}

/*
 ==============================================================================
 **********************************
 RESIZES THE WINDOW USING THE MOUSE
 **********************************
*/
void ResizeWindow(Widget wg, XtPointer p1, XtPointer p2) {
    short xx;
    short yy;

    if (!MouseMove) return;

    if (DEBUG) printf("MOUSE - re-sizing window : ");

    /* get the drawing area size */
    XtVaGetValues(DrawingArea, XmNheight, &yy, NULL);
    XtVaGetValues(DrawingArea, XmNwidth, &xx, NULL);
    if (DEBUG) printf(" %d, %d\n",xx,yy);

    /* set the pixmap */
    width = xx;
    height  = yy;

    /* resize if required */
    if (savedfunct == NULL || pixmap == 0) {
	    ResizePixmap(width, height, False);
    }
    else {
	    if (DEBUG) printf("Redraw function called %d\n",savedfunct);
	    savedfunct();
    }
}

/*
 ==============================================================================
 ****************************************
 HOLD THE DRAWING AREA REFRESH PARAMETERS
 ****************************************
*/
void SetRedrawSize(int x1, int y1, int x2, int y2) {

    if (MAX_X_REDRAW == 0) XtVaSetValues(buttonMU, XmNsensitive, True, NULL);

    if (MIN_X_REDRAW > x1) MIN_X_REDRAW = x1;
    if (MIN_X_REDRAW > x2) MIN_X_REDRAW = x2;

    if (MAX_X_REDRAW < x1) MAX_X_REDRAW = x1;
    if (MAX_X_REDRAW < x2) MAX_X_REDRAW = x2;

    if (MIN_Y_REDRAW > y1) MIN_Y_REDRAW = y1;
    if (MIN_Y_REDRAW > y2) MIN_Y_REDRAW = y2;

    if (MAX_Y_REDRAW < y1) MAX_Y_REDRAW = y1;
    if (MAX_Y_REDRAW < y2) MAX_Y_REDRAW = y2;

}	

/*
 ==============================================================================
 ****************************
 SETS THE CURSOR CO-ORDINATES
 **************************** 
*/
void SetCursorValues(Widget wg, XtPointer p1, XtPointer p2) {
    Window window = XtWindow(wg);
    XEvent *event = ((XmDrawingAreaCallbackStruct *)p2)->event;
    int i;

    if (DEBUG) printf("CURSOR HIT -- ");
        /* get position */
        CURSOR_X_HIT = event->xbutton.x;
        CURSOR_Y_HIT = event->xbutton.y;

        if (DEBUG) printf("coordinates:%d,%d -- ",CURSOR_X_HIT,CURSOR_Y_HIT);

       if (event->type == ButtonPress) {

            if (DEBUG) printf("button pressed:%d\n",event->xbutton.button);
              if (event->xbutton.button == 1) {
		    sprintf(CURSOR_B_HIT, "A");
              }
		else if (event->xbutton.button == 2) {
		    sprintf(CURSOR_B_HIT, "D");
	       }
		else {
		    sprintf(CURSOR_B_HIT, "X");
              }
	}
	else if (event->type == KeyPress) { /* key press */

	    i = XLookupString(&(event->xkey), CURSOR_B_HIT,1,NULL,NULL);
            if (DEBUG) printf("Key pressed:%s\n",CURSOR_B_HIT);
	}

}

/*
 ==============================================================================
 *************************
 EVENT HANDLER IS THREADED
 ************************* 
*/
#ifdef HAVE_LIBPTHREADS

    static pthread_t thread;  /* a thread object */

    void *xeventloop(){
	XEvent event;
	sigset_t set;
	struct sigaction action;
	if (sigemptyset(&set) != 0) { perror("sigemptyset"); }
	action.sa_handler = SIG_IGN;
#ifdef __ultrix__
	action.sa_flags = 0;
	siginterrupt(SIGINT, 0);
#else
	action.sa_flags = SA_RESTART;
#endif /* __ultrix__ */
	action.sa_mask = set;
	if (sigaction(SIGINT, &action, NULL) != 0) { perror("sigaction"); }

	pthread_setasynccancel(CANCEL_ON);

	while (1) {
		while(XtAppPending(ApplicationContext) &XtIMAll) {
		    XtAppProcessEvent(ApplicationContext, XtIMAll);
		 }
	pthread_yield();
	}
	return 0;
    }	

    void StartEventHandler() {
	    int status;

	    /* flush any events */
	    while(XtAppPending(ApplicationContext) &XtIMXEvent) {
		    if (DEBUG) printf("Pre-flushing event\n");
		    XtAppProcessEvent(ApplicationContext, XtIMXEvent);
	    }

	    if (DEBUG) printf("ASYNCHRONOUS - event handler\n");
	    if (DEBUG) printf("Creating Pthread for asynchronous event");
	    if (DEBUG) printf(" handler\n");

	    status = pthread_create( &thread, pthread_attr_default,
			(pthread_startroutine_t)xeventloop, NULL);

	    if (DEBUG) {
		if (status == 0) {
		    printf("SUCCESS - thread created\n");
		 }
		 else {
		    printf("**** FAILURE - thread NOT created\n");
		 }
	    }
/*	    status = pthread_setprio(thread,PRI_FIFO_MIN);
	    status = pthread_setprio(pthread_self(),PRI_BG_MAX_NP);*/
	}

    void StopEventHandler() {
	int status;
	pthread_addr_t thrstat;

	/* destroy the event handler */
	status = pthread_cancel(thread);
	status = pthread_join(thread,&thrstat);
		 
	if (DEBUG) printf("PTHREAD - asynchronous event handler STOPPED\n");
	}

#else

    const int PERIOD=1;            /* Alarm every 1 second for screen update */
    void xeventloop(SIGTYPE){
	signal_handler oldc;
	XEvent event;

	oldc = genie_signal(SIGINT, SIG_IGN);
	while(XtAppPending(ApplicationContext) &XtIMAll) {
	    XtAppProcessEvent(ApplicationContext, XtIMAll);
	}
	genie_signal(SIGALRM, xeventloop);             /* re-enable signal catching */
	alarm(PERIOD);   /* Reset alarm */
	genie_signal(SIGINT, oldc);
	return;
    }

    void StartEventHandler() {
	    /* flush any events */
	    while(XtAppPending(ApplicationContext) &XtIMXEvent) {
		    if (DEBUG) printf("Pre-flushing event\n");
		    XtAppProcessEvent(ApplicationContext, XtIMXEvent);
	    }

	    genie_signal(SIGALRM, xeventloop);         /* Enable signal catching */
	    alarm(PERIOD);
	    return;
	}

    void StopEventHandler() {
	/* destroy the event handler */
	alarm(0);				 /* Kill alarm */
	genie_signal(SIGALRM, (signal_handler) 0);     /* Reset signal (SIG_DFL) */
	return;
    }

#endif




/*
 ==============================================================================
 *******************************
 DESTROYS THE APPLICATION NEATLY
 ******************************* 
*/
void DestroyMyApplication(int i) {

	if (DEBUG) printf("EXIT handler called\n");


	XtUnrealizeWidget(TopWindow);

	/* free the resources */
	XFreePixmap(display, pixmap);

	/* reset the parameters */
	pixmap = 0;

	/* destroy the event handler */
	if (ASYNC) StopEventHandler();
}

/*
 ==============================================================================
 **************************************
 CALLBACK to the destroy function      
 ************************************** 
*/
void SelfDestruct(Widget wg, XtPointer p1, XtPointer p2) {

	if (DEBUG) printf("*** APPLICATION TERMINATED BY EVENT HANDLER ***\n");


	if (exitfunct == NULL) {
		XtDestroyApplicationContext(ApplicationContext);
		exit(0);
	}
	else {
		exitfunct(0);
	}

}

/*
 ==============================================================================
 ************************************
 CALLBACK TO CHANGE BACKGROUND COLOUR
 ************************************ 
*/
void ChangeColour(Widget wg, XtPointer p1, XtPointer p2) {
	int i,answer;
	XmString str1, str2;
	XGCValues GCVals;
	unsigned long temp; 
	char tempstr[15];

	/* find which colour was chosen*/
	XtVaGetValues((Widget)p1,XmNlabelString,&str1,NULL);

	/* reset the options for items 0 and 1 */
	Colours[0].pixel = Whitepix;
	Colours[1].pixel = Blackpix;

	for (i=0; i<=MAXALLOCCOLOUR; i++) {
		str2 = XmStringCreateSimple(labels[i]);
		if (XmStringCompare(str1,str2)) {
		    answer = i;
		    XtSetSensitive(COLBUT[i],False);
		}
		else {
		    XtSetSensitive(COLBUT[i],True);
		}			
		XmStringFree(str2);
	}
	XmStringFree(str1);

	if (DEBUG) printf("Background colour index: %d\n",answer);

	/* allocate the colour */
	AllocateColour(answer);
	
	/* set the background GC */
	if (AllocColour[answer] == 0) {
	    printf("FAILURE - colour not available\n");
	    return;
	}

	if (DEBUG) printf("SUCCESS - colour available\n");
	GCVals.foreground = Colours[answer].pixel;
	XChangeGC(display, gcb,GCForeground, &GCVals);


	/* set the foreground/background for the drawing GC */

	/* swap the foreground colour to a contrasting one */
	if (answer == 1 || answer == 4 || (answer>10 && answer<15)) {
		Colours[1].pixel = Whitepix;
	}
	else { 
		Colours[1].pixel = Blackpix;
	}

	    /* set background to window background */
	    Colours[0].pixel = Colours[answer].pixel;

	    /* change the GC */
	    GCVals.foreground = Colours[1].pixel;
	    XChangeGC(display, gca,GCForeground, &GCVals);

	    /* redraw if possible */
	    if (savedfunct != NULL) {
		if (DEBUG) printf("Redraw function called %d\n",savedfunct);
		savedfunct();
	     }
}

/*
 ==============================================================================
 ******************************
 CALLBACK TO USER MENU FUNCTION
 ****************************** 
*/
void UserMenuHit(Widget wg, XtPointer p1, XtPointer p2) {
	int i,answer;
	XmString str1, str2;
	unsigned long temp; 

	/* find which colour was chosen*/
	XtVaGetValues((Widget)p1,XmNlabelString,&str1,NULL);

	if (usermenu.nitems == 0) {
		    menufunct(&usermenu.nitems);
	}
	else {
	  for (i=0; i<=usermenu.nitems; i++) {
		str2 = XmStringCreateSimple(usermenu.items[i]);
		if (XmStringCompare(str1,str2)) {
		    if (DEBUG) printf("User function called with option %d\n",
				    i);
		    answer = i+1;
		    menufunct(&answer);
		}
		XmStringFree(str2);
	  }
	}
	XmStringFree(str1);
}

/*
 ==============================================================================
 ************************************ 
 CALLBACK TO CHANGE DIAGNOSTIC MODE   
 ************************************ 
*/
void Diagnostic(Widget wg, XtPointer p1, XtPointer p2) {
	XmString str;

	   DEBUG = !DEBUG;

	    if (DEBUG) {
		str = XmStringCreateSimple("Diagnostics OFF");
		printf("Diagnostics switched ON\n");
	    }
	    else {
		str = XmStringCreateSimple("Diagnostics  ON");
		printf("Diagnostics switched OFF\n");
	    }

	    XtVaSetValues(buttonFD,XmNlabelString,str,NULL);
	    XmStringFree(str);

}

/*
 ==============================================================================
 ******************************
 CALLBACK TO CHANGE COLOUR MODE   
 ****************************** 
*/
void ToggleMode(Widget wg, XtPointer p1, XtPointer p2) {
	XmString str;
	XGCValues GCVals;

	   MONO = !MONO;

	    if (MONO) {
		if (DEBUG) printf("switched to mono terminal\n");
		str = XmStringCreateSimple("Colour display");
	    }
	    else {
		str = XmStringCreateSimple("Mono display  ");
		if (DEBUG) printf("Switched to colour terminal\n");
	    }

	    XtVaSetValues(buttonFM,XmNlabelString,str,NULL);
	    XmStringFree(str);

	    /* set the GC colour index */
	    GCVals.foreground = Colours[1].pixel;
	    XChangeGC(display, gca,GCForeground, &GCVals);

}
/*
 ==============================================================================
 ******************************
 CALLBACK TO CHANGE BUFFER MODE   
 ****************************** 
*/
void setbuffering(Widget wg, XtPointer p1, XtPointer p2) {

	 if (p1 == (void*)buttonFB0) {  /* buffering OFF */
		if (DEBUG) printf("Switched buffering OFF\n");
		BUFFERING = False;
		UPDATEOFTEN = False;
		XtVaSetValues(buttonFB0, XmNsensitive, False, NULL);
		XtVaSetValues(buttonFB1, XmNsensitive, True, NULL);
		XtVaSetValues(buttonFB2, XmNsensitive, True, NULL);
		if (XtIsManaged(buttonMU)) XtUnmanageChild(buttonMU);
	}
	else if (p1 == (void*)buttonFB1) {  /* buffering PARTIAL */
		if (DEBUG) printf("Switched buffering PARTIAL\n");
		BUFFERING = True;
		UPDATEOFTEN = True;
		XtVaSetValues(buttonFB0, XmNsensitive, True, NULL);
		XtVaSetValues(buttonFB1, XmNsensitive, False, NULL);
		XtVaSetValues(buttonFB2, XmNsensitive, True, NULL);
		if (!XtIsManaged(buttonMU)) XtManageChild(buttonMU);
	}
	else {			    /* buffering FULL */
		if (DEBUG) printf("Switched buffering ON\n");
		BUFFERING = True;
		UPDATEOFTEN = False;
		XtVaSetValues(buttonFB0, XmNsensitive, True, NULL);
		XtVaSetValues(buttonFB1, XmNsensitive, True, NULL);
		XtVaSetValues(buttonFB2, XmNsensitive, False, NULL);
		if (!XtIsManaged(buttonMU)) XtManageChild(buttonMU);
	}
}

/*
 ==============================================================================
*/

void newxedriver(int *intarr, float *rbuf, char *chr) {

    /* get parameters */
    int ifunc = intarr[0];

    char WinName[100];
    char IconName[100];
    int NewWidth;
    int NewHeight;

    /* dummy command line parameters */
    int argc = 0;
    char *argv = NULL;

    /* if this is the first time then there are lots
       of things that we need to do		    */
    if (InitTime) {

	InitTime= False;
	setbuf(stdout,NULL);

	/* set the exit handler */
#ifndef __VMS_POSIX
	genie_signal(SIGINT, DestroyMyApplication);
	genie_signal(SIGTERM, DestroyMyApplication);
#endif

	/* if it is not a child then create an application context */
	if (childwindow == 0) {
		if (DEBUG) printf("Creating Application Context\n");
		TopWindow = XtAppInitialize(&ApplicationContext,
			    "PGPLOTDEFS", NULL,0, &argc, NULL, NULL, NULL, 0);
		display = XtDisplay(TopWindow);
		screen = XDefaultScreen(display);
	}
	else {
		display = XtDisplay(ParentWindow);
		screen = XDefaultScreen(display);
	}

	/* get the screen and display */
	visual = DefaultVisual(display,screen);
	Colourmap = DefaultColormap(display,screen);

	if (DEBUG) printf("Display pointer:%d,  Screen:%d\n",display, screen);

	/* get information about the screen */
	depth = XDisplayPlanes(display, screen);
	xpix = XDisplayWidth(display, screen);
	ypix = XDisplayHeight(display, screen);
	xmm = XDisplayWidthMM(display, screen);
	ymm = XDisplayHeightMM(display, screen);
	if (DEBUG) printf("Screen depth: %d\n",depth);
	if (DEBUG) printf("Screen width/height in pixels: %d,%d\n",xpix,ypix);
	if (DEBUG) printf("Screen width/height in mm: %d,%d\n",xmm,ymm);

	if (depth > 1) MONO = False;

	/* calculate screen resolution */
	resolution[0] = 25.4 * (float)xpix / (float)xmm;
	resolution[1] = 25.4 * (float)ypix / (float)ymm;
	DPI = (int)((resolution[0] + resolution[1]) / 2.0);
	if (DEBUG) printf("Screen resolution (DPI): %d\n",DPI);

    } /* END OF FirstTime */

    /* branch on the function */
    switch(ifunc) {

    case 2:  /* return plot size */
		rbuf[0] = 0.0;
		rbuf[1] = (float)width;
		rbuf[2] = 0.0;
		rbuf[3] = (float)height - 42.0;
		rbuf[4] = 0.0;
		rbuf[5] = MAXCOLOUR;
		intarr[1] = 6;
		if (DEBUG) printf("Returning plot size:%f,%f\n",
				    rbuf[1],rbuf[3]);
		break;

    case 3: /* return device resolution */
		rbuf[0] = resolution[0];
		rbuf[1] = resolution[1];
		rbuf[2] = 1.0;
		rbuf[3] = 3;		
		intarr[1] = 4;
		if (DEBUG) printf("Returning resolution:%f,%f\n",
				    rbuf[0],rbuf[1]);
		break;

    case 5: /* return default file name */
		if (userinfo.valid == 0) {
		    sprintf(chr,"%s","PGPLOT");
		    intarr[2] = 6;
		}
		else {
		    if (DEBUG) printf("USER title: %s\n",userinfo.title);
		    sprintf(chr,"%s",userinfo.title);
		    intarr[2] = userinfo.titlelen;
		}
		break;

    case 6: /* return default size */
		rbuf[0] = 0.0;
		rbuf[1] = (float)width;
		rbuf[2] = 0.0;
		rbuf[3] = (float)height;
		intarr[1] = 4;
		break;

    case 9: {/* open the workstation */
		XGCValues GCVals;

		/* I will only allow landscape (mean Eh!) */
		if (usersize.valid == 0) {
		    width = 828 * xpix / 1024;
		    height = (int)(8.5 * (float)width / 11.0);
		    if (DEBUG) printf("Default size chosen: %d,%d\n",
				    width,height);
		    }
		else if (usersize.valid == 2) {
			width = (int)(usersize.widthmm * resolution[0]);
			height = (int)(usersize.heightmm * resolution[1]);
			usersize.valid = 0;
			if (DEBUG) printf("USER size used: %d,%d\n",
				width,height);
		    }
		else {
			width = usersize.widthpix;
			height = usersize.heightpix;
			usersize.valid = 0;
			if (DEBUG) printf("USER size used: %d,%d\n",
			    width,height);
		}

		/* determine positioning if required */
		if (userpos.valid == 0) {
		    xpos = 20;
		    ypos = 20;
		    if (DEBUG) printf("Default position: %d,%d\n",xpos,ypos);
		}
		else {
		    xpos = userpos.xpix;
		    ypos = userpos.ypix;
		    userpos.valid = 0;
		    if (DEBUG) printf("USER position: %d,%d\n",xpos,ypos);
		}

		/* if the window is NOT a child */
		if (childwindow == 0) {
		    if (userinfo.valid == 0) {
				sprintf(IconName,"%s","PGPLOT");
				sprintf(WinName,"%s","PGPLOT");
		    }
		    else {
		    if (DEBUG) printf("USER title: %s\n",userinfo.title);
		    if (DEBUG) printf("USER icon name: %s\n",userinfo.iconname);
				sprintf(IconName,"%s",userinfo.iconname);
				sprintf(WinName,"%s",userinfo.title);
		   }

		    XtVaSetValues(TopWindow, XmNx, xpos, XmNy, ypos,
				    XmNtitle, WinName,
				    XmNiconName, IconName,
				    XmNallowShellResize,True,
				    XmNdeleteResponse, XmDO_NOTHING,
				    NULL);
		}

		/* allocate foreground and background */
		AllocateColour(0);
		AllocateColour(1);
		
		/* create a parent window if required */
		if (childwindow == 0 && FirstTime) {
			int i;

			if (DEBUG) printf("Creating Manager\n");
			ParentWindow = XtVaCreateManagedWidget("Manager",
					    xmMainWindowWidgetClass,
					    TopWindow, XmNborderWidth,0,
					    NULL);

			/* also create some menu items 
			**** MENU BAR */
			MenuBar = XmCreateMenuBar(ParentWindow,"menu bar",
					NULL,0);
			if (!userinfo.nomenus) XtManageChild(MenuBar);

			/* *** MENU ITEMS */
			filem = XtVaCreateManagedWidget("File",
					    xmCascadeButtonWidgetClass,
					    MenuBar,XmNmnemonic,'F',NULL);

			buttonMU= XtVaCreateManagedWidget("Flush Buffer",
				xmCascadeButtonWidgetClass,MenuBar,
				XmNborderWidth,1,XmNsensitive, False, 
				XmNmnemonic,'B',NULL);
			XtVaSetValues(MenuBar,XmNmenuHelpWidget,buttonMU,NULL);
			XtAddCallback(buttonMU,XmNactivateCallback,
				updatepicture,NULL);


			/* create a pulldown menu */
			pulldown1 = XmCreatePulldownMenu(MenuBar,"files menu",
				    NULL,0);
			/* attach it to the file button */
			XtVaSetValues(filem,XmNsubMenuId,pulldown1,NULL);

			buttonFD= XtVaCreateManagedWidget("Diagnostics  ON",
				xmPushButtonWidgetClass,pulldown1,
				XmNmnemonic,'D',NULL);
			/* adda quit callback to that button */
			XtAddCallback(buttonFD,XmNactivateCallback,
				Diagnostic,NULL);

			/* create a BUFFERING SUB MENU */
			buttonFB= XtVaCreateManagedWidget("Buffering",
				xmCascadeButtonWidgetClass,pulldown1,
				XmNmnemonic,'B',NULL);
			pulldown3 = XmCreatePulldownMenu(buttonFB,"files menu",
				    NULL,0);

			/* attach it to the buffer button */
			XtVaSetValues(buttonFB,XmNsubMenuId,pulldown3,NULL);

			/* create the sub menu items */
			buttonFB0 =  XtVaCreateManagedWidget("No Buffering",
				xmPushButtonWidgetClass,pulldown3,
				XmNmnemonic,'N',NULL);
			buttonFB1 =XtVaCreateManagedWidget("Partial Buffering",
				xmPushButtonWidgetClass,pulldown3,
				XmNmnemonic,'P',NULL);
			buttonFB2 =  XtVaCreateManagedWidget("Full Buffering",
				xmPushButtonWidgetClass,pulldown3,
				XmNsensitive, False, XmNmnemonic,'u',NULL);
			XtAddCallback(buttonFB0,XmNactivateCallback,
				setbuffering,buttonFB0);
			XtAddCallback(buttonFB1,XmNactivateCallback,
				setbuffering,buttonFB1);
			XtAddCallback(buttonFB2,XmNactivateCallback,
				setbuffering,buttonFB2);


			/* add BW option */
			if (!MONO) {
			/* create a menu item in the pulldown1 menu */
			buttonFM= XtVaCreateManagedWidget("Mono display  ",
				xmPushButtonWidgetClass,pulldown1,
				XmNmnemonic,'M',NULL);
			/* adda callback to that button */
			XtAddCallback(buttonFM,XmNactivateCallback,
				ToggleMode,NULL);
			}

			/* create a quit menu item in the pulldown1 menu */
			buttonFQ= XtVaCreateManagedWidget("Close window",
				xmPushButtonWidgetClass,pulldown1,
				XmNmnemonic,'C',NULL);
			/* adda quit callback to that button */
			XtAddCallback(buttonFQ,XmNactivateCallback,
				SelfDestruct,NULL);

			/* add colour */
			if (!MONO) {
			Colourem =  XtVaCreateManagedWidget("Background_Colour",
					    xmCascadeButtonWidgetClass,
					    MenuBar,XmNmnemonic,'a',NULL);

			/* create a pulldown menu */
			pulldown2 = XmCreatePulldownMenu(MenuBar,"Colour menu",
				    NULL,0);
			/* attach it to the colour button */
			XtVaSetValues(Colourem,XmNsubMenuId,pulldown2,NULL);

			/* add all the colours, with callbacks */
			for (i=0; i<16; i++) {
			 /* add a colour option */
			 AllocateColour(i);
			 if (AllocColour[i] != 0) {
			   MAXALLOCCOLOUR = i;
			   COLBUT[i] = XtVaCreateManagedWidget(labels[i],
				    xmPushButtonWidgetClass,pulldown2,
				    XmNbackground,Colours[i].pixel,NULL);

			   XtAddCallback(COLBUT[i],XmNactivateCallback,
				ChangeColour,COLBUT[i]);
			 }
			}
			/* set dark colours' foreground */
			if (COLBUT[1] != 0) XtVaSetValues(COLBUT[1],XmNforeground,
				    Colours[0].pixel, NULL);
			if (COLBUT[4] != 0) XtVaSetValues(COLBUT[4],XmNforeground,
				    Colours[0].pixel, NULL);
			if (COLBUT[12] != 0) XtVaSetValues(COLBUT[12],XmNforeground,
				    Colours[0].pixel, NULL);
			if (COLBUT[14] != 0) XtVaSetValues(COLBUT[14],XmNforeground,
				    Colours[0].pixel, NULL);

			/* set the sensitivity */
			if (COLBUT[0] != 0) XtSetSensitive(COLBUT[0],False);
			} /* end of colours */

			if (menufunct != NULL) { /* add user defined menu */

			Userbutton= XtVaCreateManagedWidget(usermenu.title,
				xmCascadeButtonWidgetClass,MenuBar,NULL);

			/* NO menu, just a single button */
			if (usermenu.nitems == 0) {
			    XtVaSetValues(Userbutton,XmNborderWidth,1,NULL);
			    XtAddCallback(Userbutton,XmNactivateCallback,
				UserMenuHit,Userbutton);
			}

			/* menu of up to 20 items */
			else {
			/* create a pulldown menu */
			pulldown4 = XmCreatePulldownMenu(MenuBar,"User menu",
				    NULL,0);

			/* attach it to the colour button */
			XtVaSetValues(Userbutton,XmNsubMenuId,pulldown4,NULL);

			/* add the menu items */
			for (i=0; i<usermenu.nitems; i++) {
			   Usersubmenu[i] = XtVaCreateManagedWidget(
				    usermenu.items[i],
				    xmPushButtonWidgetClass,pulldown4,NULL);
			    XtAddCallback(Usersubmenu[i],XmNactivateCallback,
				UserMenuHit,Usersubmenu[i]);
			    }
			} /* end of sub menu items */
			} /* end of user menus */
		}

		/* there MUST be a parent window */
		if (DEBUG) printf("Parent window ID: %d\n",ParentWindow);
		if (ParentWindow == NULL) DestroyMyApplication(1);

		/* create a drawing area */
		if (FirstTime) {
		    DrawingArea = XtVaCreateManagedWidget("DrawingArea",
					    xmDrawingAreaWidgetClass,
					    ParentWindow, XmNborderWidth,1,
					    XmNx, xpos, XmNy, ypos,
					    XmNwidth, width,
					    XmNheight, height,
					    XmNbackground,Colours[0].pixel,
					    NULL);
		    if (DEBUG) printf("Created Drawing area, ID: %d\n",
						    DrawingArea);

		    /* add the Expose and Redraw event callbacks */
		    XtAddCallback(DrawingArea, XmNexposeCallback, 
				RedrawWindow, NULL);
		    XtAddCallback(DrawingArea, XmNresizeCallback, 
				ResizeWindow, NULL);
 
		    /* manage the windows as required */
		    if (childwindow == 0) {

			/* set the manager window in action */
			if (userinfo.nomenus)
			    XmMainWindowSetAreas(ParentWindow, NULL, NULL,
					    NULL, NULL, DrawingArea);
			else
			    XmMainWindowSetAreas(ParentWindow, MenuBar, NULL,
					    NULL, NULL, DrawingArea);

		    }
		}

		/* set the size and position (even if NOT first time) */
		if (childwindow == 0) {
			XtVaSetValues(TopWindow,XmNx, xpos, XmNy, ypos,
					    XmNwidth, width,
					    XmNheight, height,
					    NULL);
		}

		/* realize the widget */
		if (childwindow == 0)
			XtRealizeWidget(TopWindow);

		/* create the pixmap */
		PixWidth = width;
		PixHeight = height;
		pixmap = XCreatePixmap(display, XtWindow(DrawingArea),
				    PixWidth, PixHeight, depth);
		if (DEBUG) printf("Creating pixmap, ID: %d\n",pixmap);
	
		if (FirstTime) {
		    /* create some graphics contexts */
		    if (DEBUG) printf("Creating GC\n");
		    gca = XCreateGC(display,XtWindow(TopWindow),None,NULL);
		    gcb = XCreateGC(display,XtWindow(TopWindow),None,NULL); 

		    /* set the foregrounds */
		    if (DEBUG) printf("Setting foregrounds\n");
		    if (AllocColour[1] != 0) {
			    GCVals.foreground = Colours[1].pixel;
			    XChangeGC(display, gca,GCForeground, &GCVals);
		    }
		    if (AllocColour[0] != 0) {
			GCVals.foreground = Colours[0].pixel;
		        XChangeGC(display, gcb,GCForeground, &GCVals);
		    }
		}

		/* clear the screen */
		if (DEBUG) printf("Clearing screen\n");
		XFillRectangle(display, pixmap, gcb, 0,0, width, height);

		/* set up an asynchronous event handler */
		if (childwindow == 0) {
		    if (DEBUG) printf("Calling event handler START routine\n");
		    if (ASYNC) StartEventHandler();
		}

		/* set the return values */
		rbuf[0] = (float)XtWindow(DrawingArea);
		rbuf[1] = 1.0;
		intarr[1] = 2;
		FirstTime = False;
		}
		break;

    case 10: /* close the window */

		if (childwindow == 0) {
			DestroyMyApplication(0);
		}
		else {
			if (XtIsManaged(DrawingArea))
				XtUnmanageChild(DrawingArea);
			XtDestroyWidget(DrawingArea);
		}
		break;

    case 11: { /* begin the picture */
		float temp_var;

		temp_var = rbuf[0];
		NewWidth = (int)temp_var;

		temp_var = rbuf[1];
		NewHeight = (int)temp_var;

		if (DEBUG) printf("Passed size: %d, %d\n",NewWidth,NewHeight);
		if (DEBUG) printf("Present size: %d, %d\n",width,height);

		/* re-size window if required */
		if (width != NewWidth || height != NewHeight) {

			if (DEBUG) printf("Doing resize\n");
			MouseMove = False;
			if (childwindow == 0) {
			    XtVaSetValues(TopWindow, XmNwidth, NewWidth,
					XmNheight, NewHeight, NULL);
			}
			else {
			    XtVaSetValues(DrawingArea, XmNwidth, NewWidth,
					XmNheight, NewHeight, NULL);
			}
			MouseMove = True;

			width = NewWidth;
			height = NewHeight;

		}

		/* resize the pixmap */
		if (width != PixWidth || height != PixHeight)
				ResizePixmap(width, height, True);

		/* clear the screen */
		if (DEBUG) printf("New picture Window:%d, Pixmap:%d\n",
			    DrawingArea,pixmap);
		XFillRectangle(display, pixmap, gcb, 0,0, width, height);
		XClearWindow(display, XtWindow(DrawingArea));
		XCopyArea(display, pixmap, XtWindow(DrawingArea),
					     gca, 0, 0, width,height, 0,0 );
		}

		break;

    case 12: {/* draw a line */
		int x1 = (int)rbuf[0];
		int x2 = (int)rbuf[2];
	   	int y1 = height - (int)rbuf[1];
		int y2 = height - (int)rbuf[3];

		/* draw to pixmap */
		XDrawLine(display, pixmap, gca, x1, y1, x2, y2);

		/* update redraw area */
		SetRedrawSize(x1, y1, x2, y2);

		if (!BUFFERING) {
			updatepicture(NULL, NULL, NULL);
		}
		else {
			if(--BUFFERCOUNT == 0) 
			    if (UPDATEOFTEN)
					updatepicture(NULL,NULL,NULL);
		}
		}
		break;

    case 13: {/* draw a dot */
		int x1 = (int)rbuf[0];
	   	int y1 = height - (int)rbuf[1];

		/* draw to pixmap */
		XDrawPoint(display, pixmap, gca, x1, y1);

		/* update redraw area */
		SetRedrawSize(x1, y1, x1, y1);

		if (!BUFFERING) {
			updatepicture(NULL, NULL, NULL);
		}
		else {
			if(--BUFFERCOUNT == 0) 
			    if (UPDATEOFTEN)
					updatepicture(NULL,NULL,NULL);
		}
		}
		break;

    case 14: /* end the picture */
	    if (DEBUG) printf("Ending picture\n");
		break;

    case 15: { /* allocate a colour */
		int index = (int) rbuf[0];
		XGCValues GCVals;

		AllocateColour(index);

		/* set the GC colour index */
		if (AllocColour[index] != 0 && !MONO) {
		    GCVals.foreground = Colours[index].pixel;
		    XChangeGC(display, gca,GCForeground, &GCVals);
		}
		else {
		    GCVals.foreground = Colours[1].pixel;
		    XChangeGC(display, gca,GCForeground, &GCVals);
		}

		if (DEBUG) printf("New colour index: %d\n",index);

		}
		break;

    case 16:

		updatepicture(NULL, NULL, NULL);
		break;

    case 17: { /* **** CURSOR **** */
		int x1, y1;
		XEvent event;
		Cursor cursor;

		/* stop the asnchronous event handler */
		if (ASYNC) StopEventHandler();

		/* get the input coordinates */
		x1 = (int)rbuf[0];
	   	y1 = height - (int)rbuf[1];

		/* create a cursor */
		cursor = XCreateFontCursor(display, XC_crosshair);

		/* set this for the correct window */
		XDefineCursor(display, XtWindow(DrawingArea), cursor);

		/* move the pointer to there */
		XWarpPointer(display, None, XtWindow(DrawingArea), 0,0,0,0,
				x1,y1);

		/* add cursor callback */
		XtAddCallback(DrawingArea, XmNinputCallback, 
				SetCursorValues, NULL);

		/* start a synchronous event handler */
		while (event.type != ButtonRelease &&
			    event.type != KeyRelease) {
		    XtAppNextEvent(ApplicationContext, &event);
		    XtDispatchEvent(&event);
		}

		/* set the return values */
		sprintf(chr,"%s",CURSOR_B_HIT);
		rbuf[0] = CURSOR_X_HIT;
		rbuf[1] = height - CURSOR_Y_HIT;
		intarr[1] = 2;
		intarr[2] = 1;

		/* remove the callback */
		XtRemoveCallback(DrawingArea, XmNinputCallback, 
				SetCursorValues, NULL);

		/* undefine the cursor */
		XUndefineCursor(display, XtWindow(DrawingArea));

		/* free the cursor */
		XFreeCursor(display, cursor);

		/* start the asynchronous event handler */
		if (ASYNC) StartEventHandler();

		}
		break;

    case 20: { /* POLYGON FILL */
		static int Npoints;
		static int remcal;
		static XPoint *corners;
		int x1,y1;

		/* reserve some array space */
		if (remcal == 0) {
				Npoints = (int)rbuf[0];
				corners =(XPoint *)malloc(Npoints * 
						sizeof(XPoint));
				remcal = Npoints;		
		}
		else {
				if (corners == NULL) break;
		
				/* reduce the counter */
				remcal--;

				/* set the array values */
				x1 = (int)rbuf[0];
			   	y1 = height - (int)rbuf[1];
				corners[remcal].x = x1;
				corners[remcal].y = y1;

				/* update redraw area */
				SetRedrawSize(x1, y1, x1, y1);

				/* if it is the last time - plot polygom */
				if (remcal == 0) {
				    XFillPolygon(display, pixmap, gca, corners,
					Npoints, Complex,CoordModeOrigin);

				    free(corners);
				    Npoints = 0;
				    remcal = 0;
				}	    
		}

		if (!BUFFERING) {
			updatepicture(NULL, NULL, NULL);
		}
		else {
			if (--BUFFERCOUNT == 0) 
			    if (UPDATEOFTEN)
					updatepicture(NULL,NULL,NULL);
		}
		}
		break;

    case 21: { /* SET A NEW COLOUR REPRESENTATION */

		int index = (int)rbuf[0];
		short red   = (short)(rbuf[1] * 65535.0);
		short green = (short)(rbuf[2] * 65535.0);
		short blue  = (short)(rbuf[3] * 65535.0);

		/* set the colour table values */
		RGB[index][0] = red;
		RGB[index][1] = green;
		RGB[index][2] = blue;
				
		if (DEBUG) printf("Setting colour index %d to %d,%d,%d\n",
			    index,RGB[index][0],RGB[index][1],RGB[index][2]);
		}

    case 24: { /* RECTANGLE FILL */
		int x1 = (int)rbuf[0];
		int x2 = (int)rbuf[2];
	   	int y1 = height - (int)rbuf[1];
		int y2 = height - (int)rbuf[3];
		int temp;

		/* check the dimensions for negative size */
		if (x2 <= x1) {
		    temp = x1;
		    x1 = x2;
		    x2 = temp;
		}

		if (y2 <= y1) {
		    temp = y1;
		    y1 = y2;
		    y2 = temp;
		}

		/* draw to pixmap */
		XFillRectangle(display, pixmap, gca, x1, y1, x2-x1, y2-y1);

		/* update redraw area */
		SetRedrawSize(x1, y1, x2, y2);

		if (!BUFFERING) {
			updatepicture(NULL, NULL, NULL);
		}
		else {
			if(--BUFFERCOUNT == 0) 
			    if (UPDATEOFTEN)
					updatepicture(NULL,NULL,NULL);
		}
		}
		break;

    case 26: {/* draw a line of pixels */
		int x = (int)rbuf[0];
		int y = (int)rbuf[1];
		char image[2000];
		int i;
		XImage *ximage;
		int nbuf = intarr[1];		

		/* copy the pixel colour index array */
		for(i=2; i<nbuf; i++) {
		    image[i-2] = intarr[i];
		}
		
		/* construct an image */
		ximage = XCreateImage(display, visual, depth, XYPixmap,
			    0, image, nbuf-2, 1, 8, 0);

		/* display the image */
		XPutImage(display, pixmap, gca, ximage, 0, 0, x, y,nbuf-2,1);

		/* free the image */
		XDestroyImage(ximage);

		/* update redraw area */
		SetRedrawSize(x, y, x + nbuf - 2, y + 1);

		if (!BUFFERING) {
			updatepicture(NULL, NULL, NULL);
		}
		else {
			if(--BUFFERCOUNT == 0) 
			    if (UPDATEOFTEN)
					updatepicture(NULL,NULL,NULL);
		}

		}
		break;

    } /* END OF SWITCH */

    return;
    }

#else

/* Dummy routine for non-motif machines e.g. LINUX */
void newxedriver(int *intarr, float *rbuf, char *chr) 
{
    printf("Error - newxedriver called\n");
    return;
}

#endif /* HAVE_LIBXM */
