/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *      Module:         named_unit
 *      Author:         Freddie Akeroyd, ISIS, 1/3/96
 *      Purpose:        
 */

#include "geniedefs.h"
#include <iostream.h>
#include <stdio.h>
#include <gnustring.h>
#ifdef _WIN32
#	include <io.h>
#	include "stream_nt.h"
#else
#	include <stream.h>
#endif /* _WIN32 */
#include "genievars.h"
#include "named_unit.h"

static const char* rcsid = "@(#)$Id: named_unit.cxx,v 1.5 1997/10/20 16:43:47 cmm Exp $";
USE_RCSID(rcsid);

bool SIDimensionalExponents::operator==(const SIDimensionalExponents& d) const
{
    return (length == d.length) && (mass == d.mass) && (time == d.time) && 
	   (temperature == d.temperature) && (current == d.current) && 
	   (amount == d.amount) && (intensity == d.intensity);
}

SIDimensionalExponents::operator GenieWorkspace() const
{
    GenieWorkspace gw(7);
    gw("length", length);
    gw("mass", mass);
    gw("time", time);
    gw("current", current);
    gw("temperature", temperature);
    gw("amount_of_substance", amount);
    gw("luminous_intensity", intensity);
    return gw;
}

void SIDimensionalExponents::operator+=(const SIDimensionalExponents& d)
{
    length += d.length;
    mass += d.mass;
    time += d.time;
    current += d.current;
    temperature += d.temperature;
    amount += d.amount;
    intensity += d.intensity;
    return;
}

// Divide two SI units
SIDimensionalExponents operator-(const SIDimensionalExponents& d1, const SIDimensionalExponents& d2)
{
    return SIDimensionalExponents(d1.length - d2.length, d1.mass - d2.mass, d1.time - d2.time, d1.current - d2.current, 
				  d1.temperature - d2.temperature, d1.amount - d2.amount, d1.intensity - d2.intensity);
}

// Multiply two SI units
SIDimensionalExponents operator+(const SIDimensionalExponents& d1, const SIDimensionalExponents& d2)
{
    return SIDimensionalExponents(d1.length + d2.length, d1.mass + d2.mass, d1.time + d2.time, d1.current + d2.current, 
				  d1.temperature + d2.temperature, d1.amount + d2.amount, d1.intensity + d2.intensity);
}

// Raise dimensions to some exponent
SIDimensionalExponents operator*(const SIDimensionalExponents& d, double f)
{
    return SIDimensionalExponents(d.length * f, d.mass * f, d.time * f, d.current * f, d.temperature * f,
				  d.amount * f, d.intensity * f);
}

String SIDimensionalExponents::asString() const
{
    static char buffer[32];
    String res, comp;
    if (length != 0.0) { sprintf(buffer, "%g ", length); cat("L^", buffer, comp); res += comp; }
    if (mass != 0.0) { sprintf(buffer, "%g ", mass); cat("M^", buffer, comp); res += comp; }
    if (time != 0.0) { sprintf(buffer, "%g ", time); cat("T^", buffer, comp); res += comp; }
    if (current != 0.0) { sprintf(buffer, "%g ", current); cat("A^", buffer, comp); res += comp; }
    if (temperature != 0.0) { sprintf(buffer, "%g ", temperature); cat("Th^", buffer, comp); res += comp; }
    if (amount != 0.0) { sprintf(buffer, "%g ", amount); cat("N^", buffer, comp); res += comp; }
    if (intensity != 0.0) { sprintf(buffer, "%g ", intensity); cat("C^", buffer, comp); res += comp; }
    return res;
}

ostream& SIDimensionalExponents::print(ostream& os) const
{
    return os << asString() << endl;
}
