/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef NEXUS_INTERMEDIATE_FILE
#define NEXUS_INTERMEDIATE_FILE

#include "napi.h"
#include "intermediate_file.h"

class NeXusIntermediateFile : public IntermediateFile
{
    private:
	String m_filename;
	NXaccess m_access_mode;
	bool m_sds_open;		// An SDS currently open?

    protected:
	static int countEntries(NXhandle fileid);
	int readAttributes(GenieWorkspace& gw, NXhandle fileid);
	int addGenieVariable(GenieVariable& gv, const char* name, NXhandle fileid);
	int closeFile(NXhandle* pFileid);
	int NeXusToGenieVariable(GenieWorkspace& gv, const char* name, 
			void* data, int* dims_array, int ndims, 
			int type, bool one_element_arrays);
	static int typeSize(int type);
	int readSDS(GenieWorkspace& gw, const char* name, NXhandle fileid);
	int readEntries(GenieWorkspace& gw, NXhandle fileid);
	static int cleanFilename(char* name);
	static int printHDFTag(ostream& os, int tag);

    public:
	int open(const char* filename, ModeFlags mode = Append);
	int close();
 	int numberOfBlocks() const;
	NeXusIntermediateFile();
	~NeXusIntermediateFile();
	ostream& print(ostream& os) const;
	int appendBlock(GenieVariable& gv, const char* label = "no_label", const char* comment = "<no comment>");
	int appendBlock(GenieVariableBase& gvb, const char* label = "no_label", const char* comment = "<no comment>");
	int seek(const char* label);
	int seek(int block = -1);
        int readBlock(char*& label, GenieVariable& gv);
        int readBlock(GenieVariable& gv);
        int readLabel(char*& label);
	int closeOpenSDS(NXhandle fileid);

// dummy
        int skipInteger() { return 1; }
        int readInteger(g3io_int_t& i) { return 1; }
        int writeInteger(g3io_int_t i) { return 1; }
    	int skipString() { return 1; }
    	int freeString(char*& s) { return 1; }
    	int readString(char*& s) { return 1; }
    	int writeString(const char* s) { return 1; }
    	int skipReal() { return 1; }
    	int readReal(g3io_real_t& r) { return 1; }
    	int writeReal(g3io_real_t r) { return 1; }
        int readRealArray(GenieRealArray& gra) { return 1; }
        int writeRealArray(GenieRealArray& gra) { return 1; }
        int skipRealArray() { return 1; }
   	int skipIntegerArray() { return 1; }
    	int readIntegerArray(GenieIntegerArray& ia) { return 1; }
    	int writeIntegerArray(GenieIntegerArray& ia) { return 1; }
    	int readStringArray(GenieStringArray& gsa) { return 1; }
    	int writeStringArray(GenieStringArray& gsa) { return 1; }
    	int skipStringArray() { return 1; }
    	int readItemSubclassType(char*& subclass_name) { return 1; }
    	int writeItemSubclassType(const char* subclass_name) { return 1; }
    	int skipItemSubclassType() { return 1; }
    	GenieVariableType readItemType() { return GenieTypeError; }
    	int skipItemType() { return 1; }
    	int writeItemType(GenieVariableType gvt) { return 1; }
    	int skipOffset() { return 1; }
    	int writeOffset(u_int saved_offset) { return 1; }    // write the current offset to a previosly allocated offset field
    	int saveOffset(u_int& saved_offset) { return 1; }    // allocate space for an offset field and make a not of its position
    	int readOffset(u_int& offset) { return 1; }
    	u_int setFileOffset(u_int offset) { return 1; }
    	int readArrayDims(GenieArrayBase& gab) { return 1; }
    	int writeArrayDims(const GenieArrayBase& gab) { return 1; }
};

#endif /* NEXUS_INTERMEDIATE_FILE */
