/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef PARSEDEFS__
#define PARSEDEFS__

#ifdef __cplusplus
extern "C" {
#endif


/* Common parser maximums */
#define MAXSTRLEN SHORTCHARBUFF*8   /* Maximum length of any concatenated program string */
#define MAXVARLEN SHORTCHARBUFF	    /* Maximum length of a single variable or identifier string */
#define MAXNEST 100		    /* Limits the depth of textual nesting (eg loops within loops) */
#define LEXTOKENBUFFERSIZ 512	    /* Buffer for LEX tokens */


/*
 * parse tree structures, built by yacc
 */
typedef struct snode_struct *snode;

typedef struct node_struct
{
    long valid;
    snode next;
} node;

struct snode_struct
{
    node    *side;  /* Link to next node in chain */
    char    *leaf;
};

static const node NULL_NODE = {0, NULL};
#define Valid(node)  ((node)?(node)->valid==1:0)

/*
 * name mangling to work with GENIE lex/yacc and avoid clashes with similar
 * functions in smalltalk yacc
 */

#define yyparse ggparse     
#define yylex gglex
#define yyerror ggerror		/* Avoid clash with POSIX lex declaration of yyerror! */
#define yylval gglval
#define yychar ggchar
#define yydebug ggdebug
#define yynerrs ggnerrs

/**REMOVE /* Redefine yygetc or yyin as necessary to control the input */

/**REMOVE #ifdef yygetc
/**REMOVE #undef yygetc
/**REMOVE #define yygetc() getc(InChan)
/**REMOVE #else
/**REMOVE #define yyin stdin*/
#define yyout stdout
/**REMOVE #endif


/* ********************************************
 * START - Prototypes for parser functions made available to C++, the
 * bodies for these are defined in the C definitions section of yacc source
 * file (yacccommands.y)
 */


/* exports to allow Initialization,running & diagnostics of parser
 */
int   yyparse(void);                                    /* THE parser */
void  initialize_parser(int bReadingFile, int ydebflag);/* set up before calling yyparse() */    
void  initialize_parser_stats(void);
node  *interrogate_parser(int *errcnt, int *linecnt);   /* Get parse status afterwards */    
/* diagnostic */
void  fill_buffers(void);                               /* Set up parser buffer overflow checking */    
int   check_buffers(void);                              /* See if overflows occurred anywhere */    


/* Exported parse tree handling function prototypes, see note on parse tree above
 */
void  printNode(char * buffer, node *node);             /* Diagnostic, print out a node */
void  deleteNode(node *node);                           /* destroy a node (recursive) */
int   nlength(node *node);                              /* Return length as if tree was a string */
void  p_stats(void);                                    /* Diagnostic, gives count of nodes used */


/* Imported from the C++ which supplies all input to the parser now
 */
int  getc_parser();     /* A look alike for getc but which gets input supplied by the parser */


/*
 * Global used to control diagnostic instrumented parser output
 */
extern FILE* g_fpTypeFile;

/* END - Prototypes for parser functions/globals made available to C++
 * ********************************************/


/**********************************************
 * START - variables & functions shared by lexcommands.c & yacccommands.c only
 *  (this is in lieu of a private header file shared by lex & yacc)
 */
#ifdef INSIDE_PARSER__

    /* tree bits */
    void  sweep_nodes(void);                                /* garbage collect any left over nodes */
    void  nprintf(node **result, const char* fmt, ...);     /* prints strings & nodes into a tree node */
    node  *newLeaf(const char *value);                      /* Allocate new leaf node */

    void yyerror(const char err[]);         /* yacc error reporting */
    int  yylex(void);                       /* Calls the lexical analyser */


    /* Parser record keeping */
    extern int errors_found;		/* Up to date count of errors found */
    extern int lines_parsed;		/* Count of lines successfully parsed (only incremented at the end of a line) */
    extern int continuation_line;      /* Set when we are in a continuation line */
    extern int auto_declare;           /* When this is set, declare an undeclared variable on assignment */
    extern int g_bUsingFile;		/* Tells whether the parser is in batch or interactive mode */

#endif /* INSIDE_PARSER__ */

/* END - variables shared by lex & yacc only
 **********************************************/

#ifdef __cplusplus
}
#endif
#endif /* PARSEDEFS__ */
