/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef PARSER__
#define PARSER__

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

/////////////////////////////////////////////////////////////////////////////
// parser.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CParser class
//

class CParser
{
// Construction
public:
    CParser(
        CSmalltalk& smalltalk,          // Somwhere for the parser to feed its generated output
        const char* szDebugFile,        // Default parameter set to debug filename if required
        const char* szInstrumentFile,   // File for output of parser instrumentation if required
        bool bErrorContinue = FALSE, bool bYaccDebugging = FALSE);
// Destruction
    ~CParser();

// Attributes
public:

// Operations
public:
        int CompileFile(const char* szFilename);
        int CompileString(const char* command, bool bInteractive = true);

// Implementation
public:
        // routine called from the lexical analyser (in C) to supply characters to lex
        friend int getc_parser();

private:
	static unsigned s_dwSessionID;

        CSmalltalk* m_pSmalltalk;       // Smalltalk interpreter to send output to (keep member
                                        // variable as pointer, just incase we needed to restart
                                        // the interpreter with delete & new)
	String m_gstrDebugFile;
	String m_gstrInstrumentFile;
	bool m_bErrorContinue;
	bool m_bYaccDebugging;           // State variables for diagnostic modes

        char* m_pszCommand;             // Command string to compile OR
	FILE * m_fpInputFile;           // pointer to file to compile.
        FILE*  m_fpDebugOut;            // Diagnostic file
        
        int  RunParser( bool bInteractive, bool bPrintDiagnostics, FILE* fpInput);
        void PrintBufferDiagnostics(int lines);
	int  SendToSmalltalk(node *message);
};

#endif /* PARSER__ */
