/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef PG_COORD_TRANS_FUNC_H
#define PG_COORD_TRANS_FUNC_H

#include "drawable_values.h"
#include "two_d_coord_trans_func.h"

class PGCoordinateTransformFunction;

// Must be same as coord_tran_func_t
typedef int (PGCoordinateTransformFunction::*member_ptr_t)(int* flags, const g_real* val_in, g_real* val_out);

class PGCoordinateTransformFunction : public TwoDimensionalCoordinateTransformFunction
{
    protected:
	static PGCoordinateTransformFunction* class_ptr;
	static member_ptr_t member_ptr;
	static g_real* cMatrix(g_real* matrix);

    public:
	PGCoordinateTransformFunction(coord_tran_func_t forward_func, coord_tran_func_t reverse_func = 0);
	PGCoordinateTransformFunction(g_real* matrix);	// coefficient matrix
	PGCoordinateTransformFunction(g_real* wx, int* wx_dims, int wx_ndims, CoordinateTransformMode x_mode,
						g_real* wy, int* wy_dims, int wy_ndims, CoordinateTransformMode y_mode);
	void setPointers(bool forward = 1);

        int forwardFunction(int* flags, const g_real* val_in, g_real* val_out);

        int reverseFunction(int* flags, const g_real* val_in, g_real* val_out);

	static void contourFunction(pg_int& visible, pg_float& x, pg_float& y, pg_float& z);

	static void contourLabelFunction(pg_int* flags, pg_float& x_in, pg_float& y_in, pg_float& x_out, pg_float& y_out);

	static void drawIt(pg_int& visible, pg_float& x, pg_float& y);
};

#endif /* PG_COORD_TRANS_FUNC_H */
