/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include <stdio.h>
#include <ctype.h>

#ifdef HAVE_LIBX11

#include <X11/Xlib.h>

/*
 * Module:	kill_pgplot_server
 * Author:	Freddie Akeroyd, ISIS
 * Purpose:	Terminate a "pgxwin_server" process attached to the current display
 */

static const char* rcsid = "@(#)$Id: pgkill.cxx,v 1.8 1997/10/28 17:04:23 cmm Exp $";
USE_RCSID(rcsid);


typedef int (*xlib_error_handler_t)(Display*, XErrorEvent*);
 
/*
 * Handle XLib errors 
 */
static int xlib_error_handler(Display *display, XErrorEvent *event)
{
    char errtxt[81];
    XGetErrorText(display, (int)event->error_code, errtxt, (int)sizeof(errtxt));
    *g_err << "PGKILL: " << errtxt << endl;
/*
 * Report the operation that caused it. These opcode numbers are listed in
 * <X11/Xproto.h>.
 */
    *g_err << "PGKILL: Major opcode = " << (int)event->request_code 
         << ", Resource ID = 0x" << (unsigned long)event->resourceid << "x"
         << (event->resourceid==DefaultRootWindow(display)?" (Root window)":"")
         << endl;
  return 0;
}

// Returns 0 on success, 1 on error
int kill_pgplot_server()
{
    int reply;
    Display* display = XOpenDisplay(0);
    if (display == 0)
    {
	return 0;		// cannot open display:  we can't be running under X, so no server to kill
    }
    Atom pgxwin_server = XInternAtom(display, "pgxwin_server", True);
    if (pgxwin_server == None)
    {
    	XCloseDisplay(display);
	return 0;		// No Atom, so no "pgxwin_server" process was started
    }
    Window server = XGetSelectionOwner(display, pgxwin_server);
    if (server == None)
    {
    	XCloseDisplay(display);
	return 0;		// No selection ownership, so server probably dead already
    }
// Two ways to kill the server - both work, but i'm not sure which is best
    gout << "Remove PGPLOT X windows server ? (y/n) " << flush;
    reply = gin.get();
    xlib_error_handler_t old_handler;
    if ( toupper(reply) == 'Y' )
    {
// In case the close down causes X errors, install our own error handler
// First make sure our handler is only called for our events though!
	XSync(display, False);
	old_handler = XSetErrorHandler(xlib_error_handler);
/*
 * The server should exit if the selection ownership of the
 * server atom changes and it has no clients connected
 */
	XSetSelectionOwner(display, pgxwin_server, None, CurrentTime);
	XSync(display, False);
/*
 * Alternatively a simple XKillClient() should do the trick,
 * but is a little more messy
 */
//	XKillClient(display, server);
//	XSync(display, False);

/*
 * Restore previous error handling
 */
	XSetErrorHandler(old_handler);
    }
    XCloseDisplay(display);
    return 0;
}

# else
int kill_pgplot_server() { gdbg << "Unimplemented call to kill_pgplot_server"; return 0; }
# endif /* HAVE_LIBX11 */
