/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
//+++
//
// CLASS: Polygon
//
//
//---
// ----
// define String data type
//
#include <gnustring.h>
#include "Drawable_objects.h"
#include "Polygon.h"
#include "Pginclude.h"
#include "Errors.h"
#include "SinglePrecision.h"

static const char* rcsid = "@(#)$Id: polygon.cxx,v 1.8 1997/11/04 16:50:01 cmm Exp $";
USE_RCSID(rcsid);

gPolygon::gPolygon(int sides,
		 double* xdata,
		 double* ydata,
		 int solid,
		 int colour,
		 int thickness) {

//	    int c;

	    // *** set visibility
	    invisible = 0;

	    // *** check for too few sides
	    if (sides <= 2) {
		Errors syserr;
		syserr.add_error ("Polygon ",
		    "polygon has too few sides",
		    "increase number_of sides to greater than 2");
		return;
		}

	    // *** new some space for arrays
	    X_ordinates = new float[sides];
	    Y_ordinates = new float[sides];

	    if (Y_ordinates == 0) {
		Errors syserr;
		syserr.add_error ("Polygon ",
		    "insufficient virtual memory for ordinate arrays",
		    "call procedure DELETE_PICTURES");
		return;
		}

	    // *** get number of sides
	    number_of_sides = sides;

	    // *** get line thickness
	    if (thickness == 0)
		line_thickness = LINE_THICKNESS;
	    else
		line_thickness = thickness;

	    // *** fill in ordinates
	    DoubleArrayToFloatArray(X_ordinates, xdata,sides);
	    DoubleArrayToFloatArray(Y_ordinates, ydata,sides);

	    // *** set fill characteristic
	    if (solid == 0)
		filled = 2;
	    else
		filled = 1;

	    // *** get colour
	    if (colour == -1)
		line_colour = LINE_COLOUR;
	    else
		line_colour = colour;

	}

// *** destructor
gPolygon::~gPolygon() {

	    // *** delete the NEWed arrays
	    delete [] X_ordinates;
	    delete [] Y_ordinates;

	    }

void gPolygon::display() {

	    // *** set the line thickness
	    pgslw(line_thickness);

	    // *** set the colour
	    pgsci(line_colour);

	    // *** set the fill pattern
	    pgsfs(filled);

	    // *** set the line type
	    pgsls(1);

	    // *** draw the polygon
	    pgpoly(number_of_sides,X_ordinates, Y_ordinates);

	    }
