/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef SBROUTINES_H
#define SBROUTINES_H

#if defined(__unix) || defined(HAVE_LIBF2C)
#define sbplan sbplan_
#define sbplnt sbplnt_
#define sbfint sbfint_
#define sbfcls sbfcls_
#define sbball sbball_
#define sbelip sbelip_
#define sbline sbline_
#define sbrod sbrod_
#define sbcone sbcone_
#define sbtbal sbtbal_
#define sbsurf sbsurf_
#define sbtsur sbtsur_
#define sbcpln sbcpln_
#define sb2srf sb2srf_
#define sbccrt sbccrt_
#define sbslic sbslic_
#endif 

#if defined(HAVE_LIBF2C)
#	define sbtext sbtext___
#elif defined(__unix)
#	define sbtext sbtext__
#endif

#ifdef _WIN32
#	define sbplan SBPLAN
#	define sbplnt SBPLNT
#	define sbfint SBFINT
#	define sbfcls SBFCLS
#	define sbball SBBALL
#	define sbelip SBELIP
#	define sbline SBLINE
#	define sbrod  SBROD
#	define sbcone SBCONE
#	define sbtbal SBTBAL
#	define sbsurf SBSURF
#	define sbtsur SBTSUR
#	define sbcpln SBCPLN
#	define sb2srf SB2SRF
#	define sbccrt SBCCRT
#	define sbslic SBSLIC
#	define sbtext SBTEXT
#endif

extern "C"
{
    void sbplan(const pg_float* eye, const pg_int& nv, const pg_float* vertices, 
		const pg_int& ic1, const pg_int& ic2, const pg_float* light);
    void sbplnt(const pg_float* eye, const pg_int& nv, const pg_float* vertices, 
		const pg_int& ic1, const pg_int& ic2, const pg_float* light, const 
		pg_int& transparency);
    void sbfint(const pg_float* rgb,const pg_int& ic, const pg_int& ibmode,
        const pg_int& ibuf, pg_int& maxbuf);
    void sbfcls(const pg_int& ibuf);
    void sbtext(const pg_float* eye, const STRING& text, const pg_int& icol, 
        const pg_float* pivot, const pg_float& fjust, const pg_float* orient, 
        const pg_float& size);     
    void sbelip(const pg_float* eye, const pg_float* centre, const pg_float* 
        paxes, const pg_int& ic1, const pg_int& ic2, const pg_float* light, 
        const pg_int& lshine, const pg_int& icline, const pg_float& anglin, 
        pg_float& x0, pg_float& y0, pg_float& r0);
    void sbline(const pg_float* eye, const pg_float* end1, const pg_float* end2, 
        const pg_int& colour, const pg_int& dash);
    void sbrod(const pg_float* eye, const pg_float* end1, const pg_float* end2,
	    const pg_float& radius, const pg_int& ic1, const pg_int& ic2, 
	    const pg_float* light, const pg_int& nsides, const pg_int& lend); 
    void sbcone(const pg_float* eye, const pg_float* base, const pg_float* apex, 
        const pg_float& radius, const pg_int& ic1, const pg_int& ic2, const 
        pg_float* light, const pg_int& nsides);
    void sbball(const pg_float* eye, const pg_float* centre, const pg_float& 
        radius, const pg_int& ic1, const pg_int& ic2, const pg_float* light, 
        const pg_int& lshine, pg_float& x0, pg_float& y0, pg_float& r0);
    void sbtbal(const pg_float* eye, const pg_float* centre, const pg_float& 
        radius, const pg_int& ic1, const pg_int& ic2, const pg_float* light, 
        const pg_int& lshine, pg_float& x0, pg_float& y0, pg_float& r0, const 
        pg_int& transparency);    
    void sbsurf(const pg_float* eye, const pg_float* lattice, const pg_float* 
        density, const pg_int& n1, const pg_int& n2, const pg_int& n3, const 
        pg_float& dsurf, const pg_int& ic1, const pg_int& ic2, const pg_float* 
        light, const pg_int& shine);
    void sbtsur(const pg_float* eye, const pg_float* lattice, const pg_float* 
        density, const pg_int& n1, const pg_int& n2, const pg_int& n3, const 
        pg_float& dsurf, const pg_int& ic1, const pg_int& ic2, const pg_float* 
        light, const pg_int& shine, const pg_int& transparency);
    void sbcpln(const pg_float* eye, const pg_float* lattice, const pg_int& ic1, 
        const pg_int& ic2, const pg_float* light, const pg_float* normal, const
        pg_float* apoint, const pg_int& edge, const pg_int& transparency);
    void sb2srf(const pg_float* eye, const pg_float* lattice, const pg_float* 
        density, const pg_int& n1, const pg_int& n2, const pg_float& dlow, 
        const pg_float& dhigh, const pg_float& dvert, const pg_int& ic1, const 
        pg_int& ic2, const pg_int& ncband, const pg_float* light, const 
        pg_int& shine); 
    void sbccrt(const pg_int& ibuf, const pg_int& ic1, const pg_int& ic2, 
		const pg_int* table);
    void sbslic(const pg_float* eye, const pg_float* lattice, const pg_float*
        density, const pg_int& n1, const pg_int& n2, const pg_int& n3, const
        pg_float& dlow, const pg_float& dhigh, const pg_int& ic1, const pg_int&
        ic2, const pg_float* slnorm, const pg_float* apoint, const pg_int&
        icedge);
}

#endif /* SBROUTINES_H */
