/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef SIUNIT_H
#define SIUNIT_H

#include "named_unit.h"

class SIUnit : public NamedUnit
{
    public:
        enum Code
        {
            NoCode, metre, gram, second, ampere, kelvin, mole, candela, radian, steradian, hertz, newton,
            Pascal, joule, watt, coulomb, volt, farad, ohm, siemens, weber, tesla, henry, degrees_celsius,
            lumen, lux, becquerel, gray, sievert
        }; 

        enum Prefix
	{
	    exa, peta, tera, giga, mega, kilo, hecto, deca, NoPrefix, deci, centi, milli, micro, nano, pico, femto, atto
	};

        static const char* CodeName[];
        static const char* CodeAbbreviatedName[];
        static const char* PrefixName[];
        static const char* PrefixAbbreviatedName[];
	static const int PrefixExponent[];
	static double prefixValue(Prefix prefix) { return pow(10.0, PrefixExponent[prefix]); }

    protected:
	Code m_code;
 	Prefix m_prefix;

    public:
	SIUnit(Code code = NoCode, Prefix prefix = NoPrefix) : m_code(code), m_prefix(prefix) {}
	const char* codeName() const { return codeName(m_code); }
	const char* prefixName() const { return prefixName(m_prefix); }
	double prefixValue() const { return prefixValue(m_prefix); }
	String name() const { return name(m_code, m_prefix); }
	double convertTo(const SIUnit& s) const;

	static const char* codeName(Code code);
	static const char* prefixName(Prefix prefix) { return PrefixName[prefix]; }
 	static SIDimensionalExponents dimensions(Code code);
	static String name(Code code, Prefix prefix = NoPrefix);
	static Code lookupCode(const char* name);
	static Prefix lookupPrefix(const char* name);
 	SIDimensionalExponents dimensions() const { return dimensions(m_code); }
};

#endif /* SIUNIT_H */
