/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"

#include <signal.h>
#include <setjmp.h>
#include <stdlib.h>

#include "smalltalk.h"
#include "gsignals.h"

unsigned CSmalltalk::s_dwSessionID = 0;

// Start the smalltalk interpreter, if rebuild is true, remove the
// image file before starting (forces a rebuild)
CSmalltalk::CSmalltalk(const char szImageFile[], bool bRebuild)
{
    static const char *cp, *exec_argv[5];

    // if the smalltalk becomes re-entrant or thread safe we could look at this again
    if (s_dwSessionID != 0)
    {
	    gerr << "Not allowed to create more than one smalltalk object per process - aborting" << endl;
            exit(1);
    }
    else
    {
	s_dwSessionID++;

	// Construct exec_argv from symbols or default values
        if (bRebuild)
        {
            remove(szImageFile);        // We want to start afresh
        }
	exec_argv[0] = "genie_gst";     // Genie version of GNU Smalltalk
	exec_argv[1] = "-qg";
	exec_argv[2] = "-I";
	exec_argv[3] = szImageFile ;
	exec_argv[4] = NULL;

	// Get the smalltalk up and running - it will automatically load it's kernel if needed.
	smalltalkArgs(4, (char**)exec_argv);
	initSmalltalk();
// Save a copy of the smalltalk Ctrl-C action for when we install out handler
        initializeSmalltalkSignals();		// Include smalltalk in our signal handler
	pseudoTopLevelLoop();
    }
}


// Wind things down
CSmalltalk::~CSmalltalk()
{

    unlink("gst.im");	// In case we did a save this session, this should be tidied.

    // Do this last in case something bombs in the kill
    s_dwSessionID--;
}


int CSmalltalk::EvaluateCommand(const char * comm)
{
    static int call_count = 0;	// to check for recursion
    // Pass responsibility for clean signal recovery to the smalltalk here
    // ie, let it longjump around inside itself etc. and trust it to return
    // gracefully after evalCode!

    //    if (call_count)
    //    {
    //	gprintef("ERROR: Smalltalk cannot be called recursively!!\n");
    //	return 1;
    //    }

//    jumpsigs = 0;

    ++call_count;
    int retval = evalCode((char*)comm);
    --call_count;

    // take back responsibility for signal recovery & our own longjumping
#ifndef _WIN32
//    jumpsigs = 1;	// Longjump in a separate thread doesn't work on WIN32 !
#endif

    return retval;
}
