/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef GENIE_SMALLTALK__
#define GENIE_SMALLTALK__

#if _MSC_VER >= 1000
#pragma once
#endif /* _MSC_VER >= 1000 */

// force loading of smalltalk config.h here as it will not be
// loaded by default in any GENIE routines.  (Because anything
// including smalltalk.h is by definition not a part of the smalltalk
// system itself but only a part of Open GENIE.

#ifdef __cplusplus
    extern "C" {
#endif /* __cplusplus */

/*
 * This header consists of C bits for combining the smalltalk interface with
 * Open GENIE as well as a C++ class definition for keeping track of what's
 * happening from the point of view of the C++ GENIE
 */

#include "smalltalkconfig.h"	        /* load here because we are outside smalltalk */

/* The smalltalk public interface */
#include "gstpub.h"		        /* this includes "gst.h" */

/* More private parts needed by GENIEVARS */
#include "dict.h"
#include "oop.h"
#include "sym.h"

/* Return 0 on success, 1 on an error */
int evalCode(char* str);

#ifdef __cplusplus
    }


// C++ Class definition below here (C++ from now on)

class CSmalltalk
{
// Construction
public:
    // Supply a compiled image file name, default is "gst.im", normally start from the
    // image already there, i.e. don't rebuild the image.
    CSmalltalk(const char szImageFile[] = "smalltalk.im", bool bRebuild = false);

// Destruction    
public:
    ~CSmalltalk();

// Operations
public:
    int EvaluateCommand(const char* comm);

// Implementation
private:
    static unsigned s_dwSessionID;	// Don't let anyone try to start this more than once

};

#endif /* __cplusplus */
#endif /* GENIE_SMALLTALK__ */

