/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"

#if defined (HAVE_UNISTD_H)
#  include <unistd.h>
#endif /* HAVE_UNISTD_H */

#include <sys/types.h>	/* for lseek */
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <gnustring.h>

#if defined(__VMS) && !defined(__VMS_POSIX)
#include <unixio.h>
#endif

#ifdef HAVE_IO_H
#include <io.h>
#endif

#include "fortran_types.h"
#include "genie_types.h"
#include "spectrum.h"

#define	arrayDopeOneD(__gv,__n) \
		arrayGetDims(__gv)[0]=(__n); \
		arrayGetDims(__gv)[1]=0; \
		arrayGetDope(__gv)[0]=1; \
		arrayGetDope(__gv)[1]=0

#define	MAX(__a,__b)	((__a) < (__b) ? (__b) : (__a))

#define faanew(__p_variable, __type, __len) \
    __p_variable = new __type [__len]; \
    if ( __p_variable == 0) \
    { \
	printf("New error in %s on line %i\n", __FILE__, (int)__LINE__); \
	printf("while trying to allocate array size %i\n", (int)__len); \
    } 

/* allocate space for a new gss */
g_spectrum_struct* initGSpectrum(g_integer ntc, g_integer nuse)
{
    size_t len;
    g_spectrum_struct *gss;
    gss = new g_spectrum_struct;
    len = ntc + 1;
    (*gss).x = new g_real[len];
    (*gss).y = new g_real[len];
    (*gss).e = new g_real[len];				/* errors */
    (*gss).ut = new g_real[NUSEMAX];			/* table of user parameters */
    (*gss).n = ntc;						/* number of time channels */
    (*gss).nut = nuse;					/* number of user parameters */
    (*gss).tl = " ";					/* title */
    (*gss).tm = " ";					/* time */
    (*gss).h = " ";					/* history */
    (*gss).fi = 0.0;					/* phi */
    (*gss).un = " ";					/* user_name */
    (*gss).in = " ";					/* inst_name */
    (*gss).rn = "00000";					/* run_no */
    (*gss).f = " ";					/* file */
    (*gss).d = 0.0;					/* delta */
    (*gss).dn = 0.0;					/* duration */
    (*gss).sn = "0";					/* spectrum number */
    (*gss).bg = 0;					/* bin grouping */
    (*gss).xl = " ";					/* xlabel */
    (*gss).yl = " ";					/* ylabel */
    (*gss).ym = 0.0;					/* ymax */
    (*gss).l1 = 0.0;					/* L1 */
    (*gss).l2 = 0.0;					/* L2 */
    (*gss).tt = 0.0;					/* two theta */
    (*gss).em = 0;					/* emode */
    (*gss).ef = 0.0;					/* efixed */
    return gss;
}

void deleteGSpectrum(g_spectrum_struct* gss)
{
    delete[] gss->x;
    delete[] gss->y;
    delete[] gss->e;
    delete[] gss->ut;
    delete gss;
    return;
}

void deleteFSpectrum(f_spectrum_struct* fss)
{
    delete[] fss->x;
    delete[] fss->y;
    delete[] fss->e;
    delete[] fss->ut;
    delete fss;
    return;
}

/* allocate space for a new fss */
f_spectrum_struct* initFSpectrum(fort_int ntc, fort_int nuse)
{
    int i;
    size_t len;
    f_spectrum_struct *fss;
    fss = new f_spectrum_struct;
    len = ntc + 1;
    (*fss).x = new fort_real[len];
    (*fss).y = new fort_real[len];
    (*fss).e = new fort_real[len];	/* errors */
    (*fss).ut = new fort_real[NUSEMAX];
    for(i=0; i<NUSEMAX; i++)
    {
	fss->ut[i] = 0.0;
    }
    (*fss).n = ntc;						/* number of time channels */
    (*fss).nut = nuse;					/* number of user parameters */
    (*fss).tl = " ";					/* title */
    (*fss).tm = " ";					/* time */
    (*fss).h = " ";					/* history */
    (*fss).fi = 0.0;					/* phi */
    (*fss).un = " ";					/* user_name */
    (*fss).in = " ";					/* inst_name */
    (*fss).rn = "00000";					/* run_no */
    (*fss).f = " ";					/* file */
    (*fss).d = 0.0;					/* delta */
    (*fss).dn = 0.0;					/* duration */
    (*fss).sn = "0";					/* spectrum number */
    (*fss).bg = 0;					/* bin grouping */
    (*fss).xl = " ";					/* xlabel */
    (*fss).yl = " ";					/* ylabel */
    (*fss).ym = 0.0;					/* ymax */
    (*fss).l1 = 0.0;					/* L1 */
    (*fss).l2 = 0.0;					/* L2 */
    (*fss).tt = 0.0;					/* two theta */
    (*fss).em = 0;					/* emode */
    (*fss).ef = 0.0;					/* efixed */
    return fss;
}

/* for gss, create a genie workspace */
g_variable createGSpectrum(g_spectrum_struct *gss)
{
    g_variable spectrum, tgv;
    g_integer gi, i;
    spectrum = newGworkspace(NSPECFIELD);

	gi = (*gss).n;
    	putInteger(tgv, gi);
    	addToWorkspace(spectrum, "NTC", tgv);

	tgv = newGarrayReal(gi+1);
	arrayDopeOneD(tgv, gi+1);
	for(i=0; i<gi+1; i++)
	{
	    arrayPutReal(tgv, i, (*gss).x[i]);
	}
	addToWorkspace(spectrum, "X", tgv);

	tgv = newGarrayReal(gi+1);
	arrayDopeOneD(tgv, gi+1);
	for(i=0; i<gi+1; i++)
	{
	    arrayPutReal(tgv, i, (*gss).y[i]);
	}
	addToWorkspace(spectrum, "Y", tgv);

	tgv = newGarrayReal(gi+1);
	arrayDopeOneD(tgv, gi+1);
	for(i=0; i<gi+1; i++)
	{
	    arrayPutReal(tgv, i, (*gss).e[i]);
	}
	addToWorkspace(spectrum, "E", tgv);

	gi = (*gss).nut;
    	putInteger(tgv, gi);
    	addToWorkspace(spectrum, "NUT", tgv);

	tgv = newGarrayReal(NUSEMAX);
	arrayDopeOneD(tgv, NUSEMAX);
	for(i=0; i<gi; i++)
	{
	    arrayPutReal(tgv, i, (*gss).ut[i]);
	}
	for(i=gi; i<NUSEMAX; i++)
	{
	    arrayPutReal(tgv, i, 0.0);
	}
	addToWorkspace(spectrum, "UT", tgv);

	putString(tgv, (*gss).tl, strlen(gss->tl));
	addToWorkspace(spectrum, "TITLE", tgv);

	putString(tgv, (*gss).tm, (*gss).tm.length());
	addToWorkspace(spectrum, "TIME", tgv);

	putString(tgv, (*gss).h, (*gss).h.length());
	addToWorkspace(spectrum, "HISTORY", tgv);

	putReal(tgv, (*gss).fi);
	addToWorkspace(spectrum, "FI", tgv);

	putString(tgv, (*gss).un, (*gss).un.length());
	addToWorkspace(spectrum, "USER_NAME", tgv);

	putString(tgv, (*gss).in, (*gss).in.length());
	addToWorkspace(spectrum, "INST_NAME", tgv);

	putString(tgv, (*gss).rn, (*gss).rn.length());
	addToWorkspace(spectrum, "RUN_NO", tgv);

	putString(tgv, (*gss).f, (*gss).f.length());
	addToWorkspace(spectrum, "FILE", tgv);

	putReal(tgv, (*gss).d);
	addToWorkspace(spectrum, "DELTA", tgv);

	putReal(tgv, (*gss).dn);
	addToWorkspace(spectrum, "RUN_DURATION", tgv);

	putString(tgv, (*gss).sn, (*gss).sn.length());
	addToWorkspace(spectrum, "SPEC_NO", tgv);

	putString(tgv, (*gss).xl, (*gss).xl.length());
	addToWorkspace(spectrum, "XLABEL", tgv);

	putString(tgv, (*gss).yl, (*gss).yl.length());
	addToWorkspace(spectrum, "YLABEL", tgv);

	putInteger(tgv, (*gss).bg);
	addToWorkspace(spectrum, "BIN_GROUPING", tgv);

	putReal(tgv, (*gss).ym);
	addToWorkspace(spectrum, "YMAX", tgv);

	putReal(tgv, (*gss).l1);
	addToWorkspace(spectrum, "L1", tgv);

	putReal(tgv, (*gss).l2);
	addToWorkspace(spectrum, "L2", tgv);

	putReal(tgv, (*gss).tt);
	addToWorkspace(spectrum, "TWOTHETA", tgv);

	putInteger(tgv, (*gss).em);
	addToWorkspace(spectrum, "EMODE", tgv);

	putReal(tgv, (*gss).ef);
	addToWorkspace(spectrum, "EFIXED", tgv);

    return spectrum;
}

/* for fss, create a genie workspace */
g_variable createFSpectrum(f_spectrum_struct *fss)
{
    g_variable spectrum, tgv;
    g_integer gi, i;
    g_real gr;

    spectrum = newGworkspace(NSPECFIELD);

    	gi = (g_integer) (*fss).n;
    	putInteger(tgv, gi);
    	addToWorkspace(spectrum, "NTC", tgv);

	tgv = newGarrayReal(gi+1);
	arrayDopeOneD(tgv, gi+1);
	for(i=0; i<gi+1; i++)
	{
	    gr = (g_real) (*fss).x[i];
	    arrayPutReal(tgv, i, gr);
	}
	addToWorkspace(spectrum, "X", tgv);

	tgv = newGarrayReal(gi+1);
	arrayDopeOneD(tgv, gi+1);
	for(i=0; i<gi+1; i++)
	{
	    gr = (g_real) (*fss).y[i];
	    arrayPutReal(tgv, i, gr);
	}
	addToWorkspace(spectrum, "Y", tgv);

	tgv = newGarrayReal(gi+1);
	arrayDopeOneD(tgv, gi+1);
	for(i=0; i<gi+1; i++)
	{
	    gr = (g_real) (*fss).e[i];
	    arrayPutReal(tgv, i, gr);
	}
	addToWorkspace(spectrum, "E", tgv);

    	gi = (g_integer) (*fss).nut;
    	putInteger(tgv, gi);
    	addToWorkspace(spectrum, "NUT", tgv);

	tgv = newGarrayReal(NUSEMAX);
	arrayDopeOneD(tgv, NUSEMAX);
	for(i=0; i<gi; i++)
	{
	    gr = (g_real) (*fss).ut[i];
	    arrayPutReal(tgv, i, gr);
	}
	for(i=gi; i<NUSEMAX; i++)
	{
	    arrayPutReal(tgv, i, 0.0);
	}
	addToWorkspace(spectrum, "UT", tgv);

	putString(tgv, (*fss).tl, (*fss).tl.length());
	addToWorkspace(spectrum, "TITLE", tgv);

	putString(tgv, (*fss).tm, (*fss).tm.length());
	addToWorkspace(spectrum, "TIME", tgv);

	putString(tgv, (*fss).h, (*fss).h.length());
	addToWorkspace(spectrum, "HISTORY", tgv);

	gr = (g_real) (*fss).fi;
	putReal(tgv, gr);
	addToWorkspace(spectrum, "FI", tgv);

	putString(tgv, (*fss).un, (*fss).un.length());
	addToWorkspace(spectrum, "USER_NAME", tgv);

	putString(tgv, (*fss).in, (*fss).in.length());
	addToWorkspace(spectrum, "INST_NAME", tgv);

	putString(tgv, (*fss).rn, (*fss).rn.length());
	addToWorkspace(spectrum, "RUN_NO", tgv);

	putString(tgv, (*fss).f, (*fss).f.length());
	addToWorkspace(spectrum, "FILE", tgv);

    	gr = (g_real) (*fss).d;
	putReal(tgv, gr);
	addToWorkspace(spectrum, "DELTA", tgv);

	gr = (g_real) (*fss).dn;
	putReal(tgv, gr);
	addToWorkspace(spectrum, "RUN_DURATION", tgv);

	putString(tgv, (*fss).sn, (*fss).sn.length());
	addToWorkspace(spectrum, "SPEC_NO", tgv);

	putString(tgv, (*fss).xl, (*fss).xl.length());
	addToWorkspace(spectrum, "XLABEL", tgv);

	putString(tgv, (*fss).yl, (*fss).yl.length());
	addToWorkspace(spectrum, "YLABEL", tgv);

	gi = (g_integer) (*fss).bg;
	putInteger(tgv, gi);
	addToWorkspace(spectrum, "BIN_GROUPING", tgv);

	gr = (g_real) (*fss).ym;
	putReal(tgv, gr);
	addToWorkspace(spectrum, "YMAX", tgv);

	gr = (g_real) (*fss).l1;
	putReal(tgv, gr);
	addToWorkspace(spectrum, "L1", tgv);

	gr = (g_real) (*fss).l2;
	putReal(tgv, gr);
	addToWorkspace(spectrum, "L2", tgv);

	gr = (g_real) (*fss).tt;
	putReal(tgv, gr);
	addToWorkspace(spectrum, "TWOTHETA", tgv);

	gi = (g_integer) (*fss).em;
	putInteger(tgv, gi);
	addToWorkspace(spectrum, "EMODE", tgv);

	gr = (g_real) (*fss).ef;
	putReal(tgv, gr);
	addToWorkspace(spectrum, "EFIXED", tgv);

    return spectrum;
}

f_spectrum_struct* extractFSpectrum(g_variable spectrum, g_integer ntcsize, g_integer nutsize)
{
    g_integer gi, ntc, i, nut;
    g_real gr;
    g_variable tgv;
    fort_int fint;
    f_spectrum_struct *fss;

    ntc = getInteger(getElementByField(spectrum, "NTC"));
    nut = getInteger(getElementByField(spectrum, "NUT"));

    fss = initFSpectrum(MAX(ntcsize, ntc), MAX(nutsize, nut));
	
    fint = (fort_int) ntc;
    fss->n = fint;
    fint = (fort_int) nut;
    fss->nut = fint;

    tgv = newGarrayReal(ntc+1);
    arrayDopeOneD(tgv, ntc+1);

    tgv = getElementByField(spectrum, "X");
    for(i=0; i<ntc+1; i++)
    {
	gr = arrayGetReal(tgv, i);
	fss->x[i] = (fort_real) gr;
    }

    tgv = getElementByField(spectrum, "Y");
    for(i=0; i<ntc+1; i++)
    {
	gr = arrayGetReal(tgv, i);
	fss->y[i] = (fort_real) gr;
    }

    tgv = getElementByField(spectrum, "E");
    for(i=0; i<ntc+1; i++)
    {
	gr = arrayGetReal(tgv, i);
	fss->e[i] = (fort_real) gr;
    }
	
//    tgv = newGarrayReal(nut);
    tgv = newGarrayReal(MAX(nutsize, nut));
    arrayDopeOneD(tgv, MAX(nutsize, nut));
    tgv = getElementByField(spectrum, "UT");
    for(i=0; i<nut; i++)
    {
	gr = arrayGetReal(tgv, i);
	fss->ut[i] = (fort_real) gr;
    }

	fss->in = getString(getElementByField(spectrum, "INST_NAME"));
	fss->tm = getString(getElementByField(spectrum, "TIME"));
	fss->un = getString(getElementByField(spectrum, "USER_NAME"));
	fss->tl = getString(getElementByField(spectrum, "TITLE")); 
	fss->xl = getString(getElementByField(spectrum, "XLABEL")); 
	fss->yl = getString(getElementByField(spectrum, "YLABEL")); 
	fss->f = getString(getElementByField(spectrum, "FILE")); 
	gr = getReal(getElementByField(spectrum, "DELTA"));
	fss->d = (fort_real) gr;
	fss->h = getString(getElementByField(spectrum, "HISTORY")); 
	gr = getReal(getElementByField(spectrum, "L1")); 
	fss->l1 = (fort_real) gr;
	gr = getReal(getElementByField(spectrum, "L2")); 
	fss->l2 = (fort_real) gr;
	gr = getReal(getElementByField(spectrum, "TWOTHETA")); 
	fss->tt = (fort_real) gr;
	gr = getReal(getElementByField(spectrum, "EFIXED")); 
	fss->ef = (fort_real) gr;
	gr = getReal(getElementByField(spectrum, "YMAX")); 
	fss->ym = (fort_real) gr;

	gi = getInteger(getElementByField(spectrum, "EMODE"));
	fss->em = (fort_int) gi;
	gi = getInteger(getElementByField(spectrum, "BIN_GROUPING"));
	fss->bg = (fort_int) gi;
	fss->sn = getString(getElementByField(spectrum, "SPEC_NO")); 
	fss->rn = getString(getElementByField(spectrum, "RUN_NO")); 
	gr = getReal(getElementByField(spectrum, "RUN_DURATION")); 
	fss->dn = (fort_real) gr;
	gr = getReal(getElementByField(spectrum, "FI"));
	fss->fi = (fort_real) gr;

    return fss;
}

g_spectrum_struct* extractGSpectrum(g_variable spectrum, g_integer ntcsize, g_integer nutsize)
{
    g_integer ntc, i, nut;
    g_variable tgv;
    g_spectrum_struct* gss;

    ntc = getInteger(getElementByField(spectrum, "NTC"));
    nut = getInteger(getElementByField(spectrum, "NUT"));
    gss = initGSpectrum(MAX(ntcsize, ntc), MAX(nutsize, nut));
    gss->n = ntc;
    gss->nut = nut;

    tgv = newGarrayReal(ntc+1);
    arrayDopeOneD(tgv, ntc+1);
    tgv = getElementByField(spectrum, "X");
    for(i=0; i<ntc+1; i++)
    {
	gss->x[i] = arrayGetReal(tgv, i);
    }

    tgv = getElementByField(spectrum, "Y");
    for(i=0; i<ntc+1; i++)
    {
	gss->y[i] = arrayGetReal(tgv, i);
    }

    tgv = getElementByField(spectrum, "E");
    for(i=0; i<ntc+1; i++)
    {
	gss->e[i] = arrayGetReal(tgv, i);
    }
	
    tgv = newGarrayReal(MAX(nutsize, nut));
    arrayDopeOneD(tgv, MAX(nutsize, nut));
    tgv = getElementByField(spectrum, "UT");
    for(i=0; i<nut; i++)
    {
	gss->ut[i] = arrayGetReal(tgv, i);
    }

	gss->in = getString(getElementByField(spectrum, "INST_NAME"));
	gss->tm = getString(getElementByField(spectrum, "TIME"));
	gss->un = getString(getElementByField(spectrum, "USER_NAME"));
	gss->tl = getString(getElementByField(spectrum, "TITLE")); 
	gss->xl = getString(getElementByField(spectrum, "XLABEL")); 
	gss->yl = getString(getElementByField(spectrum, "YLABEL")); 
	gss->f = getString(getElementByField(spectrum, "FILE")); 
	gss->d = getReal(getElementByField(spectrum, "DELTA"));
	gss->h = getString(getElementByField(spectrum, "HISTORY")); 
	gss->l1 = getReal(getElementByField(spectrum, "L1")); 
	gss->l2 = getReal(getElementByField(spectrum, "L2")); 
	gss->tt = getReal(getElementByField(spectrum, "TWOTHETA")); 
	gss->ef = getReal(getElementByField(spectrum, "EFIXED")); 
	gss->ym = getReal(getElementByField(spectrum, "YMAX")); 

	gss->em = getInteger(getElementByField(spectrum, "EMODE"));
	gss->bg = getInteger(getElementByField(spectrum, "BIN_GROUPING"));
	gss->sn = getString(getElementByField(spectrum, "SPEC_NO")); 
	gss->rn = getString(getElementByField(spectrum, "RUN_NO")); 
	gss->dn = getReal(getElementByField(spectrum, "RUN_DURATION")); 
	gss->fi = getReal(getElementByField(spectrum, "FI"));

    return gss;
}

/* put an fss into an existing genie workspace */
void replaceFSpectrum(g_variable* spectrum, f_spectrum_struct *fss)
{
    g_integer gi, ntc, i, nut;
    g_real gr;
    g_variable tgv;

    ntc = (g_integer) (*fss).n;
    putInteger(tgv, ntc);
//    putElementByField(*spectrum, "NTC", tgv);

    nut = (g_integer) (*fss).nut;
    putInteger(tgv, nut);
//    putElementByField(*spectrum, "NUT", tgv);

    tgv = newGarrayReal(ntc+1);
    arrayDopeOneD(tgv, ntc+1);
    for(i=0; i<ntc+1; i++)
    {
	gr = (g_real) (*fss).x[i];
	arrayPutReal(tgv, i, gr);
    }
//    putElementByField(*spectrum, "X", tgv);

    tgv = newGarrayReal(ntc+1);
    arrayDopeOneD(tgv, ntc+1);
    for(i=0; i<ntc+1; i++)
    {
	gr = (g_real) (*fss).y[i];
	arrayPutReal(tgv, i, gr);
    }
//    putElementByField(*spectrum, "Y", tgv);

    tgv = newGarrayReal(ntc+1);
    arrayDopeOneD(tgv, ntc+1);
    for(i=0; i<ntc+1; i++)
    {
	gr = (g_real) (*fss).e[i];
	arrayPutReal(tgv, i, gr);
    }
//    putElementByField(*spectrum, "E", tgv);

    tgv = newGarrayReal(nut);
    arrayDopeOneD(tgv, nut);
    for(i=0; i<nut; i++)
    {
	gr = (g_real) (*fss).ut[i];
	arrayPutReal(tgv, i, gr);
    }
//    putElementByField(*spectrum, "UT", tgv);

    putString(tgv, (*fss).in, (*fss).in.length());
//    putElementByField(*spectrum, "INST_NAME", tgv);

    putString(tgv, (*fss).tm, (*fss).tm.length());
//    putElementByField(*spectrum, "TIME", tgv);

    putString(tgv, (*fss).un, (*fss).un.length());
//    putElementByField(*spectrum, "USER_NAME", tgv);

    putString(tgv, (*fss).tl, (*fss).tl.length());
//    putElementByField(*spectrum, "TITLE", tgv);

    putString(tgv, (*fss).xl, (*fss).xl.length());
//    putElementByField(*spectrum, "XLABEL", tgv);

    putString(tgv, (*fss).yl, (*fss).yl.length());
//    putElementByField(*spectrum, "YLABEL", tgv);

    putString(tgv, (*fss).f, (*fss).f.length());
//    putElementByField(*spectrum, "FILE", tgv);

    gr = (g_real) (*fss).d;
    putReal(tgv, gr);
//    putElementByField(*spectrum, "DELTA", tgv);

    putString(tgv, (*fss).h, (*fss).h.length());
//    putElementByField(*spectrum, "HISTORY", tgv);

    gr = (g_real) (*fss).l1;
    putReal(tgv, gr);
//    putElementByField(*spectrum, "L1", tgv);

    gr = (g_real) (*fss).l2;
    putReal(tgv, gr);
//    putElementByField(*spectrum, "L2", tgv);

    gr = (g_real) (*fss).tt;
    putReal(tgv, gr);
//    putElementByField(*spectrum, "TWOTHETA", tgv);

    gr = (g_real) (*fss).ef;
    putReal(tgv, gr);
//    putElementByField(*spectrum, "EFIXED", tgv);

    gr = (g_real) (*fss).ym;
    putReal(tgv, gr);
//    putElementByField(*spectrum, "YMAX", tgv);

    gi = (g_integer) (*fss).em;
    putInteger(tgv, gi);
//    putElementByField(*spectrum, "EMODE", tgv);

    gi = (g_integer) (*fss).bg;
    putInteger(tgv, gi);
//    putElementByField(*spectrum, "BIN_GROUPING", tgv);

    putString(tgv, (*fss).sn, (*fss).sn.length());
//    putElementByField(*spectrum, "SPEC_NO", tgv);

    putString(tgv, (*fss).rn, (*fss).rn.length());
//    putElementByField(*spectrum, "RUN_NO", tgv);

    gr = (g_real) (*fss).dn;
    putReal(tgv, gr);
//    putElementByField(*spectrum, "RUN_DURATION", tgv);

    gr = (g_real) (*fss).fi;
    putReal(tgv, gr);
//    putElementByField(*spectrum, "FI", tgv);

    return;
}


/* put an gss into an existing genie workspace */
void replaceGSpectrum(g_variable* spectrum, g_spectrum_struct *gss)
{
    g_integer nut, ntc, i;
    g_variable tgv;

    ntc = (*gss).n;
    putInteger(tgv, ntc);
//    putElementByField(*spectrum, "NTC", tgv);

    nut = (*gss).nut;
    putInteger(tgv, nut);
//    putElementByField(*spectrum, "NUT", tgv);

    tgv = newGarrayReal(ntc+1);
    arrayDopeOneD(tgv, ntc+1);
    for(i=0; i<ntc+1; i++)
    {
	arrayPutReal(tgv, i, (*gss).x[i]);
    }
//    putElementByField(*spectrum, "X", tgv);

    tgv = newGarrayReal(ntc+1);
    arrayDopeOneD(tgv, ntc+1);
    for(i=0; i<ntc+1; i++)
    {
	arrayPutReal(tgv, i, (*gss).y[i]);
    }
//    putElementByField(*spectrum, "Y", tgv);

    tgv = newGarrayReal(ntc+1);
    arrayDopeOneD(tgv, ntc+1);
    for(i=0; i<ntc+1; i++)
    {
	arrayPutReal(tgv, i, (*gss).e[i]);
    }
//    putElementByField(*spectrum, "E", tgv);

    tgv = newGarrayReal(nut);
    arrayDopeOneD(tgv, nut);
    for(i=0; i<nut; i++)
    {
	arrayPutReal(tgv, i, (*gss).ut[i]);
    }
//    putElementByField(*spectrum, "UT", tgv);

    putString(tgv, (*gss).in, (*gss).in.length());
//    putElementByField(*spectrum, "INST_NAME", tgv);

    putString(tgv, (*gss).tm, (*gss).tm.length());
//    putElementByField(*spectrum, "TIME", tgv);

    putString(tgv, (*gss).un, (*gss).un.length());
//    putElementByField(*spectrum, "USER_NAME", tgv);

    putString(tgv, (*gss).tl, (*gss).tl.length());
//    putElementByField(*spectrum, "TITLE", tgv);

    putString(tgv, (*gss).xl, (*gss).xl.length());
//    putElementByField(*spectrum, "XLABEL", tgv);

    putString(tgv, (*gss).yl, (*gss).yl.length());
//    putElementByField(*spectrum, "YLABEL", tgv);

    putString(tgv, (*gss).f, (*gss).f.length());
//    putElementByField(*spectrum, "FILE", tgv);

    putReal(tgv, (*gss).d);
//    putElementByField(*spectrum, "DELTA", tgv);

    putString(tgv, (*gss).h, (*gss).h.length());
//    putElementByField(*spectrum, "HISTORY", tgv);

    putReal(tgv, (*gss).l1);
//    putElementByField(*spectrum, "L1", tgv);

    putReal(tgv, (*gss).l2);
//    putElementByField(*spectrum, "L2", tgv);

    putReal(tgv, (*gss).tt);
//    putElementByField(*spectrum, "TWOTHETA", tgv);

    putReal(tgv, (*gss).ef);
//    putElementByField(*spectrum, "EFIXED", tgv);

    putReal(tgv, (*gss).ym);
//    putElementByField(*spectrum, "YMAX", tgv);

    putInteger(tgv, (*gss).em);
//    putElementByField(*spectrum, "EMODE", tgv);

    putInteger(tgv, (*gss).bg);
//    putElementByField(*spectrum, "BIN_GROUPING", tgv);

    putString(tgv, (*gss).sn, (*gss).sn.length());
//    putElementByField(*spectrum, "SPEC_NO", tgv);

    putString(tgv, (*gss).rn, (*gss).rn.length());
//    putElementByField(*spectrum, "RUN_NO", tgv);

    putReal(tgv, (*gss).dn);
//    putElementByField(*spectrum, "RUN_DURATION", tgv);

    putReal(tgv, (*gss).fi);
//    putElementByField(*spectrum, "FI", tgv);

    return;
}

/*
 * position at start of block (1=first spectrum)
 * return -1 for invalid block, 0 for success
 */

int seekSS(int fd, int block)
{
    int nblocks, i; 
    unsigned long block_length;
    lseek(fd, 0, SEEK_SET);
    read(fd, (char*)&nblocks, sizeof(int));
    if ( block > nblocks )
    {
	return -1;
    }
    for(i=1; i<block; i++)
    {
    	read(fd, (char*)&block_length, sizeof(unsigned long));
	lseek(fd, block_length, SEEK_CUR);
    }
    return 0;
}

/*
 * write a gss assuming in correct place, then update header at start of file
 * returns number of bytes written, -1 on error
 */

unsigned long writeGSS(int fd, g_spectrum_struct* gss)
{
    unsigned long nbytes, array_length, ntot;

    updateSSHeader(fd, 1);

    array_length = gss->n + 1;	/* number of time channels + 1 */
    ntot = sizeof(g_spectrum_struct) + 3 * array_length * sizeof(g_real); 
    write(fd, (char*)&ntot, sizeof(unsigned long));
    write(fd, (char*)gss, sizeof(g_spectrum_struct));
    nbytes = array_length * sizeof(g_real);
    write(fd, (char*)(*gss).x, nbytes);
    write(fd, (char*)(*gss).y, nbytes);
    write(fd, (char*)(*gss).e, nbytes);

    return ntot;
}

/*
 * increment number of blocks, returning new number
 * resets position in file
 */
int updateSSHeader(int fd, int increment)
{
    off_t pos;
    int nblocks;
    pos = lseek(fd, 0, SEEK_SET);
    if (read(fd, (char*)&nblocks, sizeof(int)) != sizeof(int))
    {
	nblocks = 0;
    }
    else
    {
    	nblocks = nblocks + increment;
    }
    write(fd, (char*)&nblocks, sizeof(int));
    lseek(fd, pos, SEEK_SET);
    return nblocks;
}

/* 
 * read gss structure from file 
 * assume that file already positioned at correct place
 * return NULL on error
 *
 */
g_spectrum_struct* readGSS(int fd)
{
    unsigned long ntot;
    size_t nbytes, array_length;
    
    g_spectrum_struct* gss;
    gss = new g_spectrum_struct;
    read(fd, (char*)&ntot, sizeof(unsigned long));
    read(fd, (char*)&gss, sizeof(g_spectrum_struct));
    array_length = gss->n + 1;	/* number of time channels + 1 */
    gss->x = new g_real[array_length];
    gss->y = new g_real[array_length];
    gss->e = new g_real[array_length];
    nbytes = array_length * sizeof(g_real);
    read(fd, (char*)gss->x, nbytes);
    read(fd, (char*)gss->y, nbytes);
    read(fd, (char*)gss->e, nbytes);
    return gss;
}
