/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
//+++
//
// CLASS: Text
//
// 
//
//---
    // *** include string headers
#include <gnustring.h>

    // *** include PGPLOT headers
#include "Pginclude.h"

    // *** include object headers
#include "Drawable_objects.h"
#include "Text.h"

static const char* rcsid = "@(#)$Id: text.cxx,v 1.9 1997/10/28 17:04:31 cmm Exp $";
USE_RCSID(rcsid);

// *** constructor given at least a string & ordinates
Text::Text(float x,
	   float y,
	   String word,
	   float size,
	   int shade,
	   float orient,
	   int cset,
	   int thick,
	   float just) {


	    // *** set visibility
	    invisible = 0;

	    // *** set co-ordinates
		X_ordinate = x;
		Y_ordinate = y;

	    // *** set string text
		if (word != " ")
			string_text = word;
		else
			string_text = STRING_TEXT;
	    // *** set character size	
		if (size != 0.0)
			character_size = size;
		else
			character_size = CHARACTER_SIZE;

	    // *** set line colour
		if (shade != -1)
			line_colour = shade;
		else
			line_colour = LINE_COLOUR;

	    // *** set orientation
		if (orient != -1.0)
			orientation = orient;
		else
			orientation = ORIENTATION;

	    // *** set font
		if (cset != 0)
			font = cset;
		else
			font = FONT;

	    // *** set line thickness
		if (thick != 0.0)
			line_thickness = thick;
		else
			line_thickness = LINE_THICKNESS;

		justify = just;

	}


// *** destructor
Text::~Text() {
	}


void Text::display() {
	float zero = 0.0;

	// *** set character font
	pgscf(font);

	// *** set character size
	pgsch(character_size);

	// *** set colour
	pgsci(line_colour);

	// *** set line thickness
	pgslw(line_thickness);

	// *** write the text
	PGPTEXT_S(X_ordinate, Y_ordinate, orientation, justify, string_text);
	}

