/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef TRANSFORM_UTILS_H
#define TRANSFORM_UTILS_H

// _MAX and _MIN are the recommended way of avoiding the conflict with windows.h
// definitions.
#ifndef _WIN32

#	define _MIN min
#	define _MAX max

	// These are already defined in the standard C++ library
	template <class aType>
	inline aType _MAX(aType a, aType b)
	{
	    return(a > b ? a : b);
	}

	template <class aType>
	inline aType _MIN(aType a, aType b)
	{
	    return(a < b ? a : b);
	}
#endif

extern char* crunid(g_integer id);

template <class __g_real, class __g_integer>
extern int newAxis(__g_real xbounds[], __g_integer ibounds, __g_real xsteps[], 
              __g_integer isteps, __g_real xnew[], int assign,
	      __g_integer itype);

#if __GNUC__ <= 2 && __GNUC_MINOR__ <= 7	/* GCC 2.7 and below */
#define	GNUG_TEMPLATE
#else
#define GNUG_TEMPLATE	template
#endif

#ifdef __GNUG__
extern GNUG_TEMPLATE int newAxis(float[], int, float[], int, float[], int, int);
extern GNUG_TEMPLATE int newAxis(double[], long, double[], long, double[], int, g_integer);
#endif /* __GNUG__ */


template <class __g_real, class __g_integer>
extern int rebin(__g_real xold[], __g_real yold[], __g_real eold[], __g_integer ntcold, 
	  __g_real xnew[], __g_real ynew[], __g_real enew[], __g_integer ntcnew, __g_real& ymax);

#ifdef __GNUG__
extern GNUG_TEMPLATE int rebin(float[], float[], float[], int, float[], float[], float[], int, float&);
extern GNUG_TEMPLATE int rebin(double[], double[], double[], long, double[], double[], double[], long, double&);
#endif /* __GNUG__ */


template <class __g_real, class __g_integer>
extern int bracket(__g_real xval, const __g_real xarray[], __g_integer np);

#ifdef __GNUG__
extern GNUG_TEMPLATE int bracket(float, const float[], int);
extern GNUG_TEMPLATE int bracket(double, const double[], long);
#endif /* __GNUG__ */


template <class __g_real, class __g_integer>
extern int rebinRange(__g_real xold[], __g_real yold[], __g_real eold[], 
	       __g_real xnew[], __g_real ynew[], __g_real enew[],
		__g_integer& ntc, __g_real xmin, __g_real xmax, __g_real& ymax);

#ifdef __GNUG__
extern GNUG_TEMPLATE int rebinRange(double[], double[], double[], double[], double[], double[], long&, double, double, double&);
extern GNUG_TEMPLATE int rebinRange(float[], float[], float[], float[], float[], float[], int&, float, float, float&);
#endif /* __GNUG__ */


template <class __g_real_old, class __g_real_new, class __g_integer>
extern int group_bins(__g_real_old xarray[], __g_real_old yarray[], __g_real_old earray[],
	       __g_integer ntc, __g_integer nspec, __g_real_new xnew[], __g_real_new ynew[],
	       __g_real_new enew[], __g_integer nbin);

#ifdef __GNUG__
extern GNUG_TEMPLATE int group_bins(double[], double[], double[], long, long, float[], float[], float[], long);
extern GNUG_TEMPLATE int group_bins(double[], double[], double[], long, long, double[], double[], double[], long);
extern GNUG_TEMPLATE int group_bins(float[], float[], float[], int, int, float[], float[], float[], int);
#endif /* __GNUG__ */

template <class __g_real, class __g_integer>
extern int integrateSpectrum(const __g_real xarray[], const __g_real yarray[], const __g_real earray[],
			__g_integer ntc, __g_real xmin, __g_real xmax, 
			__g_real& sum, __g_real& error);

template <class __g_real, class __g_integer>
extern int integrateSpectra(__g_real xarray[], __g_real yarray[], __g_real earray[],
			__g_integer ntc, __g_integer nspec, __g_real xmin[], 
			__g_integer nmin, __g_real xmax[], __g_integer nmax, 
			__g_real sum[], __g_real error[]);

template <class __g_type>
extern int array_rank(__g_type* array, g_integer* ranks, int len, int direction);

template <class __g_type>
extern int array_sort(__g_type* array, g_integer* ranks, int len);

template <class __g_type>
void getMedian(__g_type* array, g_integer* ranks,int first, int last);

template <class __g_type>
void array_swap(__g_type* array, g_integer* ranks, int a, int b);

template <class __g_type>
void selection_sortA(__g_type* array, g_integer* ranks, int first, int last);

template <class __g_type>
void selection_sortD(__g_type* array, g_integer* ranks, int first, int last);

template <class __g_type>
void quick_sortA(__g_type* array, g_integer* ranks, int first, int last);

template <class __g_type>
void quick_sortD(__g_type* array, g_integer* ranks, int first, int last);

#endif /* TRANSFORM_UTILS_H */
