/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef	UNITS_H__
#define	UNITS_H__
/*
 *  C1 and C2 correspond to the constants used in GENIE_V2 units command,
 *  as given in UNIT_COMMAND.FOR
 */
#ifdef	G2_UNITS
/*
 * use explicit values given in UNIT_COMMAND.FOR of GENIE_V2
 */
#define	C1	252.82
#define	C2	81.787
/* #undef	PI */
#define PI	3.14159265
#else
/*
 * calculate converson factors based on best available values
 * of the fundamental atomic constants
 */
#define MASSN	1.674929E-27		/* neutron mass in Kg */
#define EV	1.6021773E-19		/* 1 eV in Joules */
#define	HPLANK	6.626076E-34		/* Planck's constant in Js */
#define	C1	MASSN/(HPLANK*1.0E4)	/* in micro seconds per angstrom */
#define	C2	(HPLANK*1.0E23)/(2.0*MASSN)*(HPLANK/EV)
					/* in meV (angstrom)**2 */
#ifndef	PI
#define PI	3.14159265358979323846
#endif /* PI */

#endif /* G2_UNITS */

#define	DEG2RAD	( PI / 180.0 )
#define MEV2WAVENUMBER		8.066

/* dimensionless for X is not a valid unit */
static const char* xunit_names[] = { "(dimensionless)", "Channel Number", 
	"Time-of-Flight (\\gms)", "Wavelength (\\A)", "d-Spacing (\\A)", 
	"Energy (meV)", "Momentum Transfer Q (\\A\\u-1\\d)", 
	"Q\\u2\\d (\\A\\u-2\\d)", "Reciprocal Velocity (\\gms/m)", "Energy Transfer \\gDE=E1-E2 (meV)", 
	"Primary Flight-Path Wavelength \\gl\\d1\\u (\\A)", "Energy Transfer \\gDE=E1-E2 (cm\\u-1\\d)" };

static const char* yunit_names[] = { "(dimensionless)", "Neutron Counts per Channel", 
	"Neutron counts / \\gms", "Neutron Counts / \\A", 
	"Neutron counts / \\A", "Neutron counts / meV", 
	"Neutron counts / \\A\\u-1\\d", "Neutron counts / \\A\\u-2\\d", 
	"Neutron counts / \\gms per metre)", "Neutron Counts / meV Transferred", 
	"Neutron counts / \\A", "Neutron counts / cm\\u-1\\d" };

/* valid unit codes range from 0 to NUNIT-1 */
#define	NUNIT	((int)(sizeof(xunit_names) / sizeof(const char*)))

typedef enum {
    Dimensionless = 0,
    ChannelNo = 1,
    TOF = 2,
    Wavelength = 3,
    dSpacing = 4,
    Energy = 5,
    MomentumTransfer = 6,
    Qsquared = 7,
    ReciprocalTOF = 8,
    DeltaE = 9,
    PrimaryWavelength = 10,
    DeltaE_inWaveNumber = 11
	} unit_code_struct;

static char error_message_buffer[200];

int convert_units_binmode(g_real xarray[], g_real yarray[], g_real earray[], 
		  g_integer ntc, const char* xunit_in,
		  unit_code_struct xunit_code_in, 
		  unit_code_struct xunit_code_out, 
		  unit_code_struct yunit_code_in, 
		  unit_code_struct yunit_code_out, 
		  g_real l1, g_real l2,
		  g_integer emode, g_real efixed, g_real SinTheta, 
		  g_real delta);

int convert_units_pointmode(g_real xarray[], g_real yarray[], g_real earray[], 
		  g_integer ntc, const char* xunit_in,
		  unit_code_struct xunit_code_in, 
		  unit_code_struct xunit_code_out, 
		  unit_code_struct yunit_code_in, 
		  unit_code_struct yunit_code_out, 
		  g_real l1, g_real l2,
		  g_integer emode, g_real efixed, g_real SinTheta, 
		  g_real delta);
#define	BIGVAL	1.0E12

#endif /* UNITS_H__ */
