/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifdef __VMS

/*
 * This module contains unprefixed versions of some DECC library functions
 */
#define _POSIX_C_SOURCE
#define FIX_IOCTL	/* comment this line out if you don't have ioctl() */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <unixio.h>
#include <unixlib.h>
#include <stdarg.h>
#ifdef FIX_IOCTL
#    include <sys/ioctl.h>
#endif

/* 
 * Remove the default DECC$ prefix - we will add it manually when required 
 */
#pragma extern_prefix save
#pragma extern_prefix ""

pid_t getpid()
{
    return decc$getpid();
}

int access(const char* path, int mode)
{
    return decc$access(path, mode);
}

int lseek(int fd, off_t offset, int where)
{
    return decc$lseek(fd, offset, where);
}

int open(const char* path, int flags, ... )
{
    va_list list;
    mode_t mode;
    va_start(list, flags);
    mode = va_arg(list, mode_t);
    va_end(list);
    return decc$open(path, flags, mode);
}

int close(int fd)
{
    return decc$close(fd);
}

int write(int filedes, const void *buffer, size_t nbytes)
{
    return decc$write(filedes, buffer, nbytes);
}

int read(int filedes, void *buffer, size_t nbytes)
{
    return decc$read(filedes, buffer, nbytes);
}

int stat(const char* path, struct stat* buffer)
{
    return decc$stat(path, buffer);
} 

int putenv(const char* var)
{
    return decc$putenv(var);
}

#ifdef FIX_IOCTL
int ioctl(int d, int request, void * arg)
{
    return decc$ioctl(d, request, arg);
}
#endif

#pragma extern_prefix restore		/* restore DECC$ prefixing */

#endif /* __VMS */
