#!/usr/local/bin/perl -w
#
# $Id: proctest.pl,v 1.4 1998/01/25 23:22:15 faa Exp $
#
# Module:	Automatic GENIE procedure test program
# Author:	Freddie Akeroyd, ISIS, 30/6/97
#
# This reads an input file (usually procdefs.out produced by a parser -s)
# and builds a GCL file (proctest.gcl) to test the procedures mentioned
#
# The procdefs.out file is of the format
#
# Hprocedure_name
# Qqualifier
# Pparamerer=type
# E
#

# upper case names of PROCEDUREs not to test as they cause problems
@leave = ( 'CHDIR', 'DEBUG', 'EXIT', 'GRIPE', 'INQUIRE', 
		'JUMP', 'QUIT', 'READ_TERMINAL', 'SEARCH', 'SHOW',
		'SYSTEM' );

# loop through files in arguments
while(<>)
{
    chop;
# Found procedure name - keep for later
    if (/^H/) 
    { 
	$proc_name = $_; 
	$proc_name =~ s/^H(.*)/$1/; 	# remove leading H
        $proc_name =~ tr/a-z/A-Z/;	# uppercase procedure name
	@qualifiers = ();		# clear list
	%parameters = ();		# clear list
	next;
    }
# Found parameter, possibly with a type
# Build an associative array, indexed by name and containing the type
    if (/^P/)
    {
	$var = $_;
	$var =~ s/^P([^=]*).*/$1/;	# name is anything before =
	$var =~ tr/a-z/A-Z/;		# uppercase
	$type = $_;
	$type =~ s/^P[^=]*=(.*)/$1/;	# type is anything after =
	$type =~ tr/a-z/A-Z/;		# uppercase
	$parameters{$var} = $type;
	next;
    }
# Found qualifier - add to list
    if (/^Q/)
    {
	$_ =~ s/^Q(.*)/$1/;		# qualifier is anything after the Q
	$_ =~ tr/a-z/A-Z/;		# uppercase
	push(@qualifiers, $_);
	next;
    }
# Finished reading in definitions, write test file
    if (/^E/) 
    {
	&build_tests($proc_name, *qualifiers, *parameters, *STDOUT);
	next;
    }
    print STDERR "Invalid input line: $_\n";
}

1;

# This is called with a complete procedure definition - it
# will build the test
sub build_tests
{
    local($proc_name, *qualifiers, *parameters, *TESTFILE) = @_;
    if ($proc_name =~ /^_/)
    {
	return 1;	# procedure starts with _; it's an internal one
    }
    foreach $q(@leave)
    {
	if ($proc_name eq $q)
	{
	    return 1;	# don't test certain procedures e.g. EXIT
 	}
    }
# Run test with no qualifiers
    print TESTFILE "PRINTIN \"*** Testing $proc_name ***\"\n";
    &test_parameters($proc_name, "", *parameters, *TESTFILE);
# Run test with each qualifier
    foreach $q1(@qualifiers)
    {
	$qq = "/".$q1;
        &test_parameters($proc_name, $qq, *parameters, *TESTFILE);
    }
# Run test with all two qualifiers combinations
    foreach $q1(@qualifiers)
    {
        foreach $q2(@qualifiers)
	{
	    if ($q1 ne $q2)
	    {
	    	$qq = "/".$q1."/".$q2;
            	&test_parameters($proc_name, $qq, *parameters, *TESTFILE);
	    }
	}
    }
    1;
}

sub test_parameters
{
    local($proc_name, $qualifiers, *parameters, *TESTFILE) = @_;
    $name = $proc_name.$qualifiers;
    foreach $p(sort keys %parameters)
    {
	&write_string(*TESTFILE, "$name $p=_");	# try undefined
	if ($parameters{$p} eq "INTEGER")
	{
	    &write_string(*TESTFILE, "$name $p=0");
	    &write_string(*TESTFILE, "$name $p=1");
	    &write_string(*TESTFILE, "$name $p=-1");
	    &write_string(*TESTFILE, "$name $p=1000");
	    &write_string(*TESTFILE, "$name $p=-1000");
	} 
	elsif ($parameters{$p} eq "REAL")
	{
	    &write_string(*TESTFILE, "$name $p=0.0");
	    &write_string(*TESTFILE, "$name $p=0.5");
	    &write_string(*TESTFILE, "$name $p=-0.5");
	    &write_string(*TESTFILE, "$name $p=1.0");
	    &write_string(*TESTFILE, "$name $p=-1.0");
	    &write_string(*TESTFILE, "$name $p=1000.0");
	    &write_string(*TESTFILE, "$name $p=-1000.0");
	}
	elsif ($parameters{$p} eq "STRING")
	{
	    &write_string(*TESTFILE, "$name $p=\"\"");
	    &write_string(*TESTFILE, "$name $p=\"junk.dat\"");
	}
	elsif ($parameters{$p} eq "WORKSPACE")
	{
	    &write_string(*TESTFILE, "$name $p=FIELDS()");
	}
	elsif ($parameters{$p} eq "REALARRAY")
	{
	    &write_string(*TESTFILE, "$name $p=Realarray.new(1)");
	}
	elsif ($parameters{$p} eq "INTEGERARRAY")
	{
	    &write_string(*TESTFILE, "$name $p=Integerarray.new(1)");
	}
	elsif ($parameters{$p} eq "STRINGARRAY")
	{
	    &write_string(*TESTFILE, "$name $p=Stringarray.new(1)");
	}
	elsif ($parameters{$p} eq "WORKSPACEARRAY")
	{
	    &write_string(*TESTFILE, "$name $p=Workspacearray.new(1)");
	}
    }
    1;
}

sub write_string()
{
    local(*FILE, $string) = @_;
    print FILE "PRINTIN \"$string\"\n";
    print FILE $string,"\n";
    1;
}
