/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user.
 *
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 *
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 *
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 *
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 *
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

#if defined(LIBC_SCCS) && !defined(lint)
/*static char *sccsid = "from: @(#)xdr_float.c 1.12 87/08/11 Copyr 1984 Sun Micro";*/
/*static char *sccsid = "from: @(#)xdr_float.c	2.1 88/07/29 4.0 RPCSRC";*/
static char *rcsid = "$Id: xdr_float.c,v 1.8 1998/04/09 20:06:39 faa Exp $";
#endif

/*
 * xdr_float.c, Generic XDR routines impelmentation.
 *
 * Copyright (C) 1984, Sun Microsystems, Inc.
 *
 * These are the "floating point" xdr routines used to (de)serialize
 * most common data items.  See xdr.h for more info on the interface to
 * xdr.
 */

#include <stdio.h>
#include <sys/types.h>

#ifdef _WIN32
#	include <winsock.h>
#else
#	include <rpc/types.h>
#endif

#include <xdr.h>
#include <geniedefs.h>

/*
 * NB: Not portable.
 * This routine works on Suns (Sky / 68000's), i386's, MIPS, NS32k and Vaxen.
 */

#ifndef __VMS 
#  define IEEEFP		1
#endif

#ifdef __VMS
#  include <cvt$routines.h>
#  include <cvtdef.h>
#  include <lib$routines.h>
#  include <libdef>
/* set up codes for use of CVT$CONVERT_FLOAT */
#  if __IEEE_FP
#    define IEEEFP		1
#    define VMS_FLOAT_NATIVE	CVT$K_IEEE_S
#    define VMS_DOUBLE_NATIVE	CVT$K_IEEE_T
#  elif __D_FLOAT
#    define VAXFP		1
#    define VMS_FLOAT_NATIVE	CVT$K_VAX_F
#    define VMS_DOUBLE_NATIVE	CVT$K_VAX_D
#  elif __G_FLOAT
#    define VAXFP		1
#    define VMS_FLOAT_NATIVE	CVT$K_VAX_F
#    define VMS_DOUBLE_NATIVE	CVT$K_VAX_G
#  else
#    error Cannot determine VMS floating point format
#  endif

static unsigned long cvt_convert_float(void* input_value, unsigned long input_type,
    void* output_value, unsigned long output_type, unsigned long options)
{
    lib$establish(lib$sig_to_ret);
    return cvt$convert_float(input_value, input_type, output_value, output_type, options);
}

#endif /* __VMS */

bool_t
xdr_float(xdrs, fp)
	register XDR *xdrs;
	register float *fp;
{
#ifndef IEEEFP
	float f;
    	unsigned long status;
#endif /* IEEEFP */
	switch (xdrs->x_op) {

	case XDR_ENCODE:
#ifdef IEEEFP
		return (XDR_PUTLONG(xdrs, (long *)fp));
#elif VAXFP
		status = cvt_convert_float(fp, VMS_FLOAT_NATIVE, &f, CVT$K_IEEE_S, 0);
		if (status & 1) /* OK */
		{
		    return (XDR_PUTLONG(xdrs, (long *)&f));
		}
		else
		{
		    gprintef("VMS error code %lu from CVT$CONVERT_FLOAT\n", status);
		    return (FALSE);
		}
#else
#error	Cannot determine Floating Point Format
#endif /* IEEEFP */

	case XDR_DECODE:
#ifdef IEEEFP
		return (XDR_GETLONG(xdrs, (long *)fp));
#elif VAXFP
		if (!XDR_GETLONG(xdrs, (long *)&f))
			return (FALSE);
		status = cvt_convert_float(&f, CVT$K_IEEE_S, fp, VMS_FLOAT_NATIVE, 0);
		if (status & 1) /* OK */
		{
		    return (TRUE);
		}
		else
		{
		    gprintef("VMS error code %lu from CVT$CONVERT_FLOAT\n", status);
		    return (FALSE);
		}
#else
#error	Cannot determine Floating Point Format
#endif /* IEEEFP */

	case XDR_FREE:
		return (TRUE);
	}
	return (FALSE);
}

/*
 * This routine works on Suns (Sky / 68000's), i386's, MIPS and Vaxen.
 */

bool_t
xdr_double(xdrs, dp)
	register XDR *xdrs;
	double *dp;
{
	register long *lp;
#ifndef IEEEFP
	double d;
	unsigned long status;
#endif /* IEEEFP */

	switch (xdrs->x_op) {

	case XDR_ENCODE:
#ifdef IEEEFP
		lp = (long *)dp;
#  ifdef WORDS_BIGENDIAN
		return (XDR_PUTLONG(xdrs, lp++) && XDR_PUTLONG(xdrs, lp));
#  else
		return (XDR_PUTLONG(xdrs, lp+1) && XDR_PUTLONG(xdrs, lp));
#  endif /* BYTE_ORDER */
#elif VAXFP
		status = cvt_convert_float(dp, VMS_DOUBLE_NATIVE, &d, CVT$K_IEEE_T, 0);
		if (status & 1) /* OK */
		{
		    lp = (long*)&d;
		    return (XDR_PUTLONG(xdrs, lp+1) && XDR_PUTLONG(xdrs, lp));
		}
		else
		{
		    gprintef("VMS error code %lu from CVT$CONVERT_FLOAT\n", status);
		    return (FALSE);
		}
#else
#  error	Cannot determine Floating Point Format
#endif /* IEEEFP */

	case XDR_DECODE:
#ifdef IEEEFP
		lp = (long *)dp;
#  ifdef WORDS_BIGENDIAN
		return (XDR_GETLONG(xdrs, lp++) && XDR_GETLONG(xdrs, lp));
#  else
		return (XDR_GETLONG(xdrs, lp+1) && XDR_GETLONG(xdrs, lp));
#  endif /* BYTE_ORDER */
#elif VAXFP
		lp = (long *)&d;
		if (XDR_GETLONG(xdrs, lp+1) && XDR_GETLONG(xdrs, lp))
	   	{
		    status = cvt_convert_float(lp, CVT$K_IEEE_T, dp, VMS_DOUBLE_NATIVE, 0);
		    if (status & 1) /* OK */
		    {
		        return (TRUE);
		    }
		    else
		    {
		        gprintef("VMS error code %lu from CVT$CONVERT_FLOAT\n", status);
		        return (FALSE);
		    }
		}
		else
		{
		    return (FALSE);
		}
#else
#  error	Cannot determine Floating Point Format
#endif /* IEEEFP */

	case XDR_FREE:
		return (TRUE);
	}
	return (FALSE);
}
