ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	SUBROUTINE ADD_RAW_FILES(pars_get,pars_put)
c
c
c	  Adapted for OpenGenie diffraction data analysis at IRIS by 
c	P J Marshall, 8/96.
c	
c	  OpenGenie module to combine raw files.  Given an array of 
c	run numbers, ADD_RAW_FILES selects the runs with compatible
c	parameters and adds them together.  The output .sum files are 
c	written to the vax SCRATCH$DISK, numbered incrementally from
c	IRS00001.  "Odd" files are also written, unchanged, to the 
c	same place for ease of reading back in the ADD program.
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c	ADDRAW:
c
c 	Program to add several raw SNS data files together.
c 	Up to 20 runs may be added together.
c 	Program checks that the run number exists and that certain
c 	parameters between runs are consistent.
c 	Files containing the summed runs are written to disk in the
c 	RAW file format.
c
c 	R C Ward, Neutron Division, RAL
c
	include 'genie_modules.inc'
	character*62 spiel
	character*40 raw_file(20),long(20)
	character*30 sum_file
	character*25 indiskdir
	character*18 outdiskdir
	character*12 rid_name(20),medium(20),file_name
	character*7 string_short(20)
	character*3 user
c
	character*25 indiskdir_scratch
	character*23 indiskdir_iris0
	character*14 indiskdir_user
c		
	integer*4 irun(20),short(20),irunex(20),nspex,ispecex(20),counter
	integer*4 check(20),num,name(20)
	external pars_get,pars_put
c
c
C *** check library version
        IF (MODULE_VERSION_OK(GENIE_MAJOR, GENIE_MINOR) .EQ. 0) RETURN
c
	call module_information('Adding raw files')
c	
c	Get variables from the Opengenie procedure:
c
	call module_get_string(pars_get,'disk_and_dir',indiskdir)
	call module_get_string(pars_get,'user',user)
	call module_get_int(pars_get,'nfiles',nfiles)
	call module_get_int_array(pars_get,'file',irun,nfiles)
c
c
c Definition of variables:
c
c sum_file		output file name eg. [irsmgr.data]irs00001.sum
c file_name		short output file name eg. irs00001.sum
c deftdir		default directory for output file
c raw_file(i)/long(i)	complete file name eg. [irsmgr.data]irs00001.raw
c rid_name(i)/medium(i)	file name for one run eg. irs00001.raw
c irun(i)/short(i)	run number eg. 12345, passed from OpenGenie
c
c
c	Get file name,run numbers and spectra to be excluded:
c
c	
	if (indiskdir.eq.'/vax/scratch/irsmgr/data/') then
	  indiskdir_scratch = indiskdir(1:25)
	  call READNAME(nfiles,raw_file,irun,rid_name,file_name,
     +	                indiskdir_scratch,user)
	  if(nfiles.eq.0) then
	     outdiskdir = 'error'
	     goto 10
	  endif   
c
	elseif (indiskdir.eq.'/vax/iris0/irsmgr/data/  ') then
	  indiskdir_iris0 = indiskdir(1:23)
	  call READNAME(nfiles,raw_file,irun,rid_name,file_name,
     +	                indiskdir_iris0,user)
	  if(nfiles.eq.0) then
	     outdiskdir = 'error'
	     goto 10
	  endif   	  
c
	elseif (indiskdir.eq.'/vax/user/'//user//'/           ') then
	  indiskdir_user = indiskdir(1:14)
	  call READNAME(nfiles,raw_file,irun,rid_name,file_name,
     +                  indiskdir_user,user)
	  if(nfiles.eq.0) then
	     outdiskdir = 'error'
	     goto 10
	  endif   	  
c	  
	endif      
c	
c
	outdiskdir = '/vax/scratch/iris/'
c
	DO N=1,20
	 check(N) = 0
	ENDDO
	num = 1
c	 
	DO I=1,(nfiles)
	 counter = 1
	 long(counter) = raw_file(I)	! Long,medium and short will be the
	 medium(counter) = rid_name(I)  ! arrays of files with consistent
	 short(counter) = irun(I)	! parameters
c
	 DO J=1,nfiles	   
c
	  IF (I.ne.J.AND.check(I).eq.0.AND.check(J).eq.0) THEN
	   call CHECKPAR(raw_file(I),raw_file(J),rid_name,nerror)
	   IF (nerror.eq.0) THEN
	    counter = counter +1	   ! Looks through all the run numbers
	    long(counter) = raw_file(J)    ! in the files array for those with
	    medium(counter) = rid_name(J)  ! the same parameters as raw_file(I)
	    short(counter) = irun(J)
	    check(J) = 1
	   ENDIF
	  ENDIF 
c
	 ENDDO
c
	 IF (check(I).eq.0) THEN
 	  write(file_name(4:8),68) short(1)
  68      format(i5.5)	   	  
 	  sum_file = outdiskdir//file_name
c
c	Sum files together:
c
	  call SUMDATA(counter,long,short,irunex,nspex,ispecex,
     +	                medium,sum_file)
c     
	  write(spiel(1:27),69) 'Run numbers in summed file '
  69      format(a27)	  
	  write(spiel(28:57),70) sum_file
  70	  format(a30)
	  write(spiel(58:62),71) ' are:'
  71      format(a5) 
	  call module_information(spiel)
	  DO K=1,counter
	   write(string_short(K),72) short(K)
  72	   format(2X,i5.5)
	   call module_information(string_short(K))
	  ENDDO 
c	  
	  name(num) = short(1)
	  num = num + 1
	 ENDIF 
c
	 check(I) = 1
c
	ENDDO
c
c	Return the location of the summed file to the procedure:
c
	call module_put_int_array(pars_put,'locations',name,(num-1))
  	call module_put_string(pars_put,'disk_and_dir',outdiskdir)
c
  10	return
	end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	subroutine READNAME(nr,raw_file,irun,rid_name,file_name,indiskdir,
     +	                    user)
c
c
c	 On screen inquiry for run numbers is replaced by them being 
c	passed to the program from the OpenGenie procedure.
c
c	 Due to the difficulty in setting up the equivalent of the VMS 
c	system logical definitions, the raw files must, in this version, 
c	come from IRIS$DISK0:[IRSMGR.DATA].  Similarly, the result sum 
c	file is only written to SCRATCH$DISK:[IRIS].
c
c	  The option of excluding spectra from the addition has been 
c	removed - if detectors are noisy they are excluded later in the ADD
c	procedure.

c
	dimension inames(9)
	character*40 raw_file(20),sum_file,deftdir,name
	character*12 rid_name(20),ins_name
	character*12 file_name
	character*3 user
	character*1 ans
c
	character*(*) indiskdir
c	
	integer*4 irun(20),iarray(10)
	integer*4 nr
	logical*4 istat
	logical found(20)
c
c
	file_name= 'INS00000.sum'
        ins_name = 'INS00000.RAW'
c
	ins_name(1:3) = 'irs'
	do i=1,20
	 rid_name(i)= ins_name
	enddo
	file_name(1:3) = user
c
 30	if(nr.le.1) then
c	 write(6,610)
c  610	 format(/,4x,'Less than two run numbers have been given!')
         call module_error('add_raw_files',
     1            'Less than two run numbers given', ' ')
  	 nr = 0
	 return
	endif
c
	do i= 1,nr-1
	 do j= i+1,nr
	  if(irun(i).eq.irun(j)) then
c	   write(6,612)
c  612	   format(/,4x,'Two identical run numbers have been given',
c     +	   /,4x,'Runs should not be summed !')
c	   write(6,606)
	   call module_error('add_raw_files',
     +       'Two identical run numbers given', ' ')
	   nr = 0
	   return
	  endif
	 enddo
	enddo
c
	do i=1,nr		! nr = number of runs inputed
	 write(rid_name(i)(4:8),1000) irun(i)
 1000	 format(i5.5)
c	 raw_file(i) = '/vax/user/pjm/data/'//rid_name(i)
	 raw_file(i) = indiskdir//rid_name(i)
	 inquire(file=raw_file(i),exist=found(i))
	 if(.not.found(i)) then
	    call module_error('add_raw_files','File not found',' ')	 
	    nr = 0
	    return
	 endif
	enddo
	do j=1,nr
	 if(.not.found(j)) then
	  nr = 0		! Signal passed to addraw that file has not 
	  return		!  not been found
	 endif
	enddo
c
c  100	write(file_name(4:8),1000) irun(1)   ! Output run no. is 1st input no.      
c	call SYS$SETDDIR(,len,deftdir)
c	sum_file= deftdir(1:len)//file_name
c
  999	return
	end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	subroutine CHECKPAR(raw_fileA,raw_fileB,rid_name,nerror)
c
c	
	PARAMETER (MAXIMUM_CHANNELS=1000000)
	character*12 rid_name(20)
	character*40 raw_fileA,raw_fileB
	integer*4 ival1(1000),ival2(1000),ierror(7)
	real*4 rval1(20),rval2(20)
c
c
	do i=1,7
	 ierror(i)=0	!set error flags to zero
	enddo
c
	call XGETPARI(raw_fileA,'NDET',ival1(1))
	ndet=ival1(1)
	call XGETPARI(raw_fileA,'NMON',ival1(2))
	call XGETPARI(raw_fileA,'SPEC',ival1(3))
	call XGETPARI(raw_fileA,'NSP1',ival1(3+ndet))
	call XGETPARI(raw_fileA,'NTC1',ival1(4+ndet))
	call XGETPARI(raw_fileA,'TCM1',ival1(5+ndet))
	call XGETPARR(raw_fileA,'TCP1',rval1(1))
c
	nerror=0		! set error counter to 0
c
	call XGETPARI(raw_fileB,'NDET',ival2(1))
	ndet=ival2(1)
	call XGETPARI(raw_fileB,'NMON',ival2(2))
	call XGETPARI(raw_fileB,'SPEC',ival2(3))
	call XGETPARI(raw_fileB,'NSP1',ival2(3+ndet))
	call XGETPARI(raw_fileB,'NTC1',ival2(4+ndet))
	call XGETPARI(raw_fileB,'TCM1',ival2(5+ndet))
	call XGETPARR(raw_fileB,'TCP1',rval2(1))
c
	if(ival1(1).ne.ival2(1)) ierror(1)=1	! NDET values
	if(ival1(2).ne.ival2(2)) ierror(2)=2	! NMON values
	do i=3,3+ndet-1
	 if(ival1(i).ne.ival2(i)) ierror(3)=3	! SPEC values
	enddo
	if(ival1(3+ndet).ne.ival2(3+ndet)) ierror(4)=4	! NSP1 values
	if(ival1(4+ndet).ne.ival2(4+ndet)) ierror(5)=5	! NTC1 values
	if(ival1(5+ndet).ne.ival2(5+ndet)) ierror(6)=6	! TCM1 values
	IF ((NSP1+1)*(NTC1+1).GT.MAXIMUM_CHANNELS) THEN
		STOP 'Raw files are too big for this version of ADDRAW'
	ENDIF
	do i=1,20
	 if(rval1(i).ne.rval2(i)) ierror(7)=7 	! TCP1 values
	enddo
c
	isum= ierror(1)		! check whether error flags are zero
	do i=2,7
	 isum= isum+ierror(i)
	enddo
c	if(isum.ne.0) then
c	 call ERROR(rid_name,j,ierror)
c	endif
c
	nerror=nerror+isum		! counter on number of errors
c
	return
	end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	subroutine ERROR(rid_name,j,ierror)
c
	character*12 rid_name(20)
	integer*4 ierror(10)
c
	
c
	if(ierror(1).eq.1) then			! NDET values
	 write(6,610) rid_name(1),rid_name(j)
  610	 format(/,4x,'Number of detectors in runs ',/,
     +	 4x,a12,' and ',a12,' are different - runs cannot be summed')
	endif
c
	if(ierror(2).eq.2) then			! NMON values
	 write(6,620) rid_name(1),rid_name(j)
  620	 format(/,4x,'Number of monitors in runs ',/,
     +	 4x,a12,' and ',a12,' are different - runs cannot be summed')
	endif
c
	if(ierror(3).eq.3) then			! SPEC values
	 write(6,630) rid_name(1),rid_name(j)
  630	 format(/,4x,'Spectra tables in runs ',/,
     +	 4x,a12,' and ',a12,' are different - runs cannot be summed')
	endif
c
	if(ierror(4).eq.4) then			! NSP1 values
	 write(6,640) rid_name(1),rid_name(j)
  640	 format(/,4x,'Number of spectra in runs ',/,
     +	 4x,a12,' and ',a12,' are different - runs cannot be summed')
	endif
c
	if(ierror(5).eq.5) then			! NTC1 values
	 write(6,650) rid_name(1),rid_name(j)
  650	 format(/,4x,'Number of time channels in runs ',/,
     +	 4x,a12,' and ',a12,' are different - runs cannot be summed')
	endif
c
	if(ierror(6).eq.6) then			! TCM1 values
	 write(6,660) rid_name(1),rid_name(j)
  660	 format(/,4x,'Time channel modes in runs ',/,
     +	 4x,a12,' and ',a12,' are different - runs cannot be summed')
	endif
c
	if(ierror(7).eq.7) then			! TCP1 values
	 write(6,670) rid_name(1),rid_name(j)
  670	 format(/,4x,'Time channel parameters in runs ',/,
     +	 4x,a12,' and ',a12,' are different - runs cannot be summed')
	endif
c
	return
	end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	subroutine SUMDATA(nr,raw_file,irun,irunex,nspex,ispecex,
     +	                   rid_name,sum_file)
c
c
	PARAMETER (MAXIMUM_CHANNELS=1000000)
c
	INCLUDE 'crptdef.par'
c
	character*40 raw_file(20),sum_file
	character*12 rid_name(20)
	character*80 cvalue
	integer*4 irun(20),irunex(20),nspex,ispecex(20)
	integer*4 ivalue(MAXIMUM_CHANNELS),i_rpb(32),idat(33000)
	integer*4 run_section,tcb_section,dat_section,log_section,
     +	nsp1,ntc1,
     +	run_duration,good_frames,total_frames,run_dur_seconds,
     +	monitor_sum1,monitor_sum2
	real*4 rvalue(MAXIMUM_CHANNELS)
	real*4 good_protons,total_protons,r_rpb(32)
	equivalence (r_rpb,i_rpb)
	equivalence (rvalue,ivalue)
	equivalence (cvalue,ivalue)
c
c	write(6,600) rid_name(1)
c  600	format(/,4x,' File ',a12,' is being read into memory',/)
c
	call XGETRUN(raw_file(1),ivalue,ierror)
	if(ierror.eq.1) then
c	 write(6,640) rid_name(1)
c  640	 format(/,4x,'Fatal error code 1 returned on accessing',
c     +	 ' file ',a12,/,4x,'Program aborted')
	 call module_error('add_raw_files',
     + 		'Fatal error accessing file', ' ')	 
	 return
	endif
c
	run_section= ivalue(AD_RUN)
	tcb_section=ivalue(AD_TCB)
	dat_section= ivalue(AD_DATA)
	log_section= ivalue(AD_LOG)
c
	nsp1= ivalue(tcb_section + T_NSP1)
	ntc1= ivalue(tcb_section + T_NTC1)
c
	run_duration= ivalue(run_section + R_DUR)
	good_protons= rvalue(run_section + R_GD_PRTN_CHRG)
	total_protons= rvalue(run_section + R_TOT_PRTN_CHRG)
	good_frames= ivalue(run_section + R_GOODFRM)
	total_frames= ivalue(run_section + R_RAWFRM)
	run_dur_seconds= ivalue(run_section + R_DUR_SECS)
	monitor_sum1= ivalue(run_section + R_MON_SUM1)
	monitor_sum2= ivalue(run_section + R_MON_SUM2)
c	
	do 10 j= 1,nsp1+1	!note there is a spectrum no. 0
	js= j-1			!start at spectrum 0
	 do jj= 1,nspex
 	  if(irun(1).eq.irunex(jj)) then
	   if(js.eq.ispecex(jj)) then
c	    write(6,620) ispecex(jj),irunex(jj)
	    sp_start= dat_section + 1 + (ntc1+1)*js
	    do k= 1,ntc1+1	!note there is a time channel 0
	     kt= k-1		!start at time channel 0
	     ivalue(sp_start+kt)= 0
	    enddo
	   endif
	  endif
	 enddo
  10	continue
c
	do 20 i= 2,nr			! If nr<2, do loop is skipped by F77
c	write(6,610) rid_name(i)
c  610	format(/,4x,'File ',a12,' is being summmed to memory',/)
	call XGETPARI(raw_file(i),'RPB',i_rpb(1))
	run_duration= run_duration + i_rpb(1)
	good_protons= good_protons + r_rpb(8)
	total_protons= total_protons + r_rpb(9)
	good_frames= good_frames + i_rpb(10)
	total_frames= total_frames + i_rpb(11)
	run_dur_seconds= run_dur_seconds + i_rpb(13)
	monitor_sum1= monitor_sum1 + i_rpb(14)
	monitor_sum2= monitor_sum2 + i_rpb(15)
c
	sp_start= dat_section + 1 + (ntc1+1)*js
	call XGETDAT(raw_file(i),js,1,idat)
	do k= 1,ntc1+1		!note there is a time channel 0
	kt= k-1			!start at time channel 0
	 ivalue(sp_start+kt)= ivalue(sp_start+kt) + idat(k)
	enddo
  30	continue
c
  20	continue
c
	max_size= log_section - 1
	ivalue(run_section + R_DUR)= run_duration
	rvalue(run_section + R_GD_PRTN_CHRG)= good_protons
	rvalue(run_section + R_TOT_PRTN_CHRG)= total_protons
	ivalue(run_section + R_GOODFRM)= good_frames
	ivalue(run_section + R_RAWFRM)= total_frames
	ivalue(run_section + R_DUR_SECS)= run_dur_seconds
	ivalue(run_section + R_MON_SUM1)= monitor_sum1
	ivalue(run_section + R_MON_SUM2)= monitor_sum2
c
	call PUTRUN(sum_file,max_size,ivalue,error)
	if(ierror.eq.1) then
c	 write(6,650) sum_file
c  650	 format(/,4x,'Fatal error code 1 returned on writing',
c     +	 ' file ',a40,/,4x,' to disk.  Program aborted')
	 call module_error('add_raw_files',
     +      'Fatal error writing file to disk', ' ')
	 return
	endif
c
	return
	end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
	function INTSEEK(returned_no,total)
c
c This funtion will read in a line from the terminal and return an array of 
c integers extracted from it. Function becomes true if integers are found
c otherwise it returns false
c
c Pass parameters
c	1) Integer*4 array to contain numbers extracted
c	2) Integer*4 which will indicate the number of array elements used.
c
c
	implicit none
	integer*4 returned_no(1),total,nchrs,strt,fin
     -	,n,n1,i,delim(2)
	logical*4 intseek
	character ip*200
c
	
c
	read(5,3010)nchrs,ip
3010	format(i5,a200)
	if (nchrs.gt.0) then
	  strt=1
5	  do 20 while (strt.lt.nchrs+1)
	    if((ip(strt:strt).gt.'/').and.(ip(strt:strt).lt.':')) then
	      total=total+1
	      fin=strt

	      do 10 while ((fin.le.nchrs).and.(ip(fin:fin).gt.'/')
     -	      			.and.(ip(fin:fin).lt.':'))
10	      fin=fin+1
	      read(ip(strt:fin-1),3020)returned_no(total)
3020	      format(i10)
	      strt=fin
	    else
	      strt=strt+1
	    end if
20	  continue
	  intseek=.true.
	else
	  intseek=.false.
	end if
	return
	end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C
C 	GET routines modified to cope with periods Feb89 !!
c
c	In genie-2 these routines were linked to in the standard fortran way -
c	as compiling modules in OpenGenie works slightly differently, they were
c	included in this same file.  GETPARR, GETPARI etc are commands
c	recognised by OpenGenie, where they have different parameter lists. 
c	For the purpose of this program they were renamed XGETPARR, XGETPARI
c	etc.
c	
c
C
	SUBROUTINE XGETPARI(RUNID,NAME,IVALUE)
C
C  Gets named integer paramter(s) from a RAW data file
C  Whole sections may also be requested
C
	CHARACTER*(*) RUNID
	CHARACTER*(*) NAME
	INTEGER*4 IVALUE(1)
C
	LOGICAL FOUND
C
	INTEGER*4 IWORK(128)
	CHARACTER*60 FILNAM
	REAL*4 WORK(128)
	COMMON/xcrpt_WORK/IWORK
	EQUIVALENCE (IWORK,WORK)
C
C
	INTEGER*4 VER1,IFORMAT(10),IVER(10),NDET,NMON,NEFF,
     +	NSEP,NTRG,NSP1,NTC1,ULEN,NPER,DATA_HEADER(32)
	LOGICAL OPENFILE
	COMMON/xcrpt_SPECIALS/ FILNAM,VER1,IFORMAT,IVER,
     +		NDET,NMON,NEFF,NSEP,NTRG,NSP1,NTC1,ULEN,
     +          OPENFILE,NPER,DATA_HEADER
C
c
	
C
C  decide whether it's CRPT or just a file
	IF (RUNID(4:8).EQ.'00000') THEN
		WRITE(6,6000)
 6000		FORMAT(' Accessing of current run not available')
		GOTO 999
	ENDIF
C
C  check name is valid & open file according to RUNID
	IF (FILNAM.NE.RUNID.OR.FILNAM.EQ.' ') THEN
		CALL XOPEN_FILE(RUNID,FOUND)
		IF (.NOT.FOUND) THEN
		  RETURN
		ENDIF  
	ENDIF
C
C  read variables into IVALUE
C ----------------------------
C
C  From now on just decide what has been requested and return it
	IF     (NAME.EQ.'VER1') THEN
		IVALUE(1)=VER1
	ELSEIF (NAME.EQ.'SFMT') THEN
		CALL XGETSECT(1,31,IVALUE)
C
C  run section
	ELSEIF (NAME.EQ.'SRUN') THEN
		CALL XGETSECT(IFORMAT(1),94,IVALUE)
	ELSEIF (NAME.EQ.'VER2') THEN
		CALL XGETSECT(IFORMAT(1),1,IVALUE)
	ELSEIF (NAME.EQ.'RUN') THEN
		CALL XGETSECT(IFORMAT(1)+1,1,IVALUE)
	ELSEIF (NAME.EQ.'RPB') THEN
		CALL XGETSECT(IFORMAT(1)+62,32,IVALUE)
C
C  instrument section
	ELSEIF (NAME.EQ.'SINS') THEN
	  IF (IVER(1).EQ.1) THEN
	    CALL XGETSECT(IFORMAT(2),70+(NMON*2)+(6+NEFF)*NDET,IVALUE)
	  ELSE
	    CALL XGETSECT(IFORMAT(2),70+(NMON*2)+(5+NEFF)*NDET,IVALUE)
	  ENDIF
	ELSEIF (NAME.EQ.'VER3') THEN
		IVALUE(1)=IVER(2)
	ELSEIF (NAME.EQ.'IVPB') THEN
		CALL XGETSECT(IFORMAT(2)+3,64,IVALUE)
	ELSEIF (NAME.EQ.'NDET') THEN
		IVALUE(1)=NDET
	ELSEIF (NAME.EQ.'NMON') THEN
		IVALUE(1)=NMON
	ELSEIF (NAME.EQ.'NEFF') THEN
		IVALUE(1)=NEFF
	ELSEIF (NAME.EQ.'NUSE') THEN
		IVALUE(1)=NEFF
	ELSEIF (NAME.EQ.'MDET') THEN
		CALL XGETSECT(IFORMAT(2)+70,NMON,IVALUE)
	ELSEIF (NAME.EQ.'MONP') THEN
		CALL XGETSECT(IFORMAT(2)+70+NMON,NMON,IVALUE)
	ELSEIF (NAME.EQ.'SPEC') THEN
		IFROM=IFORMAT(2)+70+2*NMON
		CALL XGETSECT(IFROM,NDET,IVALUE)
	ELSEIF (NAME.EQ.'CODE') THEN
		IF (IVER(2).NE.1) THEN
		IFROM=IFORMAT(2)+70+2*NMON+3*NDET
		CALL XGETSECT(IFROM,NDET,IVALUE)
		ENDIF
	ELSEIF (NAME.EQ.'TIMR') THEN
	  IF (VER1.EQ.1) THEN
		IFROM=IFORMAT(2)+70+2*NMON+NDET
	  ELSE
		IFROM=IFORMAT(4)+65+3*NDET
	  ENDIF
	  CALL XGETSECT(IFROM,NDET,IVALUE)
C
C  sample environment section
	ELSEIF (NAME.EQ.'SSEN') THEN
	  IF (IVER(3).EQ.1) THEN
		CALL XGETSECT(IFORMAT(3),34+NSEP*24,IVALUE)
	  ELSE
		CALL XGETSECT(IFORMAT(3),66+NSEP*32,IVALUE)
	  ENDIF
	  IF (NSEP.NE.0) WRITE(6,6040)
 6040	  FORMAT(' GETPAR needs adjusting to take account of SE')
	ELSEIF (NAME.EQ.'VER4') THEN
		CALL XGETSECT(IFORMAT(3),1,IVALUE)
	ELSEIF (NAME.EQ.'SPB ') THEN
	  IF (IVER(3).EQ.1) THEN
		CALL XGETSECT(IFORMAT(3)+1,32,IVALUE)
	  ELSE
		CALL XGETSECT(IFORMAT(3)+1,64,IVALUE)
	  ENDIF
	ELSEIF (NAME.EQ.'NSEP') THEN
		IVALUE(1)=NSEP
	ELSEIF (NAME.EQ.'SE01') THEN
		WRITE(6,6040)
C
C  DAE section
	ELSEIF (NAME.EQ.'SDAE') THEN
	  IF (IVER(4).EQ.1) THEN
		CALL XGETSECT(IFORMAT(4),65+3*NDET,IVALUE)
	  ELSE
		CALL XGETSECT(IFORMAT(4),65+5*NDET,IVALUE)
	  ENDIF
	ELSEIF (NAME.EQ.'VER5') THEN
		CALL XGETSECT(IFORMAT(4),1,IVALUE)
	ELSEIF (NAME.EQ.'DAEP') THEN
		CALL XGETSECT(IFORMAT(4)+1,64,IVALUE)
	ELSEIF (NAME.EQ.'CRAT') THEN
		CALL XGETSECT(IFORMAT(4)+65,NDET,IVALUE)
	ELSEIF (NAME.EQ.'MODN') THEN
		CALL XGETSECT(IFORMAT(4)+65+NDET,NDET,IVALUE)
	ELSEIF (NAME.EQ.'MPOS') THEN
		CALL XGETSECT(IFORMAT(4)+65+2*NDET,NDET,IVALUE)
	ELSEIF (NAME.EQ.'UDET') THEN
		CALL XGETSECT(IFORMAT(4)+65+4*NDET,NDET,IVALUE)
C
C  TCB section
	ELSEIF (NAME.EQ.'STCB') THEN
		IF (NTRG.NE.1) WRITE(6,6050)
		CALL XGETSECT(IFORMAT(5),288+NTC1+1,IVALUE)
	ELSEIF (NAME.EQ.'VER6') THEN
		CALL XGETSECT(IFORMAT(5),1,IVALUE)
	ELSEIF (NAME.EQ.'NTRG') THEN
		IF (NTRG.NE.1) WRITE(6,6050)
 6050	FORMAT(' Multiple time regimes....GETPAR needs changing')
		IVALUE(1)=NTRG
	ELSEIF (NAME.EQ.'NFPP') THEN
		CALL XGETSECT(IFORMAT(5)+2,1,IVALUE)
	ELSEIF (NAME.EQ.'NPER') THEN
		IVALUE(1)=NPER
	ELSEIF (NAME.EQ.'PMAP') THEN
		CALL XGETSECT(IFORMAT(5)+4,256,IVALUE)
	ELSEIF (NAME.EQ.'NSP1') THEN
		IVALUE(1)=NSP1
	ELSEIF (NAME.EQ.'NTC1') THEN
		IVALUE(1)=NTC1
	ELSEIF (NAME.EQ.'TCM1') THEN
		CALL XGETSECT(IFORMAT(5)+262,5,IVALUE)
	ELSEIF (NAME.EQ.'PRE1') THEN
		CALL XGETSECT(IFORMAT(5)+287,1,IVALUE)
	ELSEIF (NAME.EQ.'TCB1') THEN
		CALL XGETSECT(IFORMAT(5)+288,NTC1+1,IVALUE)
C  data section
	ELSEIF (NAME.EQ.'VER7') THEN
		CALL XGETSECT(IFORMAT(6),1,IVALUE)
C
C  non existant requests
	ELSE
		WRITE(6,*) ' No such INTEGER parameter as',NAME
	ENDIF
C
C
C
 999	RETURN
	END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
	SUBROUTINE XGETPARR(RUNID,NAME,RVALUE)
C
C	OSF1 and VMS compilers deal with real numbers in different ways -
C	the subroutine VAXF_TO_LOCAL converts the number read into the correct 
C	value.  TCP1 is the only real number read in to addraw and so  the 
C	only callout to VAXF_TO_LOCAL comes after the TCP1 XGETSECT call.
C
C  Gets REAL parameter(s) from RAW data file
C
	CHARACTER*(*) RUNID
	CHARACTER*(*) NAME
	INTEGER*4 RVALUE(1)
C
	LOGICAL FOUND
C
	INTEGER*4 IWORK(128)
	REAL*4 WORK(128)
	COMMON/xcrpt_WORK/IWORK
	EQUIVALENCE (IWORK,WORK)
C
	EQUIVALENCE (TEMP,ITEMP)
C
C
	CHARACTER*60 FILNAM
	INTEGER*4 VER1,IFORMAT(10),IVER(10),NDET,NMON,NEFF,
     +	NSEP,NTRG,NSP1,NTC1,ULEN,NPER,DATA_HEADER(32),IERR
	LOGICAL OPENFILE
	COMMON/xcrpt_SPECIALS/ FILNAM,VER1,IFORMAT,IVER,
     +		NDET,NMON,NEFF,NSEP,NTRG,NSP1,NTC1,ULEN,OPENFILE,NPER,
     +		DATA_HEADER
C
C
	
c
C  decide whether it's CRPT or just a file
	IF (RUNID(4:8).EQ.'00000') THEN
		WRITE(6,6000)
 6000		FORMAT(' Accessing of current run not available')
		GOTO 999
	ENDIF
C
C  check name is valid & open file according to RUNID
	IF (FILNAM.NE.RUNID.OR.FILNAM.EQ.' ') THEN
		CALL XOPEN_FILE(RUNID,FOUND)
		IF (.NOT.FOUND) RETURN
	ENDIF
C
C  read variables into RVALUE
C ----------------------------
C
C  Instrument section
	ITABLE=IFORMAT(2)+70+2*NMON
	IF (NAME.EQ.'LEN2') THEN
		CALL XGETSECT(ITABLE+2*NDET,NDET,RVALUE)
	ELSEIF (NAME.EQ.'OMEG') THEN
	  IF (IVER(2).EQ.1) THEN
		CALL XGETSECT(ITABLE+3*NDET,NDET,RVALUE)
	  ENDIF
	ELSEIF (NAME.EQ.'DELT') THEN
	  IF (IVER(2).NE.1) THEN
		CALL XGETSECT(ITABLE+NDET,NDET,RVALUE)
	  ENDIF
	ELSEIF (NAME.EQ.'TTHE') THEN
		CALL XGETSECT(ITABLE+4*NDET,NDET,RVALUE)
	ELSEIF (NAME.EQ.'PHI') THEN
	  IF (IVER(2).EQ.1) THEN
		CALL XGETSECT(ITABLE+5*NDET,NDET,RVALUE)
	  ENDIF
	ELSEIF (NAME.EQ.'EF01') THEN
		CALL XGETSECT(ITABLE+6*NDET,NDET,RVALUE)
	ELSEIF (NAME.EQ.'EF02') THEN
		CALL XGETSECT(ITABLE+7*NDET,NDET,RVALUE)
	ELSEIF (NAME.EQ.'EF03') THEN
		CALL XGETSECT(ITABLE+8*NDET,NDET,RVALUE)
	ELSEIF (NAME.EQ.'EF04') THEN
		CALL XGETSECT(ITABLE+9*NDET,NDET,RVALUE)
	ELSEIF (NAME.EQ.'EF05') THEN
		CALL XGETSECT(ITABLE+10*NDET,NDET,RVALUE)
	ELSEIF (NAME.EQ.'EF06') THEN
		CALL XGETSECT(ITABLE+11*NDET,NDET,RVALUE)
	ELSEIF (NAME.EQ.'UT01') THEN
		CALL XGETSECT(ITABLE+5*NDET,NDET,RVALUE)
	ELSEIF (NAME.EQ.'UT02') THEN
		CALL XGETSECT(ITABLE+6*NDET,NDET,RVALUE)
	ELSEIF (NAME.EQ.'UT03') THEN
		CALL XGETSECT(ITABLE+7*NDET,NDET,RVALUE)
	ELSEIF (NAME.EQ.'UT04') THEN
		CALL XGETSECT(ITABLE+8*NDET,NDET,RVALUE)
	ELSEIF (NAME.EQ.'UT05') THEN
		CALL XGETSECT(ITABLE+9*NDET,NDET,RVALUE)
	ELSEIF (NAME.EQ.'UT06') THEN
		CALL XGETSECT(ITABLE+10*NDET,NDET,RVALUE)
C  Time channel boundaries section
C     time channel area definition
	ELSEIF (NAME.EQ.'TCP1') THEN
		CALL XGETSECT(IFORMAT(5)+267,20,RVALUE)
		CALL VAXF_TO_LOCAL(RVALUE, 20, IERR)
	ELSEIF (NAME.EQ.'TCB1') THEN
C  if tcb's requested in real form then return as microsecs
		CALL XGETSECT(IFORMAT(5)+287,1,IWORK)
		IPRE1=IWORK(1)
		CALL XGETSECT(IFORMAT(5)+288,NTC1+1,RVALUE)
C  conversion loop - from clock pulses to microsecs
		DO 100 I=1,NTC1+1
		TEMP=FLOAT(RVALUE(I))/32.0*FLOAT(IPRE1)
		RVALUE(I)=ITEMP
 100		CONTINUE
C  non existant requests
	ELSE
		WRITE(6,*) ' No such REAL parameter as',NAME
	ENDIF
C
C
C
 999	RETURN
	END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
	SUBROUTINE XGETPARC(RUNID,NAME,CVALUE)
C
C  Gets character parameter(s) from RAW data file
C
	CHARACTER*(*) RUNID
	CHARACTER*(*) NAME
	CHARACTER*(*) CVALUE(*)
C
	LOGICAL FOUND
C
	INTEGER*4 ITEMP(20)
	CHARACTER*80 CTEMP
	EQUIVALENCE (ITEMP,CTEMP)
C
C
	CHARACTER*60 FILNAM
	INTEGER*4 VER1,IFORMAT(10),IVER(10),NDET,NMON,NEFF,
     +	NSEP,NTRG,NSP1,NTC1,ULEN,NPER,DATA_HEADER(32)
	LOGICAL OPENFILE
	COMMON/xcrpt_SPECIALS/ FILNAM,VER1,IFORMAT,IVER,
     +		NDET,NMON,NEFF,NSEP,NTRG,NSP1,NTC1,ULEN,OPENFILE,NPER,
     +		DATA_HEADER
C
c
	
C
C  decide whether it's CRPT or just a file
	IF (RUNID(4:8).EQ.'00000') THEN
		WRITE(6,6000)
 6000		FORMAT(' Accessing of current run not available')
		GOTO 999
	ENDIF
C
C  check name is valid & open file according to RUNID
	IF (FILNAM.NE.RUNID.OR.FILNAM.EQ.' ') THEN
		CALL XOPEN_FILE(RUNID,FOUND)
		IF (.NOT.FOUND) RETURN
	ENDIF
C
C  read variables into CVALUE
C ----------------------------
C
C  Format section
	IF (NAME.EQ.'HDR') THEN
		CALL XGETSECT(1,20,ITEMP)
		CVALUE(1)=CTEMP
C  Run section
	ELSEIF (NAME.EQ.'TITL') THEN
		CALL XGETSECT(IFORMAT(1)+2,20,ITEMP)
		CVALUE(1)=CTEMP
	ELSEIF (NAME.EQ.'USER') THEN
		CALL XGETSECT(IFORMAT(1)+22,20,ITEMP)
		CVALUE(1)=CTEMP(1:20)
		CVALUE(2)=CTEMP(21:40)
		CVALUE(3)=CTEMP(41:60)
		CVALUE(4)=CTEMP(61:80)
		CALL XGETSECT(IFORMAT(1)+42,20,ITEMP)
		CVALUE(5)=CTEMP(1:20)
		CVALUE(6)=CTEMP(21:40)
		CVALUE(7)=CTEMP(41:60)
		CVALUE(8)=CTEMP(61:80)
C  Instrument section
	ELSEIF (NAME.EQ.'NAME') THEN
		CALL XGETSECT(IFORMAT(2)+1,2,ITEMP)
		CVALUE(1)=CTEMP(1:8)
C  Notes section
	ELSEIF (NAME.EQ.'NOTE') THEN
		ILINES=(IFORMAT(8)-IFORMAT(7))/20
		IF (ILINES.LE.0) THEN
			CVALUE(1)=' No notes were made'
		ELSE
		    DO 100 I=1,ILINES
			CALL XGETSECT(IFORMAT(7)+(I-1)*20,20,ITEMP)
			CVALUE(I)=CTEMP
 100		    CONTINUE
		ENDIF
C  non existant requests
	ELSE
		WRITE(6,*) ' No such CHARACTER parameter as',NAME
	ENDIF
C
C
C
 999	RETURN
	END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
	SUBROUTINE XGETSECT(ISTART,ILONG,IVALUE)
C
C	Gets a section of the data file and places it in IVALUE - the routine 
C	that does the actual reading of the raw file.
C
C 	ISTART is the starting position in the file
C   	ILONG is the length of the required section
C
	DIMENSION IVALUE(ILONG),IWORK(128)
	COMMON/xcrpt_WORK/IWORK
C
C	
C calculate which block to start from, and first and last elements to use
	IBLOCK=(ISTART-1)/128 + 1
	IFIRST=ISTART - (IBLOCK-1)*128
	ILAST=IFIRST+ILONG-1
	IF (ILAST.GT.128) ILAST=128
C also keep a count of how many elements left to get
	ILEFT=ILONG
C
C
	J=1
c 5	TYPE *,' IBLOCK',IBLOCK
  5	READ(48,REC=IBLOCK) IWORK
C
	DO 10 I=IFIRST,ILAST
	IVALUE(J)=IWORK(I)
	J=J+1
 10	CONTINUE
C
	ILEFT=ILEFT-(ILAST-IFIRST+1)
	IF (ILEFT.LE.0) GOTO 999
	IBLOCK=IBLOCK+1
	IFIRST=1
	ILAST=ILEFT+IFIRST-1
	IF (ILAST.GT.128) ILAST=128
	GOTO 5
C
C
 999	RETURN
	END
C-------------------------------------------------------------------------
	SUBROUTINE XGETDAT(RUNID,IFSN,NOS,IDATA)
	IMPLICIT NONE
C
C  Gets data from a RAW file one or more spectra at a time
C
	CHARACTER*(*) RUNID
	INTEGER*4 IFSN,NOS,IDATA(1)
	LOGICAL FOUND
C
	INTEGER*4 IWORK(128)
	REAL*4 WORK(128)
	COMMON/xcrpt_WORK/IWORK
	EQUIVALENCE (IWORK,WORK)
C temporary parameters
	INTEGER*4 IBASE,ILONG
	INTEGER I,J,ISTART,STATUS,ICOMPRESS,IBUFFER(33000)
C
C
	CHARACTER*60 FILNAM
	INTEGER*4 VER1,IFORMAT(10),IVER(10),NDET,NMON,NEFF,
     +	NSEP,NTRG,NSP1,NTC1,ULEN,NPER,DATA_HEADER(32)
	LOGICAL OPENFILE
	COMMON/xcrpt_SPECIALS/ FILNAM,VER1,IFORMAT,IVER,
     +		NDET,NMON,NEFF,NSEP,NTRG,NSP1,NTC1,ULEN,OPENFILE,NPER,
     +		DATA_HEADER
C
	
C
C  decide whether it's CRPT or just a file
	IF (RUNID(4:8).EQ.'00000') THEN
		WRITE(6,6000)
 6000		FORMAT(' Accessing of current run not available')
		GOTO 999
	ENDIF
C
C  check name is valid & open file according to RUNID
	IF (FILNAM.NE.RUNID.OR.FILNAM.EQ.' ') THEN
		CALL XOPEN_FILE(RUNID,FOUND)
		IF (.NOT.FOUND) RETURN
	ENDIF
C
C
C  read data into IDAT ....remembering there are NTC1+1 time channels and
C -----------------------------------------------------------------------
C  NSP1+1 spectra and now also NPER periods
C
	IF (IFSN.LT.0.OR.IFSN.GT.((NSP1+1)*NPER)-1) THEN
		WRITE (6,6010) NSP1,IFSN
 6010	FORMAT(' Sorry, only ',I6,' spectra. You asked for',I6)
		RETURN
	ENDIF
	IF (IFSN+NOS-1.GT.((NSP1+1)*NPER)-1) THEN
		WRITE (6,6020) NSP1,IFSN+NOS-1
 6020	FORMAT(' Sorry, only ',I6,' spectra. You asked for up to',I6)
		RETURN
	ENDIF
	IF (VER1.EQ.1) THEN
		IBASE=IFORMAT(6)
	ELSE
		IBASE=IFORMAT(7)
	ENDIF
	IF (IVER(7).LE.1) THEN
C Original version of data section
		IBASE = IBASE + 1 + IFSN*(NTC1+1)
		ILONG = NOS*(NTC1+1)
C	TYPE *,' BASE and LONG',IBASE,ILONG
		CALL XGETSECT(IBASE,ILONG,IDATA)
	ELSE
C New version of data section (may be compressed and not necessarily consecutive
C First pick up data section header, in particular compression type.
CCCCCC		CALL XGETSECT(IBASE+1,32,DATA_header)
		ICOMPRESS=DATA_HEADER(1)
		IF (ICOMPRESS.EQ.0) THEN !uncompressed
		    IBASE = IBASE + DATA_HEADER(3) + IFSN*(NTC1+1)
		    ILONG=NOS*(NTC1+1)
		    CALL XGETSECT(IBASE,ILONG,IDATA)
		ELSEIF (ICOMPRESS.EQ.1) THEN ! byte relative compression
		    J=1
		    DO I=IFSN,IFSN+NOS-1
			ISTART=IBASE + DATA_HEADER(3) + 2*I
			ILONG=2
			CALL XGETSECT(ISTART,ILONG,IBUFFER)
			ISTART=IBASE + IBUFFER(2)
			ILONG=IBUFFER(1)
			CALL XGETSECT(ISTART,ILONG,IBUFFER)
			call xbyte_rel_expn(
     +			    IBUFFER,ILONG*4,1,IDATA(J),NTC1+1,STATUS)
			if (STATUS .NE. 1) THEN
			  WRITE(6,*) ' ERROR - decompressing spectrum',I
			  goto 999
			ENDIF	
			J = J + NTC1+1
		    ENDDO
		ELSE
		    WRITE(6,*) ' ERROR - cannot understand data section'
		ENDIF
	ENDIF
C
C
C
 999	RETURN
	END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
	SUBROUTINE XOPEN_FILE(RUNID,FOUND)
C
	IMPLICIT NONE
C
C  	Opens a RAW data file for reading. If a different file is already
C  	open it will be closed first.
C
	CHARACTER*(*) RUNID
	LOGICAL FOUND
C
	INTEGER*4 ITEMP(3),ISTATUS,I,J
C
C
	CHARACTER*60 FILNAM
	INTEGER*4 VER1,IFORMAT(10),IVER(10),NDET,NMON,NEFF,
     +	NSEP,NTRG,NSP1,NTC1,ULEN,NPER,DATA_HEADER(32)
	LOGICAL OPENFILE
	COMMON/xcrpt_SPECIALS/ FILNAM,VER1,IFORMAT,IVER,
     +		NDET,NMON,NEFF,NSEP,NTRG,NSP1,NTC1,ULEN,OPENFILE,NPER,
     +		DATA_HEADER
C
C
C  Check that the file exists
	INQUIRE(FILE=RUNID,EXIST=FOUND)
	IF (.NOT.FOUND) THEN
		WRITE(6,6000)RUNID
 6000		FORMAT(' File ',A,' not found.')
		RETURN
	ENDIF
C  If there is a file open already then close it
	IF (OPENFILE) CLOSE(48)
	OPENFILE=.FALSE.
	FILNAM=' '
C
C  Open new file:
	OPEN(UNIT=48,NAME=RUNID,STATUS='OLD',ACCESS='DIRECT',READONLY,
     +		RECL=128,SHARED,IOSTAT=ISTATUS)
	IF (ISTATUS.NE.0) THEN
		WRITE(6,6010)RUNID
 6010		FORMAT(' File could not be opened. Status=',a40)
		FOUND=.FALSE.
		RETURN
	ENDIF
C
C  Now pick out the vital parameters for future use
C
C  version number
	CALL XGETSECT(21,1,ITEMP)
	VER1=ITEMP(1)
C--  Format section
	CALL XGETSECT(22,10,IFORMAT)
C--  RUN section
	CALL XGETSECT(IFORMAT(1),1,IVER(1))
C--  Instrument section
	CALL XGETSECT(IFORMAT(2),1,IVER(2))
C  NDET,NMON,NEFF
	CALL XGETSECT(IFORMAT(2)+67,3,ITEMP)
	NDET=ITEMP(1)
	NMON=ITEMP(2)
	NEFF=ITEMP(3)
C--  SE section
	CALL XGETSECT(IFORMAT(3),1,IVER(3))
C  NSEP
	IF (IVER(3).EQ.1) THEN
		CALL XGETSECT(IFORMAT(3)+33,1,ITEMP)
	ELSE
		CALL XGETSECT(IFORMAT(3)+65,1,ITEMP)
	ENDIF
	NSEP=ITEMP(1)
C--  DAE section
	CALL XGETSECT(IFORMAT(4),1,IVER(4))
C--  TCB section
	CALL XGETSECT(IFORMAT(5),1,IVER(5))
C  NTRG
	CALL XGETSECT(IFORMAT(5)+1,1,ITEMP)
	NTRG=ITEMP(1)
	IF (NTRG.NE.1) THEN
		WRITE(6,6030) NTRG
 6030	FORMAT(' This version expects only 1 time regime. NTRG=',I9)
		FOUND=.FALSE.
		RETURN
	ENDIF
C  NPER
	CALL XGETSECT(IFORMAT(5)+3,1,ITEMP)
	NPER=ITEMP(1)
C  NSP1,NTC1
	CALL XGETSECT(IFORMAT(5)+260,2,ITEMP)
	NSP1=ITEMP(1)
	NTC1=ITEMP(2)
C--  USER section
	IF (VER1.NE.1)	THEN
		CALL XGETSECT(IFORMAT(6),1,IVER(6))
		CALL XGETSECT(IFORMAT(6)+1,1,ITEMP)
		ULEN=ITEMP(1)
	ENDIF
C--  DATA section
	IF (VER1.EQ.1) THEN
		CALL XGETSECT(IFORMAT(6),1,IVER(7))
	ELSE
		CALL XGETSECT(IFORMAT(7),1,IVER(7))
	ENDIF
C DATA section header
	do i=1,32
		data_header(i)=0
	enddo	
	if (iver(7).ge.2) then
		CALL XGETSECT(IFORMAT(7)+1,32,data_header)
	endif
C  finally store the file name and set flags to true
	FILNAM=RUNID
	FOUND=.TRUE.
	OPENFILE=.TRUE.
C	TYPE *,' FILE OPENED'
	RETURN
	END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
	SUBROUTINE XCLOSE_FILE()
C
C  	Forces the currently opened RAW file to be closed
C
C
	CHARACTER*60 FILNAM
	INTEGER*4 VER1,IFORMAT(10),IVER(10),NDET,NMON,NEFF,
     +	NSEP,NTRG,NSP1,NTC1,ULEN,NPER,DATA_HEADER(32)
	LOGICAL OPENFILE
	COMMON/xcrpt_SPECIALS/ FILNAM,VER1,IFORMAT,IVER,
     +		NDET,NMON,NEFF,NSEP,NTRG,NSP1,NTC1,ULEN,OPENFILE,NPER,
     +		DATA_HEADER
C
C	
	IF (OPENFILE) THEN
		CLOSE(UNIT=48)
		FILNAM=' '
	ENDIF
	RETURN
	END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
	SUBROUTINE XGETRUN(RUNID,IARRAY,IERROR)
C	
	IMPLICIT NONE
C
C  This routine returns the whole of a run file into the given array
C
	CHARACTER*(*) RUNID
	INTEGER*4 IARRAY(*)
	INTEGER*4 IERROR
C
	LOGICAL FOUND
	INTEGER start_of_data
C
C
	CHARACTER*60 FILNAM
	INTEGER*4 VER1,IFORMAT(10),IVER(10),NDET,NMON,NEFF,
     +	NSEP,NTRG,NSP1,NTC1,ULEN,NPER,DATA_HEADER(32)
	LOGICAL OPENFILE
	COMMON/xcrpt_SPECIALS/ FILNAM,VER1,IFORMAT,IVER,
     +		NDET,NMON,NEFF,NSEP,NTRG,NSP1,NTC1,ULEN,OPENFILE,NPER,
     +		DATA_HEADER
C
C
C	
C  check name is valid & open file according to RUNID
	IF (FILNAM.NE.RUNID.OR.FILNAM.EQ.' ') THEN
		CALL XOPEN_FILE(RUNID,FOUND)
		IF (.NOT.FOUND) THEN
			IERROR=1
			RETURN
		ENDIF
	ENDIF
C  want whole file except for log section
C get this in 2 parts: first the file up to the data section+data version number
	IF (VER1.EQ.1) THEN
	    start_of_data=IFORMAT(6)
	ELSE
	    start_of_data=IFORMAT(7)
	ENDIF
	CALL XGETSECT(1,start_of_data,IARRAY)
C	CALL XGETSECT(1,start_of_data+1+(NSP1+1)*(NTC1+1)*NPER,IARRAY)
C and now the data...
	CALL XGETDAT(RUNID,0,(NSP1+1)*NPER,IARRAY(start_of_data+1))
C for compressed file, log pointer points to wrong place.
C Recalculate for data version 2.
	IF (IARRAY(start_of_data).eq.2) then
		IARRAY(29)=IFORMAT(7)+1+(NSP1+1)*(NTC1+1)*NPER
	ENDIF
C we now have uncompressed data in a Version 1 format. Set data version to 1
	IARRAY(start_of_data)=1
 	IERROR=0
	RETURN
	END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
	SUBROUTINE PUTRUN(FILNAM,NSIZE,IARRAY,IERROR)
C
C	In genie-2 these routines were linked to in the standard fortran way -
C	as compiling modules in OpenGenie works slightly differently, they were
C	included in this same file.  In the open statement, the status of the
C	output file aws altered to "unknown" to allow overwriting of the file.
C
C
C  Puts a whole array into a RAW data file
C
	CHARACTER*(*) FILNAM
	INTEGER*4 NSIZE,IARRAY(NSIZE),IERROR
C
	LOGICAL EOD
	INTEGER NBLK
C
C	
	IUNIT=11
C  Calculate size in blocks of file
	NBLK=(NSIZE-1)/128 + 1
	OPEN(UNIT=IUNIT,FILE=FILNAM,STATUS='UNKNOWN',INITIALSIZE=NBLK,
     &		RECORDTYPE='FIXED',RECL=128,FORM='UNFORMATTED',
     &		ERR=999,IOSTAT=ISTATUS)
C  if it managed to get this far the file is OK
	EOD=.TRUE.
	CALL XWRITE_DATA(EOD,IUNIT,IARRAY,NSIZE)
	CLOSE(UNIT=IUNIT)
	IERROR=0
	RETURN
C
 999	IERROR=1
	RETURN
	END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
	SUBROUTINE XWRITE_DATA(EOD,IUNIT,IVAL,N)
C	
	IMPLICIT NONE
C
C  Routine to write out fixed length blocks(128 integers) to a file
C  It accepts an array of values and splits it into blocks. Whenever part
C  of a block is left over it is stored for blocking at a later run of 
C  the routine. To clear  out the block at the end of a file a flag,EOD should
C  be set to .TRUE.
C
C  input parameters:
C	EOD	logical, .TRUE. if this is the last record, otherwise .FALSE.
C	IUNIT	unit number to write to
C	IVAL	array containing data
C	N	number of values to be written
C
	LOGICAL EOD
	INTEGER N,IUNIT
	INTEGER IVAL(N)
	INTEGER ITOT,ILEFT,IPART,I,J,NBLOCKS,INEXT
	INTEGER IWORK(128)
C ensure IWORK cleared to start with and point to start of IWORK
	DATA IWORK/128*0/,INEXT/1/
C
	
C
C  calculate total number of values to be written
	ILEFT=N
	ITOT=ILEFT+INEXT-1
C
C  if at least 1 block then write out as many whole blocks as possible
	IF (ITOT.GT.128) THEN
C  First block is probably part IWORK, part IVAL
	    IPART=129-INEXT
	    WRITE(IUNIT) (IWORK(I),I=1,INEXT-1),(IVAL(I),I=1,IPART)
	    ITOT=ITOT-128
C  next blocks are all wholly IVAL
	    NBLOCKS=ITOT/128
	    IF (NBLOCKS.GT.0) THEN
		DO 10 I=1,NBLOCKS
		  WRITE(IUNIT) (IVAL(IPART+(I-1)*128+J),J=1,128)
 10		CONTINUE
	    ENDIF
C  then at the end there are probably a few values left over
	    ILEFT=ITOT-NBLOCKS*128
	    INEXT=1
	ENDIF
C  transfer the left overs into the temporary work space and  for next time
	IF (ILEFT.GT.0) THEN
	    DO 20 I=1,ILEFT
		IWORK(I+INEXT-1)=IVAL(N-ILEFT+I)
 20	    CONTINUE
	    INEXT=INEXT+ILEFT
	ENDIF
C  However if I've been told that this is the last block of this file
C  clear it out totally
	IF (EOD.AND.ILEFT.GT.0) THEN
	    IF (INEXT.GT.1)WRITE(IUNIT)(IWORK(I),I=1,INEXT-1)
	    INEXT=1
	ENDIF
	RETURN
	END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
	SUBROUTINE XBYTE_REL_EXPN(INDATA,NIN,NFROM,OUTDATA,NOUT,ISTATUS)
	IMPLICIT NONE
C
C Expansion of byte-relative format into 32bit format
C
C Each integer is stored relative to the previous value in byte form.The first
C is relative to zero. This allows for numbers to be within + or - 127 of
C the previous value. Where a 32bit integer cannot be expressed in this way a 
C special byte code is used (-128) and the full 32bit integer stored elsewhere.
C The final space used is (NIN-1)/4 +1 + NEXTRA longwords, where NEXTRA is the
C number of extra longwords used in giving absolute values.
C
C
C Type definitions
C   Passed parameters
	INTEGER NIN
	BYTE	INDATA(NIN)	!Array of NIN compressed bytes
	INTEGER NFROM		!pass back data from this 32bit word onwards
C
	INTEGER NOUT		!Number of 32bit words expected
	INTEGER OUTDATA(NOUT)	!outgoing expanded data
	INTEGER ISTATUS		!Status return
				!	=1  no problems!
				!	=3  NOUT .lt.NIN/5
				!	=2  NIN .le.0
				!	=4  NOUT .gt.NIN
				!	=6  number of channels lt NOUT
C   Local definitions
	INTEGER I,J,ITEMP
	BYTE BTEMP(4)
	EQUIVALENCE (ITEMP,BTEMP)	!For byte UNpacking
c	
c
C Assume innocent until proven guilty
	ISTATUS=1
c First check no slip-ups in the input parameters
	IF (NIN.LE.0) THEN
		ISTATUS=2
		GOTO 100
	ENDIF
	IF (NOUT+NFROM-1.gt.NIN) THEN
		ISTATUS=4
		GOTO 100
	ENDIF
C
C Set initial absolute value to zero and channel counter to zero
	ITEMP=0
	J=0
C
C Loop over all expected 32bit integers
	DO I=1,NFROM+NOUT-1
	    J=J+1
	    IF (J.GT.NIN) THEN	!check there are enough bytes
		ISTATUS=6
		GOTO 100
	    ENDIF
C
C if number is contained in a byte
	    IF (INDATA(J).NE.-128) THEN
		ITEMP=ITEMP+INDATA(J)	!add in offset to base
	    ELSE
C Else skip marker and pick up new absolute value
		IF (J+4.GT.NIN) THEN	!check there are enough bytes
		    ISTATUS=6
		    GOTO 100
		ENDIF
		BTEMP(1)=INDATA(J+1)	!unpack 4 bytes
		BTEMP(2)=INDATA(J+2)
		BTEMP(3)=INDATA(J+3)
		BTEMP(4)=INDATA(J+4)
		J=J+4
	    ENDIF
C update current value
	IF (I.GE.NFROM) THEN
	    OUTDATA(I-NFROM+1)=ITEMP
	ENDIF
	ENDDO
c
c
C expansion OK, but excessive number of bytes given to the routine
 100	IF (NOUT.lt.NIN/5) ISTATUS=3
C	TYPE *,'J=',J,' I=',I
	RETURN
	END
* DEC/CMS REPLACEMENT HISTORY, Element BYTE_REL_EXPN.FOR
* *1    11-AUG-1989 09:28:08 KJK 'New routine coming in at 2.4.5'
* DEC/CMS REPLACEMENT HISTORY, Element BYTE_REL_EXPN.FOR
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
