CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C      Module UNWRAP
C     ---------------
C
C  A program to "unwrap" the monitor spectrum on IRIS. (DSS, 1-MAY-1991)
C  A minimum monitor-level added for "safe" division.  (DSS,16-MAY-1991)
C  Left-right unwrap: if minimum is in the first 1/3 of the window do a 
C  left--> right unwrap, otherwise left --> right.     (DSS,17-MAY-1991)
C  Corrected last histogram XOUT point.                (DSS,23-JUL-1991)
C  Adapted for use as an OpenGenie module.             (PJM,23-JUL-1996)
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      SUBROUTINE UNWRAP(W1_get,W1_put)
      EXTERNAL W1_get,W1_put
      INTEGER mn, lptin, lptout
      PARAMETER (mn=6010)
      INCLUDE 'genie_modules.inc'
      REAL Xin(mn),Yin(mn),Ein(mn),Xout(mn),Yout(mn),Eout(mn)
C
      REAL            FTSMTH(10000),FTWRK(10000)
      LOGICAL         LFLIP
      COMMON /CHICOM/ SCLA,SCLM,SCLC,NS,NFIT
      COMMON /data/   Xout,Yout,Eout
C
C *** check library version
      IF (MODULE_VERSION_OK(GENIE_MAJOR, GENIE_MINOR) .EQ. 0) RETURN
      NFIT=30
      NFIT2=2*NFIT
      LFLIP=.FALSE.
C
      call module_get_int(W1_get, 'ntc', lptin)	
      call module_get_real_array(W1_get, 'x', Xin, lptin)	
      call module_get_real_array(W1_get, 'y', Yin, lptin)	
      call module_get_real_array(W1_get, 'e', Ein, lptin)	
C      
      NFFT=INT(LOG(FLOAT(LPTIN-1))/LOG(2.0))+1
      NFFT=2**NFFT
      IF (NFFT.GT.10000) STOP ' *** Sorry not enough array space!'
      NPAD=(NFFT-LPTIN)/2
      CALL GASMTH(FTSMTH,NFFT,LPTIN)
      CALL VRFILL(FTWRK,0.0,NFFT)
      CALL VCOPY(YIN,FTWRK,LPTIN-2)
      CALL VRFILL(FTWRK(LPTIN-1),YIN(LPTIN-2),NPAD)
      CALL VRFILL(FTWRK(NFFT-NPAD+1),YIN(1),NPAD)
      CALL FOUR2(FTWRK,NFFT,1,1,0)
      CALL CMPMLT(FTWRK,FTSMTH,NFFT/2+1,FTWRK)
      CALL FOUR2(FTWRK,NFFT,1,-1,-1)
      CALL FNDMIN(FTWRK,LPTIN-2,IBREAK,FMAX)
      IF (IBREAK.GT.NFIT2 .AND. (LPTIN-IBREAK).GT.NFIT2) THEN
        IF ((FLOAT(IBREAK)/FLOAT(LPTIN)).LE.0.33) LFLIP=.TRUE.
        CALL UNWRP(XIN,XOUT,YIN,YOUT,EIN,EOUT,LPTIN,LPTOUT,IBREAK,LFLIP)
        CALL SCALUP(XOUT,YOUT,EOUT,LPTOUT,IBREAK,LFLIP)
      ELSE
        LPTOUT=LPTIN
        CALL VCOPY(XIN,XOUT,LPTOUT+1)
        CALL VCOPY(YIN,YOUT,LPTOUT+1)
        CALL VCOPY(EIN,EOUT,LPTOUT+1)
      ENDIF
      CALL VRFILL(FTWRK,0.0,NFFT)
      CALL VCOPY(YOUT,FTWRK,LPTOUT)
      CALL VRFILL(FTWRK(LPTIN),YOUT(LPTOUT-1),NPAD)
      CALL VRFILL(FTWRK(NFFT-NPAD+1),YOUT(1),NPAD)
      CALL MSAFE(FTWRK,FMAX,NFFT)
      CALL FOUR2(FTWRK,NFFT,1,1,0)
      CALL CMPMLT(FTWRK,FTSMTH,NFFT/2+1,FTWRK)
      CALL FOUR2(FTWRK,NFFT,1,-1,-1)
      CALL VCOPY(FTWRK,YOUT,LPTOUT+1)
      XOUT(LPTOUT+1)=2.0*XOUT(LPTOUT)-XOUT(LPTOUT-1)
C
      call module_put_int(W1_put, 'ntc', lptout)	
      call module_put_real_array(W1_put, 'x', Xout, lptout)	
      call module_put_real_array(W1_put, 'y', Yout, lptout)	
      call module_put_real_array(W1_put, 'e', Eout, lptout)
      call module_information('Monitor spectrum unwrapped.')
C      
      RETURN	
      END

C******************************************************************************
C
      SUBROUTINE MSAFE(F,FMAX,N)
C     --------------------------
C                                                         
      REAL F(*)
C
      FSAFE=FMAX/50.0
      DO 10 I=1,N
  10    IF (F(I).LT.FSAFE) F(I)=FSAFE
      END
C
C***<>******************************************************************
C
      SUBROUTINE GASMTH(FTSMTH,NFFT,LPTIN)
C     ------------------------------------
C
      REAL FTSMTH(*)
C
      CALL VRFILL(FTSMTH,0.0,NFFT)
      FWHM=10.0
      SIG=FWHM/2.35
      SIG2=1.0/(2.0*SIG*SIG)
      FTSMTH(1)=1.0
      SUM=FTSMTH(1)
      DO 10 I=2,NFFT/2+1
        X=FLOAT(I-1)
        A=EXP(-X*X*SIG2)
        FTSMTH(I)=A
        FTSMTH(NFFT+2-I)=A
        SUM=SUM+FTSMTH(I)+FTSMTH(NFFT+2-I)
  10  CONTINUE
      FNORM=1.0/(FLOAT(NFFT)*SUM)
      DO 20 I=1,NFFT
  20    FTSMTH(I)=FNORM*FTSMTH(I)
      CALL FOUR2(FTSMTH,NFFT,1,1,0)
      END
C
      SUBROUTINE CMPMLT(A,B,N,C)
C     -------------------------
C
      COMPLEX A(*),B(*),C(*)
C
      DO 10 I=1,N
  10    C(I)=A(I)*B(I)
      END
C
      SUBROUTINE FNDMIN(F,N,IBREAK,FMAX)
C     ----------------------------------
C
      REAL F(*)
C
      IMIN=1
      FMIN=F(1)
      FMAX=F(1)
      DO 10 I=1,N
        IF (F(I).LT.FMIN) THEN
          FMIN=F(I)
          IMIN=I
        ELSEIF (F(I).GT.FMAX) THEN
          FMAX=F(I)
        ENDIF
  10  CONTINUE
      IBREAK=IMIN
      END
C
      SUBROUTINE UNWRP(XIN,XOUT,YIN,YOUT,EIN,EOUT,LPTIN,LPTOUT,IB,LFLIP)
C     ------------------------------------------------------------------
C
      REAL    XIN(*),XOUT(*),YIN(*),YOUT(*),EIN(*),EOUT(*)
      LOGICAL LFLIP
C
      XSHIFT=XIN(LPTIN)-XIN(1)+XIN(2)-XIN(1)
      LPTOUT=LPTIN
      J=1
      DO 10 I=IB,LPTIN
        YOUT(J)=YIN(I)
        XOUT(J)=XIN(I)-XSHIFT
        EOUT(J)=EIN(I)
        J=J+1
  10  CONTINUE
      JB=J
      DO 20 I=1,IB-1
        YOUT(J)=YIN(I)
        XOUT(J)=XIN(I)
        EOUT(J)=EIN(I)
        J=J+1
  20  CONTINUE
      YOUT(JB-1)=0.5*(YOUT(JB-2)+YOUT(JB-3))
      YOUT(JB)=0.5*(YOUT(JB+1)+YOUT(JB+2))
      XOUT(LPTOUT+1)=2.0*XOUT(LPTOUT)-XOUT(LPTOUT-1)
      YOUT(LPTOUT+1)=YOUT(LPTOUT)
      EOUT(LPTOUT+1)=EOUT(LPTOUT)
      IF (LFLIP) THEN
        PERIOD=XOUT(LPTOUT+1)-XOUT(1)
        DO 30 I=1,LPTOUT
  30      XOUT(I)=XOUT(I)+PERIOD
      ENDIF
      END
C
      SUBROUTINE SCALUP(X,Y,E,N,IB,LFLIP)
C     -----------------------------------
C
      REAL             X(*),Y(*),E(*)
      REAL             V(12),EX(9),C(4)
      INTEGER          IR(4)
      LOGICAL          LGOOD,LFLIP
      COMMON  /CHICOM/ SCLA,SCLM,SCLC,NS,NFIT
C
      SCLA=1.0
      NS=N-IB+1
      IP=NS-NFIT-1
      CALL FITLIN(E(IP),Y(IP),X(IP),NFIT,SCLM,SCLC,LGOOD)
      IF (LGOOD) THEN
        CALL VRFILL(V,1.0,3)
        CALL SIMPLEX(V,3,0.05,EX,C,IR,1000)
        SCL=EX(1)*SCLA
        IF (LFLIP) SCL=1.0/(ABS(SCL)+1.0E-10)
      ELSE
        SCL=1.0
      ENDIF
c      WRITE(*,*)
c      WRITE(*,100) SCL
 100  FORMAT(' Estimate of scaling constant = ',F7.2)
      XSHIFT=(X(NS)-X(1))/1000.0
      IF (LFLIP) XSHIFT=(X(N)-X(NS))/1000.0
c      WRITE(*,*)
c      WRITE(*,110) XSHIFT
 110  FORMAT(' unwrap> Time interval for shifted points = ',F7.2,' mS')
c     WRITE(*,*)
      IMIN=1
      IMAX=NS
      IF (LFLIP) THEN
        IMIN=NS+1
        IMAX=N+1
      ENDIF
      DO 10 I=IMIN,IMAX
        Y(I)=Y(I)*SCL
        E(I)=E(I)*SCL
  10  CONTINUE
      END
C
      SUBROUTINE FITLIN(E,Y,X,N,XM,XC,LGOOD)
C     --------------------------------------
C
      REAL    E(*),Y(*),X(*)
      LOGICAL LGOOD
C
      A=0.0
      B=0.0
      C=0.0
      D=0.0
      F=0.0
      DO 10 K=1,N
        IF (E(K).GT.1.0E-10) THEN
          SIGK=1.0/E(K)
          SIGX=SIGK*X(K)
          SIGY=SIGK*Y(K)
          A=A+SIGX*SIGX
          B=B+SIGX*SIGK
          C=C+SIGK*SIGK
          D=D+SIGX*SIGY
          F=F+SIGY*SIGK
        ENDIF
  10  CONTINUE
      DENOM=(A*C-B*B)
      IF (ABS(DENOM).GT.1.0E-10) THEN
        XM=+(D*C-B*F)/DENOM
        XC=-(B*D-A*F)/DENOM
        LGOOD=.TRUE.
      ELSE
        LGOOD=.FALSE.
      ENDIF
      END
C
      FUNCTION CCHI(V)
C     ----------------
C
	INTEGER mn
	PARAMETER (mn=6010)
	REAL Xout(mn),Yout(mn),Eout(mn)
      REAL             V(*)
      COMMON  /CHICOM/ SCLA,SCLM,SCLC,NS,NFIT
	COMMON/data/Xout,Yout,Eout
C
      SCL=SCLA*V(1)
      XM=SCLM*V(2)
      XC=SCLC*V(3)
      CHI=0.0
      DO 10 I=NS-NFIT,NS
        IF (EOUT(I).GE.1.0E-10) THEN
          YF=XM*XOUT(I)+XC
          DIF=(SCL*YOUT(I)-YF)/(SCL*EOUT(I))
          CHI=CHI+DIF*DIF
        ENDIF
  10  CONTINUE
      DO 20 I=NS+1,NS+NFIT
        IF (EOUT(I).GE.1.0E-10) THEN
          YF=XM*XOUT(I)+XC
          DIF=(YOUT(I)-YF)/EOUT(I)
          CHI=CHI+DIF*DIF
        ENDIF
  20  CONTINUE
      CCHI=CHI/FLOAT(2*NFIT+1)
      END
C
C***<Simplex>***********************************************************
C
      SUBROUTINE SIMPLEX(V,N,D,EX,C,IR,MX)
C     ------------------------------------
C
C     INPUT: N = No. of parameters.
C            V = Nx(N+1) REAL ARRAY, with the fisrt N representing
C		 an initial guess.
C	     D = Initial step size to generate the Simplex.
C	     EX= Work space of 3xN REAL ARRAY.
C	     C = REAL ARRAY of N+1 numbers. 
C	     IR= INTEGER ARRAY of N+1 numbers, used for ranking.
C            MX= Maximum no. of iterations
C
C     OUTPUT: The vector which minimises the function will be
C	      found in the first N elements of the real array EX.
C	      
C     OTHER REQUIREMENTS: The user must provide a FUNCTION CCHI(X)
C			  which evaluates the function to be minimised
C			  given a vector X.
C
      REAL    V(N,*),EX(N,*),C(*)
      INTEGER IR(*)
      LOGICAL RESTART,SLOW
      DATA    ALPHA,BETA,GAMA/1.0,0.5,2.0/
C
      SLOW=.FALSE.
      N3=3*N
      CAIM=0.0
      C(1)=CCHI(V(1,1))
      CALL SIMP0(V,D,C,IR,N)
      CMIN=C(IR(N+1))
C
      ITER=0
      NOLUCK=0
      IRSTRT=0
  10  ITER=ITER+1
      NOLUCK=NOLUCK+1
      IF (ITER.GE.MX .OR. NOLUCK.GT.100) THEN
        CALL SSORT(C,IR,N)
        CALL VCOPY(V(1,IR(N+1)),EX,N)
        IF (ITER.LT.250) THEN
          NOLUCK=0
          DSMALL=D/10.0
          CALL VCOPY(EX,V,N)
          C(1)=CCHI(V(1,1))
          CALL SIMP0(V,DSMALL,C,IR,N)
          CMIN=C(IR(N+1))
        ELSE
          RETURN
        ENDIF
      ENDIF
      CALL VRFILL(EX,0.,N3)
      CALL XCENT(V,EX(1,3),IR(1),N)
      CALL XZERO(V(1,IR(1)),EX,EX(1,3),N,ALPHA)
      C0=CCHI(EX)
      CL=C(IR(N+1))
      CH=C(IR(1))
      CS=C(IR(2))
      IF (C0.LT.CL) THEN
        CALL EXPAND(CL,EX,EX(1,2),EX(1,3),N,C0,GAMA)
      ELSEIF (C0.GT.CS) THEN 
        CALL CONTRACT(CH,CS,C0,C00,EX,EX(1,2),EX(1,3),V(1,IR(1)),BETA,N)
        IF (C00.LT.CH.AND.C00.LT.C0) THEN
           CALL VCOPY(EX(1,2),EX,N)
           C0=C00
        ELSE
           CALL CONT2(IR(N+1),V,C,N,IR)
           IRSTRT=IRSTRT+1
           IF (C(IR(N+1)).LT.CMIN) THEN
             CMIN=C(IR(N+1))
           ENDIF
           IF (IRSTRT.GE.5) THEN
             NOLUCK=0
             CALL SSORT(C,IR,N)
             CALL VCOPY(V(1,IR(N+1)),EX,N)
             RETURN
          ENDIF
        ENDIF
      ENDIF
C
      CALL VCOPY(EX,V(1,IR(1)),N)
      C(IR(1))=C0
      CALL SSORT(C,IR,N)
      IF (C(IR(N+1)).LT.CMIN) THEN
        DROP=(CMIN-C(IR(N+1)))/CMIN
        IF (ABS(DROP).LT.1.0E-4) SLOW=.TRUE.
        NOLUCK=0
        CMIN=C(IR(N+1))
      ENDIF
      IF (CMIN.GT.CAIM .AND. .NOT. SLOW) GOTO 10
      CALL VCOPY(V(1,IR(N+1)),EX,N)
      END
C
      SUBROUTINE SIMP0(V,D,C,IR,N)
C     ----------------------------
C
      REAL    V(N,*),C(*)
      INTEGER IR(*)
C
      DO 1 I=2,N+1
        CALL VCOPY(V,V(1,I),N)
        V(I-1,I)=V(I-1,I)+D
        C(I)= CCHI(V(1,I))
   1  CONTINUE
      CALL SSORT(C,IR,N)
      END
C
      SUBROUTINE SSORT(C,IR,N)
C     ------------------------
C
      REAL    C(*)
      INTEGER IR(*)
C
      IR(1)=1
      DO 20 J=2,N+1
        DO 10 I=1,J-1
          IF (C(J).LT.C(IR(I))) GOTO 10
          DO 1 II=1,J-I
   1        IR(J+1-II)=IR(J-II)
          IR(I)=J
          GOTO 20
  10    CONTINUE
        IR(J)=J
  20  CONTINUE
      END
C
      SUBROUTINE XCENT(V,XC,IH,N)
C     ---------------------------
C
      REAL V(N,*),XC(*)
C
      XNORM=1.0/FLOAT(N)
      DO 20 J=1,N+1
        IF (J.EQ.IH) GOTO 20
        DO 10 I=1,N
  10      XC(I)=XC(I)+V(I,J)
  20  CONTINUE
      DO 30 I=1,N
  30    XC(I)=XC(I)*XNORM
      END
C
      SUBROUTINE XZERO(XH,X0,XC,N,A)
C     ------------------------------
C
      REAL XH(*),X0(*),XC(*)
C
      DO 1 I=1,N
   1    X0(I)=A*(XC(I)-XH(I))+XC(I)
      END
C
      SUBROUTINE EXPAND(CL,X0,X00,XC,N,C0,G)
C     --------------------------------------
C
      REAL X0(*),X00(*),XC(*)
C
      DO 1 I=1,N
   1    X00(I)=G*(X0(I)-XC(I))+XC(I)
      C00=CCHI(X00)
      IF (C00.LT.CL) THEN
        CALL VCOPY(X00,X0,N)
        C0=C00
      ENDIF
      END
C
      SUBROUTINE CONTRACT(CH,CS,C0,C00,X0,X00,XC,XH,B,N)
C     --------------------------------------------------
C
      REAL X0(*),X00(*),XC(*),XH(*)
C
      IF (C0.LT.CH) THEN
        DO 1 I=1,N
   1      X00(I)=B*(X0(I)-XC(I))+XC(I)
      ELSE
        DO 10 I=1,N
  10      X00(I)=B*(XH(I)-XC(I))+XC(I)
      ENDIF
      C00=CCHI(X00)
      END
C
      SUBROUTINE CONT2(K,V,C,N,IR)
C     ----------------------------
C
      REAL     V(N,*),C(*)
      INTEGER  IR(*)
C
      DO 20 J=1,N+1
        IF (J.EQ.K) GOTO 20
        DO 10 I=1,N
          V(I,J)=0.5*(V(I,J)+V(I,K))
          C(J)=CCHI(V(1,J))
  10    CONTINUE
  20  CONTINUE
      CALL SSORT(C,IR,N)
      END
C
C***<FFT routine>*******************************************************
C
            SUBROUTINE FOUR2 (DATA,N,NDIM,ISIGN,IFORM)
C     COOLEY-TUKEY FAST FOURIER TRANSFORM IN USASI BASIC FORTRAN.
C     MULTI-DIMENSIONAL TRANSFORM, EACH DIMENSION A POWER OF TWO,
C     COMPLEX OR REAL DATA.
C     TRANSFORM(K1,K2,...) = SUM(DATA(J1,J2,...)*EXP(ISIGN*2*PI*SQRT(-1)
C     *((J1-1)*(K1-1)/N(1)+(J2-1)*(K2-1)/N(2)+...))), SUMMED FOR ALL
C     J1 AND K1 FROM 1 TO N(1), J2 AND K2 FROM 1 TO N(2),
C     ETC. FOR ALL NDIM SUBSCRIPTS.  NDIM MUST BE POSITIVE AND
C     EACH N(IDIM) MUST BE A POWER OF TWO.  ISIGN IS +1 OR -1.
C     LET NTOT = N(1)*N(2)*...*N(NDIM).  THEN A -1 TRANSFORM
C     FOLLOWED BY A +1 ONE (OR VICE VERSA) RETURNS NTOT
C     TIMES THE ORIGINAL DATA.  IFORM = 1, 0 OR -1, AS DATA IS
C     COMPLEX, REAL OR THE FIRST HALF OF A COMPLEX ARRAY.	TRANSFORM
C     VALUES ARE RETURNED TO ARRAY DATA.  THEY ARE COMPLEX, REAL OR
C     THE FIRST HALF OF A COMPLEX ARRAY, AS IFORM = 1, -1 OR 0.
C     THE TRANSFORM OF A REAL ARRAY (IFORM = 0) DIMENSIONED N(1) BY N(2)
C     BY ... WILL BE RETURNED IN THE SAME ARRAY, NOW CONSIDERED TO
C     BE COMPLEX OF DIMENSIONS N(1)/2+1 BY N(2) BY ....  NOTE THAT IF
C     IFORM = 0 OR -1, N(1) MUST BE EVEN, AND ENOUGH ROOM MUST BE
C     RESERVED.  THE MISSING VALUES MAY BE OBTAINED BY COMPLEX CONJUGA-
C     TION.	THE REVERSE TRANSFORMATION, OF A HALF COMPLEX ARRAY DIMEN-
C     SIONED N(1)/2+1 BY N(2) BY ..., IS ACCOMPLISHED BY SETTING IFORM
C     TO -1.  IN THE N ARRAY, N(1) MUST BE THE TRUE N(1), NOT N(1)/2+1.
C     THE TRANSFORM WILL BE REAL AND RETURNED TO THE INPUT ARRAY.
C     RUNNING TIME IS PROPORTIONAL TO NTOT*LOG2(NTOT), RATHER THAN
C     THE NAIVE NTOT**2.  FURTHERMORE, LESS ERROR IS BUILT UP.
C     WRITTEN BY NORMAN BRENNER OF MIT LINCOLN LABORATORY, JUNE 1968.
C     SEE-- IEEE AUDIO TRANSACTIONS (JUNE 1967), SPECIAL ISSUE ON FFT.
      DIMENSION DATA(1), N(1)
      NTOT=1
      DO 10 IDIM=1,NDIM
 10   NTOT=NTOT*N(IDIM)
      IF (IFORM) 70,20,20
 20   NREM=NTOT
      DO 60 IDIM=1,NDIM
      NREM=NREM/N(IDIM)
      NPREV=NTOT/(N(IDIM)*NREM)
      NCURR=N(IDIM)
      IF (IDIM-1+IFORM) 30,30,40
 30   NCURR=NCURR/2
 40   CALL BOTRV (DATA,NPREV,NCURR,NREM)
      CALL COOL2 (DATA,NPREV,NCURR,NREM,ISIGN)
      IF (IDIM-1+IFORM) 50,50,60
 50   CALL FOXRL (DATA,N(1),NREM,ISIGN,IFORM)
      NTOT=(NTOT/N(1))*(N(1)/2+1)
 60   CONTINUE
      RETURN
 70   NTOT=(NTOT/N(1))*(N(1)/2+1)
      NREM=1
      DO 100 JDIM=1,NDIM
      IDIM=NDIM+1-JDIM
      NCURR=N(IDIM)
      IF (IDIM-1) 80,80,90
 80   NCURR=NCURR/2
      CALL FOXRL (DATA,N(1),NREM,ISIGN,IFORM)
      NTOT=NTOT/(N(1)/2+1)*N(1)
 90   NPREV=NTOT/(N(IDIM)*NREM)
      CALL BOTRV (DATA,NPREV,NCURR,NREM)
      CALL COOL2 (DATA,NPREV,NCURR,NREM,ISIGN)
 100  NREM=NREM*N(IDIM)
      RETURN
      END
      SUBROUTINE FOXRL (DATA,N,NREM,ISIGN,IFORM)
C     FOR IFORM = 0, CONVERT THE TRANSFORM OF A DOUBLED-UP REAL ARRAY,
C     CONSIDERED COMPLEX, INTO ITS TRUE TRANSFORM.  SUPPLY ONLY THE
C     FIRST HALF OF THE COMPLEX TRANSFORM, AS THE SECOND HALF HAS
C     CONJUGATE SYMMETRY.  FOR IFORM = -1, CONVERT THE FIRST HALF
C     OF THE TRUE TRANSFORM INTO THE TRANSFORM OF A DOUBLED-UP REAL
C     ARRAY.  N MUST BE EVEN.
C     USING COMPLEX NOTATION AND SUBSCRIPTS STARTING AT ZERO, THE
C     TRANSFORMATION IS--
C     DIMENSION DATA(N,NREM)
C     ZSTP = EXP(ISIGN*2*PI*I/N)
C     DO 10 I2=0,NREM-1
C     DATA(0,I2) = CONJ(DATA(0,I2))*(1+I)
C     DO 10 I1=1,N/4
C     Z = (1+(2*IFORM+1)*I*ZSTP**I1)/2
C     I1CNJ = N/2-I1
C     DIF = DATA(I1,I2)-CONJ(DATA(I1CNJ,I2))
C     TEMP = Z*DIF
C     DATA(I1,I2) = (DATA(I1,I2)-TEMP)*(1-IFORM)
C 10  DATA(I1CNJ,I2) = (DATA(I1CNJ,I2)+CONJ(TEMP))*(1-IFORM)
C     IF I1=I1CNJ, THE CALCULATION FOR THAT VALUE COLLAPSES INTO
C     A SIMPLE CONJUGATION OF DATA(I1,I2).
      DIMENSION DATA(*)
      TWOPI=6.283185307*FLOAT(ISIGN)
      IP0=2
      IP1=IP0*(N/2)
      IP2=IP1*NREM
      IF (IFORM) 10,70,70
C     PACK THE REAL INPUT VALUES (TWO PER COLUMN)
 10   J1=IP1+1
      DATA(2)=DATA(J1)
      IF (NREM-1) 70,70,20
 20   J1=J1+IP0
      I2MIN=IP1+1
      DO 60 I2=I2MIN,IP2,IP1
      DATA(I2)=DATA(J1)
      J1=J1+IP0
      IF (N-2) 50,50,30
 30   I1MIN=I2+IP0
      I1MAX=I2+IP1-IP0
      DO 40 I1=I1MIN,I1MAX,IP0
      DATA(I1)=DATA(J1)
      DATA(I1+1)=DATA(J1+1)
 40   J1=J1+IP0
 50   DATA(I2+1)=DATA(J1)
 60   J1=J1+IP0
 70   DO 80 I2=1,IP2,IP1
      TEMPR=DATA(I2)
      DATA(I2)=DATA(I2)+DATA(I2+1)
 80   DATA(I2+1)=TEMPR-DATA(I2+1)
      IF (N-2) 200,200,90
 90   THETA=TWOPI/FLOAT(N)
      SINTH=SIN(THETA/2.)
      ZSTPR=-2.*SINTH*SINTH
      ZSTPI=SIN(THETA)
      ZR=(1.-ZSTPI)/2.
      ZI=(1.+ZSTPR)/2.
      IF (IFORM) 100,110,110
 100  ZR=1.-ZR
      ZI=-ZI
 110  I1MIN=IP0+1
      I1MAX=IP0*(N/4)+1
      DO 190 I1=I1MIN,I1MAX,IP0
      DO 180 I2=I1,IP2,IP1
      I2CNJ=IP0*(N/2+1)-2*I1+I2
      IF (I2-I2CNJ) 150,120,120
 120  IF (ISIGN*(2*IFORM+1)) 130,140,140
 130  DATA(I2+1)=-DATA(I2+1)
 140  IF (IFORM) 170,180,180
 150  DIFR=DATA(I2)-DATA(I2CNJ)
      DIFI=DATA(I2+1)+DATA(I2CNJ+1)
      TEMPR=DIFR*ZR-DIFI*ZI
      TEMPI=DIFR*ZI+DIFI*ZR
      DATA(I2)=DATA(I2)-TEMPR
      DATA(I2+1)=DATA(I2+1)-TEMPI
      DATA(I2CNJ)=DATA(I2CNJ)+TEMPR
      DATA(I2CNJ+1)=DATA(I2CNJ+1)-TEMPI
      IF (IFORM) 160,180,180
 160  DATA(I2CNJ)=DATA(I2CNJ)+DATA(I2CNJ)
      DATA(I2CNJ+1)=DATA(I2CNJ+1)+DATA(I2CNJ+1)
 170  DATA(I2)=DATA(I2)+DATA(I2)
      DATA(I2+1)=DATA(I2+1)+DATA(I2+1)
 180  CONTINUE
      TEMPR=ZR-.5
      ZR=ZSTPR*TEMPR-ZSTPI*ZI+ZR
 190  ZI=ZSTPR*ZI+ZSTPI*TEMPR+ZI
C     RECURSION SAVES TIME, AT A SLIGHT LOSS IN ACCURACY.	IF AVAILABLE,
C     USE DOUBLE PRECISION TO COMPUTE ZR AND ZI.
 200  IF (IFORM) 270,210,210
C     UNPACK THE REAL TRANSFORM VALUES (TWO PER COLUMN)
 210  I2=IP2+1
      I1=I2
      J1=IP0*(N/2+1)*NREM+1
      GO TO 250
 220  DATA(J1)=DATA(I1)
      DATA(J1+1)=DATA(I1+1)
      I1=I1-IP0
      J1=J1-IP0
 230  IF (I2-I1) 220,240,240
 240  DATA(J1)=DATA(I1)
      DATA(J1+1)=0.
 250  I2=I2-IP1
      J1=J1-IP0
      DATA(J1)=DATA(I2+1)
      DATA(J1+1)=0.
      I1=I1-IP0
      J1=J1-IP0
      IF (I2-1) 260,260,230
 260  DATA(2)=0.
 270  RETURN
      END
      SUBROUTINE BOTRV(DATA,NPREV,N,NREM)
C	SHUFFLE THE DATA BY 'BIT REVERSAL'.
C	DIMENSION DATA(NPREV,N,NREM)
C	DATA(I1,I2REV,I3) = DATA(I1,I2,I3), FOR ALL I1 FROM 1 TO
C	NPREV, ALL I2 FROM 1 TO N (WHICH MUST BE A POWER OF TWO), AND
C	ALL I3 FROM 1 TO NREM, WHERE I2REV-1 IS THE BITWISE REVERSAL
C	OF I2-1.  FOR EXAMPLE, N = 32, I2-1 = 10011 AND I2REV-1 =
C	11001.
C
      DIMENSION DATA(1)
      COMPLEX DATA,TEMP
      IP0=1
      IP1=IP0*NPREV
      IP4=IP1*N
      IP5=IP4*NREM
      I4REV=1
      I4MAX=IP4
      DO 60 I4=1,I4MAX,IP1
      IF(I4-I4REV)10,30,30
   10 I1MAX=I4+IP1-IP0
      DO 20 I1=I4,I1MAX,IP0
      DO 20 I5=I1,IP5,IP4
      I5REV=I4REV+I5-I4
      TEMP=DATA(I5)
      DATA(I5)=DATA(I5REV)
   20 DATA(I5REV)=TEMP
   30 IP2=IP4/2
   40 IF(I4REV-IP2)60,60,50
   50 I4REV=I4REV-IP2
      IP2=IP2/2
      IF(IP2-IP1)60,40,40
   60 I4REV=I4REV+IP2
      RETURN
      END
      SUBROUTINE COOL2(DATA,NPREV,N,NREM,ISIGN)
C	FOURIER TRANSFORM, LENGTH N, BY THE COOLEY-TUKEY ALGORITHM, IN
C	PLACE, BIT-REVERSED TO NORMAL ORDER, SANDE-TUKEY PHASE SHIFTS.
C	COMPLEX DATA
C	DIMENSION DATA(NPREV,N,NREM)
C	DATA(I1,J2,I3) = SUM(DATA(I1,I2,I3)CEXP(ISIGNC2CPICIC((I2-1)C
C	(J2-1)/N))), SUMMED OVER I2 = 1 TO N FOR ALL I1 FROM 1 TO
C	NPREV, J2 FROM 1 TO N AND I3 FROM 1 TO NREM.  N MUST BE A
C	POWER OF TWO.  FACTORING N BY FOUR'S DECREASES RUNNING TIME
C	BY ABOUT TWENTY FIVE PERCENT OVER FACTORING BY TWO'S.
C	GENERATING THE PHASE SHIFT FACTORS BY RECURSION SAVES ABOUT
C	TWENTY FIVE PERCENT OVER COMPUTING THEM, WHILE TABLING THEM
C	IS UNLIKELY TO SAVE MORE THAN ANOTHER TEN PERCENT.
C
C
      DIMENSION DATA(1)
      COMPLEX DATA,TEMP,WSTP,W,W2,W3,T0,T1,T2,T3
      TWOPI=6.28318530717958647692*FLOAT(ISIGN)
      IP0=1
      IP1=IP0*NPREV
      IP4=IP1*N
      IP5=IP4*NREM
      IP2=IP1
      NPART=N
   10 IF(NPART-2)50,30,20
   20 NPART=NPART/4
      GO TO 10
C
C	DO A FOURIER TRANSFORM OF LENGTH TWO
C
   30 IP3=IP2*2
      I1MAX=IP1
      DO 40 I1=1,I1MAX,IP0
      DO 40 I5=I1,IP5,IP3
      I3A=I5
      I3B=I3A+IP2
      TEMP=DATA(I3B)
      DATA(I3B)=DATA(I3A)-TEMP
   40 DATA(I3A)=DATA(I3A)+TEMP
      GO TO 140
C
C	DO A FOURIER TRANSFORM OF LENGTH FOUR (WITHOUT BIT REVERSAL)
C
   50 IP3=IP2*4
      THETA=TWOPI/FLOAT(IP3/IP1)
      SINTH=SIN(THETA/2.)
      WSTP=CMPLX(-2.*SINTH*SINTH,SIN(THETA))
C
C	COS(THETA)-1, FOR ACCURACY
C
      W=1.
      DO 130 I2=1,IP2,IP1
      IF(I2-1)70,70,60
   60 W2=W*W
      W3=W2*W
   70 I1MAX=I2+IP1-IP0
      DO 120 I1=I2,I1MAX,IP0
      DO 120 I5=I1,IP5,IP3
      I3A=I5
      I3B=I3A+IP2
      I3C=I3B+IP2
      I3D=I3C+IP2
      IF(I2-1)90,90,80
C
C	MULTIPLY BY THE PHASE SHIFT FACTORS
C
   80 DATA(I3B)=W2*DATA(I3B)
      DATA(I3C)=W*DATA(I3C)
      DATA(I3D)=W3*DATA(I3D)
   90 T0=DATA(I3A)+DATA(I3B)
      T1=DATA(I3A)-DATA(I3B)
      T2=DATA(I3C)+DATA(I3D)
      T3=DATA(I3C)-DATA(I3D)
      DATA(I3A)=T0+T2
      DATA(I3C)=T0-T2
      TEMP=(0.,1.)*T3
      IF(ISIGN)100,100,110
  100 TEMP=-TEMP
  110 DATA(I3B)=T1+TEMP
  120 DATA(I3D)=T1-TEMP
  130 W=W*WSTP+W
  140 IP2=IP3
      IF(IP3-IP4)50,150,150
  150 RETURN
      END
C
C***<initialise arrays>*************************************************
C
      SUBROUTINE VCOPY(X,Y,N)
C     -----------------------
C
      REAL X(*),Y(*)
C
      DO 10 I=1,N
  10    Y(I)=X(I)
      END
C
      SUBROUTINE VIFILL(INT,K,N)
C     --------------------------
C
      INTEGER INT(*)
C
      DO 10 I=1,N
  10    INT(I)=K
      END
C
      SUBROUTINE VRFILL(X,A,N)
C     ------------------------
C
      REAL X(*)
C
      DO 10 I=1,N
  10    X(I)=A
      END
C
      SUBROUTINE VLFILL(LOG,L,N)
C     --------------------------
C
      LOGICAL LOG(*),L
C
      DO 10 I=1,N
  10    LOG(I)=L
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
