C   Open GENIE, a program for neutron scattering data analysis & display
C   ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
C   Copyright (C) 1996,1997
C
C   This program is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE FNGAUS(X,Y,N,PARS)
C     -----------------------------
C
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C Purpose
C      Calculates the Y-values for a Gaussian-peak sitting on a 
C    straight-line background for a given set of X-points:
C
C       y(x) = Mx + C + A*gaus(X0,SIGMA) ,
C
C                                   1             [ - (x-X0)**2  ]  
C    where:  gaus(X0,SIGMA) = ---------------- exp|--------------| .
C                             SIGMA*sqrt(2*pi)    [ 2 * SIGMA**2 ]
C
C    PARS(1) = M ;  PARS(2) = C  ;
C    PARS(3) = A ;  PARS(4) = X0 ; PARS(5) = SIGMA .
C
C Parameters
C   ARGUMENT  TYPE  I/O  DIMENSION  DESCRIPTION
C    N        I*4    I       -      Number of points.
C    X        R*4    I       N      Array of X-coordinates.
C    Y        R*4    O       N      Array of Y-coordinates.
C    PARS     R*4    I       5      Functional parameters.
C
C History
C   D. S. Sivia      28 Nov 1995  Initial release.
C-----------------------------------------------------------------------
C
      REAL X(*),Y(*),PARS(*)
      DATA SMALL,SQRTPI,SQRT2 /1.0E-20,2.50662828,1.41421356/
C
      XM=PARS(1)
      C=PARS(2)
      A=PARS(3)
      X0=PARS(4)
      SIG=MAX(ABS(PARS(5)),SMALL)
      A=A/(SIG*SQRTPI)
      SIG2=1.0/(SIG*SQRT2)
      DO 10 I=1,N
  10    Y(I)=XM*X(I)+C+A*EXP(-(SIG2*(X(I)-X0))**2)
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
