C   Open GENIE, a program for neutron scattering data analysis & display
C   ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
C   Copyright (C) 1996,1997
C
C   This program is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE FNVOGT(X,Y,N,PARS)
C     -----------------------------
C
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C Purpose
C      Calculates the Y-values for a Voigt-peak sitting on a
C    straight-line background, for a given set of X-points:
C
C       y(x) = Mx + C + A*voigt(X0,SIG,GAM) ,
C
C
C                                       1           [ - (x-X0)**2 ]  
C    where:  voigt(X0,SIG,GAM) =  -------------- exp|-------------| 
C                                 SIG*sqrt(2*pi)    [  2 * SIG**2 ]
C
C                                                     GAM  
C                           Convolved with    ---------------------  .
C                                              pi [ x**2 + GAM**2 ]
C
C
C    PARS(1) = M  ;  PARS(2) = C   ;  PARS(3) = A ;  
C    PARS(4) = X0 ;  PARS(5) = SIG ;  PARS(6) = GAM .
C
C Parameters
C   ARGUMENT  TYPE  I/O  DIMENSION  DESCRIPTION
C    N        I*4    I       -      Number of points.
C    X        R*4    I       N      Array of X-coordinates.
C    Y        R*4    O       N      Array of Y-coordinates.
C    PARS     R*4    I       6      Functional parameters.
C
C External Calls
C   SUBROUTINE   DESCRIPTION
C     VOIGT      Evaluates a Gaussian-Lorentzian convolution (ie Voigt).
C
C History
C   D. S. Sivia      24 Nov 1995  Initial release.
C-----------------------------------------------------------------------
C
      EXTERNAL VOIGT
      REAL     X(*),Y(*),PARS(*)
      DATA     SMALL /1.0E-20/
C
      XM=PARS(1)
      C=PARS(2)
      A=PARS(3)
      X0=PARS(4)
      SIG=MAX(ABS(PARS(5)),SMALL)
      GAM=2.0*MAX(ABS(PARS(6)),SMALL)
      DO 10 I=1,N
  10    Y(I)=XM*X(I)+C+A*VOIGT(X(I)-X0,SIG,GAM)
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
