C   Open GENIE, a program for neutron scattering data analysis & display
C   ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
C   Copyright (C) 1996,1997
C
C   This program is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE PKINIT(X,Y,E,N,XW,YW,EW,XGRD,CONST,X0,XAVG,AMP,VAR,
     *                  IGOOD)
C     --------------------------------------------------------------
C
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C Purpose
C      Gives a crude (but, hopefully, good) estimate of the amplitude, 
C   (maximum and mean) position and variance of a non-parametric peak 
C    sitting on a straight-line background.
C
C Parameters
C   ARGUMENT  TYPE  I/O  DIMENSION  DESCRIPTION
C    N        I*4    I       -      Number of data.
C    X        R*4    I       N      X-coordinate of data.
C    Y        R*4    I       N      Y-coordinate of data.
C    E        R*4    I       N      Error-bar for data.
C    XW,YW,EW R*4    -       N      Work arrays.
C    XGRD     R*4    O       -      Gradient of straight-line backg.
C    CONST    R*4    O       -      Intercept "    "      "     "  .
C    X0       R*4    O       -      Position of (Y) maximum.
C    XAVG     R*4    O       -      Location of mean (<X>).
C    AMP      R*4    O       -      Area under peak.
C    VAR      R*4    O       -      Variance of peak (sigma-squared).
C    IGOOD    I*4    O       -      Negative if a problem occurs:
C                                     < -1 = serious; -1 = may be OK.
C
C Globals
C    PKCOM1
C
C History
C   D. S. Sivia      24 Nov 1995  Initial release.
C   D. S. Sivia      13 Dec 1995  Increased peak-threshold (2.5-->3.5).
C-----------------------------------------------------------------------
C
      COMMON /PKCOM1/ XSCALE,XMIDL,YSCALE
      REAL            X(*),Y(*),E(*),XW(*),YW(*),EW(*)
      LOGICAL         LBACKG
      DATA            SMALL,PKCUT,TINY,BIG /1.0E-20,3.5,1.0E-10,1.0E10/
C
      IGOOD=-4
      IF (N.LE.4) THEN
        CALL FERROR_ADD('PKINIT', 'Sorry, too few data items!', ' ')
        RETURN
      ENDIF
      IGOOD=-3
      XMIDL=(X(1)+X(N))/2.0
      XRANGE=ABS(X(N)-X(1))
      XSCALE=2.0/MIN(MAX(XRANGE,TINY),BIG)
      YMAX=TINY
      DO 10 I=1,N
        EW(I)=0.0
        XW(I)=XSCALE*(X(I)-XMIDL)
        IF (E(I).GT.SMALL) THEN
          EW(I)=1.0/E(I)
          YMAX=MAX(Y(I),YMAX)
        ENDIF
  10  CONTINUE
      YMAX=MIN(YMAX,BIG)
      YSCALE=1.0/YMAX
      DO 20 I=1,N
        YW(I)=YSCALE*Y(I)
        EW(I)=YMAX*EW(I)
  20  CONTINUE
C
      A=0.0
      B=0.0
      C=0.0
      P=0.0
      Q=0.0
      NHALF=N/2-1
      LBACKG=.FALSE.
      NBKG=0
      DO 40 I=1,NHALF
        J=N+1-I
        WI=2.0*EW(I)**2
        WJ=2.0*EW(J)**2
        IF (WI.GT.SMALL) NBKG=NBKG+1
        IF (WJ.GT.SMALL) NBKG=NBKG+1
        A=A+WI*XW(I)**2+WJ*XW(J)**2
        B=B+WI+WJ
        C=C+WI*XW(I)+WJ*XW(J)
        P=P+WI*XW(I)*YW(I)+WJ*XW(J)*YW(J)
        Q=Q+WI*YW(I)+WJ*YW(J)
        DENOM=A*B-C**2
        IF (DENOM.GT.SMALL) THEN
          XM=(B*P-C*Q)/DENOM
          XC=(A*Q-C*P)/DENOM
          CHI=0.0
          DO 30 K=1,I
            L=N+1-K
            CHI=CHI+((XM*XW(K)+XC-YW(K))*EW(K))**2
     *             +((XM*XW(L)+XC-YW(L))*EW(L))**2
  30      CONTINUE
          CHICUT=FLOAT(NBKG)+SQRT(FLOAT(2*NBKG))
          IF (LBACKG .AND. CHI.GT.CHICUT) GOTO 1
          LBACKG=.TRUE.
          XGRD=XM
          CONST=XC
        ENDIF
  40  CONTINUE
   1  IF (.NOT. LBACKG) RETURN
      IGOOD=-2
C
      SUM=0.0
      DO 50 I=1,N
        SIG2NS=(YW(I)-XGRD*XW(I)-CONST)*EW(I)
        SUM=SUM+ABS(SIG2NS)
        IF (SIG2NS.GT.PKCUT) GOTO 2
  50  CONTINUE
   2  IMIN=I
      DO 60 I=N,IMIN,-1
        SIG2NS=(YW(I)-XGRD*XW(I)-CONST)*EW(I)
        SUM=SUM+ABS(SIG2NS)
        IF (SIG2NS.GT.PKCUT) GOTO 3
  60  CONTINUE
   3  IMAX=I
      A=0.0
      B=0.0
      C=0.0
      YMAX=0.0
      IF (IMIN.LE.IMAX .AND. IMIN.LT.N) THEN
        IMIN=MAX(IMIN-2,2)
        IMAX=MIN(IMAX+2,N-1)
        DO 70 I=IMIN,IMAX
          IF (EW(I).GT.SMALL) THEN
            YI=YW(I)-XGRD*XW(I)-CONST
            IF (YI.GT.YMAX) THEN
              YMAX=YI
              X0=X(I)
            ENDIF
            YIDX=YI*0.5*ABS(XW(I+1)-XW(I-1))
            A=A+YIDX
            B=B+YIDX*XW(I)
            C=C+YIDX*XW(I)**2
          ENDIF
  70    CONTINUE
      ELSE
        SIGAVG=SUM/FLOAT(IMIN+N-IMAX)
        NBIN=MIN(MAX(NINT((2.0/SIGAVG)**2),1),(N/7))
        DO 90 J=2,N-NBIN,NBIN
          NSUM=0
          XSUM=0.0
          YSUM=0.0
          DO 80 I=J,J+NBIN-1
            IF (EW(I).GT.SMALL) THEN
              NSUM=NSUM+1
              XSUM=XSUM+XW(I)
              YSUM=YSUM+YW(I)
            ENDIF
  80      CONTINUE
          IF (NSUM.GT.0) THEN
            XWI=XSUM/FLOAT(NSUM)
            YI=YSUM/FLOAT(NSUM)-XGRD*XWI-CONST
            IF (YI.GT.YMAX) THEN
              YMAX=YI
              X0=XWI/XSCALE+XMIDL
            ENDIF
            YIDX=YI*0.5*ABS(XW(J+NBIN)+XW(J+NBIN-1)-XW(J)-XW(J-1))
            A=A+YIDX
            B=B+YIDX*XWI
            C=C+YIDX*XWI**2
          ENDIF
  90    CONTINUE
      ENDIF
      XGRD=XGRD*XSCALE/YSCALE
      CONST=CONST/YSCALE-XGRD*XMIDL
      AMP=A/(XSCALE*YSCALE)
      VAR=-SMALL
      IF (A.LT.SMALL) RETURN
      IGOOD=-1
      XAVG=B/A
      VAR=C/A-XAVG**2
      XAVG=XAVG/XSCALE+XMIDL
      VAR=VAR/XSCALE**2
      IF (VAR.GT.SMALL) IGOOD=+1
      VARMIN=(XRANGE/(2.35*FLOAT(N)))**2
      VARMAX=(XRANGE/2.35)**2
      VAR=MAX(MIN(VAR,VARMAX),VARMIN)
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
