C   Open GENIE, a program for neutron scattering data analysis & display
C   ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
C   Copyright (C) 1996,1997
C
C   This program is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE PKLEXP(X,Y,E,N,XW,YW,EW,PARS,SIGPAR,IPROPT,IGOOD)
C     ------------------------------------------------------------
C
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C Purpose
C      Estimates the parameters of a peak, consisting of a Lorentzian 
C    convolved with a sharp-edged exponential, sitting on a straight-
C    line background, given a pertinent set of data:
C
C       y(x) = Mx + C + A*lzexp(X0,GAM,TAU) ,
C
C
C                                            GAM  
C    where:  lzexp(X0,GAM,TAU) =   -------------------------
C                                  pi [ (x-X0)**2 + GAM**2 ]
C
C
C                                   {        0           if x/TAU < 0
C                  Convolved with   {                                  .
C                                   { exp(-x/TAU)/|TAU|  if x/TAU > 0 
C
C
C    PARS(1) = M  ;  PARS(2) = C   ;  PARS(3) = A ;  
C    PARS(4) = X0 ;  PARS(5) = GAM ;  PARS(6) = TAU .
C
C Parameters
C   ARGUMENT  TYPE  I/O  DIMENSION  DESCRIPTION
C    N        I*4    I       -      Number of data.
C    X        R*4    I       N      X-coordinate of data.
C    Y        R*4    I       N      Y-coordinate of data.
C    E        R*4    I       N      Error-bar for data.
C    XW,YW,EW R*4    -       N      Work arrays.
C    PARS     R*4    I       6      Parameter-values that are to be 
C                                   fixed or used for the initial guess.
C    PARS     R*4    O       6      Best estimates of the parameters.
C    SIGPAR   R*4    O       6      1-sigma error-bars for PARS; can be
C                                   negative if IGOOD < 0!
C    IPROPT   I*4    I       6      Option for treatment of parameters:
C                                     0 = no initial estimate;
C                                     1 = use as "      "    ;
C                                     2 = fix at "      "    .
C    IGOOD    I*4    O       -      Negative if a problem occurs:
C                                     < -1 = serious; -1 = may be OK.
C
C Globals
C    PKCOM1
C    PKCOM2
C
C External Calls
C   SUBROUTINE   DESCRIPTION
C     PKINIT     Gives crude non-parametric estimate of peak properties.
C     CHLEXP     Calculates "chi-squared" misfit.
C
C History
C   D. S. Sivia      24 Nov 1995  Initial release.
C-----------------------------------------------------------------------
C
      EXTERNAL        CHLEXP
      REAL            CHLEXP
      REAL            X(*),Y(*),E(*),XW(*),YW(*),EW(*),PARS(*),SIGPAR(*)
      INTEGER         IPROPT(*)
      COMMON /PKCOM1/ XSCALE,XMIDL,YSCALE
      INCLUDE 'pkdefs.inc'
      DATA            SMALL /1.0E-20/
C
      IGOOD=-10
      NPARS=0
      DO 10 I=1,6
        IF (IPROPT(I).EQ.2) THEN
          PKPARS(I)=PARS(I)
          SIGPAR(I)=0.0
        ELSE
   1      NPARS=NPARS+1
          IPARS(NPARS)=I
          IF (IPROPT(I).EQ.1) PKPARS(I)=PARS(I)
        ENDIF
  10  CONTINUE
      IF (NPARS.EQ.0 .OR. N.LE.NPARS) RETURN
      CALL PKINIT(X,Y,E,N,XW,YW,EW,PARS(1),PARS(2),XYMAX,XAVG,PARS(3),
     *            VAR,IGOOD)
      IF (IGOOD.LT.-2) THEN
        CALL VCOPY(PKPARS,PARS,6)
        RETURN
      ENDIF
      PARS(4)=XYMAX
      PARS(5)=0.5*SQRT(ABS(VAR)+SMALL)
      PARS(6)=1.73205*PARS(5)
      IF (VAR.LT.SMALL) PARS(5)=-PARS(5)
      IF (XAVG.LT.XYMAX) PARS(6)=-PARS(6)
      DO 20 I=1,NPARS
        J=IPARS(I)
        IF (IPROPT(J).EQ.0) PKPARS(J)=PARS(J)
  20  CONTINUE
      IF (PKPARS(3).LT.SMALL .OR. PKPARS(5).LE.SMALL) THEN
        CALL VCOPY(PKPARS,PARS,6)
        PARS(3)=5.0*SMALL
        PARS(5)=0.1*ABS(X(N)-X(1))
        PARS(6)=PARS(5)
        RETURN
      ENDIF
      XSCL1=1.0/XSCALE
      YSCL1=1.0/YSCALE
      PKPARS(2)=(PKPARS(1)*XMIDL+PKPARS(2))*YSCALE
      PKPARS(1)=PKPARS(1)*YSCALE*XSCL1
      PKPARS(3)=PKPARS(3)*XSCALE*YSCALE
      PKPARS(4)=(PKPARS(4)-XMIDL)*XSCALE
      PKPARS(5)=PKPARS(5)*XSCALE
      PKPARS(6)=PKPARS(6)*XSCALE
      DPAR(1)=0.1*((MIN(ABS(PKPARS(1)),1.0E10))**2+1.0)
      DPAR(2)=0.1*MAX(ABS(PKPARS(2)),MAX(SMALL,E(2),E(N-1)))
      DPAR(3)=0.1*PKPARS(3)
      DPAR(4)=0.1*SQRT(PKPARS(5)**2+PKPARS(6)**2)
      DPAR(5)=0.15*PKPARS(5)
      DPAR(6)=0.15*PKPARS(6)
      DO 30 I=1,NPARS
        J=IPARS(I)
        PARS(I)=PKPARS(J)
        DPAR(I)=DPAR(J)
  30  CONTINUE
      IGOOD=ISMPLX(PARS,DPAR,COVAR,NPARS,.FALSE.,CHLEXP,XW,YW,EW,N)
      DO 40 I=1,NPARS
        J=IPARS(I)
        PKPARS(J)=PARS(I)
        SIG2=COVAR((I-1)*NPARS+I)
        SIGPAR(J)=SQRT(ABS(SIG2)+SMALL)
        IF (SIG2.LT.SMALL) THEN
          SIGPAR(J)=-SIGPAR(J)
          IGOOD=-1
        ENDIF
  40  CONTINUE
      PARS(1)=PKPARS(1)*XSCALE*YSCL1
      PARS(2)=PKPARS(2)*YSCL1-PARS(1)*XMIDL
      PARS(3)=PKPARS(3)*XSCL1*YSCL1
      PARS(4)=PKPARS(4)*XSCL1+XMIDL
      PARS(5)=PKPARS(5)*XSCL1
      PARS(6)=PKPARS(6)*XSCL1
      SIGPAR(1)=SIGPAR(1)*XSCALE*YSCL1
      IF (IPROPT(2).NE.2) THEN
        IF (IPROPT(1).EQ.2) THEN
          SIGPAR(2)=SIGPAR(2)*YSCL1
        ELSE
          XSM=XSCALE*XMIDL
          SIGC2=COVAR(NPARS+2)+COVAR(1)*XSM**2-2.0*XSM*COVAR(2)
          SIGPAR(2)=SQRT(ABS(SIGC2)+SMALL)*YSCL1
          IF (SIGC2.LT.SMALL) THEN
            SIGPAR(2)=-SIGPAR(2)
            IGOOD=-1
          ENDIF
        ENDIF
      ENDIF
      SIGPAR(3)=SIGPAR(3)*XSCL1*YSCL1
      SIGPAR(4)=SIGPAR(4)*XSCL1
      SIGPAR(5)=SIGPAR(5)*XSCL1
      SIGPAR(6)=SIGPAR(6)*XSCL1
      END
C
      FUNCTION CHLEXP(PARS,NPARS,X,D,EW,N,IOFF)
C     ------------------------------------
C
      EXTERNAL        LZEXP
      REAL            PARS(*),X(*),D(*),EW(*),LZEXP
      INCLUDE 'pkdefs.inc'
      DATA            SMALL /1.0E-20/
C
      CHLEXP=1.0E10
      DO 10 I=1,NPARS
  10    PKPARS(IPARS(I))=PARS(I)
      XM=PKPARS(1+IOFF)
      C=PKPARS(2+IOFF)
      A=PKPARS(3+IOFF)
      X0=PKPARS(4+IOFF)
      GAM=2.0*PKPARS(5+IOFF)
      TAU=MAX(ABS(PKPARS(6+IOFF)),SMALL)
      TSIGN=1.0
      IF (PKPARS(6+IOFF).LT.0.0) TSIGN=-1.0 
      IF (A.LE.SMALL .OR. GAM.LE.SMALL) RETURN
      CHI=0.0
      DO 20 I=1,N
        Y=XM*X(I)+C+A*LZEXP(TSIGN*(X(I)-X0),GAM,TAU)
        CHI=CHI+((Y-D(I))*EW(I))**2
  20  CONTINUE
      CHLEXP=CHI
      END
C
C***<Bill David's routines (renamed & modified)>************************
c
	REAL function LZEXP(x,fwhm,tau)
C       -------------------------------
C
c..	This function corresponds to a normalised convolution of
c	the exponential function     (1/tau)*exp(-x/tau)    and 
c	the Lorentzian function	  (fwhm/2pi)/[(fwhm/2)**2 + x**2]
c..	W.I.F.David		23-Jan-86
c
	beta= 1./tau
	ax= -x*beta
	ay= 0.5*fwhm*beta
	anorm=  0.31830988*beta
	call EXPINT(ax,ay,eimag)
	LZEXP= -anorm*eimag
	end
c
	subroutine EXPINT(ax,ay,ansy)
C       -----------------------------
c
c..	ansx and ansy are the real and imaginary components
c..	of the function   exp(z)E(z)       where   E(z)  is
c..	the exponential integral function.  The complex
c..	argument z  equals   ax + i ay.
c..	W.I.F.David	1-SEP-84
c
	implicit complex*8	(b-h,o-z)
c
	z= cmplx(ax,ay)
	aych= -1.
	if (ax .lt. 0.) aych= 0.125*ax+4.
        a2= ax**2+ay**2
	if (a2 .lt. 16. .or. abs(ay) .lt. aych) then
          a= sqrt(a2)
          go to 50
        endif
	if (ax .lt. 0.) then
          a=sqrt(0.111111*(ax+29.)*(ax+29.)+ay*ay)
        else
          a= sqrt(a2)
        endif
	iter= 4 + nint(128./a)
	tem= float(iter)
	add= tem
	do 10 i=1,iter-1
	tem= tem - 1.
	sum= z + add
	rat= tem/sum
 10	add= tem/(1.+rat)
	zans= 1./(z+add)
	go to 999
c
 50	iter= 10 + 4*nint(a)
	euler= 0.5772156649
	cln= clog(z)
c
	tem= float(iter)*z/float((iter+1)*(iter+1))
	sum= tem
	do 40 j= iter,2,-1
	t= 1./float(j)
	sub= 1.-sum
	tem= tem*(1.+t)*(1-t*t)
 40	sum= tem*sub
	sum= -euler-cln+z*(1.-sum)
	zans= sum*cexp(z)
c
 999    ansy= aimag(zans)
	end
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
