C   Open GENIE, a program for neutron scattering data analysis & display
C   ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
C   Copyright (C) 1996,1997
C
C   This program is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE PKLORZ(X,Y,E,N,XW,YW,EW,PARS,SIGPAR,IPROPT,IGOOD)
C     ------------------------------------------------------------
C
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C Purpose
C      Estimates the parameters of a Lorentzian-peak sitting on a
C    straight-line background, given a pertinent set of data:
C
C       y(x) = Mx + C + A*lorentz(X0,GAMMA) ,
C
C                                          GAMMA  
C    where:  lorentz(X0,GAMMA) = ---------------------------  .
C                                pi [ (x-X0)**2 + GAMMA**2 ]
C
C    PARS(1) = M ;  PARS(2) = C  ;
C    PARS(3) = A ;  PARS(4) = X0 ; PARS(5) = GAMMA .
C
C Parameters
C   ARGUMENT  TYPE  I/O  DIMENSION  DESCRIPTION
C    N        I*4    I       -      Number of data.
C    X        R*4    I       N      X-coordinate of data.
C    Y        R*4    I       N      Y-coordinate of data.
C    E        R*4    I       N      Error-bar for data.
C    XW,YW,EW R*4    -       N      Work arrays.
C    PARS     R*4    I       5      Parameter-values that are to be 
C                                   fixed or used for the initial guess.
C    PARS     R*4    O       5      Best estimates of the parameters.
C    SIGPAR   R*4    O       5      1-sigma error-bars for PARS; can be
C                                   negative if IGOOD < 0!
C    IPROPT   I*4    I       5      Option for treatment of parameters:
C                                     0 = no initial estimate;
C                                     1 = use as "      "    ;
C                                     2 = fix at "      "    .
C    IGOOD    I*4    O       -      Negative if a problem occurs:
C                                     < -1 = serious; -1 = may be OK.
C
C Globals
C    PKCOM1
C    PKCOM2
C
C External Calls
C   SUBROUTINE   DESCRIPTION
C     PKINIT     Gives crude non-parametric estimate of peak properties.
C     CHLORZ     Calculates "chi-squared" misfit for Lorentzian peak.
C
C History
C   D. S. Sivia      24 Nov 1995  Initial release.
C-----------------------------------------------------------------------
C
      EXTERNAL        CHLORZ
      REAL            CHLORZ
      REAL            X(*),Y(*),E(*),XW(*),YW(*),EW(*),PARS(*),SIGPAR(*)
      INTEGER         IPROPT(*)
      COMMON /PKCOM1/ XSCALE,XMIDL,YSCALE
      INCLUDE 'pkdefs.inc'
      DATA            SMALL /1.0E-20/
C
      IGOOD=-10
      NPARS=0
      DO 10 I=1,5
        IF (IPROPT(I).EQ.2) THEN
          PKPARS(I)=PARS(I)
          SIGPAR(I)=0.0
        ELSE
          NPARS=NPARS+1
          IPARS(NPARS)=I
          IF (IPROPT(I).EQ.1) PKPARS(I)=PARS(I)
        ENDIF
  10  CONTINUE
      IF (NPARS.EQ.0 .OR. N.LE.NPARS) RETURN
      CALL PKINIT(X,Y,E,N,XW,YW,EW,PARS(1),PARS(2),XYMAX,PARS(4),
     *            PARS(3),VAR,IGOOD)
      IF (IGOOD.LT.-2) THEN
        CALL VCOPY(PKPARS,PARS,5)
        RETURN
      ENDIF
      PARS(5)=SQRT(ABS(VAR)+SMALL)
      IF (VAR.LT.SMALL) PARS(5)=-PARS(5)
      DO 20 I=1,NPARS
        J=IPARS(I)
        IF (IPROPT(J).EQ.0) PKPARS(J)=PARS(J)
  20  CONTINUE
      IF (PKPARS(3).LT.SMALL .OR. PKPARS(5).LE.SMALL) THEN
        CALL VCOPY(PKPARS,PARS,5)
        PARS(3)=5.0*SMALL
        PARS(5)=0.1*ABS(X(N)-X(1))
        RETURN
      ENDIF
      XSCL1=1.0/XSCALE
      YSCL1=1.0/YSCALE
      PKPARS(2)=(PKPARS(1)*XMIDL+PKPARS(2))*YSCALE
      PKPARS(1)=PKPARS(1)*YSCALE*XSCL1
      PKPARS(3)=PKPARS(3)*XSCALE*YSCALE
      PKPARS(4)=(PKPARS(4)-XMIDL)*XSCALE
      PKPARS(5)=PKPARS(5)*XSCALE
      DPAR(1)=0.1*((MIN(ABS(PKPARS(1)),1.0E10))**2+1.0)
      DPAR(2)=0.1*MAX(ABS(PKPARS(2)),MAX(SMALL,E(2),E(N-1)))
      DPAR(3)=0.1*PKPARS(3)
      DPAR(4)=0.1*PKPARS(5)
      DPAR(5)=0.1*PKPARS(5)
      DO 30 I=1,NPARS
        J=IPARS(I)
        PARS(I)=PKPARS(J)
        DPAR(I)=DPAR(J)
  30  CONTINUE
      IGOOD=ISMPLX(PARS,DPAR,COVAR,NPARS,.FALSE.,CHLORZ,XW,YW,EW,N)
      DO 40 I=1,NPARS
        J=IPARS(I)
        PKPARS(J)=PARS(I)
        SIG2=COVAR((I-1)*NPARS+I)
        SIGPAR(J)=SQRT(ABS(SIG2)+SMALL)
        IF (SIG2.LT.SMALL) THEN
          SIGPAR(J)=-SIGPAR(J)
          IGOOD=-1
        ENDIF
  40  CONTINUE
      PARS(1)=PKPARS(1)*XSCALE*YSCL1
      PARS(2)=PKPARS(2)*YSCL1-PARS(1)*XMIDL
      PARS(3)=PKPARS(3)*XSCL1*YSCL1
      PARS(4)=PKPARS(4)*XSCL1+XMIDL
      PARS(5)=PKPARS(5)*XSCL1
      SIGPAR(1)=SIGPAR(1)*XSCALE*YSCL1
      IF (IPROPT(2).NE.2) THEN
        IF (IPROPT(1).EQ.2) THEN
          SIGPAR(2)=SIGPAR(2)*YSCL1
        ELSE
          XSM=XSCALE*XMIDL
          SIGC2=COVAR(NPARS+2)+COVAR(1)*XSM**2-2.0*XSM*COVAR(2)
          SIGPAR(2)=SQRT(ABS(SIGC2)+SMALL)*YSCL1
          IF (SIGC2.LT.SMALL) THEN
            SIGPAR(2)=-SIGPAR(2)
            IGOOD=-1
          ENDIF
        ENDIF
      ENDIF
      SIGPAR(3)=SIGPAR(3)*XSCL1*YSCL1
      SIGPAR(4)=SIGPAR(4)*XSCL1
      SIGPAR(5)=SIGPAR(5)*XSCL1
      END
C
      FUNCTION CHLORZ(PARS,NPARS,X,D,EW,N,IOFF)
C     ------------------------------------
C
      REAL            PARS(*),X(*),D(*),EW(*)
      INCLUDE 'pkdefs.inc'
      DATA            SMALL,PI /1.0E-20,3.141592654/
C
      CHLORZ=1.0E10
      DO 10 I=1,NPARS
  10    PKPARS(IPARS(I))=PARS(I)
      XM=PKPARS(1+IOFF)
      C=PKPARS(2+IOFF)
      A=PKPARS(3+IOFF)
      X0=PKPARS(4+IOFF)
      GAM=PKPARS(5+IOFF)
      IF (A.LE.SMALL .OR. GAM.LE.SMALL) RETURN
      A=A*GAM/PI
      GAM2=GAM**2
      CHI=0.0
      DO 20 I=1,N
        Y=XM*X(I)+C+A/((X(I)-X0)**2+GAM2)
        CHI=CHI+((Y-D(I))*EW(I))**2
  20  CONTINUE
      CHLORZ=CHI
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
